<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;
use Monolog\Handler\StreamHandler;

class BenefitTypesCloneCommand extends Command
{
    protected $signature = 'sync:benefit_types';
    protected $description = 'Sync tables from server to client';

    public function handle()
    {
        try {
            Log::channel('custom-command-log')->info('Benefit Types Command Initiated successfully.');

            $allowed_tables = ["benefit_types"];

            foreach ($allowed_tables as $tableName) {
                // Access the API endpoint
                $response = Http::get(env('CONSOLE_API') . 'get-benefit-types-clone/' . $tableName);


                if ($response->successful()) {
                    $data = $response->json();

                    // Drop table if exist statement
                    DB::statement("DROP TABLE IF EXISTS $tableName");
                    $this->info("Table Dropped If Exist\n");

                    // Run the create table statement
                    DB::statement($data['create_table_statement']);
                    $this->info("Create Statements executed successfully\n");

                    // Run the insert statement
//                if($tableName == 'merchant_branches'){
//                    var_dump($data['insert_statement']);
//                    exit();
//                }
                    // DB::statement($data['insert_statement']);
                    $this->insertOrUpdate($tableName, $data['table_data']);
                    $this->info("Insert Statements executed successfully\n");

                } else {
                    $this->error("Benefit Types Error accessing API endpoint: {$response->status()}");
                }
            }
        } catch (\Exception $e) {
            // Log any exception or error that occurred
            Log::channel('custom-command-log')->info('Benefit Types Error executing command: ' . $e->getMessage() );

        }finally {
            // Create a new log file for the next command execution
            $this->rotateLogFile();
        }
    }

    private function insertOrUpdate($table,array $rows){

        if(count($rows) <= 0){
            return true;
        }

        $first = reset($rows);

        $columns = implode( ',',
            array_map( function( $value ) { return "$value"; } , array_keys($first) )
        );
        $values = implode( ',', array_map( function( $row ) {
                return '('.implode( ',',
                        array_map( function( $value ) {
                            if(!$value){
                                return var_export($value, true);
                            }else{
                                return '"'.str_replace('"', '""', $value).'"';
                            }
                            } , $row )
                    ).')';
            } , $rows )
        );
        $updates = implode( ',',
            array_map( function( $value ) { return "$value = VALUES($value)"; } , array_keys($first) )
        );
        $sql = "INSERT INTO {$table}({$columns}) VALUES {$values} ON DUPLICATE KEY UPDATE {$updates}";

        return \DB::statement( $sql );
    }

    private function rotateLogFile()
    {
        Log::channel('custom-command-log')->info('Benefit Types Command completed successfully on - '. now()->format('Y-m-d_H-i-s') );

    }

}
