<?php

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Route;
use \Illuminate\Support\Arr;
use \Illuminate\Support\Str;
use \App\Models\Role;
use \App\Models\Applicant;
use \App\Models\Beneficiary;
use \App\Models\Benefit;
use App\Models\BenefitAmount;
use App\Models\ContactDetail;
use App\Models\EmploymentDetail;
use App\Models\GeneralNotification;
use \App\Models\LookupCode;
use \App\Models\Instance;
use \Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Storage;
use Illuminate\Http\Request;
use \App\Models\Office;
use \App\Http\Controllers\Api\LocationApiController;
use \App\Http\Controllers\Api\MerchantsApiController;
use App\Models\Deployment;
use App\Models\Pmt;
use App\Models\Region;
use App\Models\Village;
use App\Repositories\LocationRepository;

function getTableColumns($table)
{
    return DB::getSchemaBuilder()->getColumnListing($table);

    // OR

//    return \Doctrine\DBAL\Schema\Schema::getColumnListing($table);

}

function getActiveInstance()
{
    if (\Illuminate\Support\Facades\Auth::check()) {
        if (auth()->user()->last_instance_opened == null) return 0;
        return auth()->user()->last_instance_opened;
    }
    else {
        return 0;
    }

}

function get_last_instance($id){
    $deployment = Deployment::find($id);
    if($deployment){
        return $deployment;
    }else{
        return 0;
    }   
    
}



function systemLogo($placeholder = null)
{
    $filePath = \setting('logo');
    if (is_null($placeholder)) $placeholder = config('app_settings.logo');

    $disk = collect(Config::get('app_settings.sections.app.inputs'))
        ->where('disk','!=',null)
        ->pluck('disk')->first();

    if (!empty($filePath)) {
        return Storage::disk($disk)->exists($filePath) ?
            Storage::disk($disk)->url($filePath) : asset($placeholder);
    } else return asset($placeholder);

}

function verificationCode($characters=4){
    if (!env('APP_DEBUG')) {
        return strtoupper(Str::random($characters));
    }
    else {
        return '0000';
    }
}

function canTour($page) {
    $can_tour = false;
    $touring_areas = [];
    if (!is_null(auth()->user()->touring_areas)) $touring_areas = auth()->user()->touring_areas;
    if(!auth()->user()->has_toured && in_array($page,$touring_areas)) $can_tour = true;

    return $can_tour;
}

function updateNotification($subject_id,$table_name){
    $receivers = GeneralNotification::whereJsonContains('receivers',auth()->id())
            ->where('table','=',$table_name)
            ->where('subject_id','=',$subject_id)
            ->first();

    if ($receivers) {
        $new_receivers = findReplaceItem(auth()->id(),$receivers->receivers);

        $receivers->receivers = $new_receivers;
        if ($receivers->has_read == null) {
            $receivers->has_read = array();
        }
        $has_read = $receivers->has_read;
        $has_read[] = auth()->id();
        $receivers->has_read = $has_read;
        $receivers->save();
    }
}

function sendSMS($contact,string $message) {
        $response = \Illuminate\Support\Facades\Http::get(env('SMS_URI'),[
            'key'   =>  env('SMS_API_KEY'),
            'type'   =>  0,
            'text'   =>  $message,
            'sender'   =>  env('SMS_SENDER_ID'),
            'to'   =>  $contact,
        ]);

        return $response->successful();
}

function sendMultipleSMS(array $contacts,string $message) {
    foreach ($contacts as $key=>$contact) {
        $response = sendSMS($contact,$message);
    }
}

function randomBgColor() {
    $background_colors = ['bg-radial-gradient-success','bg-radial-gradient-primary','bg-radial-gradient-dark','bg-gradient-primary',
//        'bg-gradient-dark','bg-gradient-success','bg-gradient-warning','bg-gradient-danger'
//        ,'bg-gradient-secondary','bg-gradient-info','bg-radial-gradient-info',
//        'bg-radial-gradient-warning'
//        ,'bg-radial-gradient-danger'
    ];
    return Arr::random($background_colors);
}

function findReplaceItem($item, array $array) {
    $pos = array_search($item, $array);
    if ($pos !== false) {
        unset($array[$pos]);
    }
    return $array;
}

function getNextStage($current_lookup_id) {
    $getLookup = Lookup::find($current_lookup_id);
    $next_status = 0;
    if ($getLookup) {
        $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->pluck('id')->toArray();
        $checkOrder = StageEmail::whereIn('lookup_id', $get_other_options)->pluck('order','lookup_id')->toArray();
        $current_order = $checkOrder[$current_lookup_id];
        foreach ($checkOrder as $lookup_id=>$order) {
            if ($checkOrder[$lookup_id] > $current_order) {
                if (isDeclinedProcess($lookup_id)){
                    continue;
                }else{
                    $next_status = $lookup_id;
                    break;
                }
            }
        }
    }

    return $next_status;
}

function getDeclinedStage($current_lookup_id) {
    $getLookup = Lookup::find($current_lookup_id);
    $next_status = 0;
    if ($getLookup) {
        $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->pluck('id')->toArray();
        $checkOrder = StageEmail::whereIn('lookup_id', $get_other_options)->pluck('lookup_id','order')->toArray();
        foreach ($checkOrder as $order=>$lookup_id) {
            if (isDeclinedProcess($lookup_id)){
                $next_status = $lookup_id;
                break;
            }
        }
    }

    return $next_status;
}

function getPreviousStage($current_lookup_id) {
    $getLookup = Lookup::find($current_lookup_id);
    $next_status = 0;
    if ($getLookup) {
        $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->pluck('id')->toArray();
        $get_current_stage = StageEmail::where('lookup_id', '=', $current_lookup_id)->first();
        $checkOrder = StageEmail::whereIn('lookup_id', $get_other_options)->orderBy('order', 'desc')->pluck('lookup_id','order')->toArray();
//        dd($checkOrder);
//        $current_order = $checkOrder[$current_lookup_id];
//        $count = 1;
        for ($count = 1; $count <= count($checkOrder); $count--) {
            if (isset($checkOrder[$get_current_stage->order - $count])) {
//                dd($checkOrder[$get_current_stage->order - $count]);
//                dd($checkOrder[$get_current_stage->order - $count]);
//                if ($checkOrder[$get_current_stage->order - $count] < $get_current_stage->order) {
                if (isStartProcess($checkOrder[$get_current_stage->order])) {
                    $next_status = 0; break;
                }
                else {
                    $next_status = $checkOrder[$get_current_stage->order - $count];
                    break;
                }
//                }
            }
        }
//        foreach ($checkOrder as $order=>$lookup_id) {
//            if (isset($checkOrder[$get_current_stage->order - $count])) {
////                dd($checkOrder[$get_current_stage->order - $count]);
//                if ($checkOrder[$get_current_stage->order - $count] == $lookup_id) {
//                    if (isStartProcess($lookup_id)) {
//                        $next_status = 0; break;
//                    }
//                    else {
//                        $next_status = $lookup_id;
//                        break;
//                    }
//                }
//            }
//            else {
//                $count+=1;
//            }
//
//
//        }
    }
//    if($next_status == null) {
//        $next_status = null;
//    }

    return $next_status;
}

/**
 * Get Active Lookup Option by the Lookup Code Parent ID
 * @param $lookupCode_id
 * @return mixed
 */
function getActiveOptionsFor($lookupCode_id) {
    return LookupCode::activeOptions($lookupCode_id);
}

function getActiveOptions($lookupCode_id) {

    return LookupCode::activeOptionsFull($lookupCode_id);
}
/**
 * Get Selected Lookup Option by the Lookup ID
 * @param $field_id
 * @param $value
 * @return mixed
 */
function getLookupSelectedOptionTextByValue($value) {
    if (is_numeric($value)) {
        $lookup_text = \App\Models\Lookup::find($value);
        if (isset($lookup_text)) {
            return $lookup_text->fullName;
        }
        else {
            return $value;
        }
    }
    return $value;
}
/**
 * Get All Lookup Option(Active & Inactive) by the Lookup Code Parent ID
 * @param $lookupCode_id
 * @return mixed
 */
function getAllOptionsFor($lookupCode_id) {
    return LookupCode::allOptions($lookupCode_id);
}

/**
 * @param array $route
 * @param bool $multilevel
 * @param string $output
 * @return string
 */
function isRouteActive( array $route, $multilevel = false, $output = 'active' ) {

    $current_route_name = Route::currentRouteName();

    if ( $multilevel ) {
        return ( in_array($current_route_name,$route) ) ? 'active' : '';
    }

    return (in_array($current_route_name,$route) ) ? $output : '';
}

/**
 * Helper function to show flash messages using bootstrap notify
 *
 * @param string $title
 * @param $message
 * @param string $alert_type
 * @param string $position
 */
function notify(string $body, string $alert_type = 'primary', string $header = 'Notification',string $position = 'top-right' ) {
    $icons = [
        "success" => 'fa fa-check-circle-o text-success',
        "danger"  => 'fa fa-exclamation-triangle',
        "warning" => 'fa fa-exclamation-triangle',
        "info"    => 'fa fa-exclamation-circle',
        "primary" => 'fa fa-dot-circle-o',
        "error" => 'fa fa-times-circle-o'
    ];

    $icon = $icons[$alert_type];
    if (!isset($icons[$alert_type])) {
        $icon = 'primary';
    }
    if ($alert_type == 'error') {
        $alert_type = 'danger';
    }

    session()->flash('notification',true);
    session()->flash('header','<i style="font-size:20px" class="text-'.$alert_type.' me-2 '.$icon.'"></i><h6 class="mb-0">'.$header.'</h6>');
    session()->flash('body',$body);
    session()->flash('position',$position);
}

/**
 * Helper function to show flash messages using notify
 *
 * @param string $title
 * @param $message
 * @param string $type
 */
function alertNotify( $message, $title = null,  $type = 'info') {
    session()->flash( 'ShowNotify',true);
    session()->flash( 'notifyTitle',$title);
    session()->flash( 'notifyMessage',$message);
    session()->flash( 'notifyAlertType',$type);
}

function clearNotify() {
    session()->forget(['ShowNotify', 'notifyTitle','notifyMessage','notifyAlertType']);
}
function clearErroPage() {
    session()->forget(['error-page', 'page-message', 'page-message-title']);
}

/**
 *Get the current route name. The name returned is the route name specified in the web routes file.
 *
 * @return string
 */
function currentRouteName() {
    $route_name = Route::currentRouteName();
    $breadcrumb = str_replace( [ ".", "_" ], [ " / ", " " ], $route_name );

    return $breadcrumb;
}

/**
 * Get last auto increment id of a model
 *
 * @param Model $model
 *
 * @return int
 */
function getLastId( Model $model ) {
    $models = $model::all();

    if ( $models->isEmpty() ) {
        return 1;
    } else {
        return (int) $models->last()->id + 1;
    }
}

/**
 * Generate a unique Id for a model
 *
 * @param Model $model
 * @param string $name
 * @param string $prefix
 * @param bool $is_random
 *
 * @return string
 * @throws Exception
 */
function generateModelId( Model $model, $name, $prefix, $is_random = false ) {
    $initials = getNameAcronym( $name );
    $prefix   = strtoupper( $prefix );

    $length = 5 - strlen($initials);
    $suffix   = ( $is_random ) ? getRandomString( $length, '1234567890' ) : getLastId( $model );

    return $prefix . $initials . $suffix;
}

function generateHouseholdCode($lastID,$prefix=null)
    {
        $length = 6;
        $lastID+=1;

        if(!is_null($prefix)) $code=$prefix.sprintf("%0".$length."d", $lastID);
        else $code=sprintf("%0".$length."d", $lastID);

        return $code;
    }

    function setNewMember($applicant_number,$household_reference_id) {
        $record = new ContactDetail();
        $record->applicant_number = $applicant_number;
        $record->type = config('settings.contact_type.household');
        $record->type_name = getLookupSelectedOptionTextByValue(config('settings.contact_type.household'));
        $record->household_number = $household_reference_id;
        $record->is_household_head = 0;
        $record->status = 'Verified';
        $record->addedBy = auth()->id();

        if($record->save()) {
            return true;
        }
        else {
            return false;
        }
    }


/**
 * @param $fullname
 * @return string
 */

function getNameAcronym( $fullname)
{
    $names = explode(" ", $fullname);
    $acronym = "";

    foreach ($names as $name) {
        $acronym .= $name[0];
    }

    return strtoupper($acronym);
}

function getDateRange($date_range) {
    //split date range value to from and to
    // dd(str_replace(' ', '', $date_range));
    // dd(str_replace(' ', '', $date_range));
    $split_dates = explode('-',str_replace(' ', '', $date_range));
    $from = date('Y-m-d 00:00:00',strtotime($split_dates[0]));
    $to = date('Y-m-d 23:59:59',strtotime($split_dates[1]));

    return array('from'=>$from,'to'=>$to);
}

function defaultDateFormat($date_string, $time=false)
{
    if($date_string == null) return null;

    if($time) return date('Y-m-d @ h:ia',strtotime($date_string));
    return date('Y-m-d',strtotime($date_string));
}

function easyDateNormal($date_string, $time=false)
{
    if($date_string == null) return null;

    if($time) return date('d.m.Y @ h:ia',strtotime($date_string));
    return date('d.m.Y',strtotime($date_string));
}

function getDateTRange($date_range) {
    //split date range value to from and to
    $split_dates = explode('-',str_replace(' ', '', $date_range));
    $from = date('Y-m-d 00:00:00',strtotime($split_dates[0]));
    $to = date('Y-m-d 23:59:59',strtotime($split_dates[1]));

    return array('from'=>$from,'to'=>$to);
}
function archiveBenefit(Benefit $benefit) {

    //Restore an Archived Benefit
    if ($benefit->archived) {
        $benefit->archived = 0;
        $benefit->save();
        alertNotify('Benefit has been restored.',null,'success');
    }
    else {
        $benefit->archived = 1;
        $benefit->save();

        Beneficiary::where('form_id','=',$benefit->form_id)->update(['archived'=>1]);

        alertNotify('Benefit has been archived.',null,'success');
    }

    return back();
}

/**
 * Get Selected Lookup Option by the Lookup ID
 * @param $field_id
 * @param $value
 * @return mixed
 */
function getSelectedOptionTextByFieldID($field_id,$value) {
    $field = \App\Models\Field::find($field_id);
    if (in_array($field->input_type,['select','checkbox','radio'])) {
        if($field->has_auto_options == 1 && $field->auto_options > 0) {
            $lookup_text = \App\Models\Lookup::find((integer)$value);
            if ($lookup_text){
                return $lookup_text->fullName;
            }
            else{
                return null;
            }

        }
    }
    return $value;
}
/**
 * @param $length
 * @param string $key_characters
 * @return string
 * @throws Exception
 */
function getRandomString($length, $key_characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ')
{
    $string = '';
    $max = mb_strlen( str_shuffle($key_characters), '8bit') - 1;
    for ($i = 0; $i < $length; ++$i) {
        $string .= $key_characters[random_int(0, $max)];
    }
    return $string;
}

/**
 * @return mixed
 *
 */
function getRandomName()
{
    $name = ["adamant", "adroit", "amatory", "animistic", "antic",
        "arcadian", "baleful", "bellicose", "bilious", "boorish", "calamitous", "caustic",
        "cerulean", "comely", "concomitant", "contumacious", "corpulent", "crapulous",
        "defamatory", "didactic", "dilatory", "dowdy", "efficacious", "effulgent",
        "egregious", "endemic", "equanimous", "execrable", "fastidious", "feckless",
        "fecund", "friable", "fulsome", "garrulous", "guileless", "gustatory", "heuristic",
        "histrionic", "hubristic", "incendiary", "insidious", "insolent", "intransigent",
        "inveterate", "invidious", "irksome", "jejune", "jocular", "judicious", "lachrymose",
        "limpid", "loquacious", "luminous", "mannered", "mendacious", "meretricious", "minatory",
        "mordant", "munificent", "nefarious", "noxious", "obtuse", "parsimonious", "pendulous",
        "pernicious", "pervasive", "petulant", "platitudinous", "precipitate", "propitious", "puckish",
        "querulous", "quiescent", "rebarbative", "recalcitant", "redolent", "rhadamanthine", "risible",
        "ruminative", "sagacious", "salubrious", "sartorial", "sclerotic", "serpentine", "spasmodic",
        "strident", "taciturn", "tenacious", "tremulous", "trenchant", "turbulent", "turgid",
        "ubiquitous", "uxorious", "verdant", "voluble", "voracious", "wheedling", "withering",
        "zealous"];

    return Arr::random($name);
}

function getRandomColor()
{
    $color = ["e84e40", "ec407a", "ab47bc", "7e57c2", "5c6bc0",
        "738ffe", "29b6f6", "26c6da", "26a69a", "2baf2b", "9ccc65", "d4e157",
        "ffee58", "ffa726", "ff7043", "8d6e63", "bdbdbd", "78909c"];

    return Arr::random($color);
}

function cryptThis($id) {
    return \Illuminate\Support\Facades\Crypt::encrypt($id);
}


function generateRefID(Request $request)
{
    $base_code = getBaseRegID($request);
    // $existing_applicants = Applicant::where('regID','like',"$base_code%")->count();
    $existing_applicants = Applicant::where('regID','like',"$base_code%")->orderBy('created_at','desc')->pluck('id')->first();
    $base_code.=sprintf("%03d", $existing_applicants+1);
    return $base_code;
}
function getBaseRegID(Request $request)
{
    $prefix = 'MB';
    $dob = date('Ymd',strtotime($request->dobs));
    $gender = ($request->genders == 'Male')?1:2;

    $refID = "$prefix-".$dob."-".$gender."-";
    return $refID;
}

function getProjectName(){
    return settings()->get('app_project_name',env('APP_NAME'));
}

function getProjectFooter(){
    $preference = Instance::find(1);
    if ($preference) {
        if ($preference->copyright != NULL) {
            return $preference->copyright;
        }
    }
    return "@2022 ".env('APP_NAME');
}

function getProjectFooterLogo(){
    $preference = Instance::find(1);
    if ($preference) {
        if ($preference->footer_logo_url != NULL) {
            return $preference->footer_logo_url;
        }
    }
    return env('APP_ABBREV');
}

function getProjectLogo(){

    if (is_null(getActiveDeployment('logo'))) {
        return 'images/icons/Alleviate.png';
    }
    else {
        $deployment = getActiveDeployment();
        if(!$deployment){
            return 'images/icons/Alleviate.png';
            exit();
        }
        if (file_exists($deployment->logo)) return asset($deployment->logo);
    }

    return 'images/icons/Alleviate.png';
}

function get_issuance_mode_id($string){
    switch($string){
        case 'Electronic Transfers':
            $id = 200;
        break;
        case 'Vouchers or Coupons':
            $id = 201;
        break;
        case 'In-Kind Assistance':
            $id = 202;
        break;
        case 'Provider Payments':
            $id = 203;
        break;
        case 'Tax credits':
            $id = 204;
        break;
        case 'Direct Cash Payments':
            $id = 205;
        break;
        default:
            $id = null;

    }

    return $id;
}

function getActiveDeployment(string $field=null) {
    $deployment = \App\Models\Deployment::find(getActiveInstance());
    if (!is_null($field)) {
        if (isset($deployment->$field)){
            return $deployment->$field;
        }
        elseif($field == 'currency_code') {
            return env('DEFAULT_CURRENCY_CODE');
        }
        return 'Alleviate';
    }

    return $deployment;
}

function getProjectFavicon(){
    return asset('images/icons/favicon.png');
}

function benefitAmount(array $benefit_id)
{
    $get_min_amount = BenefitAmount::select(['benefit_id','min_value','max_value'])
        ->distinct()
        ->whereIn('benefit_id',$benefit_id)
        ->where('year','=',date('Y'))
        ->where('month',date('m'))
        ->get()->toArray();
        return $get_min_amount;
}

function getActiveBenefit() {
    $benefits = Benefit::where('instance_id','=',getActiveInstance())->where('archived','=',0)->pluck('titles','id');
    return $benefits;
}

function getIdentificationType() {
    $units = getActiveOptionsFor(config('settings.identification_code'));
    return $units;
}

function getRelationshipType() {
    $units = getActiveOptionsFor(config('settings.family_type_code'));
    return $units;
}

function getEmploymentSectorType() {
    $units = getActiveOptionsFor(config('settings.employment_sector_code'));
    return $units;
}

function getEmploymentType() {
    $units = getActiveOptionsFor(config('settings.employment_type_code'));
    return $units;
}

function getImpairmentType() {
    $units = getActiveOptionsFor(config('settings.impairment_type_code'));
    return $units;
}

function getSurveyType() {
    $units = getActiveOptionsFor(config('settings.survey_type_code'));
    return $units;
}

function getSurveyCategory() {
    $units = getActiveOptionsFor(config('settings.survey_category_code'));
    return $units;
}

function getSurveyStatus() {
    $units = getActiveOptionsFor(config('settings.survey_status_code'));
    return $units;
}

function getMaritalStatusType() {
    $units = getActiveOptionsFor(config('settings.marriage_status_code'));
    return $units;
}

function getMaritalType() {
    $units = getActiveOptionsFor(config('settings.marriage_type_code'));
    return $units;
}

function getTeamRoles() {
    $units = getActiveOptionsFor(config('settings.team_role_code'));
    return $units;
}

function getTeamTypes() {
    $units = getActiveOptionsFor(config('settings.team_type_code'));
    return $units;
}
function getBusinessTypes() {
    $units = getActiveOptionsFor(config('settings.business_type_code'));
    return $units;
}

//function getBanks() {
//    $banks = \App\Models\Bank::where('status','=',1)->orderBy('name','asc')->whereIn('type',[105,106])->pluck('name','id')->toArray();
//    return $banks;
//}
//
//function getMerchants() {
//    $banks = \App\Models\Bank::where('status','=',1)->orderBy('name','asc')->pluck('name','id')->toArray();
//    return $banks;
//}

//function getBanks() {
//    $units = getActiveOptionsFor(config('settings.bank_code'));
//    return $units;
//}

//function getMerchBranches($bank_id) {
//    $branches = \App\Models\BankBranch::where('status','=',1)->where('bank_id','=',$bank_id)->pluck('name','id')->toArray();
//    return $branches;
//}

//function getBankBranchesOptions($bank_id) {
//    $branches = getMerchBranches($bank_id);
//    $options = "<option>Choose Branch</option>";
//    foreach ($branches as $key=>$branch) {
//        $options .= "<option value='$key'>$branch</option>";
//    }
//    return $branches;
//}

//function getBranches() {
//    $branches = \App\Models\BankBranch::where('status','=',1)->pluck('name','id')->toArray();
//    return $branches;
//}

function getCaseCategories() {
    $units = getActiveOptionsFor(config('settings.case_categories_code'));
    return $units;
}

function getCaseTypes() {
    $units = getActiveOptionsFor(config('settings.case_types_code'));
    return $units;
}

function hashAccountNumber($account_number) {
    $first = substr($account_number, 0, 3);
//    $middle = substr($account_number, 4, 3);
    $last = substr($account_number, -4);

    return "$first *** **** $last";
}
//
//function getBranches() {
//    $units = getActiveOptionsFor(config('settings.branch_code'));
//    return $units;
//}

function getHouseholdTypes() {
    $units = getActiveOptionsFor(27);
    return $units;
}

function getHouseholdAreas() {
    $units = getActiveOptionsFor(28);
    return $units;
}

function getHouseholdTenures() {
    $units = getActiveOptionsFor(29);
    return $units;
}

function getHouseholdHeadship() {
    $units = getActiveOptionsFor(30);
    return $units;
}

function getHouseholdIncomeStatus() {
    $units = getActiveOptionsFor(31);
    return $units;
}

function getHoldingTypes() {
    $units = getActiveOptionsFor(3);
    return $units;
}

function getWarehouseTypes() {
    $units = getActiveOptionsFor(4);
    return $units;
}

function getLocations() {
    $locations = DB::table('applicants')
        ->join('applicant_deployment','applicants.id','=','applicant_deployment.applicant_id')
        ->selectRaw('Distinct(applicants.city_id) as city_id,applicants.place_of_births')
        ->where('applicant_deployment.deployment_id','=', auth()->user()->last_instance_opened)
        ->pluck('place_of_births','city_id');
    return $locations;
}

//function getLocations() {
//    $locations = getActiveOptionsFor(9);
//    return $locations;
//}

function getAccount() {
    $locations = getActiveOptionsFor(config('settings.account_type_code'));
    return $locations;
}

function getLandType() {
    $locations = getActiveOptionsFor(6);
    return $locations;
}

function getProduceType() {
    $types = getActiveOptionsFor(7);
    return $types;
}

function getProduceMethod() {
    $types = getActiveOptionsFor(13);
    return $types;
}

function getProductionStage() {
    $types = getActiveOptionsFor(8);
    return $types;
}

function getCertificationTypes() {
    $types = getActiveOptionsFor(10);
    return $types;
}

function getCertificationStandards() {
    $types = getActiveOptionsFor(11);
    return $types;
}

function getLandOwnership() {
    $types = getActiveOptionsFor(15);
    return $types;
}

function getUserTypes() {
    $types = Role::where('status','=',1)->where('instance_id','=',getActiveInstance())->orWhere('is_general','=',1)->pluck('display_name','id');
    return $types;
}

function getGender() {
    $types = getActiveOptionsFor(1);
    return $types;
}

function getEducationalLevels() {
    $types = getActiveOptionsFor(37);
    return $types;
}

function getOfficeTypes() {
    $types = getActiveOptionsFor(23);
    return $types;
}

function getBenefitClasses() {
    $types = getActiveOptionsFor(24);
    return $types;
}

function getBenefitClassesTerminology() {
    $types = getActiveOptions(24);

    return $types;
}

function getSupportMode() {
    $support_mode = \App\Models\Lookup::query()
        ->where('lookup_code_id',44)
        ->select('id','fullName','description')
        ->get();
    return $support_mode;
}

function getTypesTerminology()
{
    $benefits_type = \App\Models\BenefitType::query()->select('name','class','description')
        ->get();
    $data = array();
    foreach( getBenefitClassesTerminology() as $class){
        $data[] = [
            'class' => $class->fullName,
            'types' => $benefits_type->where('class',$class->id)->toArray(),
        ];
    }

    return $data;
}

function getBankTypes() {
    $types = getActiveOptionsFor(25);
    return $types;
}

function getMerchantTypes() {
//    $types = getActiveOptionsFor(25);
    return config('settings.merchant_types');
}

function getContactTypes() {
    $types = getActiveOptionsFor(26);
    return $types;
}

function getPaymentVoucherStatus() {
    $types = getActiveOptionsFor(config('settings.payment_voucher_status'));
    return $types;
}

function getProgramPartnerTypes() {
    $types = getActiveOptionsFor(32);
    return $types;
}

function getBenefitClassTypes($class_id) {
    $types = \App\Models\BenefitType::where('class','=',$class_id)->pluck('name','id');
    return $types;
}

function getNationalities() {
//    $types = \App\Models\Country::orderBy('country_name')->pluck('nationality','id');

    $getNationalities = new LocationApiController();
    $getNationalities = $getNationalities->getNationalities();
    $getNationalities = $getNationalities->getOriginalContent();
    if (is_null($getNationalities)) {
        $countries = null;
    }
    elseif (!$getNationalities['status']) {
        $countries = null;
    }
    else {
        $countries = $getNationalities['data'];
    }
    return $countries;
}

function getMerchants($ids=null)
{
    $getMerchants = new MerchantsApiController();
    $merchants = $getMerchants->getMerchants($ids);
    $merchants = $merchants->getOriginalContent();

    if (is_null($merchants)) {
        $merchants = null;
    }
    elseif (!$merchants['status']) {
        $merchants = null;
    }
    else {
        $merchants = $merchants['data'];
    }

    return $merchants;
}

function getMerchant($id,$key=null) {
    //Get Countries
    $responses = getMerchants([$id]);

    $data = null;
    if (isset($responses[0])) {
        $data = $responses[0];
    }

    if (!is_null($key)) {
        return $data[$key];
    }

    return $data;

}

function getMerchantBranch($id,$key=null) {
    $getBranch = new MerchantsApiController();
    $getBranch = $getBranch->getBranchById($id);
    $getBranch = $getBranch->getOriginalContent();

    if (is_null($getBranch)) {
        $data = null;
    }
    elseif (!$getBranch['status']) {
        $data = null;
    }
    else {
        $data = $getBranch['data'];
    }

    if (!is_null($key)) {
        return $data[$key];
    }

    return $data;

}

function getMerchantCountries() {
    //Get Countries
    $responses = Http::get(env('CONSOLE_API').'get-merchant-countries');
    $responseData = $responses->json();
    if (is_null($responseData)) {
        $data = null;
    }
    elseif (!$responseData['status']) {
        $data = null;
    }
    else {
        $data = $responseData['data'];
    }

    return $data;

}

function getBankMerchantTypes() {
    $getMerchantTypes = new MerchantsApiController();
    $getMerchantTypes = $getMerchantTypes->getMerchantTypes();
    $getMerchantTypes = $getMerchantTypes->getOriginalContent();
    if (is_null($getMerchantTypes)) {
        $merchants_types = null;
    }
    elseif (!$getMerchantTypes['status']) {
        $merchants_types = null;
    }
    else {
        $merchants_types = $getMerchantTypes['data'];
    }
    return  $merchants_types;
}

function getMerchantType($id) {
    //Get Countries
    $allMerchantTypes = array_values(getMerchantTypes());
    $name='Merchant';
    foreach ($allMerchantTypes as $key=>$branch) {
        if ($branch['id'] == $id) {
            $name = $branch['name'];
            break;
        }
    }

    return $name;
}

function getMerchantTypesByCountry($country_id) {
    //Get Countries
    $allMerchantTypes = Http::get(env('CONSOLE_API').'get-types-by-country/'.$country_id);
    $merchantsData = $allMerchantTypes->json();
    if (is_null($merchantsData)) {
        $merchants_types = null;
    }
    elseif (!$merchantsData['status']) {
        $merchants_types = null;
    }
    else {
        $merchants_types = $merchantsData['data'];
    }

    return $merchants_types;

}

function getMerchantTypesOnlyByCountry($country_id) {
    $getTypes = new MerchantsApiController();
    $getTypes = $getTypes->getTypesOnlyUnderCountry($country_id);
    $getTypes = $getTypes->getOriginalContent();
    if (is_null($getTypes)) {
        $merchants_types = null;
    }
    elseif (!$getTypes['status']) {
        $merchants_types = null;
    }
    else {
        $merchants_types = $getTypes['data'];
    }

    return $merchants_types;

}

function getCountry($id,string $value=null) {
    //Get Countries
    $country = Http::get(env('CONSOLE_API').'get-country/'.$id);
    $countryData = $country->json();
    if (is_null($countryData)) {
        $country = null;
    }
    elseif (!$countryData['status']) {
        $country = null;
    }
    else {
        $country = $countryData['data'];
    }
    if ($value == 'name') $value='country_name';
    if (!is_null($value) && isset($country[$value])) {
        return $country[$value];
    }

    return $country;

}

function getState($id,string $value=null) {
    //Get Countries
    $state = Http::get(env('CONSOLE_API').'get-state/'.$id);
    $stateData = $state->json();
    if (is_null($stateData)) {
        $state = null;
    }
    elseif (!$stateData['status']) {
        $state = null;
    }
    else {
        $state = $stateData['data'];
    }

    if (!is_null($value) && isset($state[$value])) {
        return $state[$value];
    }
    return $state;

}

function getVillages($country_id) {
    //Get Countries
    if(isset($country_id)){
        $villages = Village::where('country_id', $country_id)->orderBy('name', 'asc')->get();
    }else{
        $villages = Village::orderBy('name', 'asc')->get();
    }

    return $villages;

}

function getVillage($id, $value)
{
    $village = Village::find($id);
    if(!$village){
        return 'null';
        exit;
    }
    if(isset($value)){
        return $village->$value;
    }else{
        return $village;
    }
}

function getCounty($id,string $value=null) {
    //Get Countries
    $county = Http::get(env('CONSOLE_API').'get-county-district/'.$id);
    $countyData = $county->json();
    if (is_null($countyData)) {
        $county = null;
    }
    elseif (!$countyData['status']) {
        $county = null;
    }
    else {
        $county = $countyData['data'];
    }
    if ($value == 'name') $value='county';
    if (!is_null($value) && isset($county[$value])) {

        return $county[$value];
    }
    return $county;

}

function getCounties($id) {
    //Get counties
    $counties = Http::get(env('CONSOLE_API').'get-state-counties/'.$id);
    $statesData = $counties->json();
    if (is_null($statesData)) {
        $counties = [];
    }
    elseif (!$statesData['status']) {
        $counties = [];
    }
    else {
        $counties = $statesData['data']['counties'];
    }

    return $counties;
}

function getStates($id) {
    //Get Countries
    $states = new LocationApiController();
    $states = $states->getStatesOrRegions($id);
    $states = $states->getOriginalContent();
    if (!$states['status']) {
        $states = [];
    }
    else {
        $states = $states['data']['states'];
    }

    return $states;

}
function getCountries() {
    $getCountries = new LocationApiController();
    $countries = $getCountries->getCountries();
    $countries = $countries->getOriginalContent();
    if (!$countries['status']) {
        $countries = null;
    }
    else {
        $countries = $countries['data'];
    }

    return $countries;
}
function getCountriesRaw() {
    $countries = Http::get(env('CONSOLE_API').'get-countries-json/');
    return $countries;
}

function getUserTypeID($user = null) {
    if (!is_null($user)){
        $type = $user->roles->pluck('id')->first();
    }
    else {
        $type = \Illuminate\Support\Facades\Auth::user()->roles->pluck('id')->first();
    }

    return $type;
}

function getUserTypeName($user = null) {
    if (!is_null($user)){
        $type = $user->roles->pluck('display_name')->first();
    }
    else {
        $type = \Illuminate\Support\Facades\Auth::user()->roles->pluck('display_name')->first();
    }

    return ucwords($type);
}

function addApplicantDocument($applicant_id,$file_url,$type) {
    if ($applicant_id == '' || $file_url == '' || $type == '') {

    }
    else {
        \App\Models\ApplicantDocument::create([
            'applicant_id'  =>  $applicant_id,
            'url'  =>  $file_url,
            'type'  =>  $type,
            'status'  =>  1,
            'created_by'  =>  Auth::id(),
        ]);
    }
}

function measureSystem($measurement=null,$instance_id=null) {
    if ($instance_id == null) {
        $instance_id = 1;
    }
    $instance = \App\Models\Instance::find($instance_id);

    if ($measurement != null) {
        $measure = $instance->measurement_system->measurement_units->where('measurement','=',$measurement)->pluck('unit')->first();
    }
    else {
        $measure = $instance->measurement_system->pluck('name')->first();
    }
    return $measure;
}

function validateObject($object) {
    if (!$object) {
        return abort(404);
    }

    return true;
}

function getDefault($array, $key, $default=null) {
    return isset($array[$key])?$array[$key]:$default;
}

function canDo(string|array $role_keys = null,string|array $permission_keys=null) {

    if (is_null($role_keys) && is_null($permission_keys)) {
        return true;
    }

    if (!is_null($permission_keys)) {
        if (!is_array($permission_keys)) {
            $permission_keys = explode('|',str_replace(' ','',$permission_keys));
        }
    }
    else {
        $permission_keys = [];

//        if (in_array(session()->get('role'),$role_keys) || systemUserCan()) {
//            return true;
//        }
    }
//    dd();

    if (!is_array($role_keys)) {
        $role_keys = explode('|',str_replace(' ','',$role_keys));
    }

    if (is_null(session()->get('my_permissions')) || is_null(session()->get('role'))) {
        return false;
    }

    if (sizeof(array_intersect(session()->get('my_permissions'),$permission_keys)) > 0 || in_array(session()->get('role'),$role_keys) || systemUserCan()) {
//    if (array_intersect(session()->get('my_permissions'),$permission_keys) || array_intersect(session()->get('my_roles'),$role_keys) || systemUserCan()) {
        return true;
    }
    else {
        return false;
    }
}

function systemUserCan(string|array $permission_keys=null)
{
    if (auth()->user()->user_type == 'administrator') {
        if (is_null($permission_keys)) {
            return true;
        }
        else{
            return hasPermissionTo($permission_keys);
        }
    }
    return false;
}

function hasPermissionTo(string|array $permission_keys) {

    if (!is_array($permission_keys)) {
        $permission_keys = explode('|',str_replace(' ','',$permission_keys));
    }

    if (!is_null($permission_keys) && sizeof(array_intersect(session()->get('my_permissions'),$permission_keys) > 0)) {
        return true;
    }
    else {
        return false;
    }
}

function setTextLabel($string) {
    if (substr($string, -1) != ':') {
        $string .= ' :';
    }
    return Str::headline($string);
}

function money($figure)
{
    return number_format($figure, 2, '.', ',');
}

function easyCount($figure)
{
    return number_format($figure, null, null, ',');
}
function easyMoney($figure)
{
    return getCurrency().' '.number_format($figure, null, null, ',');
}

function easyDate($date) {
    $timestamp = strtotime($date);

    return \Illuminate\Support\Carbon::parse($timestamp)->diffForHumans();
}

function getAgeByDate($date) {
    $age = date_diff(date_create($date), date_create('now'));
    if($age->y > 0) {
        if($age->y > 1) {
            return $age->y." years";
        }
        return $age->y." year";
    }
    else {
        if($age->m > 1) {
            return $age->m." months";
        }
        return $age->m." month";
    }
}

function getRawAgeByDate($date) {
    $age = date_diff(date_create($date), date_create('now'));
    return $age->y;
}

/**
 * @param $date
 * @param $days
 * @return false|string
 */
function subtractDaysFromDate($date, $days) {
    $created_date=date_create(date('Y-m-d', strtotime($date)));
    date_sub($created_date,date_interval_create_from_date_string("$days days"));
    return date_format($created_date,"Y-m-d");
}

function getBeneficiaryTableNameByID($beneficiary_id) {
    $beneficiary = Beneficiary::find($beneficiary_id);
    $table_name = $beneficiary->form->table_name;
    return $table_name;
}

function getBeneficiaryAgeByID($beneficiary_id) {
    $beneficiary = Beneficiary::find($beneficiary_id);
    $table_name = $beneficiary->form->table_name;
    $dob_field = $beneficiary->form->fields->where('input_type','=', 'dob')->pluck('name')->first();
    if($dob_field) {
        $dob = DB::table($table_name)->where('id','=',$beneficiary->benefitRegistration->application_id)->pluck($dob_field)->first();
        $age = date_diff(date_create($dob), date_create('now'))->y;
    }
    else {
        $age = 0;
    }

    return $age;
}

function getBeneficiaryAgeInMonthsByID($beneficiary_id) {
    $beneficiary = Beneficiary::find($beneficiary_id);
    $table_name = $beneficiary->form->table_name;
    $dob_field = $beneficiary->form->fields->where('input_type','=', 'dob')->pluck('name')->first();
    $dob = DB::table($table_name)->where('id','=',$beneficiary->benefitRegistration->application_id)->pluck($dob_field)->first();
    $age = date_diff(date_create($dob), date_create('now'));
    return $age->m;
}

/**
 * @param array $roles
 *
 * @return \Illuminate\Support\Collection
 */
function getUsersWithRoles(array $roles) {
    $users = DB::table('users')->join('role_user', 'users.id','=','role_user.user_id')
        ->join('roles', 'roles.id','=','role_user.role_id')
        ->whereIn('role_id',$roles)
        ->select(['users.*'])->get();
    return $users;
}

/**
 * @param array $roles
 * @param $user_id
 *
 * @return \Illuminate\Support\Collection
 */
function isUserEligibleByRole(array $roles,$user_id) {
    $users = DB::table('users')
        ->join('role_user', 'users.id','=','role_user.user_id')
        ->join('roles', 'roles.id','=','role_user.role_id')
        ->whereIn('roles.id',array_values($roles))
        ->where('users.id','=',$user_id)->get();
    return $users;
}

/**
 * @param array $roles
 *
 * @return array
 */
function getUsersEmailAndNameByRoles(array $roles) {
    $users = DB::table('users')
        ->join('role_user', 'users.id','=','role_user.user_id')
        ->join('roles', 'roles.id','=','role_user.role_id')
        ->whereIn('role_user.role_id',$roles)
        ->pluck('users.name','users.email')->toArray();
    return $users;
}

/**
 * @param array $users
 *
 * @return array
 */
function getUsersEmailAndNameByUsers(array $users) {
    $users = DB::table('users')
        ->whereIn('id',$users)
        ->pluck('name','email')->toArray();
    return $users;
}

/**
 * @param array $roles
 *
 * @return array
 */
function getRolesByID(array $roles) {
    $rs = DB::table('roles')
        ->whereIn('id',$roles)
        ->pluck('name','id')->toArray();
    return $rs;
}

/**
 * FOR THIS PROJECT
 */
function getUserActiveProfile($user_id) {
    $profile = \App\Models\AppPhoto::where('user_id','=',$user_id)->where('status','=',1)->first();

    if (is_null($profile)) {
        return 'assets/img/avatar.png';
    }
    else {
        return $profile->url;
    }
}
function getUserActiveProfileAb($user_id) {
    $profile = \App\Models\AppPhoto::where('user_id','=',$user_id)->where('status','=',1)->first();

    if (is_null($profile)) {
        return 'assets/img/avatar.png';
    }
    else {
        return $profile->scan;
    }
}

function getCurrency($code=true) {
    return getActiveDeployment('currency_code');
//    if ($code) {
//        $currency = settings()->get('app_main_currency_code',env('DEFAULT_CURRENCY_CODE'));
//    }
//    else {
//        $currency = settings()->get('app_main_currency_symbol',env('DEFAULT_CURRENCY_SYMBOL'));
//    }
//    if (session()->has('configurations')){
//        if (isset(session()->get('configurations')['app_main_currency_code'])) {
//            return session()->get('configurations')['app_main_currency_code'];
//        }
//    }

//    return env('DEFAULT_CURRENCY_CODE','USD');
    // return $currency;
}

function getInstanceCounties(){
    $counties = \App\Models\County::get();
    return $counties;
}

function __i($key,) {
    $value = \App\Models\ContentConfig::where('key','=',$key)
        ->where('instance_id','=',Auth::user()->instance_id)
        ->pluck('text')
        ->first();

    if (is_null($value)) {
        $value = \App\Models\ContentConfig::where('key','=',$key)
            ->where('instance_id','=',0)
            ->pluck('text')
            ->first();
    }
//    if ($plural){
//        return Str::plural($value);
//    }
//    else {
    return $value;
//    }
}

function getApplicantAgeByDOB($date) {
    if (is_null($date)) return 0;
    $age = date_diff(date_create($date), date_create('now'))->y;
    return $age;
}

function makeString($value) {
    return "$value";
}

//function isApplicantEligibleObj(\App\Models\Applicant $applicant,Benefit $benefit) {
function isApplicantEligibleObj(\App\Models\Applicant $applicant,Benefit $benefit) {

    $age = getApplicantAgeByDOB($applicant->dobs);

    $employment_sector = $applicant->employmentDetails->where('removed','=',0)->where('employment_end_date','>',date('Y-m-d',strtotime(now())))->orWhere('employment_end_date','=',null)->pluck('employment_sector')->toArray();
    $employment_types = $applicant->employmentDetails->where('removed','=',0)->where('employment_end_date','>',date('Y-m-d',strtotime(now())))->orWhere('employment_end_date','=',null)->pluck('employment_type')->toArray();
    $island = $applicant->contactDetails->where('status','=',1)->where('island','!=',null)->pluck('island')->toArray();
    $impairments = $applicant->healthDetails->where('status','=',1)->pluck('health_type')->toArray();

    $is_eligible = true;
    //Check Age
    if(!is_null($benefit->min_age) || !is_null($benefit->max_age)) {
        $from_age = $benefit->min_age;
        $to_age = $benefit->max_age;
        if ($to_age <= 0 && $age < $from_age) {
            $is_eligible = false;
        }
        else {
            if ($age >= $from_age && $age <= $to_age) {
                $is_eligible = true;
            }
            else {
                $is_eligible = false;
            }
        }
    }

    if(is_null($benefit->min_age) && is_null($benefit->max_age)) {
        $is_eligible = true;
    }

    //Check Gender
    if(!is_null($benefit->gender_criteria)) {
        if(!in_array($applicant->genders,$benefit->gender_criteria)) {
            $is_eligible = false;
        }
    }

    //Check Residence
    if(!is_null($benefit->residence_criteria)) {
        if(!is_null($island)) {
            if(sizeof(array_intersect($island,$benefit->residence_criteria)) < 1) {
                $is_eligible = false;
            }
        }
        else {
            $is_eligible = false;
        }
    }

    //Check nationality
    if(!is_null($benefit->nationalities)) {
        if(!in_array($applicant->nationalities,$benefit->nationalities)) {
            $is_eligible = false;
        }
    }

    //Check Impairment
    if(!is_null($benefit->impairments)) {
        if(!is_null($impairments)) {
            if(sizeof(array_intersect($impairments,$benefit->impairments)) < 1) {
                $is_eligible = false;
            }
        }
        else {
            $is_eligible = false;
        }
    }

    //Check Employment Type
    if(!is_null($benefit->employment_type_criteria)) {
        if(!is_null($employment_types)) {
            if(sizeof(array_intersect($employment_types,$benefit->employment_type_criteria)) < 1) {
                $is_eligible = false;
            }
        }
        else {
            $is_eligible = false;
        }
    }

    //Check Employment Sector
    if(!is_null($benefit->employment_criteria)) {
        if(!is_null($employment_sector)) {
            if (sizeof(array_intersect($employment_sector, $benefit->employment_criteria)) < 1) {
                $is_eligible = false;
            }
        }
        else {
            $is_eligible = false;
        }
    }
    return $is_eligible;
}

function isApplicantEligible($applicant_number,$benefit_id) {

    $benefit = Benefit::find($benefit_id);

    $applicant = \App\Models\Applicant::where('regID','=',$applicant_number)->first();
    if(!$applicant) return ['status'=>false,'message'=>'Account does not exist.'];
    else $age = getApplicantAgeByDOB($applicant->dobs);

    $employment_sector = EmploymentDetail::where('applicant_number','=',$applicant_number)->where('status','=','Verified')->where('removed','=',0)->where('employment_end_date','>',date('Y-m-d',strtotime(now())))->orWhere('employment_end_date','=',null)->pluck('employment_sector')->toArray();
    $employment_types = EmploymentDetail::where('applicant_number','=',$applicant_number)->where('status','=','Verified')->where('removed','=',0)->where('employment_end_date','>',date('Y-m-d',strtotime(now())))->orWhere('employment_end_date','=',null)->pluck('employment_type')->toArray();
    //$island = $applicant->contactDetails->where('type_name','=','Residential Address')->where('status','=','Verified')->where('city','!=',null)->pluck('city')->toArray();
    $island = $applicant->contactDetails->where('type_name','=','Residential Address')->where('status','=','Verified')->where('village','!=',null)->pluck('village')->toArray();

    $impairments = $applicant->healthDetails->where('status','=','Verified')->pluck('health_type')->toArray();
    // dd($benefit->residence_criteria);

    $is_eligible = true;
    $message = '<ul>';
    // foreach ($benefits as $benefit) {
        //Check Age
    if(!is_null($benefit->min_age) || !is_null($benefit->max_age)) {
        $from_age = $benefit->min_age;
        $to_age = $benefit->max_age;

        if ($benefit->age_format == 'Above'){
            $is_eligible = true;
            if ($age < $from_age) {
                $is_eligible = false;
                $message .= "<li>Age must be from $from_age years</li>";
            }
        }
        elseif ($benefit->age_format == 'Below'){
            $is_eligible = true;
            if ($age > $to_age) {
                $is_eligible = false;
                $message .= "<li>Age must be below $to_age years</li>";
            }
        }
        else {
            if ($age >= $from_age && $age <= $to_age) {
                $is_eligible = true;
            }
            else {
                $is_eligible = false;
                $message .= "<li>Age must be from $from_age years to $to_age years</li>";
            }
        }
    }

    if(is_null($benefit->min_age) && is_null($benefit->max_age)) {
        $is_eligible = true;
    }

        //Check Gender
        if(!is_null($benefit->gender_criteria)) {
            if(!in_array($applicant->genders,$benefit->gender_criteria)) {
                $is_eligible = false;
                $message .= "<li>Gender must be ".implode(', ',$benefit->gender_criteria)."</li>";
            }
        }

        //Check Residence
        if(!is_null($benefit->residence_criteria)) {
            if(!is_null($island)) {
                if(sizeof(array_intersect($island,$benefit->residence_criteria)) < 1) {
                    $is_eligible = false;
                    $message .= "<li>Residence must be ".implode(', ',$benefit->residence_criteria)."</li>";
                }
            }
            else {
                $is_eligible = false;
                $message .= "<li>Residence must be ".implode(', ',$benefit->residence_criteria)."</li>";
            }
        }

        //Check nationality
        if(!is_null($benefit->nationalities)) {
            if(!in_array($applicant->nationalities,$benefit->nationalities)) {
                $is_eligible = false;
                $message .= "<li>Nationality must be ".implode(', ',$benefit->nationalities)."</li>";
            }
        }

        //Check Impairment
        if(!is_null($benefit->impairments)) {
            if(!is_null($impairments)) {
                if(sizeof(array_intersect($impairments,$benefit->impairments)) < 1) {
                    $is_eligible = false;
                    $message .= "<li>Impairments must be ".implode(', ',$benefit->impairments)."</li>";
                }
            }
            else {
                $is_eligible = false;
                $message .= "<li>Impairments must be ".implode(', ',$benefit->impairments)."</li>";
            }
        }

        //Check Employment Type
        if(!is_null($benefit->employment_type_criteria)) {
            if(!is_null($employment_types)) {
                if(sizeof(array_intersect($employment_types,$benefit->employment_type_criteria)) < 1) {
                    $is_eligible = false;
                    $message .= "<li>Current Employment Type must be ".implode(', ',$benefit->employment_type_criteria)."</li>";
                }
            }
            else {
                $is_eligible = false;
                $message .= "<li>Current Employment Type must be ".implode(', ',$benefit->employment_type_criteria)."</li>";
            }
        }

        //Check Employment Sector
        if(!is_null($benefit->employment_criteria)) {
            if(!is_null($employment_types)) {
                if (sizeof(array_intersect($employment_sector, $benefit->employment_criteria)) < 1) {
                    $is_eligible = false;
                    $message .= "<li>Current Employment Sector must be ".implode(', ',$benefit->employment_criteria)."</li>";
                }
            }
            else {
                $is_eligible = false;
            }
        }
        $message .= "</ul>";
    // }
    return ['status'=>$is_eligible,'message'=>$message];
}

function applicationDate($filter=null)
{
    return getActiveDeployment('deployment_date');
//    if($filter == null) {
//        return settings()->get('app_deployment_date');
//    }
//    else{
//        return date($filter,strtotime(settings()->get('app_deployment_date')));
//    }

}

function applicant_eligible_benefits($applicant_number=null) {

   // Get list of active benefits
    $benefits = Benefit::where('archived','=',0)->whereNotNull('form_id')->get();
    $age = $employment_sector = $employment_types = $island = $gender = $impairments = $nationalities = null;
   // if (auth()->user()->applicant_number) {
    if (is_null($applicant_number) && !is_null(auth()->user()->applicant_number)) {
        $applicant = Applicant::where('regID','=',auth()->user()->applicant_number)->first();
    }
    else {
        $applicant = Applicant::where('regID','=',$applicant_number)->first();
    }

    $age = getApplicantAgeByDOB($applicant->dobs);
    $employment_sector = $applicant->employmentDetails->where('status','=','Verified')->where('removed','=',0)->pluck('employment_sector')->toArray();
    $employment_types = $applicant->employmentDetails->where('status','=','Verified')->where('removed','=',0)->pluck('employment_type')->toArray();
    $island = $applicant->contactDetails->where('type_name','=','Residential Address')->where('status','=','Verified')->pluck('island')->toArray();
    $gender = $applicant->genders;
    $impairments = $applicant->impairments;
    $nationalities = $applicant->nationalities;

    $benefits = $benefits->filter(function ($item) use($applicant) {
        if ($item->visibility) {
            //Get applicant invited deployments
            $invited_deployments = $applicant->deployments()->pluck('deployment_id')->toArray();
            if (in_array($item->instance_id,$invited_deployments)) {
                return true;
            }
            else{
                return false;
            }
        }
        else{
            return true;
        }
    });


    $eligible_benefits = [];
    foreach ($benefits as $benefit) {
        $is_eligible = true;

        //Check Age
        if(!is_null($benefit->min_age) || !is_null($benefit->max_age)) {
            if ($benefit->max_age < 1) $benefit->max_age = 210;
            $from_age = $benefit->min_age;
            $to_age = $benefit->max_age;
            if ($to_age <= 0 && $age < $from_age) {
                continue;
            }
            else {
                if ($age >= $from_age && $age <= $to_age) {
                    $is_eligible = true;
                }
                else {
                    continue;
                }
            }
        }

        if(is_null($benefit->min_age) && is_null($benefit->max_age)) {
            $is_eligible = true;
        }

        //Check Gender
        if(!is_null($benefit->gender_criteria)) {
            if(!in_array($gender,$benefit->gender_criteria)) {
                continue;
            }
        }

        //Check Residence
        if(!is_null($benefit->residence_criteria)) {
            if(sizeof(array_intersect($island,$benefit->residence_criteria)) < 1) {
                continue;
            }
        }

        //Check Nationality
        if(!is_null($benefit->nationalities)) {
            if(!in_array($nationalities,$benefit->nationalities)) {
                continue;
            }
        }

        //Check Impairment
        if(!is_null($benefit->impairments)) {
            if(!is_null($applicant->impairments)) {
                if(sizeof(array_intersect($impairments,$benefit->impairments)) < 1) {
                    continue;
                }
            }
            else {
                continue;
            }
        }

        //Check Employment Sector
        if(!is_null($benefit->employment_criteria)) {
            if(!is_null($employment_types)) {
                if (sizeof(array_intersect($employment_sector, $benefit->employment_criteria)) < 1) {
                    continue;
                }
            }
            else {
                continue;
            }
        }

        //Check Employment Type
        if(!is_null($benefit->employment_type_criteria)) {
            if(!is_null($employment_types)) {
                if(sizeof(array_intersect($employment_types,$benefit->employment_type_criteria)) < 1) {
                    continue;
                }
            }
            else {
                continue;
            }
        }

        if ($is_eligible) {
            $eligible_benefits[] = $benefit->id;
        }
    }
    return $eligible_benefits;
}

function applicantCompletion($applicant_number) {
    $applicant = \App\Models\Applicant::withCount([
        'idDetails'=> function ($query) {
            $query->where('removed',0);
        },
        'contactDetails'=> function ($query) {
            $query->whereNotIn('status',['Rejected']);
        },
        'maritalDetails'=> function ($query) {
            $query->whereNotIn('status',['Rejected']);
        },
        'employmentDetails'=> function ($query) {
            $query->whereNotIn('status',['Rejected']);
        },
        'accountDetails'=> function ($query) {
            $query->whereNotIn('status',['Rejected']);
        },
        'familyDetails'=> function ($query) {
            $query->whereNotIn('status',['Rejected']);
        }
    ])->where('regID',$applicant_number)->first();

    $idDetails = $contactDetails = $maritalDetails = $employmentDetails = $accountDetails = $familyDetails =  $total = $personal = 0; //$healthDetails =;

    if ($applicant) {
        if ($applicant->id_details_count > 0) $idDetails = 100; $total += $idDetails;
        if ($applicant->contact_details_count > 0) $contactDetails = 100; $total += $contactDetails;
        if ($applicant->marital_details_count > 0) $maritalDetails = 100; $total += $maritalDetails;
        if ($applicant->employment_details_count > 0) $employmentDetails = 100; $total += $employmentDetails;
        if ($applicant->account_details_count > 0) $accountDetails = 100; $total += $accountDetails;
        if ($applicant->family_details_count > 0) $familyDetails = 100; $total += $familyDetails;

        if (!is_null($applicant->surnames)) $personal += 10;
        if (!is_null($applicant->first_names)) $personal += 10;
        if (!is_null($applicant->dobs)) $personal += 20;
        if (!is_null($applicant->genders)) $personal += 20;
        if (!is_null($applicant->place_of_births)) $personal += 20;
        if (!is_null($applicant->country_of_births)) $personal += 10;
        if (!is_null($applicant->nationalities)) $personal += 10;
    }


    $total += $personal;
    $code = 'danger';
    if ($total > 0) {
        $total = floor(($total/700)*100);

        if ($total >= 100) $code = 'success';
        elseif($total >= 80) $code = 'info';
        elseif($total >= 65) $code = 'primary';
        elseif($total > 50 && $total < 65) $code = 'warning';
        else $code = 'danger';
    }

    return ['total'=>$total,'personal'=>$personal,'idDetails'=>$idDetails,'contactDetails'=>$contactDetails,
        'maritalDetails'=>$maritalDetails,'employmentDetails'=>$employmentDetails,
        'accountDetails'=>$accountDetails,'familyDetails'=>$familyDetails, 'code'=>$code]; //'healthDetails'=>$healthDetails,
}

function appplicationDate($filter=null)
{
    if($filter == null) {
        return settings()->get('app_deployment_date');
    }
    else{
        return date($filter,strtotime(settings()->get('app_deployment_date')));
    }

}

function phoneNumberFormat($phone,$code=null)
{
    if (is_null($code)) $code = env('DEFAULT_CALL_CODE');
    //0501375124
    $phone = preg_replace('/[^0-9]/', '', cleanString($phone));
    if(Str::length($phone) > 10) {
        return '+'.$phone;
    }
    else {

        if(!str_starts_with($code, '+')) {
            $code = '+'.$code;
        }

        if(substr($phone, 0, 1) == 0) {
            return $code.ltrim($phone, '0');
        }
        else {
            return "$code$phone";
        }
    }

}

function logUser($action=null,$user_id = null) {
    if ($user_id == null) {
        $user_id = Auth::id();
    }
    if ($action == null) {
        \App\Models\AccessLog::create([
            'user_id'   =>  $user_id,
            'ip_address'   =>  $_SERVER["REMOTE_ADDR"],
            'user_agent'   =>  $_SERVER['HTTP_USER_AGENT']
        ]);
    }
    else {
        $latest = \App\Models\AccessLog::where('user_id','=',$user_id)->orderBy('created_at','desc')->first();
        if ($latest) {
            $latest->updated_at = date('Y-m-d H:i:s',strtotime(now()));
            $latest->save();
        }

    }
}

function userName($user) {
    return "<strong>".$user->name."</strong>, ".$user->firstname." ".$user->othername;
}
/////
///
/// Applicant Helpers
function getDefaultResidence($applicant_id) {
    $applicant = \App\Models\Applicant::find($applicant_id);
    $applicant = $applicant->contactDetails->where('status','=','Verified')->where('type','=',config('settings.contact_type.residential'))->first();
    if ($applicant) {
        return $applicant;
    }
    return null;
}

function getDefaultPostal($applicant_id) {
    $applicant = \App\Models\Applicant::find($applicant_id);
    $applicant = $applicant->contactDetails->where('status','=','Verified')->where('type','=',config('settings.contact_type.postal'))->first();
    if ($applicant) {
        return $applicant;
    }
    return null;
}

function isSettingActive($key) {
    if (is_null(settings()->get($key))) return false; //Previous

//    if (session()->has('configurations')) {
//        if (isset(session('configurations')[$key])) {
//            if (session('configurations')[$key] != "") return true;
//        }
//    }
    return true;
}

function setSocialLink($key) {
    $link = config('app_settings.'.$key);
    if (isSettingActive($key)) $link = settings()->get($key); //Previous
//    if (isSettingActive($key)) $link = session('configurations')[$key];
    return $link;
}

function setAppDefault($key) {
    $name = config('app_settings.'.$key);
    if (isSettingActive($key)) $name = settings()->get($key); //Previous
//    if (isSettingActive($key)) $name = session('configurations')[$key];
    return $name;
}

function application_style($status,$type=null) {
    $style = config('settings.application_status_style');
    if (isset($style[$status])) {
        if ($type != NULL) {
            if (isset($style[$status][$type])) {
                return $style[$status][$type];
            }
        }
        return $style[$status]['text'];
    }
    return 'text-muted';
}

function canPerform($module)
{
    $modules = \App\Models\Module::where('id','=',1)->first()->pluck('modules')->toArray();
    if (in_array($module,$modules[0])) {
        return true;
    }
    else {
        return false;
    }
}

function menu() {
    if (!is_null(auth()->user()->last_instance_visited)) {
        $modules =  \App\Models\Module::find(auth()->user()->last_instance_visited);
    }
    else {
        $modules =  \App\Models\Module::find(auth()->user()->deployments()->pluck('id')->first());
    }

    if (auth()->user()->hasRole('administrator')) $modules = \App\Models\Module::find(1);

    if (!isset($modules)) {
        return [];
    }
    else {
        if ($modules->active == 0) {
            return [];
        }
        else {
            return $modules->pluck('menus')->first();
        }
    }
}

function canPublish($announcement)
{
    if ($announcement->status == 'Publish' && strtotime($announcement->publish_date) < strtotime(now())) {
        return true;
    }
    else {
        return false;
    }
}

function hasReadAnnouncement($announcement_id)
{
    $read_announcements = auth()->user()->readAnnouncements->pluck('announcement_id')->toArray();
    if (in_array($announcement_id,$read_announcements)) return true;
    else return false;
}

function content_excerpt($content,$length=200)
{
    $text = substr(strip_tags($content),0,$length);
    if (strlen(strip_tags($content))>$length) $text.='...';
    return $text;
}

function benefitRespondents($role) {
    return DB::table('users')
        ->join('role_user','role_user.user_id','=','users.id')
        ->join('roles','roles.id','=','role_user.role_id')
        ->selectRaw('users.id,users.name,users.firstname,users.othername,users.email,roles.name')
        ->where('roles.id','=',$role)
        ->get();
}

function pendingSurvey()
{
    $self_respondents = ['applicants','all-users'];
    if (auth()->user()->is_admin) $self_respondents = ['administrators','all-users'];
    $active_surveys = \App\Models\Survey::where('mode','=','self')->active()
        ->whereIn('self_respondents',$self_respondents)
        ->pluck('id')
        ->toArray();
    $pending_entries = DB::table('entries')->select(['survey_id'])->distinct()
        ->where('participant_id','=', auth()->id())
        ->pluck('survey_id')->toArray();

    $collection = collect($active_surveys);
    $diff = $collection->diff($pending_entries);

    return $diff->count();
}

function getGeneralNotifications()
{
    if (session()->get('role') == 'beneficiary') {
        if (!auth()->user()->applicant) {
            return [];
        }
    }
    return auth()->user()->generalNotifications()->get();
}



function hasGeneralNotifications()
{
    if (session()->get('role') == 'beneficiary') {
        if (!auth()->user()->applicant) {
            return false;
        }
    }

    return auth()->user()->generalNotifications()->count() > 0;
}
function cleanString($string,$whitespaces=true)
{
    if ($whitespaces) $string = str_replace(' ','',$string);
    return preg_replace('~[+(*&$#!)-]~','',trim($string));
}
function deploymentCode()
{
    $last_deployment_id = DB::table('deployments')->latest()->pluck('id')->first() + 1;
//    if (is_null($last_deployment_id)) $last_deployment_id = 1;

    switch (true) {
        case $last_deployment_id < 100:
            $outputAlphabet = 'A';
            break;
        case $last_deployment_id < 200 && $last_deployment_id >= 100:
            $outputAlphabet = 'B';
            break;
        case $last_deployment_id < 300 && $last_deployment_id >= 200:
            $outputAlphabet = 'C';
            break;
        case $last_deployment_id < 400 && $last_deployment_id >= 300:
            $outputAlphabet = 'D';
            break;
        case $last_deployment_id < 500 && $last_deployment_id >= 400:
            $outputAlphabet = 'E';
            break;
        case $last_deployment_id < 600 && $last_deployment_id >= 500:
            $outputAlphabet = 'F';
            break;
        case $last_deployment_id < 700 && $last_deployment_id >= 600:
            $outputAlphabet = 'G';
            break;
        case $last_deployment_id < 800 && $last_deployment_id >= 700:
            $outputAlphabet = 'H';
            break;
        case $last_deployment_id < 900 && $last_deployment_id >= 800:
            $outputAlphabet = 'I';
            break;
        case $last_deployment_id < 1000 && $last_deployment_id >= 900:
            $outputAlphabet = 'J';
            break;
        case $last_deployment_id < 1100 && $last_deployment_id >= 1000:
            $outputAlphabet = 'K';
            break;
        case $last_deployment_id < 1200 && $last_deployment_id >= 1100:
            $outputAlphabet = 'L';
            break;
        case $last_deployment_id < 1300 && $last_deployment_id >= 1200:
            $outputAlphabet = 'M';
            break;
        case $last_deployment_id < 1400 && $last_deployment_id >= 1300:
            $outputAlphabet = 'N';
            break;
        case $last_deployment_id < 1500 && $last_deployment_id >= 1400:
            $outputAlphabet = 'O';
            break;
        default:
            $outputAlphabet = 'Z'; // Default case
            break;
    }

    $code = $outputAlphabet.sprintf("%02d", ($last_deployment_id%100));

    return $code;
}

function isAdmin()
{
    return auth()->user()->hasRole('administrator');
}

function isMember()
{
    return auth()->user()->hasRole('beneficiary');
}

//Deployments
function getDeploymentConfig(\App\Models\Deployment $deployment,string $key, string $default=null) {
    if (isset($deployment->$key)){
        return $deployment->$key;
    }
    else {
        return $default;
    }
}
function getCountryById($location_id){
    return \App\Models\Country::find($location_id)?->country_name  ??  null;
}

function get_num_of_office_users($office_id)
{
    $office_users =  \App\Models\User::where('office_id', $office_id)->get();
    return $office_users->count();
}

function getOfficeNameById($office_id)
{
    return Office::find($office_id)->name  ??  null;
}

function getOfficeLocationById($office_id)
{
    return Office::find($office_id)?->location  ??  null;
}

function getOfficesList()
{
    $offices = \App\Models\Office::query()
        ->select('name','id')
        ->active()
        ->get();

    return $offices;
}

function getOfficeTypeName($office_id)
{
    if($office_id == null){
        return null;
    }

    $office_type = \App\Models\Office::find($office_id)?->type;
    return \App\Models\Lookup::find($office_type)->fullName  ??  null;
}

function status_past($string)
{
    switch(strtolower($string)){
        case 'publish':
            $string_value = 'Published';
            break;
        case 'draft':
            $string_value = 'Drafted';
            break;
    }

    return $string_value;
}
function getAssignedUsersCount($role_id) {
    // Find the role by ID
    $role = Role::find($role_id);

    if (!$role) {
        // Handle the case where the role is not found
        return -1;
    }

    // Use the users relationship to get the users assigned to the role
    $users = $role->users;

    // Get the count of assigned users
    return $users->count();
}

function getPmtDetail($pmtScore)
{
    $pmt = Pmt::where('mini', '<=', $pmtScore)->where('maxi', '>=', $pmtScore)->first();
    return $pmt ? $pmt->detail : $pmtScore;
}

function get_region_by_name($name, $column=null)
{
    $region = Region::where([['name', $name], ['country_id', 115]])->first();
    if($column !== null){
        return $region->$column;
    }else{
        return $region;
    }
    
}
