<?php

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Route;
use \Illuminate\Support\Arr;
use \Illuminate\Support\Str;
use \App\Models\Benefit;
use \App\Models\LookupCode;
use \App\Models\Beneficiary;
use \App\Models\Application;
use \App\Models\BenefitRegistration;
use \App\Models\BeneficiariesFormMapping;
use \Illuminate\Support\Facades\DB;
use \App\Models\Lookup;
use \App\Models\CaseManagement;
use \App\Models\StageEmail;

function getTableColumns($table)
{
    return DB::getSchemaBuilder()->getColumnListing($table);

    // OR

//    return \Doctrine\DBAL\Schema\Schema::getColumnListing($table);

}

function randomBgColor() {
    $background_colors = ['bg-radial-gradient-success','bg-radial-gradient-primary','bg-radial-gradient-dark','bg-gradient-primary',
//        'bg-gradient-dark','bg-gradient-success','bg-gradient-warning','bg-gradient-danger'
//        ,'bg-gradient-secondary','bg-gradient-info','bg-radial-gradient-info',
//        'bg-radial-gradient-warning'
//        ,'bg-radial-gradient-danger'
    ];
    return Arr::random($background_colors);
}

function getNextStage($current_lookup_id) {
    $getLookup = Lookup::find($current_lookup_id);
    $next_status = 0;
    if ($getLookup) {
        $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->where('activeFlag','=',1)->orderBy('order','asc')->pluck('order','id')->toArray();
        $current_order = $get_other_options[$current_lookup_id];
        foreach ($get_other_options as $lookup_id=>$order) {
            if ($get_other_options[$lookup_id] > $current_order) {
                if (isDeclinedProcess($lookup_id)){
                    continue;
                }else{
                    $next_status = $lookup_id;
                    break;
                }
            }
        }
    }

    return $next_status;
}

function getDeclinedStage($current_lookup_id) {
    $getLookup = Lookup::find($current_lookup_id);
    $next_status = 0;
    if ($getLookup) {
        $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->orderBy('order','asc')->pluck('order','id')->toArray();
        foreach ($get_other_options as $order=>$lookup_id) {
            if (isDeclinedProcess($lookup_id)){
                $next_status = $lookup_id;
                break;
            }
        }
    }

    return $next_status;
}

function getPreviousStage($current_lookup_id) {
    $getLookup = Lookup::find($current_lookup_id);
    $next_status = 0;
    if ($getLookup) {
        $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->orderBy('order','asc')->pluck('id','order')->toArray();
        for ($count = 1; $count <= count($get_other_options); $count--) {
            if (isset($get_other_options[$getLookup->order - $count])) {
                    if (isStartProcess($get_other_options[$getLookup->order])) {
                        $next_status = 0; break;
                    }
                    else {
                        $next_status = $get_other_options[$getLookup->order - $count];
                        break;
                    }
//                }
            }
        }
    }

    return $next_status;
}

/**
 * Get Active Lookup Option by the Lookup Code Parent ID
 * @param $lookupCode_id
 * @return mixed
 */
function getActiveOptionsFor($lookupCode_id) {
    return LookupCode::activeOptions($lookupCode_id);
}

/**
 * Get Selected Lookup Option by the Lookup ID
 * @param $field_id
 * @param $value
 * @return mixed
 */
function getLookupSelectedOptionTextByValue($value) {
    if (is_numeric($value)) {
        $lookup_text = \App\Models\Lookup::find($value);
        return $lookup_text->fullName;
    }
    return $value;
}
/**
 * Get Selected Lookup Option by the Lookup ID
 * @param $field_id
 * @param $value
 * @return mixed
 */
function getSelectedOptionTextByFieldID($field_id,$value) {
    $field = \App\Models\Field::find($field_id);
    if (in_array($field->input_type,['select','checkbox','radio'])) {
        if($field->has_auto_options == 1 && $field->auto_options > 0) {
            $lookup_text = \App\Models\Lookup::find((integer)$value);
            if ($lookup_text){
                return $lookup_text->fullName;
            }
            else{
                return null;
            }

        }
    }
        return $value;
}
/**
 * Get All Lookup Option(Active & Inactive) by the Lookup Code Parent ID
 * @param $lookupCode_id
 * @return mixed
 */
function getAllOptionsFor($lookupCode_id) {
    return LookupCode::allOptions($lookupCode_id);
}

/**
 * @param array $route
 * @param bool $multilevel
 * @param string $output
 * @return string
 */
function isRouteActive( array $route, $multilevel = false, $output = 'menu-item-active' ) {

    $current_route_name = Route::currentRouteName();

	if ( $multilevel ) {
		return ( in_array($current_route_name,$route) ) ? 'menu-item-open menu-item-here' : '';
	}

	return (in_array($current_route_name,$route) ) ? $output : '';
}

/**
 * Helper function to show flash messages using bootstrap notify
 *
 * @param string $title
 * @param $message
 * @param string $alert_type
 * @param string $position
 */
function notify( $message, $title = 'Notice',  $alert_type = 'primary', $position = 'right' ) {
	$icons = [
		"success" => 'ui-1_check',
		"danger"  => 'ui-1_simple-remove',
		"warning" => 'objects_support-17',
		"info"    => 'travel_info',
		"primary" => 'ui-2_like'
	];

	session()->flash( 'notification', [
		'title'      => $title,
		'message'    => $message,
		'alert_type' => $alert_type,
		'position'   => $position,
		'icon'       => $icons[ $alert_type ]
	] );

	return;
}

/**
 * Helper function to show flash messages using notify
 *
 * @param string $title
 * @param $message
 * @param string $type
 */
function alertNotify( $message, $title = null,  $type = 'info') {
    session()->flash( 'ShowNotify',true);
    session()->flash( 'notifyTitle',$title);
    session()->flash( 'notifyMessage',$message);
    session()->flash( 'notifyAlertType',$type);
}

function clearNotify() {
    session()->forget(['ShowNotify', 'notifyTitle','notifyMessage','notifyAlertType']);
}

/**
 *Get the current route name. The name returned is the route name specified in the web routes file.
 *
 * @return string
 */
function currentRouteName() {
	$route_name = Route::currentRouteName();
	$breadcrumb = str_replace( [ ".", "_" ], [ " / ", " " ], $route_name );

	return $breadcrumb;
}

/**
 * Get last auto increment id of a model
 *
 * @param Model $model
 *
 * @return int
 */
function getLastId( Model $model ) {
	$models = $model::all();

	if ( $models->isEmpty() ) {
		return 1;
	} else {
		return (int) $models->last()->id + 1;
	}
}

/**
 * Generate a unique Id for a model
 *
 * @param Model $model
 * @param string $name
 * @param string $prefix
 * @param bool $is_random
 *
 * @return string
 * @throws Exception
 */
function generateModelId( Model $model, $name, $prefix, $is_random = false ) {
	$initials = getNameAcronym( $name );
	$prefix   = strtoupper( $prefix );

	$length = 5 - strlen($initials);
	$suffix   = ( $is_random ) ? getRandomString( $length, '1234567890' ) : getLastId( $model );

    return $prefix . $initials . $suffix;
}

/**
 * @param $fullname
 * @return string
 */

function getNameAcronym( $fullname)
{
    $names = explode(" ", $fullname);
    $acronym = "";

    foreach ($names as $name) {
        $acronym .= $name[0];
    }

    return strtoupper($acronym);
}

function getDateRange($date_range) {
    //split date range value to from and to
    $split_dates = explode('/',str_replace(' ', '', $date_range));
//    dd($split_dates[1]);
    $from = date('Y-m-d',strtotime($split_dates[0]));
    $to = date('Y-m-d',strtotime($split_dates[1]));

    return array('from'=>$from,'to'=>$to);
}

/**
 * @param $length
 * @param string $key_characters
 * @return string
 * @throws Exception
 */
function getRandomString($length, $key_characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ')
{
    $string = '';
    $max = mb_strlen( str_shuffle($key_characters), '8bit') - 1;
    for ($i = 0; $i < $length; ++$i) {
        $string .= $key_characters[random_int(0, $max)];
    }
    return $string;
}

/**
 * @param $id
 * @param string $prefix
 *
 * @return string
 */
function getRandomID($id,$prefix='REG')
{
    $string = $prefix.'-'.strtotime(now()).$id;
    return $string;
}

/**
 * @return mixed
 *
 */
function getRandomName()
{
    $name = ["adamant", "adroit", "amatory", "animistic", "antic",
        "arcadian", "baleful", "bellicose", "bilious", "boorish", "calamitous", "caustic",
        "cerulean", "comely", "concomitant", "contumacious", "corpulent", "crapulous",
        "defamatory", "didactic", "dilatory", "dowdy", "efficacious", "effulgent",
        "egregious", "endemic", "equanimous", "execrable", "fastidious", "feckless",
        "fecund", "friable", "fulsome", "garrulous", "guileless", "gustatory", "heuristic",
        "histrionic", "hubristic", "incendiary", "insidious", "insolent", "intransigent",
        "inveterate", "invidious", "irksome", "jejune", "jocular", "judicious", "lachrymose",
        "limpid", "loquacious", "luminous", "mannered", "mendacious", "meretricious", "minatory",
        "mordant", "munificent", "nefarious", "noxious", "obtuse", "parsimonious", "pendulous",
        "pernicious", "pervasive", "petulant", "platitudinous", "precipitate", "propitious", "puckish",
        "querulous", "quiescent", "rebarbative", "recalcitant", "redolent", "rhadamanthine", "risible",
        "ruminative", "sagacious", "salubrious", "sartorial", "sclerotic", "serpentine", "spasmodic",
        "strident", "taciturn", "tenacious", "tremulous", "trenchant", "turbulent", "turgid",
        "ubiquitous", "uxorious", "verdant", "voluble", "voracious", "wheedling", "withering",
        "zealous"];

    return Arr::random($name);
}

function getRandomColor()
{
    $color = ["e84e40", "ec407a", "ab47bc", "7e57c2", "5c6bc0",
        "738ffe", "29b6f6", "26c6da", "26a69a", "2baf2b", "9ccc65", "d4e157",
        "ffee58", "ffa726", "ff7043", "8d6e63", "bdbdbd", "78909c"];

    return Arr::random($color);
}

function cryptThis($id) {
    return \Illuminate\Support\Facades\Crypt::encrypt($id);
}

function validateObject($object) {
    if (!$object) {
        return abort(404);
    }
}

function setTextLabel($string) {
    if (substr($string, -1) != ':') {
        $string .= ' :';
    }
    return Str::headline($string);
}

function money($figure)
{
    return number_format($figure, 2, '.', ',');
}

function easyCount($figure)
{
    return number_format($figure, null, null, ',');
}

function getAgeByDate($date) {
    $age = date_diff(date_create($date), date_create('now'));
    if($age->y > 0) {
        return $age->y." years";
    }
    else {
        return $age->m." months";
    }
}

function archiveBenefit(Benefit $benefit) {

    //Restore an Archived Benefit
    if ($benefit->archived) {
        $benefit->archived = 0;
        $benefit->save();
        alertNotify('Benefit has been restored.',null,'success');
    }
    else {
        $benefit->archived = 1;
        $benefit->save();

        Beneficiary::where('form_id','=',$benefit->form_id)->update(['archived'=>1]);

        alertNotify('Benefit has been archived.',null,'success');
    }

    return back();
}

function transferBeneficiaryInformation($benefit_registration_id)
{
    $benefit_registration = BenefitRegistration::find($benefit_registration_id);
    try{

        //Get Information from Registration form
        $form = $benefit_registration->form;
        $registration_information = DB::table($form->table_name)->where('id','=',$benefit_registration->application_id)->first();

        //Get the mapped form
        $mapped_fields = BeneficiariesFormMapping::where('form_id','=',$form->id)->pluck('form_fields')->first();
        $data_collected = null;
        foreach ($mapped_fields as $key=>$mapped_field) {
            if (is_null($mapped_field)) continue;
            $data_collected[$key] = $registration_information->$mapped_field;
        }
        $data_collected['applicant_number'] = $benefit_registration->applicant_number;
        $data_collected['beneficiary_number'] = getRandomID($benefit_registration->application_id,$prefix='BFT');


        $beneficiary = Beneficiary::updateOrCreate(['benefit_registration_id'   =>  $benefit_registration_id],[
            'form_id'   =>  $form->id
        ]);

        $save = \App\Models\BeneficiariesDetail::updateOrCreate([
            'beneficiary_id'    =>  $beneficiary->id
        ],$data_collected);

        if ($save) {

            return true;
        }
    }
    catch (\Exception $exception) {
        return $exception->getMessage();
    }
}

function addCase($application)
{
    if (Str::startsWith(strtolower($application->form->benefit->titles),['destitute','infirm'])) {
        $has_column = $application->form->fields->where('name','=','review_requireds')->count();
        if ($has_column > 0) {
            $value = DB::table($application->form->table_name)->where('review_requireds','=','Yes')->count();

            if ($value > 0) {
                $case = new CaseManagement();
                $case->beneficiary_id = $application->applicant_number;
                $case->user_id = Auth::id();
                $case->title = $application->type.' claim review';
                $case->description = 'This Beneficiary on the '.$application->type.' claim requires a review';
                $case->severity = 'Normal';
                $case->stage = 'Opened';
                $case->save();
            }
        }
    }
}

function getBeneficiaryTableNameByID($beneficiary_id) {
    $beneficiary = Beneficiary::find($beneficiary_id);
    $table_name = $beneficiary->form->table_name;
    return $table_name;
}

function getBeneficiaryAgeByID($application_id) {
    $application = Application::find($application_id);
    $dob = $application->applicant->pluck('dobs')->first();
    if($dob) {
        $age = date_diff(date_create($dob), date_create('now'))->y;
    }
    else {
        $age = 0;
    }

    return $age;
}

function getBeneficiaryAgeInMonthsByID($beneficiary_id) {
    $beneficiary = Beneficiary::find($beneficiary_id);
    $table_name = $beneficiary->form->table_name;
    $dob_field = $beneficiary->form->fields->where('input_type','=', 'dob')->pluck('name')->first();
    $dob = DB::table($table_name)->where('id','=',$beneficiary->benefitRegistration->application_id)->pluck($dob_field)->first();
    $age = date_diff(date_create($dob), date_create('now'));
    return $age->m;
}

function getBeneficiaryAgeGroupTaxByID($application_id,$tax=null) {
    $age = getBeneficiaryAgeByID($application_id);
    $age_group = null;
    $amount = 0;
    $tax = 0;
    $application = Application::find($application_id);
    $table_name = $application->form->table_name;
    $benefit = Benefit::find($application->benefit_id);
    if($benefit->paymentSchedule) {
        if ($table_name == config('settings.pension_benefit')) {
            $pension_amount = $benefit->paymentSchedule->pension_amount;
            $age_groups = config('settings.pension_ages');
            if (in_array($age,$age_groups['60+'])) {
                $amount = $pension_amount['60+']['amount'];
                $tax = $pension_amount['60+']['tax'];
                $age_group = '60+';
            }
            elseif (in_array($age,$age_groups['65+'])) {
                $amount = $pension_amount['65+']['amount'];
                $tax = $pension_amount['65+']['tax'];
                $age_group = '65+';
            }
            else {
                $amount = $pension_amount['70+']['amount'];
                $tax = $pension_amount['70+']['tax'];
                $age_group = '70+';
            }
        }
        elseif ($table_name == config('settings.funeral_allowance')) {
            $funeral_amount = $benefit->paymentSchedule->funeral_amount;
            $table_name = $application->form->table_name;
            $benefit_type = DB::table($table_name)
                ->where('id','=',$application->application_id)
                ->pluck('benefit_types')
                ->first();
            if (in_array(strtoupper($benefit_type),array_keys($funeral_amount))) {
                $amount = $funeral_amount[$benefit_type];
                $tax = $benefit->paymentSchedule->tax;
            }
        }
        else {
            $tax = $benefit->paymentSchedule->tax;
            $amount = $benefit->paymentSchedule->amount;
        }
    }

    return ['age'=>$age,'age_group'=>$age_group,'tax'=>$tax,'amount'=>$amount];
}

function isStartProcess($lookup_id) {
    $lookup = \App\Models\Lookup::find($lookup_id);
    $is_start = false;
    if ($lookup){
        if ($lookup->process_action == 1) {
            $is_start = true;
        }
    }
    return $is_start;
}

function isApprovedProcess($lookup_id) {
    $lookup = \App\Models\Lookup::find($lookup_id);
    $is_start = false;
    if ($lookup){
        if ($lookup->process_action == 100) {
            $is_start = true;
        }
    }
    return $is_start;
}

function isDeclinedProcess($lookup_id) {
    $lookup = \App\Models\Lookup::find($lookup_id);
    $is_start = false;
    if ($lookup){
        if ($lookup->process_action == 101) {
            $is_start = true;
        }
    }
    return $is_start;
}

function easyDate($date) {
    $timestamp = strtotime($date);

    return \Illuminate\Support\Carbon::parse($timestamp)->diffForHumans();
}

function getStartProcessCode($lookupCode_id) {
    $lookupCode = \App\Models\LookupCode::find($lookupCode_id);
    $start_id = $lookupCode->options->where('activeFlag',true)->where('process_action','=',1)->pluck('id')->first();
    if ($start_id){
        return $start_id;
    }
    return 1;
}

function inWhichProcessStage($lookup_id) {
    $lookup = \App\Models\Lookup::find($lookup_id);
    $process_stage = null;
    if ($lookup){
        if ($lookup->process_action == 1) {
            $process_stage['name'] = $lookup->fullName;
            $process_stage['stage'] = 'start';
        }
        elseif ($lookup->process_action == 100) {
            $process_stage['name'] = $lookup->fullName;
            $process_stage['stage'] = 'approved';
        }
        elseif ($lookup->process_action == 101) {
            $process_stage['name'] = $lookup->fullName;
            $process_stage['stage'] = 'declined';
        }
        else {
            $process_stage['name'] = null;
            $process_stage['stage'] = null;
        }
    }
    return $process_stage;
}

/**
 * @param array $roles
 *
 * @return \Illuminate\Support\Collection
 */
function getUsersWithRoles(array $roles) {
    $users = DB::table('users')->join('role_user', 'users.id','=','role_user.user_id')
        ->join('roles', 'roles.id','=','role_user.role_id')
        ->whereIn('role_id',$roles)
        ->select(['users.*'])->get();
    return $users;
}

/**
 * @param array $roles
 * @param $user_id
 *
 * @return \Illuminate\Support\Collection
 */
function isUserEligibleByRole(array $roles,$user_id) {
    $users = DB::table('users')->join('role_user', 'users.id','=','role_user.user_id')
        ->join('roles', 'roles.id','=','role_user.role_id')
        ->whereIn('role_id',$roles)
        ->where('users.id','=',$user_id)->get();
    return $users;
}
function getApplicantAgeByDOB($date) {
    $age = date_diff(date_create($date), date_create('now'))->y;
    return $age;
}

function applicant_eligible_benefits($applicant_number) {

//    Get list of active benefits
    $benefits = Benefit::where('archived','=',0)->whereNotNull('form_id')->get();
    $applicant = \App\Models\Applicant::where('regID','=',$applicant_number)->first();
    $age = getApplicantAgeByDOB($applicant->dobs);
    $employment_sector = $applicant->employmentDetails->where('removed','=',0)->where('employment_end_date','>',date('Y-m-d',strtotime(now())))->pluck('employment_sector')->toArray();
    $employment_types = $applicant->employmentDetails->where('removed','=',0)->where('employment_end_date','>',date('Y-m-d',strtotime(now())))->pluck('employment_type')->toArray();
    $island = $applicant->contactDetails->where('status','=',1)->pluck('island')->toArray();

    $eligible_benefits = [];
    foreach ($benefits as $benefit) {
        $is_eligible = true;

        //Check Age
        if(!is_null($benefit->age_ranges) || !is_null($benefit->age_tos)) {
            $from_age = $benefit->age_ranges;
            $to_age = $benefit->age_tos;
            if ($to_age <= 0 && $age < $from_age) {
                $is_eligible = false;
                continue;
            }
            else {
                if ($age >= $from_age && $age <= $to_age) {
                    $is_eligible = true;
                }
                else {
                    $is_eligible = false;
                    continue;
                }
            }
        }

        //Check Gender
        if(!is_null($benefit->gender_criteria)) {
            if(!in_array($applicant->genders,$benefit->gender_criteria)) {
                $is_eligible = false;
                continue;
            }
        }

        //Check Residence
        if(!is_null($benefit->residence_criteria)) {
            if(!in_array($island,$benefit->residence_criteria)) {
                $is_eligible = false;
                continue;
            }
        }

        //Check Impairment
        if(!is_null($benefit->impairments)) {
            if(!is_null($applicant->impairments)) {
                if(sizeof(array_intersect($applicant->impairments,$benefit->impairments)) < 1) {
                    $is_eligible = false;
                    continue;
                }
            }
            else {
                $is_eligible = false;
                continue;
            }
        }

        //Check Employment Sector
        if(!is_null($benefit->employment_criteria)) {
            if(!is_null($employment_types)) {
                if (sizeof(array_intersect($employment_sector, $benefit->employment_criteria)) < 1) {
                    $is_eligible = false;
                    continue;
                }
            }
            else {
                $is_eligible = false;
                continue;
            }
        }

        //Check Employment Type
        if(!is_null($benefit->employment_type_criteria)) {
            if(!is_null($employment_types)) {
                if(sizeof(array_intersect($employment_types,$benefit->employment_type_criteria)) < 1) {
                    $is_eligible = false;
                    continue;
                }
            }
            else {
                $is_eligible = false;
                continue;
            }
        }

        //Check Benefit Type
//        if(!is_null($benefit->benefit_types)) {
//            if(!in_array($employment_types,$benefit->benefit_types)) {
//                $is_eligible = false;
//                continue;
//            }
//        }

        if ($is_eligible) {
            $eligible_benefits[] = $benefit->id;
        }
    }
    return $eligible_benefits;
}

/**
 * @param array $roles
 *
 * @return array
 */
function getUsersEmailAndNameByRoles(array $roles) {
    $users = DB::table('users')
        ->join('role_user', 'users.id','=','role_user.user_id')
        ->join('roles', 'roles.id','=','role_user.role_id')
        ->whereIn('role_id',$roles)
        ->pluck('users.name','users.email')->toArray();
    return $users;
}

/**
 * @param array $users
 *
 * @return array
 */
function getUsersEmailAndNameByUsers(array $users) {
    $users = DB::table('users')
        ->whereIn('id',$users)
        ->pluck('name','email')->toArray();
    return $users;
}

/**
 * @param array $roles
 *
 * @return array
 */
function getRolesByID(array $roles) {
    $rs = DB::table('roles')
        ->whereIn('id',$roles)
        ->pluck('name','id')->toArray();
    return $rs;
}


//Old Functions
function getBenefitAgeLimitsOld() {
    $benefits = Benefit::where('age_ranges','<>',null)->orWhere('age_ranges','<>','')
        ->where('age_ranges','<>',null)->orWhere('age_ranges','<>','')
        ->pluck('age_ranges','id');

    return $benefits;
}

function isEligibilityOld(Beneficiary $beneficiary,$benefit_id) {
    $beneficiary_age = getBeneficiaryAgeByID($beneficiary->id);
    $benefit = Benefit::find($benefit_id);

    $limits = $benefit->age_ranges;
    $is_eligible = false;

    $from_age = 0;$to_age = 0;
    $age_limit = explode(',',trim($limits));
    if (isset($age_limit[0])) {
        $from_age = $age_limit[0];
    }
    if (isset($age_limit[1])) {
        $to_age = $age_limit[1];
    }

    if ($to_age <= 0) {
        if ($beneficiary_age >= $from_age) {
            $is_eligible = true;
        }
    }
    else {
        if ($beneficiary_age >= $from_age && $beneficiary_age <= $to_age) {
            $is_eligible = true;
        }
    }

    return $is_eligible;
}

function checkEligibilityOld(Beneficiary $beneficiary) {
    $benefit_age_limits = getBenefitAgeLimits();
    $beneficiary_age = getBeneficiaryAgeByID($beneficiary->id);
    $message = ''; $count = 0;
    foreach ($benefit_age_limits as $benefit_id=>$limits) {
        if ($benefit_id == $beneficiary->form->benefit->id) continue;
        $is_eligible = false;
        $benefit = Benefit::find($benefit_id);
        $from_age = 0;$to_age = 0;
        $age_limit = explode(',',trim($limits));
        if (isset($age_limit[0])) {
            $from_age = $age_limit[0];
        }
        if (isset($age_limit[1])) {
            $to_age = $age_limit[1];
        }

        if ($to_age <= 0) {
            if ($beneficiary_age >= $from_age) {
                $is_eligible = true;
            }
        }
        else {
            if ($beneficiary_age >= $from_age && $beneficiary_age <= $to_age) {
                $is_eligible = true;
            }
        }

        if ($is_eligible) {
            $count +=1;
            if ($count > 1){
                $message .= ", ".$benefit->titles;
            }
            else {
                $message .= "Eligible for ".$benefit->titles;
            }

        }
    }


    return $message;
}
//End


function getBenefitAgeLimits() {
    $benefits = Benefit::where('age_ranges','<>',null)->orWhere('age_ranges','<>','')
        ->where('age_tos','<>',null)->orWhere('age_tos','<>','')
        ->select(['id','age_ranges','age_tos'])->get();

    return $benefits;
}

function isEligibility(Beneficiary $beneficiary,$benefit_id) {
    $beneficiary_age = getBeneficiaryAgeByID($beneficiary->id);
    $benefit = Benefit::find($benefit_id);

    $from_age = $benefit->age_ranges;
    $to_age = $benefit->age_tos;
    $is_eligible = false;

    if ($to_age <= 0) {
        if ($beneficiary_age >= $from_age) {
            $is_eligible = true;
        }
    }
    else {
        if ($beneficiary_age >= $from_age && $beneficiary_age <= $to_age) {
            $is_eligible = true;
        }
    }

    return $is_eligible;
}

function checkEligibility(Beneficiary $beneficiary) {
    $benefit_age_limits = getBenefitAgeLimits();
    $beneficiary_age = getBeneficiaryAgeByID($beneficiary->id);
    $message = ''; $count = 0;
    foreach ($benefit_age_limits as $limit) {
        $benefit_id = $limit->id;
        if ($benefit_id == $beneficiary->form->benefit->id) continue;
        $is_eligible = false;
        $benefit = Benefit::find($benefit_id);
        $from_age = $limit->age_ranges;
        $to_age = $limit->age_tos;

        if ($to_age <= 0) {
            if ($beneficiary_age >= $from_age) {
                $is_eligible = true;
            }
        }
        else {
            if ($beneficiary_age >= $from_age && $beneficiary_age <= $to_age) {
                $is_eligible = true;
            }
        }

        if ($is_eligible) {
            $count +=1;
            if ($count > 1){
                $message .= ", ".$benefit->titles;
            }
            else {
                $message .= "Eligible for ".$benefit->titles;
            }

        }
    }

    return $message;
}

