<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Traits\HttpResponses;
use App\Models\Deployment;
use App\Models\Plan;
use Illuminate\Http\Request;

class DeploymentApiController extends Controller
{
    use HttpResponses;
    public $successStatus = 200;
    public $noContentStatus = 204;
    public $badRequestStatus = 400;
    public $invalidAuthStatus = 401;
    public $hiddendates = ['created_at','updated_at','deleted_at'];
    public function getDeploymentSetting($deployment_code)
    {
        $deployment = Deployment::where('code','=',$deployment_code)->first();
        if ($deployment) {
            $deployment->makeHidden($this->hiddendates);
            return $this->successResponse($deployment->toArray(),'Loaded deployment settings');
        }
        else {
            return $this->errorResponse(null,'Invalid Deployment code!',$this->badRequestStatus);
        }
    }

    public function hasActiveSubscription($deployment_code)
    {
        $deployment = Deployment::where('code','=',$deployment_code)->first();

        if ($deployment) {
            if ($deployment->planSubscription($deployment->code)) {
                if ($deployment->planSubscription($deployment->code)->active() && $deployment->instance == 'Online') {
                    return $this->successResponse(['active'=>true],'Has active subscription');
                }
                else {
                    return $this->successResponse(['active'=>false,'instance'=>$deployment->instance],'Has no active subscription');
                }
            }
            else {
                return $this->errorResponse(['active'=>false],'No subscription',$this->badRequestStatus);
            }
        }
        else {
            return $this->errorResponse(['active'=>false],'Invalid deployment',$this->badRequestStatus);
        }
    }

    public function deploymentModules($deployment_code,$namesOnly=null)
    {
        $deployment = Deployment::where('code','=',$deployment_code)->first();

        if ($deployment) {

            if ($deployment->planSubscription($deployment->code)) {
                if ($deployment->planSubscription($deployment->code)->active()) {
                    $plan = Plan::with(['features'])->where('slug','=',$deployment->package)->first();
                    if (isset($plan->features)) {
                        if (is_null($namesOnly)) {
                            $features = $plan->features->setVisible(['name','value','icon','route','description'])->toArray();
                        }
                        else {
                            $features = $plan->features->pluck('name')->toArray();
                        }

                        return $this->successResponse($features,'Loaded all subscription modules');
                    }
                    else {
                        return $this->successResponse(null,'No modules available');
                    }
                }
                else {
                    return $this->successResponse(['active'=>$deployment->planSubscription($deployment->code)->active()],'Checked for active subscription');
                }
            }
            else {
                return $this->errorResponse(null,'No subscription',$this->badRequestStatus);
            }

        }
        else {
            return $this->errorResponse(null,'Invalid deployment',$this->badRequestStatus);
        }
    }
}
