<?php

namespace App\Http\Controllers;

use App\Http\Helpers\FormRender;
use App\Http\Traits\MembershipTrait;
use App\Http\Traits\ModelTracker;
use App\Models\AccountDetail;
use App\Models\Applicant;
use App\Models\ApplicantContact;
use App\Models\ApplicantRecord;
use App\Models\Application;
use App\Models\Bank;
use App\Models\BankBranch;
use App\Models\Benefit;
use App\Models\CaseManagement;
use App\Models\ContactDetail;
use App\Models\County;
use App\Models\CustomField;
use App\Models\Deployment;
use App\Models\EmploymentDetail;
use App\Models\Family;
use App\Models\HealthDetail;
use App\Models\HouseHold;
use App\Models\IdentificationDetail;
use App\Models\Lookup;
use App\Models\MaritalDetail;
use App\Models\SubCounty;
use App\Models\Village;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Yajra\DataTables\DataTables;

class ApplicantController extends Controller
{
    use ModelTracker, MembershipTrait;
//    public function __construct()
//    {
//        $this->middleware(['auth', 'verified'])->except(['login']);
//    }

    /**
     * Show Customer's login form
     *
     */
    public function loginForm()
    {
        return view('auth.customer-login');
    }

    /**
     * Get the list of Application DataTable
     * @param null $query
     * @param null $value
     *
     * @return mixed
     * @throws \Exception
     */
//    public function getApplicationsInfoList($query=null,$value=null,$gender=null,$location=null,$date=null)
    /*public function getApplicationsInfoList(Request $request)
    {
        $deployment = Deployment::find(getActiveInstance());

        $request_data = $request->all();
        $clean_request_data = [];
        foreach ($request_data as $key=>$value) {
            $key = str_replace('amp;','',$key);
            $clean_request_data[$key] = $value;
        }

        $applicants = $deployment->applicants()->orderBy('surnames','desc');
        if (isset($clean_request_data['date_range'])) {
            $dates = getDateRange($clean_request_data['date_range']);
            $applicants = $applicants->where('created_at', '>=', $dates['from'])
                ->where('created_at', '<=', $dates['to']);
        }

        if (isset($clean_request_data['query_option'])){
            
            if ($clean_request_data['query_option'] == 'regID'){
                $refID = $clean_request_data['regID'];
                $applicants = $applicants->where('regID','like',"%$refID%");
            }
            elseif ($clean_request_data['query_option'] == 'dobs') {
                $dobs = $clean_request_data['dobs'];
                $applicants = $applicants->whereDate('dobs','=',date('Y-m-d',strtotime($dobs)));
            }
            elseif ($clean_request_data['query_option'] == 'pmt') {
                $pmtFrom = $clean_request_data['pmt_from'];
                $pmtTo = $clean_request_data['pmt_to'];
            
                // Assuming there's a 'pmtScore' column in the applicants table
                // Filtering applicants whose pmtScore is between $pmtFrom and $pmtTo
                $applicants = $applicants->whereBetween('pmtScore', [$pmtFrom, $pmtTo]);
            }
            elseif ($clean_request_data['query_option'] == 'beneficiary') {
                $beneficiary_value = $clean_request_data['beneficiary_option'];
                $applicant_numbers = [];
                if($beneficiary_value == 'archived') {
                    $applicant_numbers = Application::where('is_beneficiary','=',1)
                        ->where('archived','=',1)
                        ->pluck('applicant_number')->toArray();
                }
                elseif($beneficiary_value == 'active') {
                    $applicant_numbers = Application::where('is_beneficiary','=',1)
                        ->pluck('applicant_number')->toArray();
                }
                elseif($beneficiary_value == 'ended') {
                    $applicant_numbers = Application::where('is_beneficiary','=',1)
                        ->where('archived','=',1)
                        ->pluck('applicant_number')->toArray();
                }

                $applicants = $applicants->whereIn('regID',array_unique($applicant_numbers));
            }
        }
        
        if (isset($clean_request_data['gender'])){
            $applicants = $applicants->where('genders','=',$clean_request_data['gender']);
        }

        if (isset($clean_request_data['location'])) {
            $applicants = $applicants->where('city_id','=',$clean_request_data['location']);
        }
        
        $applicants = $applicants->get();

        

        return DataTables::of($applicants)
            ->addColumn('action', function ($applicant) {
                $view = '<a href="'.route('applicant.viewApplicantInfo',['app_id'=>cryptThis($applicant->id)]).'"
                class="btn btn-sm btn-link text-info mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("View Applicant") .'"><i class=" fa fa-user"></i></a>';

                $edit = '<a href="'.route('applicant.applicantRegistration',[cryptThis($applicant->id)]).
                    '" class="btn btn-sm btn-link text-primary mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Edit Applicant") .'"><i class="fa fa-edit"></i></a>';

                if($applicant->status == 0) {
                    $archive = '<a href="'.route('applicant.deleteApplicantInfo',[cryptThis($applicant->id)]).
                        '" onclick="return confirm(\'Are you sure you want to restore this Applicant?\')"
                        class="btn btn-sm btn-link text-success mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Restore Applicant").'"><i class="fa fa-play"></i></a>';

                }
                else {

                    $archive = '<a href="'.route('applicant.deleteApplicantInfo',[cryptThis($applicant->id)]).
                        '" onclick="return confirm(\'Are you sure you want to archive this Applicant?\')"
                        class="btn btn-sm btn-link text-danger mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Delete Applicant").'"><i class=" fa fa-pause"></i></a>';
                }

                $options = '<div class="dropdown dropdown-inline mr-4">
                                <button type="button" class="btn btn-light-primary btn-icon btn-xs" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                    <i class="ki ki-bold-more-hor"></i>
                                </button>
                                <div class="dropdown-menu drop-left">'.$view. $edit. $archive.'</div>
                            </div>';

                if (!canDo('administrator','view-member-info')) {
                    $view = '';
                }
                if (!canDo('administrator','edit-beneficiary')) {
                    $edit = '';
                }
                if (!canDo('administrator','delete-beneficiary')) {
                    $archive = '';
                }

                return $view. $edit;
//                return $view. $edit. $archive;

            })
            ->editColumn('surnames', function ($applicant) {

                if (!isset($applicant->surnames)) {$surname = null;}
                else{$surname = $applicant->surnames;}

                if (!isset($applicant->first_names)) {$first_name = null;}
                else{$first_name = $applicant->first_names;}

                $other_name = $applicant->other_names;

                if (!is_null($applicant->avatar)) $link = asset($applicant->avatar);
                else $link =  asset('assets/img/avatar.png');

                $text = '<div class="d-flex align-items-center">
                <img src="'.$link.'" class="rounded-circle sm avatar" alt="">
                <div class="ms-2 mb-0"><strong>'.strtoupper($surname).'</strong>, '.ucwords($first_name).' '.ucwords($other_name).'
                <br><small class="text-muted">'.$applicant->regID.'</small></div>

              </div>';
                return $text;
                // return "<strong>".strtoupper($surname)."</strong>, ".ucwords($first_name)." ".ucwords($other_name);
            })
            ->editColumn('marital_status', function ($applicant) {
                if(count($applicant->maritalDetails) > 0){
                    $marital_status = ($applicant->maritalDetails->sortByDesc('created_at')->first())->marriage_status;
                }else{
                    $marital_status = 'No Marital Records';
                }

                if(count($applicant->healthDetails) > 0){

                    $health_status = implode(', ', $applicant->healthDetails->pluck('health_type')->toArray());
                }else{
                    $health_status = 'No Health Conditions';
                }
                return $marital_status.'<br><small class="text-muted">'.$health_status.'</small>';
            })
            ->editColumn('id', function ($applicant) {
                return $applicant->id;
            })
            ->editColumn('bio', function ($applicant) {
                return $applicant->genders."<br><span style='display: none'>".date('Y-m-d',strtotime($applicant->dobs)).
                    "</span><small class='text-muted'>".getAgeByDate($applicant->dobs)."</small>";
            })
            ->editColumn('citizenship', function ($applicant) {
                return $applicant->nationalities.'<br><small class="text-muted">Born in '.$applicant->place_of_births.', '.$applicant->country_of_births.'</small>';
            })
            // ->editColumn('dobs', function ($applicant) {
            //     return ;
            // })
            ->escapeColumns([])
            ->make();
    }*/

    public function getApplicationsInfoList(Request $request)
    {
        
        $deployment = Deployment::find(getActiveInstance());

        $request_data = $request->all();
        $clean_request_data = [];
        foreach ($request_data as $key => $value) {
            $key = str_replace('amp;', '', $key);
            $clean_request_data[$key] = $value;
        }

        $applicants = $deployment->applicants()->orderBy('surnames', 'desc');

        if (isset($request->pmt_to) && isset($request->pmt_from)) {
            $pmtFrom = $clean_request_data['pmt_from'];
            $pmtTo = $clean_request_data['pmt_to'];
            //dd('Here');

            // Ensure pmtScore is being properly filtered
            $applicants = $applicants->whereBetween('pmtScore', [$pmtFrom, $pmtTo]);
        }
        if (isset($request->applicant_name)) {
            $name = request()->get('applicant_name'); 

            $applicants = $applicants->where('surnames','like',"%$name%");
        }

        $applicants = $applicants->get(); // Final query execution

        return DataTables::of($applicants)
            ->addColumn('action', function ($applicant) {
                $view = '<a href="'.route('applicant.viewApplicantInfo',['app_id'=>cryptThis($applicant->id)]).'"
                class="btn btn-sm btn-link text-info mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("View Applicant") .'"><i class=" fa fa-user"></i></a>';

                $edit = '<a href="'.route('applicant.applicantRegistration',[cryptThis($applicant->id)]).
                    '" class="btn btn-sm btn-link text-primary mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Edit Applicant") .'"><i class="fa fa-edit"></i></a>';

                if($applicant->status == 0) {
                    $archive = '<a href="'.route('applicant.deleteApplicantInfo',[cryptThis($applicant->id)]).
                        '" onclick="return confirm(\'Are you sure you want to restore this Applicant?\')"
                        class="btn btn-sm btn-link text-success mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Restore Applicant").'"><i class="fa fa-play"></i></a>';

                }
                else {

                    $archive = '<a href="'.route('applicant.deleteApplicantInfo',[cryptThis($applicant->id)]).
                        '" onclick="return confirm(\'Are you sure you want to archive this Applicant?\')"
                        class="btn btn-sm btn-link text-danger mr-2" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Delete Applicant").'"><i class=" fa fa-pause"></i></a>';
                }

                $options = '<div class="dropdown dropdown-inline mr-4">
                                <button type="button" class="btn btn-light-primary btn-icon btn-xs" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                    <i class="ki ki-bold-more-hor"></i>
                                </button>
                                <div class="dropdown-menu drop-left">'.$view. $edit. $archive.'</div>
                            </div>';

                if (!canDo('administrator','view-member-info')) {
                    $view = '';
                }
                if (!canDo('administrator','edit-beneficiary')) {
                    $edit = '';
                }
                if (!canDo('administrator','delete-beneficiary')) {
                    $archive = '';
                }

                return $view. $edit;
//                return $view. $edit. $archive;
            })
            ->editColumn('surnames', function ($applicant) {

                if (!isset($applicant->surnames)) {$surname = null;}
                else{$surname = $applicant->surnames;}

                if (!isset($applicant->first_names)) {$first_name = null;}
                else{$first_name = $applicant->first_names;}

                $other_name = $applicant->other_names;

                if (!is_null($applicant->avatar)) $link = asset($applicant->avatar);
                else $link =  asset('assets/img/avatar.png');

                $text = '<div class="d-flex align-items-center">
                <img src="'.$link.'" class="rounded-circle sm avatar" alt="">
                <div class="ms-2 mb-0"><strong>'.strtoupper($surname).'</strong>, '.ucwords($first_name).' '.ucwords($other_name).'
                <br><small class="text-muted">'.$applicant->regID.'</small></div>

              </div>';
                return $text;
                // return "<strong>".strtoupper($surname)."</strong>, ".ucwords($first_name)." ".ucwords($other_name);
            })
            ->editColumn('marital_status', function ($applicant) {
                if(count($applicant->maritalDetails) > 0){
                    $marital_status = ($applicant->maritalDetails->sortByDesc('created_at')->first())->marriage_status;
                }else{
                    $marital_status = 'No Marital Records';
                }

                if(count($applicant->healthDetails) > 0){

                    $health_status = implode(', ', $applicant->healthDetails->pluck('health_type')->toArray());
                }else{
                    $health_status = 'No Health Conditions';
                }
                return $marital_status.'<br><small class="text-muted">'.$health_status.'</small>';
            })
            ->editColumn('id', function ($applicant) {
                return $applicant->id;
            })
            ->editColumn('bio', function ($applicant) {
                return $applicant->genders."<br><span style='display: none'>".date('Y-m-d',strtotime($applicant->dobs)).
                    "</span><small class='text-muted'>".getAgeByDate($applicant->dobs)."</small>";
            })
            ->editColumn('citizenship', function ($applicant) {
                return $applicant->nationalities.'<br><small class="text-muted">Born in '.$applicant->place_of_births.', '.$applicant->country_of_births.'</small>';
            })
            // ->editColumn('dobs', function ($applicant) {
            //     return ;
            // })
            ->escapeColumns([])
            
            ->make();
    }


    /**
     * Get Applicant Data
     * @param Request $request
     *
     * @return string
     */
    /*public function getApplicantInfo(Request $request)
    {
        $page_title = __('Beneficiaries');
        $applicant = $query = $value = $date_period = null;
        $gender = $location = 0;
        if (sizeof($request->all()) > 0) {
            $query = $request->query_option;
            if (!is_null($request->dobs)) {
                $value = $request->dobs;
            }
            elseif (!is_null($request->refID)) {
                $value = $request->refID;
            }
            elseif (!is_null($request->beneficiary_option)) {
                $value = $request->beneficiary_option;
                $page_title .= ' - '.ucwords($value).' Beneficiary';
            }
            if ($request->query == 'dobs' && is_null($request->dobs)) {
                alertNotify('Please select a date to proceed.','','info');
                return back()->withInput();
            }
        }
        if (isset($request->gender)) { $gender = $request->gender; }
        if (isset($request->location)) { $location = $request->location; }
        if (isset($request->date_range)) { $date_period = $request->date_range; }

        $deployment = getActiveDeployment();
        $applicants = $deployment->applicants()->count();
        $active = $deployment->applicants()->active()->count();
        $inactive = $applicants - $active;
        return view('applicant.get-applicant', compact(
            'applicant','page_title','query','value','gender',
            'location','applicants','active','inactive','date_period'));
    }*/


    public function getApplicantInfo(Request $request)
    {
        $page_title = __('Beneficiaries');
        $applicant = $query = $value = $date_period = null;
        $gender = $location = 0;
        if (sizeof($request->all()) > 0) {
            $query = $request->query_option;
            if (!is_null($request->dobs)) {
                $value = $request->dobs;
            }
            elseif (!is_null($request->refID)) {
                $value = $request->refID;
            }
            
            elseif (!is_null($request->beneficiary_option)) {
                $value = $request->beneficiary_option;
                $page_title .= ' - '.ucwords($value).' Beneficiary';
            }
            if ($request->query == 'dobs' && is_null($request->dobs)) {
                alertNotify('Please select a date to proceed.','','info');
                return back()->withInput();
            }
        }
        if (isset($request->gender)) { $gender = $request->gender; }
        if (isset($request->location)) { $location = $request->location; }
        if (isset($request->date_range)) { $date_period = $request->date_range; }

        $deployment = getActiveDeployment();
        $applicants = $deployment->applicants()->count();
        $active = $deployment->applicants()->active()->count();
        $inactive = $applicants - $active;
        return view('applicant.get-applicant', compact(
            'applicant','page_title','query','value','gender',
            'location','applicants','active','inactive','date_period'));
    }

    /**
     * Member Form
     * @param Request $request
     *
     * @return string
     */
    public function applicantRegistration($regID=null)
    {
        $page_title = __('Create');
        $applicant = null;
        $applicantCustomFieldValues = null;
        $applicant_docs = null;
        $pending_approval = false;
        $is_new = true;
        if (isset($regID)) {
            $page_title = __('Edit');
            $applicant = Applicant::find(Crypt::decrypt($regID));
            $applicant_docs = $applicant->documents;
            $is_new = false;

            if ($applicant->history->where('status','=',0)->where('approved','=',0)->count() > 0) {
                $pending_approval = true;
            }

            // Fetch the custom field values for specific applicant
            $applicantCustomFieldValues = DB::table('applicants_custom_field_values')
            ->where('applicant_id', $applicant->id)
            ->pluck('value', 'custom_field_id')
            ->toArray();
        }

        $villages = Village::orderBy('name')->get();
        $counties = County::where([['country_id', 115], ['status', 1]])->orderBy('county')->get();
        $subcounties = SubCounty::where('country_id', 115)->orderBy('name')->get();
        $customFields = CustomField::where('status', 1)->with('options')->get();

        

        return view('applicant.applicant-registration', compact('applicantCustomFieldValues','subcounties','counties','customFields','villages','page_title','is_new','applicant','applicant_docs'));
    }

    /**
     * Save Applicant Data
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveApplicantRegistration(Request $request)
    {
        return $this->createMemberShip($request);
//        if($applicant = ) {
//            return redirect()->route('applicant.applicantRegistration',[cryptThis($applicant->id)]);
//        }
//        return back();
    }

    /**
     * Check If Applicant already Exists
     * @param Request $request
     *
     * @return string
     */
    public function checkApplicantInformation($request)
    {
        $base_code = $this->getBaseRegID($request);

        $applicants = Applicant::where('regID','like',"$base_code%")
            ->where('surnames','like','%'.$request->surnames.'%')
            ->where('first_names','like','%'.$request->first_names.'%')
            ->where('mother_surnames','like','%'.$request->mother_surnames.'%')
            ->where('mother_first_names','like','%'.$request->mother_first_names.'%')
            ->where('father_surnames','like','%'.$request->father_surnames.'%')
            ->where('father_first_names','like','%'.$request->father_first_names.'%');

        if ($request->other_names != null) {
            $applicants->where('other_names','like','%'.$request->other_names.'%');
        }

        if ($request->mother_other_names != null) {
            $applicants->where('mother_other_names','like','%'.$request->mother_other_names.'%');
        }

        if ($request->father_other_names != null) {
            $applicants->where('father_other_names','like','%'.$request->father_other_names.'%');
        }

        return $applicants->get();
    }

    /**
     * View Applicant Information
     * @param Request $request
     *
     * @return string
     */
    public function viewApplicantInfo(Request $request)
    {
        $applicant = Applicant::with(['applications','idDetails', 'contactDetails','maritalDetails','employmentDetails','accountDetails','familyDetails','healthDetails','documents'])->find(Crypt::decrypt($request->app_id));
        $manual_verification = ($applicant->idDetails->where('status','=','Verified')->count() > 0)?"Manually":"Unverified";
        $eligible_benefits = applicant_eligible_benefits($applicant->regID);
        $view = true;
        $islands = Lookup::where('lookup_code_id','=', config('settings.island_code'))->pluck('fullName','short_code')->toArray();
        $countries = DB::table('countries')->pluck('en_short_name','alpha_3_code')->toArray();
        $place_of_birth = array_merge($islands,$countries);
        $nationalities = DB::table('countries')->pluck('nationality','alpha_3_code')->toArray();
        $total_bank_accounts = AccountDetail::where('regID','=',$applicant->regID)->where('removed','=',0)->count();
        $cases = CaseManagement::where('applicant_number','=',$applicant->regID)->get();
        $benefits = Benefit::where('archived','=',0)->whereNotNull('form_id')->orderBy('titles','asc')->get();
        $add_application = false;
        $customFields = CustomField::where('status', 1)->with('options')->get();
        // Fetch the custom field values for specific applicant
        $applicantCustomFieldValues = DB::table('applicants_custom_field_values')
        ->where('applicant_id', $applicant->id)
        ->pluck('value', 'custom_field_id')
        ->toArray();

        $render_form =  null;
        if (isset($request->benefit_type)) {
            $get_render_form = new FormRender();
            $benefit = Benefit::find(Crypt::decrypt($request->benefit_type));
            $get_render_form->submitUrl(route('benefit_registration.save',[cryptThis($benefit->id),$applicant->regID]));
            $render_form = $get_render_form->renderFormView($benefit->form->table_name);
            $add_application = true;
        }

        $applications = $applicant->applications;

        return view('applicant.applicant-view', compact('applicantCustomFieldValues','customFields','place_of_birth','nationalities','cases','manual_verification',
            'applicant','view','applications','total_bank_accounts','benefits','render_form','eligible_benefits','add_application'));
    }

    /**
     * View Applicant Record Information
     * @param Request $request
     * @return string
     *
     */
    public function viewApplicantRecordInfo(Request $request)
    {
        $record = DB::table($request->table)->where('id','=',(int)$request->record)->first();
        $details = true;
        $view = null;
        $title = 'DETAILS';
        $action = $request->action;

        if ($action == 'remove') {
            $record_id = $record->id;
            $form_type = $request->table;
            $view = view('applicant.applicant-form.delete-record',compact('record_id','form_type'))->render();
            return response()->json(['the_view'=>$view,'title'=>Str::singular(Str::headline($form_type))]);
        }

        if ($action == 'edit'|| $action == 'delete') $details = null;

        if ($request->table == 'family') {
            $familyDetail = Family::find((int)$request->record);
            if (!is_null($familyDetail->type)) $title = $familyDetail->surname.', '.$familyDetail->first_name.' - '.$familyDetail->type;
            $view = view('applicant.applicant-form.family-fields',compact('details','familyDetail','action','request'))->render();
        }
        elseif ($request->table == 'contact_details') {
            $contactDetail = ContactDetail::find((int)$request->record);
            if (!is_null($contactDetail->type_name)) $title = $contactDetail->type_name;
            $view = view('applicant.applicant-form.contact-fields',compact('details','contactDetail','action','request'))->render();
        }
        elseif ($request->table == 'identification_details') {
            $idDetail = IdentificationDetail::find((int)$request->record);
            if (!is_null($idDetail->id_type)) $title = $idDetail->id_type.' - '.$idDetail->id_number;
            $view = view('applicant.applicant-form.identifications-fields',compact('details','idDetail','action','request'))->render();
        }
        elseif ($request->table == 'employment_details') {
            $employmentDetail = EmploymentDetail::find((int)$request->record);
            if (!is_null($employmentDetail->employer_name)) $title = $employmentDetail->employer_name.' - '.$employmentDetail->employment_type;
            $view = view('applicant.applicant-form.employment-fields',compact('details','employmentDetail','action','request'))->render();
        }
        elseif ($request->table == 'account_details') {
            $accountDetail = AccountDetail::find((int)$request->record);
            if (!is_null($accountDetail->bank_name)) $title = $accountDetail->bank_name.' - '.$accountDetail->account_type;
            $view = view('applicant.applicant-form.banking-fields',compact('details','accountDetail','action','request'))->render();
        }
        elseif ($request->table == 'marital_details') {
            $maritalDetail = MaritalDetail::find((int)$request->record);
            if (!is_null($maritalDetail->marriage_status)) $title = $maritalDetail->marriage_status.' - '.$maritalDetail->marriage_type;
            $view = view('applicant.applicant-form.marital-fields',compact('details','maritalDetail','action','request'))->render();
        }
        elseif ($request->table == 'health_details') {
            $healthDetail = HealthDetail::find((int)$request->record);
            if (!is_null($healthDetail->health_type)) $title = $healthDetail->health_type;
            $view = view('applicant.applicant-form.health-fields',compact('details','healthDetail','action','request'))->render();
        }

        return response()->json(['the_view'=>$view,'title'=>$title]);
    }

    /**
     * Update Applicant Record
     * @param Request $request
     *
     * @return string
     */
    public function updateApplicantRecord(Request $request)
    {
        return $this->updateMembershipRecord($request);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function verifyContact(Request $request)
    {
        $record = ContactDetail::where('id','=', $request->record_id)->where('code','=', $request->code)->update([
           'code'   =>  null,
           'status'   =>  'Verified',
           'default'   =>  true,
        ]);

        if ($record) {
            $message = 'The contact has been verified.';
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = 'Invalid code or record cannot be found. Please try again';
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    public function getPendingUpdates($applicant_number = null)
    {
        if (!is_null($applicant_number)) {
            $applicant = Applicant::where('regID','=',$applicant_number)->first();
            if ($applicant) {
                $pendingRequests = $applicant->history;
            }
        }
        else {
            $pendingRequests = ApplicantRecord::pending()->get();
        }

        return $pendingRequests;
    }

    public function savePendingUpdates(Request $request)
    {
        if ($request->approve == 1) {

            $pendingRequest = ApplicantRecord::find(Crypt::decrypt($request->pending_record));

            if ($pendingRequest) {
                $applicant = Applicant::where('regID','=',$pendingRequest->applicant_number)->first();
                if ($applicant) {
                    $applicant->surnames = $pendingRequest->surnames;
                    $applicant->first_names = $pendingRequest->first_names;
                    $applicant->other_names = $pendingRequest->other_names;
                    $applicant->dobs = $pendingRequest->dobs;
                    $applicant->genders = $pendingRequest->genders;
                    $applicant->place_of_births = $pendingRequest->place_of_births;
                    $applicant->country_of_births = $pendingRequest->country_of_births;
                    $applicant->nationalities = $pendingRequest->nationalities;
                    $applicant->updated_by = $pendingRequest->updated_by;
                    $applicant->upload_passport_pic = $pendingRequest->upload_passport_pic;
                    $applicant->avatar = $pendingRequest->avatar;
                    $applicant->save();

                    $pendingRequest->status = 1;
                    $pendingRequest->modified_by = Auth::id();
                    $pendingRequest->save();

                    $message = 'Record has been updated';
                    session()->flash('status', 'success');
                }
                else {
                    $message = 'Something went wrong. Please try again';
                    session()->flash('status', 'error');
                }
            }
            else {
                $message = 'Something went wrong. Please try again';
                session()->flash('status', 'error');
            }
        }
        else {
            $message = 'Record has been updated';
            session()->flash('status', 'success');
        }

        session()->flash('message', $message);
        return back();
    }

    public function getAllMerchants(Request $request) {
        $merchants = Bank::withCount(['branches'])->where('type','=',$request->account_type)->where('status','=',1)->get();

        $options = "<option value=''>-- ".__('Choose Merchant')." --</option>";
        foreach ($merchants as $key=>$agent) {
            if ($agent->branches_count <= 0) continue;
            $options .= "<option value='$agent->id'>$agent->name</option>";
        }
        return response()->json($options);
    }

    public function getMerchantBranches(Request $request) {
        $branches = getMerchBranches($request->bank_id);
        $options = "<option value=''>-- ".__('Choose Branch')." --</option>";
        foreach ($branches as $key=>$branch) {
            $options .= "<option value='$key'>$branch</option>";
        }
        return response()->json($options);
    }

    /**
     * Delete/Deactivate Applicant Data
     * @param Request $request
     *
     * @return string
     */
    public function deleteApplicantInfo($id)
    {
        $applicant = Applicant::destroy(Crypt::decrypt($id));

        if ($applicant) {
            alertNotify('Applicant has been removed','','success');
        }
        else {
            alertNotify('Sorry, Something went wrong or applicant account does not exist.','','danger');
        }
        return back();
    }

    /**
     * View Applicant Financials
     * @param Request $request
     *
     * @return string
     */
    public function financialStatements($applicant_number)
    {
        $applicant_info = Applicant::where('regID','=',$applicant_number)->first();
        $benefits = Application::where('applicant_number','=',$applicant_number)->where('is_beneficiary','=',1)->pluck('type','benefit_id');
        return view('applicant.financial-statement', compact('applicant_info','benefits'));
    }

    public function duplicateDetection(Request $request)
    {
        return view('duplicate-detection.index');
    }

    public function getDuplicates(Request $request)
    {
        //Get all active applicant application numbers
        $applicant_application_numbers = Applicant::active()->pluck('regID')->toArray();

        //Remove the trailing 3 characters from all the application numbers
        $applicant_application_numbers_cleaned = [];
        foreach($applicant_application_numbers as $key => $value) {
            $applicant_application_numbers_cleaned[] = substr($value, 0, -4);
        }

        //Get only duplicated application numbers
        $duplicated_application_numbers_cleaned = array_values(array_unique(array_diff_assoc($applicant_application_numbers_cleaned, array_unique($applicant_application_numbers_cleaned))));

        //Loop through the applicants with the $duplicated_application_numbers_cleaned comparing the applicant numbers and names
        $duplicates_container = array();
        for ($offset=0; $offset < sizeof($duplicated_application_numbers_cleaned); $offset++) {
            $subject = $duplicated_application_numbers_cleaned[$offset];
            $duplicates = array();
            $duplicates = DB::table('applicants')
                ->select('id', 'regID', 'surnames', 'first_names','other_names','dobs','genders')
                ->where('regID','like',"$subject%")
                ->get()->toArray();

            if(sizeof($duplicates) > 1) {
                $duplicates_container[] = $duplicates;
            }
        }
        // dd($duplicates_container);

        $view = view('duplicate-detection.duplication-result', compact('duplicates_container'))->render();

        return response()->json(['view' => $view]);
    }

    public function removeProfileImage(Request $request)
    {
        $applicant = Applicant::where('regID','=',$request->regID)->first();

        if (!$applicant) {
            notify('Invalid request','error','Sorry!');
            return back();
        }

        $applicant->avatar = null;
        $applicant->upload_passport_pic = null;
        $applicant->save();
        notify('Profile image has been removed','success','Successful!');
        return back();
    }
}
