<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Auth\Traits\AuthenticatesUsers;
use App\Http\Controllers\Controller;
use App\Models\Customer;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;

class CustomerLoginController extends Controller
{

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/dashboard';
    protected $maxAttempts = 5; // Default is 5
    protected $decayMinutes = 1; // Default is 1

    /**
     * Show Customer's login form
     *
     */
    public function customerLoginForm()
    {
        return view('auth.login');
    }

//    protected function authenticated(Request $request, $user)
//    {
//        $user->last_login = date('Y-m-d H:i:s');
//        $user->save();
//    }


    protected function authenticated(Request $request, $user)
    {
        Auth::logoutOtherDevices($request->password);
        Auth::user()->last_login = date('Y-m-d H:i:s');
        Auth::user()->save();
    }

    /**
     * Get the login username to be used by the controller.
     *
     * @return string
     */
    public function username()
    {
        if(is_numeric(cleanString(request()->phone))) {
            return 'phone';
        }
        else {
            return 'email';
        }
    }
    /**
     * Get the login username to be used by the controller.
     *
     * @return string
     */
    public function redirectPath()
    {
        $prefix = config('app.portal_base_url','portal');
        return "/$prefix".$this->redirectTo;
    }

    /**
     * Get the guard to be used during authentication.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
//    protected function guard()
//    {
//        return Auth::guard('customer');
//    }

    /**
     * Log the user out of the application.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function logout(Request $request)
    {
        Auth::logout();

        $request->session()->invalidate();

        $request->session()->regenerateToken();

        return redirect()->route('login');
    }

    public function forgotPassword(Request $request,$phone=null,$reset_password_token=null)
    {

        if (isset($request->mobile)) {
            $customer = User::where('phone', cleanString($request->mobile))->first();

            if ($customer) {
                if ($customer->deactivated_at != null) {
                    $message = "Sorry! This account is inactive. Please contact support.";
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return redirect()->route('customer.forgotPassword');
                }
                if ($customer->account_verified_at == null) {
                    $message = "Sorry! This account has not been verified yet. Enter the code sent to your phone number to verify.";
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return redirect()->route('customer.forgotPassword');
                }
                $reset_password_token = verificationCode(24);
                $code = verificationCode();
                $phone = cleanString($customer->phone);
                $customer->reset_password_token = $reset_password_token;
                $customer->verification_code = $code;
                if($customer->save()) {
                    $message = "A code has been sent to your phone number. Use the code to reset your password.";
                    session()->flash('status', 'success');
                    session()->flash('message', $message);
                    $sms = setAppDefault('app_project_name').": Use the verification code - $code to reset your password. Thank you.";
                    sendSMS(cleanString($request->mobile),$sms);
                    return redirect()->route('customer.forgotPassword',[$phone,$reset_password_token]);
                }
                else {
                    $message = "Sorry! Something went wrong. Please try again.";
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return redirect()->route('customer.forgotPassword');
                }

            }
            else {
                $message = "Sorry! This phone number cannot be found in our records.";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back();
            }
        }

        if ($reset_password_token) {
            $reset_password_token_ = User::where('reset_password_token', $reset_password_token)->first();
            if (!$reset_password_token_) {
                $message = "Sorry! Invalid reset password token or token has expired.";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back();
            }
        }
        return view('auth.customer-forgot-password',compact('reset_password_token','phone'));
    }

    public function forgotPasswordSet(Request $request)
    {
        $this->validate($request, [
            'code' => 'required|string|min:4|max:4',
            'password' => 'required|string|min:6|confirmed',
        ]);

        $customer = User::where('reset_password_token',$request->reset_password_token)->where('verification_code',$request->code)->first();

        if ($customer) {

            if($customer->deactivated_at != null){
                $message = "Sorry! This account is inactive. Please contact support.";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back();
            }

            $customer->verification_code = null;
            $customer->reset_password_token = null;
            $customer->password = Hash::make($request->password);
            if ($customer->save()) {
                $message = 'Successful! Please login.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                return redirect()->route('login');
            }
            else {
                $message = "Sorry! Please try again.";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back();
            }

        }
        else {
            $message = "Sorry! Invalid token or code.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }

    }

    public function credentials(Request $request)
    {
        if ($this->username() == 'phone') {
            $credentials = $request->only($this->username(), 'password','call_code');
        }
        else {
            $credentials = $request->only($this->username(), 'password');
        }
//        dd($credentials);

        $credentials['is_admin'] = 0;
        return $credentials;
    }
}
