<?php

namespace App\Http\Controllers;

use App\Http\Helpers\FormRender;
use App\Http\Traits\BlackBox;
use App\Http\Traits\FormSubmission;
use App\Models\Application;
use App\Models\BeneficiariesArchivedLog;
use App\Models\BeneficiariesDetail;
use App\Models\BeneficiariesFormMapping;
use App\Models\Beneficiary;
use App\Models\Benefit;
use App\Models\BenefitFormOption;
use App\Models\BenefitRegistration;
use App\Models\DebtRecovery;
use App\Models\Form;
use App\Models\Payroll;
use App\Models\PayrollHistory;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use \Illuminate\Support\Facades\Schema;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Yajra\DataTables\DataTables;

class BeneficiaryController extends Controller
{
    use FormSubmission;
    use BlackBox;

    public function __construct()
    {
        $this->middleware(['auth', 'verified']);
    }

    /**
     * Display a listing of Beneficiaries.
     * @param null $benefit_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function index($benefit_id = null)
    {
        $benefit = null;
        if (isset($benefit_id)) {
            $benefit = Benefit::find(Crypt::decrypt($benefit_id));
        }

        return view('beneficiaries.list',compact('benefit_id','benefit'));
    }

    /**
     * Get the list of Beneficiaries DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getBeneficiariesList($benefit_id = null)
    {
        if (is_null($benefit_id)) {
            $applications = Application::where('is_beneficiary','=',1)->where('archived','=',0)->orderBy('confirmed_date','desc')->get();
        }
        else {
            $benefit_form = Benefit::where('id','=', Crypt::decrypt($benefit_id))->pluck('form_id')->first();
            $applications = Application::where('is_beneficiary','=',1)->where('archived','=',0)->where('form_id','=',$benefit_form)->orderBy('confirmed_date','desc')->get();
        }

        return DataTables::of($applications)
            ->addColumn('action', function ($application) {
//                    $edit = '<a href="'.route('benefit_registration.edit',[cryptThis($beneficiary->form_id),cryptThis($beneficiary->application_id)]).
//                        '" class=" btn btn-icon btn-xs btn-light-primary mb-2  mr-1" data-toggle="tooltip" title="'. __("Edit Application") .'"><i class="la la-edit"></i></a>';

                    $applicant = '<a href="'.route('benefit_registration.viewApplicantInfo',['app_id'=>cryptThis($application->applicant->id)]).'" 
                    class="btn btn-success btn-xs btn-icon mb-2 mr-1" data-toggle="tooltip" title="'. __("View Applicant") .'"><i class="la la-user"></i></a>';
                    $view = '<a href="'.route('benefit_registration.view',[cryptThis($application->id)]).'" 
                    class="btn btn-primary btn-xs btn-icon mb-2 mr-1" data-toggle="tooltip" title="'. __("View Application") .'"><i class="la la-eye"></i></a>';

//                    $delete = '<a data-container="body" data-toggle="tooltip" href="'.route('beneficiaries.delete',[cryptThis($application->id)]).
//                        '" onclick="return confirm(\'The beneficiary will be taken off the list completely. Do you want to proceed?\')"
//                        class="btn btn-sm mb-2 btn-icon mr-1 btn-danger" title="'. __("Delete Beneficiary").'"><i class="la la-trash"></i></a>';

//                $archive = '<a data-container="body" data-toggle="tooltip" href="'.route('beneficiaries.archive',[cryptThis($application->id)]).'"
//                        onclick="return confirm(\'Archiving will prevent the beneficiary from receiving this benefit. Do you want to proceed?\')"
//                        class="btn btn-xs mb-2 mr-1 btn-icon btn-light-danger" title="'. __("Archive Beneficiary") .'"><i class="la la-lock"></i></a>';

                    $archive = '<a data-container="body" data-toggle="tooltip" href="'.route('beneficiaries.archive',[cryptThis($application->id)]).'"
                        onclick="return confirm(\'Archiving will prevent the beneficiary from receiving this benefit. Do you want to proceed?\')"
                        class="btn btn-xs mb-2 mr-1 btn-icon btn-light-danger" title="'. __("Archive Beneficiary") .'"><i class="la la-trash"></i></a>';

                return $applicant.' '.$view
                    .' '.$archive
                    .'
                            <a href="javascript:;" onclick="setId('.$application->id.')"
                        class="btn btn-xs mb-2 btn-icon btn-warning" data-toggle="modal" data-target="#staticBackdrop" data-beneficiary="'.cryptThis($application->id).'"
                        data-toggle="tooltip" title="'.__("Perform a Debt Recovery on this Beneficiary").'"><i class="la la-money"></i></a>';
            })
            ->editColumn('benefit_id', function ($application) {
//                $benefit = BenefitFormOption::where('form_id','=',$application->form_id)->first();
                return ucwords($application->type);
            })
            ->editColumn('surname', function ($application) {
                $name = "";
//                $surname = BeneficiariesDetail::where('beneficiary_id','=',$application->id)->pluck('surnames')->first();
                if(isset($application->applicant->surnames)) {
                    $name .= "<strong>".strtoupper($application->applicant->surnames)."</strong>";
                }
//                $first_name = BeneficiariesDetail::where('beneficiary_id','=',$application->id)->pluck('first_names')->first();
                if(isset($application->applicant->first_names)) {
                    $name .= ", ".ucwords($application->applicant->first_names);
                }

                return ucwords($name);
            })
//            ->editColumn('first_name', function ($application) {
//                $first_name = BeneficiariesDetail::where('beneficiary_id','=',$application->id)->pluck('first_names')->first();
//                return ucwords($first_name);
//            })
            ->editColumn('applicant_number', function ($application) {
//                $applicant_number = BeneficiariesDetail::where('beneficiary_id','=',$application->id)->pluck('applicant_number')->first();
                return $application->applicant_number;
            })
            ->editColumn('dob', function ($application) {
//                $dob = BeneficiariesDetail::where('beneficiary_id','=',$application->id)->pluck('dobs')->first();
                return "<span style='display:none'>".$application->applicant->dobs."</span>".date('d M, Y',strtotime($application->applicant->dobs))." (".getAgeByDate($application->applicant->dobs).")";
            })
            ->editColumn('confirmed_date', function ($application) {
                return date('d M, Y',strtotime($application->confirmed_date));
            })
            ->editColumn('bci_account_numbers', function ($application) {
                $account_numbers = BeneficiariesDetail::where('beneficiary_id','=',$application->id)->pluck('bci_account_numbers')->first();
                return $account_numbers;
            })
            ->escapeColumns([])
            ->make();
    }
//    public function getBeneficiariesList($benefit_id = null)
//    {
//        if (is_null($benefit_id)) {
//            $beneficiaries = Beneficiary::where('archived','=',0)->orderBy('updated_at','desc')->get();
//        }
//        else {
//            $benefit_form = Benefit::where('id','=', Crypt::decrypt($benefit_id))->pluck('form_id')->first();
//            $beneficiaries = Beneficiary::where('archived','=',0)->where('form_id','=',$benefit_form)->orderBy('updated_at','desc')->get();
//        }
//
//        return DataTables::of($beneficiaries)
//            ->addColumn('action', function ($beneficiary) {
//                $edit = '<a href="'.route('beneficiaries.edit',[cryptThis($beneficiary->id)]).
//                    '" data-container="body" data-toggle="tooltip" class="btn btn-sm mb-2 mr-1 btn-icon btn-light-primary" title="'. __("Edit Beneficiary") .'"><i class="la la-edit"></i></a>';
//
//                $view = '<a href="'.route('beneficiaries.view',[cryptThis($beneficiary->id)]).
//                    '" data-container="body" data-toggle="tooltip" class="btn btn-sm mb-2 mr-1 btn-icon btn-primary" title="'. __("View Details") .'"><i class="la la-eye"></i></a>';
//
//                $delete = '<a data-container="body" data-toggle="tooltip" href="'.route('beneficiaries.delete',[cryptThis($beneficiary->id)]).
//                    '" onclick="return confirm(\'The beneficiary will be taken off the list completely. Do you want to proceed?\')"
//                        class="btn btn-sm mb-2 btn-icon mr-1 btn-danger" title="'. __("Delete Beneficiary").'"><i class="la la-trash"></i></a>';
//
//                $archive = '<a data-container="body" data-toggle="tooltip" href="'.route('beneficiaries.archive',[cryptThis($beneficiary->id)]).'"
//                        onclick="return confirm(\'Archiving will prevent the beneficiary from receiving this benefit. Do you want to proceed?\')"
//                        class="btn btn-sm mb-2 mr-1 btn-icon btn-light-danger" title="'. __("Archive Beneficiary") .'"><i class="la la-lock"></i></a>';
//
//                return $edit.' '.$view.''.$archive.'
//                            <a href="javascript:;" onclick="setId('.$beneficiary->id.')"
//                        class="btn btn-sm mb-2 btn-icon btn-warning" data-toggle="modal" data-target="#staticBackdrop" data-beneficiary="'.cryptThis($beneficiary->id).'"
//                        data-toggle="tooltip" title="'.__("Perform a Debt Recovery on this Beneficiary").'"><i class="la la-money"></i></a>';
//            })
//            ->editColumn('form_id', function ($beneficiary) {
//                $benefit = BenefitFormOption::where('form_id','=',$beneficiary->form_id)->first();
//                return ucwords($benefit->benefit->titles);
//            })
//            ->editColumn('surname', function ($beneficiary) {
//                $name = "";
//                $surname = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('surnames')->first();
//                if(isset($surname)) {
//                    $name .= "<strong>".strtoupper($surname)."</strong>";
//                }
//                $first_name = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('first_names')->first();
//                if(isset($first_name)) {
//                    $name .= ", ".ucwords($first_name);
//                }
//
//                return ucwords($name);
//            })
////            ->editColumn('first_name', function ($beneficiary) {
////                $first_name = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('first_names')->first();
////                return ucwords($first_name);
////            })
//            ->editColumn('applicant_number', function ($beneficiary) {
//                $applicant_number = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('applicant_number')->first();
//                return $applicant_number;
//            })
//            ->editColumn('dob', function ($beneficiary) {
//                $dob = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('dobs')->first();
//                return date('d M, Y',strtotime($dob))." (".getBeneficiaryAgeByID($beneficiary->id).")";
//            })
//            ->editColumn('created_at', function ($beneficiary) {
//                return date('d M, Y',strtotime($beneficiary->created_at));
//            })
//            ->editColumn('bci_account_numbers', function ($beneficiary) {
//                $account_numbers = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('bci_account_numbers')->first();
//                return $account_numbers;
//            })
//            ->escapeColumns([])
//            ->make();
//    }

    /**
     * List of archived Beneficiaries
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function archives()
    {
        return view('beneficiaries.archived-list');
    }

    /**
     * Get the list of Beneficiaries DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getArchivedBeneficiariesList($benefit_id = null)
    {
//        $beneficiaries = Beneficiary::where('archived','=',true)->orderBy('confirmed_date','desc')->get();

        if (is_null($benefit_id)) {
            $beneficiaries = Application::where('is_beneficiary','=',1)->where('archived','=',1)->orderBy('confirmed_date','desc')->get();
        }
        else {
            $benefit_form = Benefit::where('id','=', Crypt::decrypt($benefit_id))->pluck('form_id')->first();
            $beneficiaries = Application::where('is_beneficiary','=',1)->where('archived','=',1)->where('form_id','=',12)->orderBy('confirmed_date','desc')->get();
        }
//        dd($benefit_form);

        return DataTables::of($beneficiaries)
            ->addColumn('action', function ($beneficiary) {
                    return '<a href="'.route('beneficiaries.archive',[cryptThis($beneficiary->id)]).'" 
                        onclick="return confirm(\'Restoring this beneficiary will be eligible for Benefit. Do you want to proceed?\')"
                        class="btn btn-xs btn-icon mb-2 btn-success" data-toggle="tooltip" title="'.__("Restore Beneficiary").'"><i class="la la-trash-restore"></i></a>
                        <a href="javascript:;" onclick="setId('.$beneficiary->id.')"
                        class="btn btn-xs mb-2 btn-icon btn-warning" data-toggle="modal" data-target="#staticBackdrop" data-beneficiary="'.cryptThis($beneficiary->id).'"
                        data-toggle="tooltip" title="'.__("Perform a Debt Recovery on this Beneficiary").'"><i class="la la-money"></i></a>';
            })
            ->editColumn('type', function ($beneficiary) {
//                $benefit = BenefitFormOption::where('form_id','=',$beneficiary->form_id)->first();
                return ucwords($beneficiary->type);
            })
            ->editColumn('surname', function ($beneficiary) {
                $name = "";
//                $surname = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('surnames')->first();
                if(isset($beneficiary->applicant->surnames)) {
                    $name .= "<strong>".strtoupper($beneficiary->applicant->surnames)."</strong>";
                }
//                $first_name = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('first_names')->first();
                if(isset($beneficiary->applicant->first_names)) {
                    $name .= ", ".ucwords($beneficiary->applicant->first_names);
                }

                return ucwords($name);
            })
//            ->editColumn('first_name', function ($beneficiary) {
//                $first_name = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('first_names')->first();
//                return ucwords($first_name);
//            })
            ->editColumn('applicant_number', function ($beneficiary) {
//                $applicant_number = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('applicant_number')->first();
                return $beneficiary->applicant_number;
            })
            ->editColumn('dob', function ($beneficiary) {
//                $dob = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('dobs')->first();
                return date('d M, Y',strtotime($beneficiary->applicant->dobs))." (".getAgeByDate($beneficiary->applicant->dobs).")";
            })
            ->editColumn('confirmed_date', function ($beneficiary) {
                return date('d M, Y',strtotime($beneficiary->confirmed_date));
            })
//            ->editColumn('bci_account_numbers', function ($beneficiary) {
//                $account_numbers = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('bci_account_numbers')->first();
//                return $account_numbers;
//            })
            ->escapeColumns([])
            ->make();
    }
//    public function getArchivedBeneficiariesList($benefit_id = null)
//    {
////        $beneficiaries = Beneficiary::where('archived','=',true)->orderBy('updated_at','desc')->get();
//
//        if (is_null($benefit_id)) {
//            $beneficiaries = Beneficiary::where('archived','=',1)->orderBy('updated_at','desc')->get();
//        }
//        else {
//            $benefit_form = Benefit::where('id','=', Crypt::decrypt($benefit_id))->pluck('form_id')->first();
//            $beneficiaries = Beneficiary::where('archived','=',1)->where('form_id','=',12)->orderBy('updated_at','desc')->get();
//        }
////        dd($benefit_form);
//
//        return DataTables::of($beneficiaries)
//            ->addColumn('action', function ($beneficiary) {
//                    return '<a href="'.route('beneficiaries.archive',[cryptThis($beneficiary->id)]).'"
//                        onclick="return confirm(\'Restoring this beneficiary will be eligible for Benefit. Do you want to proceed?\')"
//                        class="btn btn-sm btn-icon mb-2 btn-primary" data-toggle="tooltip" title="'.__("Restore Beneficiary").'"><i class="la la-box-open"></i></a>
//                        <a href="javascript:;" onclick="setId('.$beneficiary->id.')"
//                        class="btn btn-sm mb-2 btn-icon btn-warning" data-toggle="modal" data-target="#staticBackdrop" data-beneficiary="'.cryptThis($beneficiary->id).'"
//                        data-toggle="tooltip" title="'.__("Perform a Debt Recovery on this Beneficiary").'"><i class="la la-money"></i></a>';
//            })
//            ->editColumn('form_id', function ($beneficiary) {
//                $benefit = BenefitFormOption::where('form_id','=',$beneficiary->form_id)->first();
//                return ucwords($benefit->benefit->titles);
//            })
//            ->editColumn('surname', function ($beneficiary) {
//                $name = "";
//                $surname = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('surnames')->first();
//                if(isset($surname)) {
//                    $name .= "<strong>".strtoupper($surname)."</strong>";
//                }
//                $first_name = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('first_names')->first();
//                if(isset($first_name)) {
//                    $name .= ", ".ucwords($first_name);
//                }
//
//                return ucwords($name);
//            })
//            ->editColumn('applicant_number', function ($beneficiary) {
//                $applicant_number = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('applicant_number')->first();
//                return $applicant_number;
//            })
//            ->editColumn('dob', function ($beneficiary) {
//                $dob = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('dobs')->first();
//                return date('d M, Y',strtotime($dob))." (".getBeneficiaryAgeByID($beneficiary->id).")";
//            })
//            ->editColumn('created_at', function ($beneficiary) {
//                return date('d M, Y',strtotime($beneficiary->created_at));
//            })
//            ->editColumn('bci_account_numbers', function ($beneficiary) {
//                $account_numbers = BeneficiariesDetail::where('beneficiary_id','=',$beneficiary->id)->pluck('bci_account_numbers')->first();
//                return $account_numbers;
//            })
//            ->escapeColumns([])
//            ->make();
//    }

    /**
     * Edit Beneficiary information
     * @param $beneficiary_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function edit($beneficiary_id)
    {
        $beneficiary = Beneficiary::find(Crypt::decrypt($beneficiary_id));
        $get_render_form = new FormRender();
        $get_render_form->request_id(cryptThis($beneficiary->benefitRegistration->application_id));
        $get_render_form->submitUrl(route('beneficiaries.update',[$beneficiary_id]));
        $render_form = $get_render_form->renderFormView($beneficiary->form->table_name);
        $page_title = __('Edit Details');
        return view('beneficiaries.edit', compact('render_form','page_title','beneficiary'));
    }

    /**
     * Edit Beneficiary information
     * @param $beneficiary_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function view($beneficiary_id)
    {
        $beneficiary = Beneficiary::find(Crypt::decrypt($beneficiary_id));
        $applications = BenefitRegistration::where('applicant_number','=',$beneficiary->benefitRegistration->applicant_number)->get();

        //Get the various benefits the applicant is on
        $applicant_ids = $applications->pluck('id')->toArray();
        $benefits = array();
        $beneficiary_ids = array();
        foreach ($applicant_ids as $key=>$registration_id) {
            $beneficiary = Beneficiary::where('benefit_registration_id','=',$registration_id)->first();
            $beneficiary_ids[] = $beneficiary->id;
            $benefits[] = $beneficiary->form->benefit->id;
        }
//        dd($beneficiary_ids);
        $beneficiary_payrolls = Payroll::whereIn('benefit_id',$benefits)->where('cleared','=',1)->get();

        $get_render_form = new FormRender();
        $get_render_form->request_id(cryptThis($beneficiary->benefitRegistration->application_id));
        $get_render_form->displayAsDetails(true);
//        $get_render_form->submitUrl(route('beneficiaries.update',[$beneficiary_id]));
        $render_form = $get_render_form->renderFormView($beneficiary->form->table_name);
        $page_title = __('View Details');
        $audit = \App\Models\BlackBox::where('auditable_type','=',$beneficiary->form->table_name)
            ->where('auditable_id','=',$beneficiary->benefitRegistration->application_id)->get();
//        dd($beneficiary->benefitRegistration->application_id);
        return view('beneficiaries.edit', compact('render_form','page_title','audit','beneficiary','applications','beneficiary_payrolls','beneficiary_ids'));
    }

    /**
     * Update Beneficiary information
     * @param Request $request
     * @param $beneficiary_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Request $request, $beneficiary_id)
    {
        $beneficiary = Beneficiary::find(Crypt::decrypt($beneficiary_id));
        $benefit_registration_id = $beneficiary->benefit_registration_id;
        try {
            $benefit_registration = BenefitRegistration::find($benefit_registration_id);
            $oldData = $this->getData($benefit_registration->form->table_name,$benefit_registration->application_id);
            $beneficiary = $this->submit($request);
//            $newData = $this->getData($benefit_registration->form->table_name,$benefit_registration->application_id);

//            $amended = $this->getAmendedData((array)$oldData,(array)$newData);
            $this->initiateBlackBox($benefit_registration->form->table_name,$benefit_registration->application_id,(array)$oldData,$request);
//            dd($beneficiary);

            if ($beneficiary && transferBeneficiaryInformation($benefit_registration_id)) {
                $message = "The information has been updated.";
                $type = "success";
                alertNotify($message,null,$type);

                return redirect()->route('beneficiaries.list');
            }
            else {
                $message = "Sorry! Something went wrong. Please try again.";
                $type = "warning";
                alertNotify($message,'Error!',$type);
                return back();
            }
        }
        catch (\Exception $exception) {
            $message = $exception->getMessage();
            $type = "warning";
            alertNotify($message,'Error!',$type);
            return back();
        }
    }

    /**
     * Get the lists of Benefit Forms
     */
    public function formMapping()
    {
//        $forms = BenefitFormOption::all();
        $benefits = Benefit::where('form_id', '>', 0)->get();

        return view('beneficiaries.form-mappings', compact('benefits'));
    }

    /**
     * Mapping Benefit form columns to Beneficiaries table to extract information
     * @param $form_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function formMappingDetails($form_id)
    {
        $benefit_form = Form::find(Crypt::decrypt($form_id));
        if (is_null($benefit_form)) {
            alertNotify('Sorry! This Benefit has not been assigned a form. Please update this Benefit by selecting a form.',null,'danger');
            return back();
        }
        $benefit_form_fields = $benefit_form->fields;

        //Get existing form mapping information
        $form_mapping = BeneficiariesFormMapping::where('form_id','=',Crypt::decrypt($form_id))->first();
//dd($form_mapping);
        //Get Beneficiaries Details form fields
        $beneficiary_details= new BeneficiariesDetail();
        $table = $beneficiary_details->getTable();
        $beneficiary_details_form_info = Form::where('table_name','=',$table)->first();
        $beneficiary_details_form_fields = $beneficiary_details_form_info->fields;

//        $beneficiary_details_form_fields  = Schema::getColumnListing($table);

        return view('beneficiaries.map-forms', compact('beneficiary_details_form_info','beneficiary_details_form_fields',
            'benefit_form','benefit_form_fields','form_mapping'));
    }

    /**
     * Save Mapping Benefit form columns to Beneficiaries table to extract information
     * @param Request $request
     * @param $form_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveFormMappingDetails(Request $request, $form_id)
    {
        $form_mapping = BeneficiariesFormMapping::where('form_id','=',Crypt::decrypt($form_id))->first();

        try{
            if (!$form_mapping) {
                BeneficiariesFormMapping::updateOrCreate([
                    'form_id'   =>  Crypt::decrypt($form_id)
                ],[
                    'form_fields'   =>  $request->benefit_fields,
                    'show_fields_on_table'   =>  $request->show_on_voucher,
                    'added_by'   =>  Auth::id(),
                    'updated_by'   =>  Auth::id(),
                ]);
                alertNotify('Forms have been mapped.','Successful','success');
                return redirect()->route('beneficiaries.formMapping');
            }
            else {
                $form_mapping->form_fields = $request->benefit_fields;
                $form_mapping->show_fields_on_table = $request->show_on_voucher;
                $form_mapping->updated_by = Auth::id();
                $form_mapping->save();
                alertNotify('Forms have been mapped.','Successful','success');
                return redirect()->route('beneficiaries.formMapping');
            }
        }
        catch (\Exception $exception) {
//            dd($exception->getMessage());
            alertNotify('Please try again','Error','warning');
            return back();
        }
    }

    /**
     * Archive/Restore a Beneficiary
     * @param $application_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function archive($application_id)
    {
        $application = Application::find(Crypt::decrypt($application_id));
//        $benefit_id = BenefitFormOption::where('form_id','=',$application->form_id)
//            ->pluck('benefit_id')->first();
//        $benefit = Benefit::find($benefit_id);

        if ($application->benefit->archived == 1 && $application->archived == 1) {
            alertNotify('Beneficiary cannot be restored. The Benefit associated to this beneficiary has been archived.',null,'warning');
        }
        else {
            if ($application->archived == 1) {
                $application->archived = false;

                if ($application->save()) {
                    BeneficiariesArchivedLog::create([
                        'user_id'   =>  Auth::id(),
                        'application_id'   =>  $application->id,
                        'restored_date'   =>  date('Y-m-d H:i:s')
                    ]);
                    alertNotify('Beneficiary has been restored.',null,'success');
                }
            }
            else {
                $application->archived = true;
                if ($application->save()) {
                    BeneficiariesArchivedLog::create([
                        'user_id'   =>  Auth::id(),
                        'application_id'   =>  $application->id,
                        'archived_date'   =>  date('Y-m-d H:i:s')
                    ]);
                    alertNotify('Beneficiary has been archived.',null,'success');
                }
            }
        }

        return back();
    }

    /**
     * Delete a Beneficiary
     * @param $application_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function delete($application_id)
    {
        if (Beneficiary::destroy(Crypt::decrypt($application_id))) {
            alertNotify('Beneficiary has been deleted.',null,'success');
        }
        else {
            alertNotify('Sorry! Something went wrong. Try again.',null,'danger');
        }
        return back();
    }

    /**
     * Debt Recovery on Beneficiary
     *
     * @param Request $request
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function recoverDebtForm(Request $request)
    {
        if (!isset($request->date_range)) {
            alertNotify('Please select a date range',null,'danger');
            return back();
        }
//        $application_id = 3;
        $application_id = (int)$request->beneficiary;
        $beneficiary = Application::find($application_id);
        if (isset($beneficiary)) {

//            if ($beneficiary->archived == 0) {
////                alertNotify('Beneficiary must be archived.','Action Denied!','danger');
////                return back();
////            }

            //split date range value to from and to
            $split_dates = explode('/',str_replace(' ', '', $request->date_range));
            $from = date($split_dates[0]);
            $to = date($split_dates[1]);

            $payroll_ids = DB::table('payroll_history')
                ->join('payrolls','payroll_history.payroll_id','=','payrolls.id')
                ->where('payrolls.payment_date','>=',date('Y-m-d', strtotime($from)))
                ->where('payrolls.payment_date','<=',date('Y-m-d', strtotime($to)))
                ->pluck('payroll_history.benefit_id','payroll_history.payroll_id')->toArray();

//            dd($payroll_ids);
            $total_amount = 0;
            $totalPerBenefit[] = null;
            if (empty($payroll_ids)) {
                alertNotify('Sorry! No Payroll identified for this period.','No results','info');
            }
            else{
                foreach ($payroll_ids as $payroll_id=>$benefit_id) {
                    $beneficiary_info = PayrollHistory::where('payroll_id','=',$payroll_id)->pluck('beneficiaries_data')->first();

                    if (isset($beneficiary_info[$application_id])) {
                        $total_amount += $beneficiary_info[$application_id];

                        if (isset($totalPerBenefit[$benefit_id])) {
                            $totalPerBenefit[ $benefit_id ] += $beneficiary_info[$application_id];
                        }
                        else {
                            $totalPerBenefit[ $benefit_id ] = $total_amount;
                        }
                    }
                }
                $debt = null;
                foreach ($totalPerBenefit as $benefit=>$amount) {
                    if ($benefit == 0) continue;

                    $debt_exist = DB::table('debt_recoveries')->whereBetween('from_date',[$from,$to])
                        ->where('to_date','>=',date('Y-m-d', strtotime($from)))
                        ->where('to_date','<=',date('Y-m-d', strtotime($to)))
//                        ->OrWhereBetween('to_date',[$from,$to])
                        ->where('benefit_id','=',$benefit)
                        ->where('application_id','=',$application_id)
                        ->first();

                    if ($debt_exist) {

                        alertNotify('A Debt Recovery has already been created with a the date range '
                            .date("Y-m-d",strtotime($debt_exist->from_date)).' to '.date("Y-m-d",strtotime($debt_exist->to_date)),__('Date Range conflict'),'info');
                        continue;
                    }

                    $debt = DebtRecovery::updateOrCreate([
                        'benefit_id'        =>  $benefit,
                        'application_id'    =>  $application_id,
                        'from_date'         =>  date('Y-m-d', strtotime($from)),
                        'to_date'           =>  date('Y-m-d', strtotime($to)),
                    ],[
                        'date_processed'     => date('Y-m-d', strtotime(now())),
                        'user_id'           =>  Auth::id(),
                        'amount_due'        =>  $amount
                    ]);
                }

                if ($debt) {
                    alertNotify('Debt Recovery has been created.',null,'success');
                }
            }
//            dd($totalPerBenefit);
        }
        else {
            alertNotify('Sorry! Beneficiary does not exist.',null,'danger');
        }
        return back();
    }
}
