<?php

namespace App\Http\Controllers;

use App\Http\Helpers\FormRender;
use App\Http\Traits\FormSubmission;
use App\Http\Traits\VerificationProcesses;
use App\Models\AccountDetail;
use App\Models\Applicant;
use App\Models\Application;
use App\Models\ApplicationApproval;
use App\Models\ApplicationComment;
use App\Models\ApplicationDecline;
use App\Models\ApplicationsProcessing;
use App\Models\BeneficiariesFormMapping;
use App\Models\Beneficiary;
use App\Models\Benefit;
use App\Models\CaseManagement;
use App\Models\ContactDetail;
use App\Models\EmploymentDetail;
use App\Models\Family;
use App\Models\Form;
use App\Models\IdentificationDetail;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Http\Traits\EmailSender;
use App\Models\ApplicationItem;
use App\Models\GeneralNotification;
use App\Models\InventoryItem;
use App\Models\ItemIssuance;
use App\Models\MaritalDetail;
use App\Models\StageEmail;
use App\Models\SubCounty;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;
use Yajra\DataTables\DataTables;

class BenefitRegistrationController extends Controller
{
    use FormSubmission;
    use VerificationProcesses;
    use EmailSender;

    /**
     * Display a listing of Benefit Registrations.
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function index(Request $request)
    {
        $benefits = getActiveBenefit();
        $applications = Application::where('instance_id','=',getActiveInstance())->count();
        $new = Application::submitted()->where('instance_id','=',getActiveInstance())->count();
        $approved = Application::approved()->where('instance_id','=',getActiveInstance())->count();

        return view('benefit-registration.list', compact('request',
            'benefits','new','approved','applications'));
    }

    /**
     * Get the list of Application DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getApplicationsList(Request $request)
    {
        $request_data = $request->all();
        $clean_request_data = [];
        foreach ($request_data as $key=>$value) {
            $key = str_replace('amp;','',$key);
            $clean_request_data[$key] = $value;
        }

        $registrations = Application::with(['approval'])->where('instance_id','=',getActiveInstance())->orderBy('updated_at','desc');

        if (isset($clean_request_data['status'])){
            $registrations = $registrations->where('status','=',$clean_request_data['status']);
        }
        else {
            //$registrations = $registrations->where('status','!=','Approved');
        }


        if (isset($clean_request_data['year'])){
            $registrations = $registrations->whereYear('created_at','=',$clean_request_data['year']);
        }

        if (isset($clean_request_data['benefit_type'])){
            $registrations = $registrations->where('benefit_id','=',$clean_request_data['benefit_type']);
        }

        if (isset($clean_request_data['applicant_name'])){
            $applicant_name = $clean_request_data['applicant_name'];
            $applicants = Applicant::where('surnames','like',"%$applicant_name%")
            ->orWhere('first_names','like',"%$applicant_name%")
            ->orWhere('other_names','like',"%$applicant_name%")
            ->orWhere('regID','like',"%$applicant_name%")
                ->pluck('regID')->toArray();
            $registrations = $registrations->whereIn('applicant_number',$applicants);
        }

        $registrations = $registrations->get();

        return DataTables::of($registrations)
            
            ->addColumn('action', function ($registration) {
                $edit = $submit = $view = $comment = null;
                if ($registration->status == 'Returned For Correction' && canDo('administrator','edit-application')) {
                    $edit = '<a href="'.route('benefit_registration.edit',[cryptThis($registration->id)]).
                        '" class="btn-xs btn btn-link text-primary mb-1" data-bs-toggle="tooltip" data-bs-placement="top" title="'. __("Edit Application") .'"><i class="fa fa-edit"></i></a>';
                }

                if (canDo('administrator','view-application-info')) {
                    $view = '<a href="' . route('benefit_registration.view', [cryptThis($registration->id)]) .
                        '" class=" btn-xs btn btn-link text-info mb-1" data-bs-toggle="tooltip" data-bs-placement="top" title="' . __("View Application") . '"><i class="fa fa-file-text"></i></a>';
                }
                if(canDo('administrator','view-application-info')){
                    $paymentList = '<button type="button" onclick="showissuancerecords('.$registration->id.')" class="btn btn-link btn-xs mb-1 text-info" data-bs-toggle="modal" data-bs-target="#issuance-modal"  title="' . __("View Issuance Records") . '"><i class="fa fa-file"></i></button>';
                }
                if (canDo('administrator','view-application-info')) {

                    if($registration->status == 'Approved'){

                        if($registration->last_issuance_date  == null){
                            $payment = '<button type="button" onclick="showpaymentmodal('.$registration->id.')" class="btn btn-link btn-xs mb-1 text-info" data-bs-toggle="modal" data-bs-target="#paymentBox"  title="' . __("Issuance") . '"><i class="fa fa-credit-card"></i></button>';
                        }else{

                            if($registration->next_issuance_date > now() || $registration->next_issuance_date == null){
                                $payment ='<button type="button"  class="btn btn-link btn-xs mb-1 text-muted"  title="' . __("Not Due for Issuance") . '"><i class="fa fa-credit-card"></i></button>';
                            }else{
                                $payment = '<button type="button" onclick="showpaymentmodal('.$registration->id.')" class="btn btn-link btn-xs mb-1 text-info" data-bs-toggle="modal" data-bs-target="#paymentBox"  title="' . __("Issuance") . '"><i class="fa fa-credit-card"></i></button>';
                            }

                        }
                        

                    }elseif($registration->status == 'Issued' || $registration->status == 'Paid'){

                        if($registration->next_issuance_date > now() || $registration->next_issuance_date == null){
                            $payment ='<button type="button"  class="btn btn-link btn-xs mb-1 text-muted"  title="' . __("Not Due for Issuance") . '"><i class="fa fa-credit-card"></i></button>';
                        }else{
                            $payment = '<button type="button" onclick="showpaymentmodal('.$registration->id.')" class="btn btn-link btn-xs mb-1 text-info" data-bs-toggle="modal" data-bs-target="#paymentBox"  title="' . __("Issuance") . '"><i class="fa fa-credit-card"></i></button>';
                        }


                    }else{

                        $payment ='<button type="button"  class="btn btn-link btn-xs mb-1 text-muted"  title="' . __("Has not been approved") . '"><i class="fa fa-credit-card"></i></button>';

                    }
                    


                }
                if (canDo('administrator','view-application-comment|add-application-comment')) {
                    $comment = '<button type="button" class="btn btn-link btn-xs mb-1 text-muted" data-bs-toggle="modal" onclick="showComment('.$registration->id.')"
                                        data-bs-placement="top" data-bs-target="#commentBox" title="comments"><i class="fa fa-comments"></i>
                                    </button>';
                }
                if ($registration->status == 'Submitted' && canDo('administrator','delete-application')) {
                    $delete = '<a href="'.route('benefit_registration.delete',[cryptThis($registration->id)]).
                        '" onclick="return confirm(\'Are you sure you want to delete this Application?\')"
                        class=" btn-xs text-danger btn-link mb-2  me-3" data-bs-toggle="tooltip" title="'. __("Delete Registration").'"><i class="fa fa-trash"></i></a>';
                }

                    return "$edit $view $submit $comment $payment $paymentList";
            })
            ->editColumn('application_id', function ($registration) {
                $applicant = $registration->applicant;

                if (!isset($applicant->surnames)) {$surname = null;}
                else{$surname = $applicant->surnames;}

                if (!isset($applicant->first_names)) {$first_name = null;}
                else{$first_name = $applicant->first_names;}

                if (canDo('administrator','view-member-info')) {
                    $applicant = '<a target="_blank" href="' . route('applicant.viewApplicantInfo', ['app_id' => cryptThis($applicant->id)]) . '"
                class="text-primary mr-2" style="text-decoration:none" href="javascript:;" data-bs-toggle="tooltip" data-bs-placement="top" title="' . __("View Applicant") . '">
                <strong>' . strtoupper($surname) . '</strong>, ' . ucwords($first_name) . '</a>';

                }
                else {
                    $applicant = '<strong>' . strtoupper($surname) . '</strong>, ' . ucwords($first_name);
                }
                return $applicant.'<br><small class="text-muted">'.$registration->applicant_number.'</span>';
            })
            ->editColumn('form_id', function ($registration) {
                $archived = '';
                if ($registration->benefit->archived) {
                    $archived = '<span class="text-danger mx-2"> (Archived)</span>';
                }
                return ucwords($registration->benefit->titles).' '.$archived.'<br><small class="text-muted">'.$registration->benefit->financial.'</span>';
            })
            // ->editColumn('applicant_number', function ($registration) {
            //     return $registration->applicant_number;
            // })
            // ->editColumn('bci_account_numbers', function ($registration) {
            //     return $registration->applicant->accountDetails->where('removed','=',0)->pluck('account_number')->first();
            // })
            ->editColumn('status', function ($registration) {
                $progress = '<span class="'.application_style($registration->status).'">'.$registration->status.'</span>';
                if ($registration->status == 'Approved' && isset($registration->approval)) {
                    $progress.=' - <small class="text-muted">'.easyMoney($registration->approval->amount).'</small>';
                }
                return $progress.'<br>  <small class="text-muted">'.date('d M Y', strtotime($registration->updated_at)).'</small>';
            })
            ->editColumn('created_at', function ($registration) {
                return "<span style='display: none'>".date('Y-m-d',strtotime($registration->created_at))."</span>".date('d M Y',strtotime($registration->created_at))
                .'<br><small class="text-muted">'.$registration->addedBy->fullName.'</span>';
            })
            ->escapeColumns([])
            ->make();
    }

    public function uploadDocument($key='attachment',$location='application-comment', $validation = 'file|mimes:jpeg,png,jpg,gif,docx,pdf,txt,xlsx|max:10048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('local')->putFileAs($location, request()->file($key),$name);
        }
        return $file;
    }

    public function openComments(Request $request)
    {
        $application_id = $request->id;
        $comments = ApplicationComment::with(['user'])->where('application_id','=',$application_id)->get();
        $view = view('benefit-registration.comments', compact('comments','application_id'))->render();

        return response()->json(['view'=>$view]);
    }

    public function addComment(Request $request)
    {
        $request->validate([
            'application_id' =>  'required|integer',
            'comment' =>  'required|string',
            'attachment' =>  'nullable|file',
        ]);

        $attachment = null;
        if (isset($request->attachment)) {
            $attachment = $this->uploadDocument();
        }

        $comments = ApplicationComment::create(
            [
                'application_id' =>  $request->application_id,
                'comment' =>  $request->comment,
                'user_id' =>  auth()->id(),
                'attachment' =>  $attachment,
            ]
        );

        if ($comments) {
            session()->flash('status', 'success');
            session()->flash('message', 'Your comment has be saved');
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Please try again');
        }

        return back();
    }

    public function checkApplicantInformation($request)
    {
        $base_code = $this->getBaseRegID($request);

        $applicants = Applicant::where('regID','like',"$base_code%")
            ->where('surnames','like','%'.$request->surnames.'%')
            ->where('first_names','like','%'.$request->first_names.'%')
            ->where('mother_surnames','like','%'.$request->mother_surnames.'%')
            ->where('mother_first_names','like','%'.$request->mother_first_names.'%')
            ->where('father_surnames','like','%'.$request->father_surnames.'%')
            ->where('father_first_names','like','%'.$request->father_first_names.'%');

        if ($request->other_names != null) {
            $applicants->where('other_names','like','%'.$request->other_names.'%');
        }

        if ($request->mother_other_names != null) {
            $applicants->where('mother_other_names','like','%'.$request->mother_other_names.'%');
        }

        if ($request->father_other_names != null) {
            $applicants->where('father_other_names','like','%'.$request->father_other_names.'%');
        }

        return $applicants->get();
    }

    /**
     * Get the list of Application DataTable
     * @param null $query
     * @param null $value
     *
     * @return mixed
     * @throws \Exception
     */
    public function getApplicationsInfoList($query=null,$value=null)
    {
        $applicants = Applicant::orderBy('surnames','desc')->get();
        if ($query == 'refID'){
            $refID = $value;
            $applicants = Applicant::where('regID','like',"%$refID%")->orderBy('surnames','desc')->get();
        }
        elseif ($query == 'dobs') {
            $dobs = $value;
            $applicants = Applicant::whereDate('dobs','=',date('Y-m-d',strtotime($dobs)))->orderBy('surnames','desc')->get();
        }

        return DataTables::of($applicants)
            ->addColumn('action', function ($applicant) {
//                $view = '<a href="javascript:;" class=" btn btn-icon btn-sm btn-primary mb-2  mr-1"  onclick="getApplicantInfo('.$applicant->id.')"
//                     data-toggle="modal" data-target="#staticBackdrop" title="'. __("View Applicant") .'"><i class="la la-eye"></i></a>';
                $view = '<a href="'.route('benefit_registration.viewApplicantInfo',['app_id'=>cryptThis($applicant->id)]).'"
                class="dropdown-item text-secondary" href="javascript:;" title="'. __("View Applicant") .'"><i class="la la-eye mr-2 text-secondary"></i>View</a>';

                    $edit = '<a href="'.route('benefit_registration.applicantRegistration',[cryptThis($applicant->id)]).
                        '" class="dropdown-item text-primary" href="javascript:;" title="'. __("Edit Applicant") .'"><i class="la la-edit mr-2 text-primary"></i> Edit</a>';

//                    $edit = '<a href="'.route('benefit_registration.applicantRegistration',[cryptThis($applicant->id)]).
//                        '" class=" btn btn-icon btn-xs btn-light-primary mb-2  mr-1" title="'. __("Edit Applicant") .'"><i class="la la-edit"></i></a>';

                    if($applicant->status == 0) {
                        $archive = '<a href="'.route('benefit_registration.deleteApplicantInfo',[cryptThis($applicant->id)]).
                            '" onclick="return confirm(\'Are you sure you want to restore this Applicant?\')"
                        class="dropdown-item text-success" href="javascript:;" data-toggle="tooltip" title="'. __("Restore Applicant").'"><i class="la la-trash-restore mr-2 text-success"></i> Restore</a>';

//                        $archive = '<a href="'.route('benefit_registration.deleteApplicantInfo',[cryptThis($applicant->id)]).
//                            '" onclick="return confirm(\'Are you sure you want to restore this Applicant?\')"
//                        class="btn btn-icon btn-xs btn-light-success mb-2  mr-1" data-toggle="tooltip" title="'. __("Restore Applicant").'"><i class="la la-trash-restore"></i></a>';
                    }
                    else {
//                        $archive = '<a href="'.route('benefit_registration.deleteApplicantInfo',[cryptThis($applicant->id)]).
//                            '" onclick="return confirm(\'Are you sure you want to archive this Applicant?\')"
//                        class="btn btn-icon btn-xs btn-light-danger mb-2  mr-1" data-toggle="tooltip" title="'. __("Delete Applicant").'"><i class="la la-trash"></i></a>';

                        $archive = '<a href="'.route('benefit_registration.deleteApplicantInfo',[cryptThis($applicant->id)]).
                            '" onclick="return confirm(\'Are you sure you want to archive this Applicant?\')"
                        class="dropdown-item text-danger" href="javascript:;" data-toggle="tooltip" title="'. __("Delete Applicant").'"><i class="la la-trash mr-2 text-danger"></i> Remove</a>';
                    }

                    $options = '<div class="dropdown dropdown-inline mr-4">
                                <button type="button" class="btn btn-light-primary btn-icon btn-xs" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                    <i class="ki ki-bold-more-hor"></i>
                                </button>
                                <div class="dropdown-menu drop-left">'.$view. $edit. $archive.'</div>
                            </div>';


//                $account = '<a href="javascript:;" onclic="return confirm(\'Are you sure you want to delete this Applicant?\')"
//                        class="btn btn-icon btn-sm btn-light-info mb-2  mr-1" data-toggle="tooltip" title="'. __("Account Applicant").'"><i class="la la-bank"></i></a>';
                    return "$options";

            })
            ->editColumn('surnames', function ($applicant) {

                if (!isset($applicant->surnames)) {$surname = null;}
                else{$surname = $applicant->surnames;}

                if (!isset($applicant->first_names)) {$first_name = null;}
                else{$first_name = $applicant->first_names;}

                $other_name = $applicant->other_names;

                return "<strong>".strtoupper($surname)."</strong>, ".ucwords($first_name)." ".ucwords($other_name);
            })
            ->editColumn('id', function ($applicant) {
                return $applicant->id;
            })
            ->editColumn('genders', function ($applicant) {
                return $applicant->genders;
            })
            ->editColumn('place_of_births', function ($applicant) {
                return $applicant->place_of_births;
            })
            ->editColumn('dobs', function ($applicant) {
                return date('d M Y',strtotime($applicant->dobs));
            })
            ->escapeColumns([])
            ->make();
    }

    public function viewApplicantInfo(Request $request)
    {
        $applicant = Applicant::find(Crypt::decrypt($request->app_id));
        $eligible_benefits = applicant_eligible_benefits($applicant->regID);
        $view = true;
        $islands = Lookup::where('lookup_code_id','=', config('settings.island_code'))->pluck('fullName','short_code')->toArray();
        $countries = DB::table('countries')->pluck('en_short_name','alpha_3_code')->toArray();
        $place_of_birth = array_merge($islands,$countries);
        $nationalities = DB::table('countries')->pluck('nationality','alpha_3_code')->toArray();
        $total_bank_accounts = AccountDetail::where('regID','=',$applicant->regID)->where('removed','=',0)->count();
        $cases = CaseManagement::where('applicant_number','=',$applicant->regID)->get();
        $benefits = Benefit::where('archived','=',0)->whereNotNull('form_id')->orderBy('titles','asc')->get();
        $add_application = false;

            $render_form =  null;
        if (isset($request->benefit_type)) {
            $get_render_form = new FormRender();
            $benefit = Benefit::find(Crypt::decrypt($request->benefit_type));
            $get_render_form->submitUrl(route('benefit_registration.save',[cryptThis($benefit->id),$applicant->regID]));
            $render_form = $get_render_form->renderFormView($benefit->form->table_name);
            $add_application = true;
        }

        $applications = Application::where('applicant_number','=',$applicant->regID)->get();

        return view('benefit-registration.applicant-view', compact('place_of_birth','nationalities','cases',
            'applicant','view','applications','total_bank_accounts','benefits','render_form','eligible_benefits','add_application'));
    }

    public function viewApplicantRecordInfo(Request $request)
    {
        $record = DB::table($request->table)->where('id','=',(int)$request->record)->first();
        $details = true;
        $view = null;
        $title = 'DETAILS';
        $action = $request->action;
        if ($action == 'edit') $details = null;

        if ($request->table == 'family') {
            $familyDetail = $record;
            if (!is_null($familyDetail->type)) $title = $familyDetail->surname.', '.$familyDetail->first_name.' - '.$familyDetail->type;
            $view = view('benefit-registration.applicant-form.family-fields',compact('details','familyDetail','action','request'))->render();
        }
        elseif ($request->table == 'contact_details') {
            $contactDetail = $record;
            if (!is_null($contactDetail->village)) $title = $contactDetail->village.' - RESIDENCE';
            $view = view('benefit-registration.applicant-form.contact-fields',compact('details','contactDetail','action','request'))->render();
        }
        elseif ($request->table == 'identification_details') {
            $idDetail = $record;
            if (!is_null($idDetail->id_type)) $title = $idDetail->id_type.' - '.$idDetail->id_number;
            $view = view('benefit-registration.applicant-form.identifications-fields',compact('details','idDetail','action','request'))->render();
        }
        elseif ($request->table == 'employment_details') {
            $employmentDetail = $record;
            if (!is_null($employmentDetail->employer_name)) $title = $employmentDetail->employer_name.' - '.$employmentDetail->employment_type;
            $view = view('benefit-registration.applicant-form.employment-fields',compact('details','employmentDetail','action','request'))->render();
        }
        elseif ($request->table == 'account_details') {
            $accountDetail = $record;
            if (!is_null($accountDetail->bank_name)) $title = $accountDetail->bank_name.' - '.$accountDetail->account_type;
            $view = view('benefit-registration.applicant-form.banking-fields',compact('details','accountDetail','action','request'))->render();
        }
        elseif ($request->table == 'marital_details') {
            $maritalDetail = $record;
            if (!is_null($maritalDetail->marriage_status)) $title = $maritalDetail->marriage_status.' - '.$maritalDetail->marriage_type;
            $view = view('benefit-registration.applicant-form.marital-fields',compact('details','maritalDetail','action','request'))->render();
        }

        return response()->json(['the_view'=>$view,'title'=>$title]);
    }

    public function updateApplicantRecord(Request $request)
    {
        if ($request->table = 'family') {
            $record = DB::table($request->table)->where('id','=',Crypt::decrypt($request->record_id))->update([
                'type'                      =>      $request->relationship_types,
                'surname'                   =>      $request->relationship_surname,
                'first_name'                =>      $request->relationship_first_name,
                'other_name'                =>      $request->relationship_other_name,
                'date_of_birth'             =>      $request->relationship_dobs,
                'dependent_reference_id'    =>      $request->relationship_reg_no,
                'is_dependent'              =>      $request->is_dependant,
            ]);
        }

        if ($record) {
            $message = 'Record has been updated successfully';
            session()->flash('status', 'success');
            return back();
        }
        else {
            $message = 'Sorry! Record could not be found.';
            session()->flash('status', 'error');
        }
        session()->flash('message', $message);
        return back();
    }

    public function deleteApplicantInfo($id)
    {
        $applicant = Applicant::find(Crypt::decrypt($id));
        if ($applicant->status == 0) {
            $applicant->status = 1;
            $message = 'Applicant has been restored';
        }
        else {
            $applicant->status = 0;
            $message = 'Applicant has been archived';
        }

        if ($applicant->save()) {
            alertNotify($message,'','success');
        }
        else {
            alertNotify('Sorry, Something went wrong or applicant account does not exist.','','danger');
        }
        return back();
    }

    /**
     * New Benefit Registrations.
     * @param Request $request
     * @param null $form_id
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\RedirectResponse
     */
    public function create(Request $request,$benefit_id)
    {
        $application_form =  null;
        $applicant_regID =  null;

       /*$applicant = \App\Models\Applicant::where('regID','=','MB-1260')->first();
        $island = $applicant->contactDetails->where('type_name','=','Residential Address')->where('status','=','Verified')->where('city','!=',null)->pluck('city')->toArray();
        $benefit = Benefit::find(7);

        
        dd([$island, $benefit->residence_criteria]);
        
        */

        

        $benefit = Benefit::find(Crypt::decrypt($benefit_id));
        $benefit_name =  null;
        if (isset($request->get_applicant)) {
            $applicant_regID =  $request->get_applicant;
        }

        $application_form_view =  'general-form';

        if($benefit) {
            $benefit_id = $benefit->id;
            $benefit_name = $benefit->titles;
            $application_form_view = $benefit->form->view_file;
        }
        else {
            $message = 'Sorry! This benefit does not have a form.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        if (is_null($application_form_view)) {
            $message = 'Sorry! This benefit does not have a form.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
        }

        $header = 'General Application Form';

        return view("benefit-registration.applications.$application_form_view",
            compact('header','application_form','benefit_id','benefit_name','applicant_regID'));
    }

    /**
     * Edit Benefit Registrations.
     * @param Request $request
     * @param null $form_id
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\RedirectResponse
     */
    public function edit($application_id)
    {
        $application = Application::findOrFail(Crypt::decrypt($application_id));
        $application_form =  null;
        $applicant_regID = auth()->user();

        $benefit = Benefit::with('form')->where('id','=',$application->benefit_id)->first();
        $benefit_name =  null;

        $return_message = null;
        if ($application->status == 'Returned For Correction'){
            $app = $application->latestReturnForCorrection();
            $return_message = $app->remarks;
        }

        $application_form_view =  'edit-general-form';

        if($benefit) {
            $benefit_id = $benefit->id;
            $benefit_name = $benefit->titles;
            $application_form_view = 'edit-backend-'.$benefit->form->view_file;
        }
        else {
            $message = 'Sorry! This benefit does not have a form.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        if (is_null($application_form_view)) {
            $message = 'Sorry! This benefit does not have a form.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        $header = 'General Application Form';

        return view("benefit-registration.applications.$application_form_view",
            compact('header','return_message','application_form','benefit_id','application','benefit_name','applicant_regID'));
    }

    public function validateUser(Request $request)
    {
        $applicant = Applicant::where('regID','=',$request->applicant)->first();
        $has_existing_application = Application::where('benefit_id','=',$request->benefit)
            ->where('applicant_number','=',$request->applicant)
            ->whereNotIn('status',['Declined','Rejected','Returned For Correction'])->count();

        if ($has_existing_application > 0) {
            return response()->json(['error'=>'The account has already applied for this Benefit','status'=>0]);
        }

        $eligible_data = isApplicantEligible($request->applicant,$request->benefit);
        if(!$eligible_data['status']) {
            return response()->json(['error'=>'Applicant is not eligible for this benefit claim','message'=>$eligible_data['message'],'status'=>0]);
        }

        $application_bank_account_id = null;
        if (isset($request->application_bank_account_id)) $application_bank_account_id = $request->application_bank_account_id;


        if ($applicant) {
            $benefit = Benefit::find($request->benefit);
            $documents = $benefit->supporting_documents;
            $applicant_documents = $applicant->documents->pluck('type','id')->toArray();

            $documents = view('benefit-registration.applications.documents-info', compact('applicant_documents','documents'))->render();
            $view = view('benefit-registration.applications.basic-info', compact('applicant','benefit','application_bank_account_id'))->render();
            return response()->json(['applicant'=>$applicant->surnames." ".$applicant->first_names,
                'id'=>cryptThis($applicant->id),
                'view'=>$view,  'documents'   => $documents ]);
        }
        else {
            return response()->json(['error'=>'No Record Found','status'=>0]);
        }
    }

    public function applicantAccountDetails(Request $request)
    {
        $accountDetail = AccountDetail::find($request->num);
        if ($accountDetail) {
            $view = view('benefit-registration.applications.bank-account-info', compact('accountDetail'))->render();
            return response()->json($view);
        }
        else {
            return response()->json('<h5 class="text-center mt-5 text-danger">No Record Found</h5>');
        }
    }

    public function applicantChannelDetails($channelId)
    {
        $accountDetail = AccountDetail::find($channelId);
        if ($accountDetail) {
            return  view('benefit-registration.applications.bank-account-info', compact('accountDetail'))->render();
        }
        else {
            return '<h5 class="text-center mt-5 text-danger">No Record Found</h5>';
        }
    }

    /**
     * @param Request $request
     * @param $benefit_id
     * @param $applicant_number
     * Remember to modify the Corresponding ENDPOINT
     * @return \Illuminate\Http\RedirectResponse
     */
    public function save(Request $request,$benefit_id,$applicant_number=null)
    {
        $benefit = Benefit::find(Crypt::decrypt($benefit_id));
        if ($request->comfirmation != 1) {
            $message = 'Sorry! You must agree and confirm the terms of this application to proceed.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        if (is_null($benefit)) {
            $message = 'Sorry! Benefit does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        $docs = $request->documentations_existing;

        $applicant = Applicant::where('regID','=',$request->applicant_number)->first();
        if (is_null($applicant)) {
            $message = 'Sorry! Applicant does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        if($benefit->financial == 'Financial') {
            $account = DB::table('account_details')
                ->where('regID','=',$applicant->regID)
                ->where('id','=',$request->bank_account)
                ->first();
            if (!$account) {
                $message = 'Bank Account details does not exist.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
                return back();
            }
        }

        try{
            if (!is_null($request->documentations_new)) {
                foreach($request->documentations_new as $key=>$document){
                    $doc = Str::snake($document);
                    $doc_name = "upload_$doc";
                    if($request->file($doc_name)) {
                        $directory = 'public/applicants/'.$applicant->regID;
                        Storage::putFileAs("$directory", $request->file($doc_name), $request->file($doc_name)->getClientOriginalName());
                        $file_path = $directory.'/'.$request->file($doc_name)->getClientOriginalName();

                        $docs[] = $document;

                        addApplicantDocument($applicant->id,$file_path,$document);
                    }
                }
            }

            $app = Application::create([
                'instance_id' => $benefit->instance_id,
                'form_id' => $benefit->form_id,
                'benefit_id' => $benefit->id,
                'applicant_number' => $request->applicant_number,
                'applicationable_id' => Auth::id(),
                'applicationable_type' => 'App/Models/User',
                'modified_by' => Auth::id(),
                'type' => $benefit->titles,
                'bank_account_id' => $request->bank_account??null,
                'reporter_surname' => $request->reporter_surname,
                'reporter_first_name' => $request->reporter_first_name,
                'reporter_contact' => $request->reporter_contact,
                'reporter_id_type' => $request->reporter_id_type,
                'reporter_relationship' => $request->reporter_relationship,
                'documentations' => $docs,
                'status' => 'Submitted',
            ]);

            if ($app) {
                ApplicationsProcessing::create([
                    'instance_id'    =>  $benefit->instance_id,
                    'application_id'    =>  $app->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  $app->status,
                    'access_type'            =>  'Portal',
                    'stage'             =>  'New Application',
                    'remarks'             =>  ''
                ]);
            }


            $message = "The information has been saved.";
            session()->flash('status', 'success');
            session()->flash('message', $message);

            // return redirect()->route('benefit_registration.viewApplicantInfo',['app_id'=>cryptThis($applicant->id)]);
            return redirect()->route('benefit_registration.list');
        }
        catch (\Exception $exception){
            $message = 'Sorry! Submission failed. Try again. '.$exception->getMessage();
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

    }

    /**
     * @param Request $request
     * @param $form_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function updateApplication(Request $request)
    {
        $application = Application::findOrFail(decrypt($request->application_id));

        $benefit = Benefit::find($application->benefit_id);
        if ($request->comfirmation != 1) {
            $message = 'Sorry! You must agree and confirm the terms of this application to proceed.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        if (is_null($benefit)) {
            $message = 'Sorry! Benefit does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        $docs = $request->documentations_existing;

        $applicant = Applicant::where('regID','=',$application->applicant_number)->first();
        if (is_null($applicant)) {
            $message = 'Sorry! Applicant does not exist.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        try{
            if (!is_null($request->documentations_new)) {
                foreach($request->documentations_new as $key=>$document){
                    $doc = Str::snake($document);
                    $doc_name = "upload_$doc";
                    $file_path = $this->uploadFile($doc_name,'/applicants/'.$applicant->regID.'/application');
                    $docs[] = $document;
                    addApplicantDocument($applicant->id,$file_path,$document);
                }
            }

            $app = Application::updateOrCreate([
                'id'    =>  Crypt::decrypt($request->application_id),
            ],[
                'form_id' => $benefit->form_id,
                'benefit_id' => $benefit->id,
                'applicant_number' => $application->applicant_number,
                'applicationable_id' => Auth::id(),
                'applicationable_type' => 'App/Models/User',
                'modified_by' => Auth::id(),
                'type' => $benefit->titles,
                'bank_account_id' => $request->bank_account,
                'reporter_id_number' => $request->reporter_id_number,
                'reporter_surname' => $request->reporter_surname,
                'reporter_first_name' => $request->reporter_first_name,
                'reporter_contact' => $request->reporter_contact,
                'reporter_id_type' => $request->reporter_id_type,
                'reporter_relationship' => $request->reporter_relationship,
                'user_type' => 'applicant',
                'documentations' => $docs,
                'status' => 'Resubmitted',
            ]);

            $access_type = 'Portal';
            if ($access_type != 'Portal') {
                $access_type = 'Mobile App';
            }

            if ($app) {
                ApplicationsProcessing::create([
                    'instance_id'    =>  $benefit->instance_id,
                    'application_id'    =>  $app->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  'Resubmitted',
                    'access_type'            =>  $access_type,
                    'stage'             =>  'Correction',
                    'remarks'             =>  'Correction made on behalf of applicant.'
                ]);
            }


            $message = "Application has been submitted.";
            session()->flash('status', 'success');
            session()->flash('message', $message);

            return redirect()->route('benefit_registration.list');
        }
        catch (\Exception $exception){
            $message = 'Sorry! Submission failed. Try again. '.$exception->getMessage();
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }

    }

    /**NOT USED
     * @param $id
     * @return mixed
     */
    public function submitApplication($id)
    {
        $application = Application::find(Crypt::decrypt($id));
        $old_stage = $application->stage;
        $applicant = Applicant::where('regID','=',$application->applicant_number)->first();

        //Temporary Function
        $application->status = 'Approved';
        $application->confirmed_date = date('Y-m-d H:i:s');
        $application->modified_by = auth()->id();
        if($application->save()){
            ApplicationsProcessing::create([
//                'instance_id'    =>  $application->instance_id,
                'application_id'    =>  $application->id,
                'applicable_id'           =>  Auth::id(),
                'applicable_type'           =>  'App\Models\User',
                'action'            =>  $application->status,
                'access_type'            =>  'Portal',
                'stage'             =>  'New Application',
                'remarks'             =>  ''
            ]);
            $message = 'Application been Approved successfully.';
                session()->flash('message', $message);
                session()->flash('status', 'success');
        }
        else {
            $message = 'Sorry, the application could not be processed. Please try again.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
        }

        // $lookup_id = $this->getStartBenefitVerificationStatusID(Crypt::decrypt($id));
        // $action = Lookup::find($lookup_id);
        // $application->stage = $lookup_id;
        // if($old_stage == 0) {
        //     $application->status = 'Resubmitted';
        //     $message = __('Application has been Resubmitted for Processing.');
        // }
        // else {
        //     $application->status = $action->process_status;
        //     $message = __('Application has been Submitted for Processing.');
        // }

        // $application->started_process = 1;
        // $application->modified_by = Auth::id();
        // if ($application->save()) {

        //     $saved = ApplicationsProcessing::create([
        //         'application_id'    =>  $application->id,
        //         'applicable_id'           =>  Auth::id(),
        //         'applicable_type'           =>  'App\Models\User',
        //         'action'            =>  $application->status,
        //         'access_type'            =>  'Portal',
        //         'stage'             =>  'New Order',
        //         'remarks'             =>  ''
        //     ]);

        //     $getRole = StageEmail::where('lookup_id','=',$application->stage)->pluck('roles')->first();
        //     $application->responsible_role = $getRole;
        //     $application->save();

        //     if ($saved) {
        //         //Send Email Notification
        //         $email = $applicant->contactDetails->where('status','=',1)->pluck('email')->toArray();
        //         if(!is_null($email) && $old_stage != 0 ) {
        //             $message = 'Dear '.$applicant->surnames.' '.$applicant->first_names.'<br>';
        //             $message .= __("Your Benefit Application for ".strtoupper($application->type)." has been submitted successfully.");
        //             $message .= "<br><br> Thank you.<br><br>Best Regards,";
        //             $subject = "A New Benefit Application Submitted";

        //             $this->blast_mails($message,$subject,$email);
        //         }
        //         $this->send_email($lookup_id,'applications',Crypt::decrypt($id));
        //     }

        //     alertNotify('Application been submitted successfully',null, 'success');
        // }
        // else{
        //     alertNotify('Sorry! Something went wrong',null, 'danger');
        // }
        return back();
    }

    public function applicationProcessing(Request $request,$application_id)
    {
        if ($request->responder == 1) {
            $request->validate([
                'responder' =>  'required|numeric',
                'application_state'   =>  [
                    'required',
                    Rule::in(['pass', 'fail']),
                ],
                'reason_failed'   =>  'required_if:application_state,fail|string'
            ],[
                'responder.required'=>'In appropriate responding officer'
            ]);

            $status = 'Pending Approval';
            $stage_title = 'Review';
            $message = 'Application has been processed for approval.';
            $remarks = 'All required criteria has been met.';
            $notification = 'Your application has passed review.';

            if ($request->application_state == 'fail') {
                $status = 'Returned For Correction';
                $remarks = $request->reason_failed;
                $message = 'The application has been returned to the applicant for correction.';
                $notification = 'Your application required your attention since it failed after review.';
            }

            $application = Application::findOrFail(Crypt::decrypt($application_id));
            $application->modified_by = auth()->id();
            $application->status = $status;
            if($application->save()){
                ApplicationsProcessing::create([
                    'instance_id'    =>  $application->instance_id,
                    'application_id'    =>  $application->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  $application->status,
                    'access_type'            =>  'Portal',
                    'stage'             =>  $stage_title,
                    'remarks'             =>  $remarks
                ]);

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Sorry, the application could not be processed. Please try again.';
                session()->flash('message', $message);
                session()->flash('status', 'error');

                return back();
            }
        }
        else {
            $application = Application::findOrFail(Crypt::decrypt($application_id));
            $get_min_values = benefitAmount([$application->benefit_id]);
            $min_value = $max_value = 0;

            $request->validate([
                'payment_amount' =>  'required|numeric|min:0',
                'comment'   =>  'required|string',
                'submit'   =>  'required|string'
            ]);

            if ($request->submit == 'Approved'){
                foreach($request->input('inventoryItem', []) as $item){
                    if($item['quantity'] ==null){
                        return back()->with('error', 'Enter a value in the quantity field for all items.');
                        exit();
                    }
                }

                if (!empty($get_min_values)) {
                    if(isset($get_min_values[0])) $min_value = $get_min_values[0]['min_value']; $max_value = $get_min_values[0]['max_value'];
                }
                $request->validate([
                    'payment_amount' =>  'required|numeric|min:'.$min_value.'|max:'.$max_value,
                ]);
            }

            $application->status = $request->submit;
            if ( in_array($request->submit,['Approved','Declined'])) $application->confirmed_date = date('Y-m-d H:i:s');
            $application->modified_by = auth()->id();

            if($application->save()){
                $message = 'The application has been updated';

                if ($request->submit == 'Approved'){
                    $application_approval = ApplicationApproval::updateOrCreate([
                        'application_id'    =>  $application->id,
                        'instance_id'    =>  $application->instance_id
                    ],[
                        'created_by'           =>  Auth::id(),
                        'comment'            =>  $request->comment,
                        'amount'            =>  $request->payment_amount,
                    ]);

                    if(count($request->input('inventoryItem', [])) > 0){
                        foreach($request->input('inventoryItem', []) as $item){
                            if($item['item'] !== null && $item['quantity'] !==null){

                                /*ItemIssuance::create([
                                    'application_id'=>$application->id,
                                    'application_approval_id' => $application_approval->id,
                                    'item_id' => $item['item'],
                                    'item_quantity_approved' => $item['quantity'],
                                    'issuance_date' => now(),
                                    'quantity_issued' => $item['quantity'],
                                    'user_id'=>auth()->user()->id
                                ]);*/

                                ApplicationItem::create([
                                    'application_id'=>$application->id,
                                    'item_id'=>$item['item'],
                                    'quantity_approved'=>$item['quantity']
                                ]);

                                $application->next_issuance_date = now();
                                $application->save();

                            }
                            
    
                        }
                    }

                    $message = 'The application has been approved';
                    $stage = 'Decision';
                    $notification = 'Your application has been approved';
                }

                if ($request->submit == 'Declined'){
                    ApplicationDecline::create([
                        'application_id'    =>  $application->id,
                        'instance_id'    =>  $application->instance_id,
                        'created_by'           =>  Auth::id(),
                        'comment'            =>  $request->comment,
                        'amount'            =>  $request->payment_amount,
                    ]);

                    $message = 'The application has been declined';
                    $stage = 'Decision';
                    $notification = 'Your application has been declined';
                }

                if ($request->submit == 'On Hold'){
                    $message = 'The application has been paused';
                    $stage = 'Investigation';
                    $notification = 'Your application has been halted for further review.';
                }

                ApplicationsProcessing::create([
                    'instance_id'    =>  $application->instance_id,
                    'application_id'    =>  $application->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  $application->status,
                    'access_type'            =>  'Portal',
                    'stage'             =>  $stage,
                    'remarks'             =>  $request->comment
                ]);

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Sorry, the application could not be processed. Please try again.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
                return back();
            }
        }

        if(isset($application->applicant->owner)) {
            GeneralNotification::create([
                'user_id'   =>  auth()->id(),
                'receivers'   =>  [$application->applicant->owner->id],
                'subject_id'   =>  $application->id,
                'table'   =>  $application->getModel()->getTable(),
                'instance_id'   =>  $application->instance_id,
                'message'   =>  '<strong>Application Update:</strong> '.$notification,
                'route'   =>  route('customer.viewApplication',[cryptThis($application->id)]),
            ]);
        }


        return redirect()->route('benefit_registration.list');
    }

    /**
     * @param $benefit_registration_id
     * @return mixed
     */
    public function delete($benefit_registration_id)
    {
        Application::destroy([Crypt::decrypt($benefit_registration_id)]);
        return back();
    }

    /**
     * @param $app_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function view($app_id)
    {
        //Get the Benefit Registration data
        $application = Application::find(Crypt::decrypt($app_id));
        $application_processing = ApplicationsProcessing::where('application_id','=',Crypt::decrypt($app_id))->orderBy('created_at','desc')->get();

        $application_status = $application->stage;
        $status_information = getVerificationStatusData($application_status);
        $application_id = $application->application_id;

        //Get form information
        $form_id = $application->form_id;
        $form = Form::find($form_id);

        //Build the form
        $get_render_form = new FormRender();
        $get_render_form->request_id(cryptThis($application->application_id));
        $get_render_form->displayAsDetails(true);
        $get_render_form->submitUrl(route('benefit_registration.update',[cryptThis($form_id)]));
        $render_form = $get_render_form->renderFormView($form->table_name);
        $page_title = __('View Application');

        $benefit = Benefit::find($application->benefit_id);
        $view = true;
        $workflow_stages = LookupCode::activeOptionsFull(config('settings.workflow_code'))->sortBy('order');

        $verification_processes = LookupCode::activeApplicationProcessOptions();
        $is_a_process = $this->isProcessForm($form_id);

        $permitted_roles = explode(',',$application->responsible_role);
        $permitted = false;
        if (in_array(Auth::user()->roles->pluck('id')->first(),$permitted_roles)) {
            $permitted = true;
        }

        return view('benefit-registration.view',compact('form','render_form','page_title','application_id',
            'verification_processes','is_a_process','status_information','application','view','permitted','benefit',
            'workflow_stages','application_processing'));
    }

    /**
     * @param $app_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function viewApplication($app_id)
    {
        //Get the Benefit Registration data
        $application = Application::with(['benefit','addedBy','applicant'])->where('id',Crypt::decrypt($app_id))->first();
        $applicant = Applicant::where('regID','=',$application->applicant_number)->first();
        $application->markRead();
        $used_documents = $applicant->documents->whereIn('type',$application->documentations);

        $channel = null;
        if ($application->benefit->financial == "Financial") $channel = $this->applicantChannelDetails($application->bank_account_id);

        $get_min_values = benefitAmount([$application->benefit_id]);
        $min_value = $max_value = 0;
        if (!empty($get_min_values)) {
            if(isset($get_min_values[0])) $min_value = $get_min_values[0]['min_value']; $max_value = $get_min_values[0]['max_value'];
        }

        if(canDo('administrator','settings')){
            $inventory_items = InventoryItem::orderBy('item')->get();
        }else{
            $userDeploymentIds = auth()->user()->deployments->pluck('id');
            $inventory_items = InventoryItem::whereIn('deployment_id', $userDeploymentIds)->orderBy('id', 'desc')->get();
        }

        return view('benefit-registration.applications.general-view',
        compact('applicant','application','channel','used_documents','min_value','max_value', 'inventory_items'));
    }

    /**
     * @param Request $request
     * @param $benefit_registration_id
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function verificationProcess(Request $request,$benefit_registration_id)
    {
        $application = Application::find(Crypt::decrypt($benefit_registration_id));
        $application->is_beneficiary = false;

        //Check if the user has the permission to change status
        $permitted_roles = explode(',',$application->responsible_role);
        if (!in_array(Auth::user()->roles->pluck('id')->first(),$permitted_roles)) {
            alertNotify('You are not permitted to perform this action!',null,'danger');
            return back();
        }

        $action_taken = 'Processed';
        $feedback = '';
        if (isset($request->message)) {
            $feedback = $request->message;
        }

        //Get Next Stage
        $next_stage = 0;
        $lookup = Lookup::find((int)$request->stage);

        switch (Str::snake($lookup->fullName)) {
            case 'receive':
                if ($request->app_action == 'Yes' && $request->docs_action == 'Yes') {
                    $next_stage = $lookup->id;
                    $message = 'The Application has been moved to the Next stage.';
                    $action_taken = 'Accepted';
                }
                else {
                    $next_stage = getPreviousStage($application->stage);
                    $message = 'The Application has been moved to the previous stage for correction.';
                    $action_taken = 'Rejected';
                }
                break;

            case 'review':
                if ($request->app_action == 'Yes' && $request->docs_action == 'Yes' && $request->recom_action == 'Approved') {
                    $next_stage = $lookup->id;
                    $message = 'The Application has been moved to the Next stage.';
                    $action_taken = 'Approved';
                }
                elseif ($request->recom_action == 'Returned As Draft') {
                    $next_stage = 0;
                    $message = 'The Application has been returned as draft.';
                    $action_taken = 'Returned As Draft';
                }
                else {
                    $next_stage = getDeclinedStage($application->stage);;
                    $message = 'The Application has been Declined.';
                    $action_taken = 'Declined';
                }
                break;
            default:
//                dd('Nothing to proceed');
                break;
        }

        $application->stage = $next_stage;
        $application->status = $lookup->process_status;
        if($next_stage == 0) {
            $application->started_process = 0;
            if ($application->save()) {

                ApplicationsProcessing::create([
                    'instance_id'    =>  $application->instance_id,
                    'application_id'    =>  $application->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'            =>  $action_taken,
                    'access_type'            =>  'Portal',
                    'stage'             =>  $lookup->fullName,
                    'remarks'             =>  $feedback
                ]);

                alertNotify('The Application has been returned for Re-Submission.',null,'success');
            }
        }
        else {
            $getRole = StageEmail::where('lookup_id','=',$next_stage)->pluck('roles')->first();
            $application->responsible_role = $getRole;

            if ($application->save()) {

                ApplicationsProcessing::create([
                    'application_id'    =>  $application->id,
                    'applicable_id'           =>  Auth::id(),
                    'applicable_type'           =>  'App\Models\User',
                    'action'      =>  $action_taken,
                    'access_type'            =>  'Portal',
                    'stage'             =>  $lookup->fullName,
                    'remarks'           =>  $feedback
                ]);

                //$this->send_email($next_stage,'applications',$application->id);

                if (endVerification($next_stage)){
                    $application->is_beneficiary = true;
                    $application->save();
                    addCase($application);
                    //transferBeneficiaryInformation($application->id);

                    alertNotify('The Application has been closed successfully. Thank you!',null,'success');
                }elseif (endVerificationByRejecting($next_stage)){
                    $application->rejected = 1;
                    $application->is_beneficiary = false;
                    $application->confirmed_date = now();
                    $application->save();
                    alertNotify('The Application has been rejected and closed. Thank you!',null,'success');
                }
                else {
                    $application->save();
                    if($next_stage > 0){
                        alertNotify($message,null,'success');
                    }
                }

            }
        }
        return redirect()->route('benefit_registration.list');
    }

    public function getStageComponent(Request $request)
    {
        $view = view('components.stages.'.Str::snake($request->stage))->render();

        return response()->json(['the_view'=>$view,'title'=>strtoupper($request->stage)]);
    }

}
