<?php

namespace App\Http\Controllers;

use App\Models\Applicant;
use App\Models\CustomField;
use App\Models\CustomFieldOption;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class CustomFieldController extends Controller
{
    //
    public function index(Request $request){
        $custom_fields = CustomField::all();
        return view('custom-fields.index', compact('custom_fields'));
    }

    public function create(Request $request){
        $request->validate([
            'name'=>'required|unique:custom_fields,name',
            'type'=>'required'
        ]);

        //dd('sasas');

        $custom_field = CustomField::create([
            'name' => $request->name,
            'type'=> $request->type,
            'status'=>$request->status
        ]);
        return redirect()->back()->with('success', '1 Custom Field Added');
    }

    public function create_option(Request $request, $custom_field_id)
    {
        $request->validate([
            'option' => 'required',
        ]);

        $custom_field = CustomField::findOrFail($custom_field_id);

        // Fetch the saved options' values only
        $saved_options = $custom_field->options->pluck('option_value')->toArray();

        // Check if the new option is already saved
        if (in_array($request->option, $saved_options)) {
            return back()->with('error', 'The value you entered is already saved as an option under the "'.$custom_field->name.'" custom field.');
        }

        // If not a duplicate, save the new option
        CustomFieldOption::create([
            'custom_field_id' => $custom_field->id,
            'option_value' => $request->option,
        ]);

        return back()->with('success', '"'.$request->option.'" has been added as an option under the "'.$custom_field->name.'" dropdown field.');
    }

    public function update_custom_attributes(Request $request, $applicant_id)
    {

        $applicant = Applicant::findOrFail($applicant_id);

        if(isset($request->orphan_status)){

            if($request->orphan_status == 'maternal'){
                $applicant->mother_alive = 0 ;
                $applicant->father_alive = 1 ;
            }elseif($request->orphan_status == 'paternal'){
                $applicant->mother_alive = 1 ;
                $applicant->father_alive = 0 ;
            }elseif($request->orphan_status == 'double'){
                $applicant->mother_alive = 0 ;
                $applicant->father_alive = 0 ;
            }

        }

        $applicant->household_condition = $request->household_condition;
        $applicant->learning_status = $request->learning_status;
        $applicant->highest_education_level = $request->education_level;
        $applicant->pmtScore = $request->pmt;

        $applicant->save();

        if ($request->has('custom_fields')) {
            DB::table('applicants_custom_field_values')
                ->where('applicant_id', $applicant->id)
                ->delete();
    
            // Save the new custom field values
            foreach ($request->input('custom_fields') as $fieldId => $value) {
                if (!empty($value)) { 
                    $existingCustomField = DB::table('applicants_custom_field_values')
                    ->where('applicant_id', $applicant->id)
                    ->where('custom_field_id', $fieldId)
                    ->first();

                    if ($existingCustomField) {
                        // Update the existing custom field value
                        DB::table('applicants_custom_field_values')
                            ->where('id', $existingCustomField->id)
                            ->update([
                                'value' => $value,
                                'updated_at' => now(),
                            ]);
                    }else{
                        // Insert a new custom field value if it doesn't exist
                        DB::table('applicants_custom_field_values')->insert([
                            'applicant_id' => $applicant->id,
                            'custom_field_id' => $fieldId,
                            'value' => $value,
                            'created_at' => now(),
                            'updated_at' => now(),
                        ]);
                    }
                    
                }
            }
        }

        return back()->with('success', 'Done');
    }

    public function update(Request $request, $fieldId)
    {
        

        $request->validate([
            'name'=>'required'
        ]);

        $custom_field = CustomField::findOrFail($fieldId);

        $custom_field->name = $request->name;
        $custom_field->type = $request->type;
        $custom_field->status = $request->status;
        $custom_field->save();

        return redirect()->back()->with('success', 'Custom Field Updated');
    }

    public function get_custom_field($fieldId)
    {
        $custom_field = CustomField::find($fieldId);
        return response()->json($custom_field);
    }

}
