<?php

namespace App\Http\Controllers;

use App\Mail\DeploymentSuspended;
use App\Mail\SendInstanceInvitation;
use App\Mail\ShipDeployment;
use App\Mail\ShipRedeployment;
use App\Models\Client;
use App\Models\Country;
use App\Models\CountryPlan;
use App\Models\Deployment;
use App\Models\DeploymentInvitation;
use App\Models\DeploymentsLog;
use App\Models\GeneralNotification;
use App\Models\Plan;
use App\Models\PlanSubscription;
use Carbon\Carbon;
use \App\Models\User;
use Illuminate\Database\Query\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class DeploymentController extends Controller
{

    public function __construct()
    {

        $this->middleware(['auth:web']);
    }

    /**
     * Function for upload image
     *
     * @param $key
     * @param $client_id
     * @param $validation
     * @return string|null
     */
    public function uploadFile($key, $client_id, $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:5048|sometimes')
    {
        $location = "documents/$client_id";
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('local')->putFileAs($location, request()->file($key),$name);
        }
        return $file;
    }
    public function avatarLogo($key, $client_id,$deployment_id, $validation = 'image|mimes:jpeg,png,jpg|max:5048|sometimes')
    {
        $location = "$deployment_id/logo";
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($location, request()->file($key),$name);
        }
        return 'storage/'.$file;
    }

    public function switchSubscription(Deployment $deployment)
    {
        if (auth()->user()->deployments->where('id','=',$deployment->id)->first()) {
            auth()->user()->last_instance_opened = $deployment->id;
            auth()->user()->save();
            session([
                'my_permissions'    =>  auth()->user()->allPermissions()->pluck('name')->toArray(),
                'role'    =>  auth()->user()->roles()->pluck('name')->first(),
                'role_name'    =>  auth()->user()->roles()->pluck('display_name')->first(),
            ]);

            notify("You have switched to ".$deployment->name." subscription",'success','You are in!');
        }
        else {
            notify("Attempting an invalid subscription",'error','Sorry');
        }

        return redirect()->route('dashboard');
    }

    public function index(Request $request) {

        $statuses = array_keys(config('settings.deployments_status'));
        $instances = array_keys(config('settings.deployments_instance'));
        $deployments = auth()->user()->deployments()->with(['client'])->orderBy('created_at');

        if (isset($request->organisation) && $request->organisation != 'Select'){
            $clients = Client::where('name','like','%'.$request->organisation.'%')->pluck('id')->toArray();
            $deployments->whereIn('client_id',$clients);
        }

        if (isset($request->status) && $request->status != 'Select'){
            $deployments->where('status','=', $request->status);
        }

        if (isset($request->instance) && $request->instance != 'Select'){
            $deployments->where('instance','=', $request->instance);
        }

        $isChannelPartner=false;

        $clients = Client::pluck('name','id')->toArray();
        $plansArray = Plan::whereIsActive(true)->orderBy('sort_order','asc')->pluck('name','slug')->toArray();
        $deployments = $deployments->paginate(25);
        $overdue = Deployment::whereNotNull('expiry')
            ->where('expiry','<',date('Y-m-d'))
            ->where('instance','=','Deployed')
            ->count();
        $thirty_days_due = Deployment::whereNotNull('expiry')
            ->where('expiry','<=',date('Y-m-d', strtotime('+30 days')))
            ->where('instance','=','Deployed')
            ->count();



        return view( 'subscriptions.index', compact('deployments','isChannelPartner','overdue','thirty_days_due',
            'statuses','instances','plansArray','clients'));
    }

    /**
     * Create a deployment request
     * @param $client_id
     * @param $user_id
     * @return bool
     */
    public function deployment($client_id,$user_id=null)
    {
        //Checks if the $user_id is provided if not the provided the logged-in user's  ID
        if (is_null($user_id)) $user_id = auth()->id();

        if (Client::find($client_id)){

            //Check if the Client has already initiated a deployment request
            $deploy = Deployment::where('client_id','=',$client_id)->first();

            if (!$deploy) {

                Deployment::create([
                    'client_id' => $client_id,
                    'status'    =>  'New',
                    'instance'    =>  'Pending',
                    'expiry'    =>  null,
                    'amount'    =>  0,
                    'created_by'    =>  $user_id,
                ]);

                DeploymentsLog::create([
                    'client_id' => $client_id,
                    'status'    =>  'New',
                    'instance'    =>  'Pending',
                    'expiry'    =>  null,
                    'amount'    =>  0,
                    'created_by'    =>  $user_id,
                ]);

            }

            return true;
        }

        return false;

    }

    public function create($deployment_id)
    {
        $deployment = Deployment::find($deployment_id);

        if (!$deployment) {
            notify("Sorry! Invalid deployment selected.",'error');
            return back();
        }

        return view('deployment.create', compact('deployment'));
    }

    public function save(Request $request)
    {
        $this->validate($request, [
            'deployment_id' => 'required|numeric',
            'username' => 'required|string|min:2',
            'abbreviation' => 'required|string|min:1|max:3',
            'db_name' => 'required|string|min:4',
            'db_user' => 'required|string|min:2',
            'db_password' => 'required|string|min:8',
            'subscription_interval' => 'required|numeric|min:1',
        ]);

        $deployment = Deployment::with(['client'=>['owner']])->find($request->deployment_id);

        if(!$deployment) {
            notify("Sorry! Invalid deployment",'error');
            return back();
        }

        if ($deployment->instance == 'Pending') {
            $deployment->username = $request->username;
            $deployment->password = Hash::make('P@$$word.');
            $deployment->abbreviation = $request->abbreviation;
            $deployment->db_name = $request->db_name;
            $deployment->db_user = $request->db_user;
            $deployment->db_password = $request->db_password;
            $deployment->subscription_interval = $request->subscription_interval;
            $deployment->status = 'Deployed';
            $deployment->instance = 'Online';
            $deployment->deployed_by = auth()->id();

            if ($deployment->save()) {
                $plan = Plan::whereSlug($deployment->package)->first();
                $deployment_date = Carbon::now();
                if (isset($deployment->configurations['app_deployment_date'])) {
                    if (!is_null($deployment->configurations['app_deployment_date'])) {
                        $client_deployment_date = dateInCarbon($deployment->configurations['app_deployment_date']);

                        if ($client_deployment_date > $deployment_date) $deployment_date = $client_deployment_date;
                    }
                }

                $subscribed = $deployment->newPlanSubscription($deployment->code, $plan,$deployment_date);

                $deployment->expiry = $subscribed->ends_at;
                $deployment->save();

                $deploymentLog = new DeploymentsLog();
                $deploymentLog->status =  $deployment->status;
                $deploymentLog->instance =  $deployment->instance;
                $deploymentLog->expiry =  $deployment->expiry;
                $deploymentLog->amount =  $deployment->amount;
                $deploymentLog->country_id =  $deployment->country_id;
                $deploymentLog->client_id =  $deployment->client_id;
                $deploymentLog->package =  $deployment->package;
                $deploymentLog->subscription_interval =  $deployment->subscription_interval;
                $deploymentLog->deployment_id =  $request->deployment_id;
                $deploymentLog->created_by =  auth()->id();
                $deploymentLog->save();

                Mail::to([$deployment->client->owner->email])->send(new ShipDeployment($deployment));

                notify("Congratulation! A new deployment has been shipped.",'success','Successful Deployment!');

            }
            else {
                notify("Sorry! Something went wrong. Please check your information provided.",'error','NOTICE');
                return back();
            }
        }
        else {
            $deployment->username = $request->username;
            $deployment->password = Hash::make('P@$$word.');
            $deployment->abbreviation = $request->abbreviation;
            $deployment->db_name = $request->db_name;
            $deployment->db_user = $request->db_user;
            $deployment->db_password = $request->db_password;
            $deployment->subscription_interval = $request->subscription_interval;
            $deployment->status = 'Deployed';
            $deployment->instance = 'Online';
            $deployment->deployed_by = auth()->id();

            if ($deployment->save()) {
                $deploymentLog = new DeploymentsLog();
                $deploymentLog->status =  $deployment->status;
                $deploymentLog->instance =  $deployment->instance;
                $deploymentLog->expiry =  $deployment->expiry;
                $deploymentLog->amount =  $deployment->amount;
                $deploymentLog->client_id =  $deployment->client_id;
                $deploymentLog->country_id =  $deployment->country_id;
                $deploymentLog->package =  $deployment->package;
                $deploymentLog->subscription_interval =  $deployment->subscription_interval;
                $deploymentLog->deployment_id =  $request->deployment_id;
                $deploymentLog->created_by =  auth()->id();
                $deploymentLog->save();

                PlanSubscription::whereSubscriberId($deployment->id)->update([
                    'canceled_at'   =>  null,
                    'ends_at'   =>  dateInCarbon($deployment->expiry)
                ]);

                Mail::to([$deployment->client->owner->email])->send(new ShipRedeployment($deployment));

                notify("Congratulation! Re-Deployment was successful.",'success','Successful Deployment!');
            }
        }

        return back();
    }

    public function suspension(Request $request)
    {
        $deployment = Deployment::with(['client'=>['owner']])->find($request->deployment_id);
        if(!$deployment) {
            notify("Sorry! Invalid deployment",'error');
            return back();
        }

        $deployment->instance = 'Suspended';
        $deployment->updated_by = auth()->id();

        if ($deployment->save()) {
            $deployment->planSubscription($deployment->code)->cancel(true);

            $deploymentLog= new DeploymentsLog();
            $deploymentLog->deployment_id =  $deployment->id;
            $deploymentLog->status =  $deployment->status;
            $deploymentLog->instance =  $deployment->instance;
            $deploymentLog->expiry =  $deployment->expiry;
            $deploymentLog->amount =  $deployment->amount;
            $deploymentLog->client_id =  $deployment->client_id;
            $deploymentLog->country_id =  $deployment->country_id;
            $deploymentLog->package =  $deployment->package;
            $deploymentLog->created_by =  auth()->id();
            $deploymentLog->save();

            Mail::to([$deployment->client->owner->email])->send(new DeploymentSuspended($deployment));

            notify("Done! A deployment has been suspended with immediate effect.",'success','Successful Suspension!');
            return redirect()->route('dashboard');
        }
        else {
            notify("Sorry! Something went wrong. Please check your information provided.",'error','NOTICE');
            return back();
        }
    }

    public function increaseExpiration(Deployment $deployment)
    {
        if(!$deployment) {
            notify("Sorry! Invalid deployment",'error');
            return back();
        }

        $deployment->updated_by = auth()->id();

        if ($deployment->save()) {
            $date = $deployment->setNewPeriod();
//            $deployment->planSubscription($deployment->code)->ends_at = $deployment->planSubscription($deployment->code)->ends_at->addMonths();
//            $deployment->planSubscription($deployment->code)->canceled_at = null;
//            $deployment->planSubscription($deployment->code)->save();
//            dd($deployment->setNewPeriod());

            $deployment->expiry = $date->toDate()->format('Y-m-d');
            if ($deployment->expiry > now()) {
                $deployment->instance = 'Online';
                $deployment->status = 'Deployed';
            }
            $deployment->save();

            $deploymentLog= new DeploymentsLog();
            $deploymentLog->deployment_id =  $deployment->id;
            $deploymentLog->status =  $deployment->status;
            $deploymentLog->instance =  $deployment->instance;
            $deploymentLog->expiry =  $deployment->expiry;
            $deploymentLog->amount =  $deployment->amount;
            $deploymentLog->client_id =  $deployment->client_id;
            $deploymentLog->country_id =  $deployment->country_id;
            $deploymentLog->package =  $deployment->package;
            $deploymentLog->created_by =  auth()->id();
            $deploymentLog->save();

//            Mail::to([$deployment->client->owner->email])->send(new DeploymentSuspended($deployment));

            notify("One month extension period has been added to the subscription expiry date.",'success','Successful Suspension!');
            return redirect()->route('dashboard');
        }
        else {
            notify("Sorry! Something went wrong. Please try again",'error','NOTICE');
            return back();
        }
    }

    public function projectSettings($client_id,$deployment_id=null)
    {
        $plan = $current_plan = null;
        if (is_null($deployment_id)) {
            $deployment = null;
        }
        else {
            $deployment = Deployment::withCount(['applicants','cases','benefits','applications','users'])
                ->with(['applicantCodeGen','client'])
                ->where('client_id','=',$client_id)
                ->where('id','=',$deployment_id)
                ->first();
            if(!$deployment) {
                abort('404');
            }
            if ($deployment->activePlanSubscriptions()->first()) {
                $plan = $deployment->activePlanSubscriptions()->first()->plan;
            }

            if (!is_null($deployment->package)) {
                $current_plan = Plan::whereSlug($deployment->package)->first();
            }
        }

        $plans = Plan::whereIsActive(true)->with(['features'])->where('is_custom','=',0)->orWhereNull('is_custom')->orderBy('price','asc')->get();
        $country_pricing = CountryPlan::with([
            'currency:id,currency_code',
            'planPricing'=>[
                'plan:id,name,description,year_discount_percent'
            ]])
            ->where('country_id','=',$deployment->country_id)
            ->first();
        $page = 'deployment.settings-page';

        return view($page, compact('deployment','client_id','plan','current_plan','plans','country_pricing'));
    }

    public function saveProjectSettings(Request $request)
    {
        $client = Client::find(Crypt::decrypt($request->client_id));
        if(!$client) {
            notify("Sorry! This client does not exist.",'error');
            return back();
        }

        $logo =  $app_contact = $deployment = null;
        if (isset($request->deployment_id)) {

            $this->validate($request,[
                'app_project_name'   =>    'required|string|min:2',
                'app_email'   =>    'required|string|min:2',
                'client_id'   =>    'required|string',
                'app_country'   =>    'required|string',
            ]);

            $deployment = Deployment::find(Crypt::decrypt($request->deployment_id));
            if(!$deployment) {
                notify("Sorry! Invalid deployment",'error');
                return back();
            }
            if (isset($deployment->logo)) $logo = $deployment->logo;

            if (isset($request->app_logo)) {
                $logo = $this->avatarLogo('app_logo',$client->id,$deployment->id);
            }

            $app_project_module = $deployment->module;
        }

        if (isset($request->app_contact)) {
            $request->validate([
                'app_contact' => 'required|regex:/^([0-9\s\-\+\(\)]*)$/|min:10',
            ]);

            $app_contact = cleanString($request->app_contact);
        }

        $call_code = getCountry($request->app_country,'phone_code');
        $country_name = getCountry($request->app_country,'country_name');
        $currency_symbol = getCountry($request->app_main_currency,'currency_symbol');
        $currency_code = getCountry($request->app_main_currency,'currency_code');

        $deployment->logo = $logo;
        $deployment->name = $request->app_project_name;
        $deployment->country_id = $request->app_country;
        $deployment->country_name = $country_name;
        $deployment->currency_symbol = $currency_symbol;
        $deployment->currency_code = $currency_code;
        $deployment->call_code = $call_code;
        $deployment->module = $app_project_module;
        $deployment->address = $request->app_address;
        $deployment->description = $request->app_description;
        $deployment->contact = $app_contact;
        $deployment->client_id = Crypt::decrypt($request->client_id);
        $deployment->language = $request->language;

        if ($deployment->save()) {
            $message = "You have successfully updated your account configuration preferences.";
            notify($message,'success','Saved!');
        }
        else {
            notify("Sorry! Something went wrong. Please check your information provided.",'error','NOTICE');
        }
        return back();
    }

    public function saveTerms(Request $request)
    {
        $newData = Arr::except($request->all(), ['_token']);
        settings()->set($newData);
        $message = 'Terminology has been saved successfully';
        session()->flash('status', 'success');
        session()->flash('message', $message);
        return back();
    }

    public function saveApplicantGenCode(Request $request,Deployment $deployment)
    {
        $request->validate([
            'gen_code_1'    =>  'required|string',
            'gen_code_2'    =>  'required|string',
            'gen_code_3'    =>  'required|string',
        ]);

        $format = [$request->gen_code_1];

        if (in_array($request->gen_code_2,$format)) {
            notify("Sorry! ".Str::headline($request->gen_code_2)." has already been selected.",'error','NOTICE');
            return back();
        }
        $format[] = $request->gen_code_2;

        if (in_array($request->gen_code_3,$format)) {
            notify("Sorry! ".Str::headline($request->gen_code_3)." has already been selected.",'error','NOTICE');
            return back();
        }

        $format[] = $request->gen_code_3;


        $code = Deployment::where('id','=',$deployment->id)->update([
            'applicant_code_format'  =>  $format
        ]);

        if ($code) {
            notify('Reference code format has been saved','success','Saved!');
            return back();
        }
        else {
            notify("Sorry! Something went wrong. Please check your information provided.",'error','NOTICE');
            return back();
        }
    }

    public function checkSubdomain(Request $request)
    {
        // Check if the subdomain exists in the database
        $exists = Deployment::where('domain', $request->subdomain)->exists();

        return response()->json(['exists' => $exists]);
    }

    public function updateProject($id) {
        $deployment = Deployment::find($id);
        if (!$deployment) {
            notify("Sorry! Project cannot be found",'error','NOTICE');
            return back();
        }
        $url = getDeploymentUrl($deployment).'/api/update-project/';
        if (env('SYSTEM_DEBUG_MODE')) $url = 'http://127.0.0.1:8000/api/update-project/';

        $response = Http::get($url.$deployment->code);
        $jsonData = $response->json();

//        $client = new \GuzzleHttp\Client();
//        $res = $client->request('GET', $url.$deployment->code);
//        echo $res->getStatusCode();
//        echo $res->getHeader('content-type')[0];
//        echo $res->getBody();
        dd($jsonData);
        return back();
    }

    public function sendInvitation(Request $request)
    {
        $request->validate([
            'email' =>  'required|email'
        ]);

        $user = User::where('email','=',$request->email)->first();

        if (!$user) {
            session()->flash('message','User does not exist');
            session()->flash('status','error');
            return back()->withInput();
        }

        $invited = DeploymentInvitation::updateOrCreate([
            'email' =>  $request->email,
            'deployment_id' =>  getActiveInstance(),
        ],
        [
            'user_id'   =>  auth()->id(),
            'name'   =>  $user->fullName,
            'role_id'   =>  $request->role
        ]);

        $view = null;

        if ($invited) {

            $deployment = Deployment::findOrFail(getActiveInstance());

            GeneralNotification::create([
                'user_id'   =>  auth()->id(),
                'receivers'   =>  [$user->id],
                'subject_id'   =>  $invited->id,
                'table'   =>  $invited->getModel()->getTable(),
                'instance_id'   =>  getActiveInstance(),
                'message'   =>  '<strong>Collaboration:</strong> '.$deployment->name.' has invited to collaborate',
                'route'   =>  route('account.editAccount'),
            ]);

            $invitations = $deployment->invitations;
            $view = view('user_management.list-invitations', compact('invitations'))->render();
            $message = 'Invitation has been sent successfully.';
            $status = 1;

            Mail::to([$request->email])->send(new SendInstanceInvitation($invited,$deployment));
        }
        else {
            $message = 'Invitation was not sent due to some challenges. Try again.';
            $status = 0;
        }
        return response()->json(['status'=>$status,'message'=>$message,'view'=>$view]);
    }

    public function checkEmail(Request $request)
    {
        // Check if the email exists in the database
        $exists = User::where('email', $request->email)->exists();

        return response()->json(['exists' => $exists]);
    }

    public function cancelInvitation(Request $request)
    {
        if (DeploymentInvitation::where('deployment_id','=', getActiveInstance())->where('id','=',$request->invitation)->delete()) {
            $deployment = getActiveDeployment();
            $invitations = $deployment->invitations;
            $view = view('user_management.list-invitations', compact('invitations'))->render();
            $message = 'Invitation has been cancelled successfully.';
            $status = 1;
        }
        else {
            $message = 'Invitation was not cancelled due to some challenges. Try again.';
            $status = 0;
        }
        return response()->json(['status'=>$status,'message'=>$message,'view'=>$view]);
    }
}
