<?php

namespace App\Http\Controllers;

use App\Models\ContentConfig;
use App\Models\Country;
use App\Models\County;
use App\Models\Currency;
use App\Models\Instance;
use App\Models\MeasurementSystem;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Storage;

class InstancesController extends Controller
{

    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index(Request $request)
    {
//        $instances = Instance::orderBy('project_name');
//        if (isset($request->value)) {
//            $instances = $instances->where('project_name','like','%'.$request->value.'%')
//                            ->orWhere('email','like','%'.$request->value.'%')
//                            ->orWhere('owner','like','%'.$request->value.'%')
//                            ->orWhere('contact','like','%'.$request->value.'%');
//        }
//        if (isset($request->country)) {
//            $instances = $instances->where('country','=',$request->country);
//        }
//        if (isset($request->status)) {
//            $instances = $instances->where('status','=',$request->status);
//        }
//
//        $instances = $instances->get();
//        $countries = Country::all();
//        $currencies = Currency::all();
//        return view('instances.index', compact('instances','countries','currencies'));
    }

    public function createInstance()
    {
//        $instance = null;
//        $countries = Country::all();
//        $currencies = Currency::all();
//        $measure = MeasurementSystem::all();
//        $counties = County::where('status','=',1)->get();
//        $action_type = 'Create';
//        return view('instances.instance-form', compact('instance','counties','action_type','measure','countries','currencies'));
    }

    public function myInstance()
    {
        $instance = Instance::find(1);
        $countries = Country::all();
        $currencies = Currency::all();
        $measure = MeasurementSystem::all();
        $counties = County::where('status','=',1)->get();
        if ($instance) {
            $counties = County::where('status','=',1)->where('country_id','=',$instance->country_id)->get();
        }
        $action_type = 'Preferences';
        return view('instances.instance-form', compact('instance','counties','action_type','countries','currencies','measure'));
    }

    public function editInstance($instance_id)
    {
//        $instance = Instance::find(1);
//        $countries = Country::all();
//        $currencies = Currency::all();
//        $measure = MeasurementSystem::all();
//        $counties = County::where('status','=',1)->where('country_id','=',$instance->country_id)->get();
//        $action_type = 'Edit';
//        return view('instances.instance-form', compact('instance','counties','action_type','countries','currencies','measure'));
    }

    public function viewInstance($instance_id)
    {
//        $instance = Instance::find(1);
//        $view = view('instances.instance-view', compact('instance',))->render();
//        return response()->json(['the_view'=>$view]);
    }

    public function saveInstance(Request $request)
    {
        $this->validate($request,[
            'project_name' => 'required|string|min:2',
            'owner' => 'required|string|min:2',
            'email' => 'required|string|min:3',
            'main_currency' => 'required|numeric|min:1',
            'alt_currency_id' => 'nullable|numeric|min:1',
            'country_id' => 'required|numeric|min:1',
            'contact' => 'required|string|min:9',
            'copyright' => 'required|string|min:2',
            'advisory_url' => 'nullable|url|min:10',
            'counties' => 'nullable|array',
            'lms_url' => 'nullable|url|min:10',
            'deployment_date' => 'required|string',
            'measurement_system_id' => 'required|numeric|min:1',
            'description' => 'nullable|string',
            'logo' => 'nullable|file|mimes:jpg,jpeg,bmp,png,webp|max:5024',
            'favicon_url' => 'nullable|file|mimes:jpg,jpeg,bmp,png,webp,ico,svg|max:5024',
            'footer_logo_url' => 'nullable|file|mimes:jpg,jpeg,bmp,png,webp,ico,svg|max:5024',
            'bg_image' => 'nullable|file|mimes:jpg,jpeg,bmp,png,webp|max:5024',
        ]);

        try {
            if (isset($request->instance_id)) {
                $instance = Instance::find(Crypt::decrypt($request->instance_id));
                $instance->updated_by = Auth::id();
            }
            else {
                $instance = new Instance();
                $instance->created_by = Auth::id();
                $instance->status = 1;
            }

            $instance->project_name = $request->project_name;
            $instance->owner = $request->owner;
            $instance->currency_id = $request->main_currency;
            $instance->alt_currency_id = $request->alt_currency_id;
            $instance->country_id = $request->country_id;
            $instance->counties = $request->counties;
            $instance->email = $request->email;
            $instance->contact = $request->contact;
            $instance->deployment_date = date('Y-m-d', strtotime($request->deployment_date));
            $instance->lms_url = $request->lms_url;
            $instance->advisory_url = $request->advisory_url;
            $instance->copyright = $request->copyright;
            $instance->measurement_system_id = $request->measurement_system_id;
            $instance->description = $request->description;
            $instance->save();

            if($request->file('logo')) {
                $file = $request->file('logo');
                $name = $file->hashName();
                $extension = $file->getClientOriginalExtension();
                $full_name = "$name.$extension";
                $url = 'instances/'.$instance->id.'/logo';
                $extension = $request->file('logo')->extension();
                $name = time().".$extension";
                Storage::disk('public_uploads')->putFileAs($url, $request->file('logo'),$full_name);

                $instance->logo_url = "app-uploads/$url/$full_name";
            }

            if($request->file('favicon_url')) {
                $file = $request->file('favicon_url');
                $name = $file->hashName();
                $extension = $file->getClientOriginalExtension();
                $full_name = "$name.$extension";
                $url = 'instances/'.$instance->id.'/favicon';
                Storage::disk('public_uploads')->putFileAs($url, $request->file('favicon_url'),$full_name);

                $instance->favicon_url = "app-uploads/$url/$full_name";
            }

            if($request->file('footer_logo_url')) {
                $file = $request->file('footer_logo_url');
                $name = $file->hashName();
                $extension = $file->getClientOriginalExtension();
                $full_name = "$name.$extension";
                $url = 'instances/'.$instance->id.'/logo';
                Storage::disk('public_uploads')->putFileAs($url, $request->file('footer_logo_url'),$full_name);

                $instance->footer_logo_url = "app-uploads/$url/$full_name";
            }

            if($request->file('bg_image')) {
                $file = $request->file('bg_image');
                $name = $file->hashName();
                $extension = $file->getClientOriginalExtension();
                $full_name = "$name.$extension";
                $url = 'instances/'.$instance->id.'/bg_image';
                Storage::disk('public_uploads')->putFileAs($url, $request->file('bg_image'),$full_name);
                $instance->bg_image_url = "app-uploads/$url/$full_name";
            }
            $instance->save();

            $message = 'Preference has been set';
            session()->flash('status', 'success');
            if (isset($request->instance_id)) {
                $message = 'Preference has been updated successfully';
            }
            session()->flash('message', $message);
            if ($request->action_type == 'Profile') {
                return back();
            }
            return back();
//            return redirect()->route('instances.editInstance',[cryptThis($instance->id)]);
        }
        catch (\Exception $exception) {
            $message = 'Sorry something went wrong. Please try again. '.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back()->withInput();
        }
    }

    public function saveContentConfig(Request $request)
    {
        $saved = null;
        foreach ($request->all() as $key=>$content) {
            $saved = ContentConfig::updateOrCreate([
                'key'    => $key,
                'instance_id'    => Crypt::decrypt($request->instance_id),
            ],[
                'text'    => $content,
                'updated_by'    => Auth::id(),
            ]);
        }

        if ($saved) {
            $message = 'Configuration Content has been saved';
            session()->flash('status', 'success');
        }
        else{
            $message = 'All Configurations may not be saved. Please check.';
            session()->flash('status', 'success');
        }

        session()->flash('message', $message);
        return back();
    }

    public function removeInstance(Request $request)
    {
//        $instance = Instance::find(Crypt::decrypt($request->instance));
//        if ($instance) {
//            if ($instance->status == 1) {
//                $instance->status = 0;
//                $instance->save();
//                $message = 'Instance Preference has been deactivated!';
//                session()->flash('status', 'success');
//            }
//            else {
//                $instance->status = 1;
//                $instance->save();
//                $message = 'Instance Preference has been restore!';
//                session()->flash('status', 'success');
//            }
//        }
//        else {
//            $message = 'Sorry something went wrong. Please try again.';
//            session()->flash('status', 'error');
//        }
//        session()->flash('message', $message);
        return back();
    }
}
