<?php

namespace App\Http\Controllers;

use App\Models\Applicant;
use App\Models\Application;
use App\Models\Benefit;
use App\Models\Constituency;
use App\Models\Deployment;
use App\Models\District;
use App\Models\HealthDetail;
use App\Models\IdentificationDetail;
use App\Models\InventoryItem;
use App\Models\MaritalDetail;
use App\Models\Mdata;
use App\Models\User;
use App\Models\Village;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use PhpOffice\PhpSpreadsheet\IOFactory;

class MdataController extends Controller
{
    //
    public function index()
    {
        //dd(Benefit::liveBenefits()->get());

        return view('mdata.index');
    }

    public function import(Request $request)
    {
        // Set the maximum execution time to 300 seconds (5 minutes)
        set_time_limit(1200);

        $request->validate([
            'file' => 'required|mimes:xlsx,xls,csv'
        ]);

        // Load the uploaded file
        $file = $request->file('file');
        $spreadsheet = IOFactory::load($file->getPathname());
        $worksheet = $spreadsheet->getActiveSheet();

        //Check if the headers (all columns ) exist

        //Assume the first row is the header
        $rowIndex = 2;
        
        while($worksheet->getCell('A' . $rowIndex)->getValue() !== null){

            $unixDate = ($worksheet->getCell('BD'. $rowIndex)->getValue() - 25569) * 86400;
            $dob = \Carbon\Carbon::createFromTimestamp($unixDate)->format('Y-m-d H:i:s');


            Mdata::create([
                'countyName' => $worksheet->getCell('A'. $rowIndex)->getValue(),
                'constituencyName' => $worksheet->getCell('B'. $rowIndex)->getValue(),
                'districtName' => $worksheet->getCell('C'. $rowIndex)->getValue(),
                'divisionName' => $worksheet->getCell('D'. $rowIndex)->getValue(),
                'locationName' => $worksheet->getCell('E'. $rowIndex)->getValue(),
                'subLocationName' => $worksheet->getCell('F'. $rowIndex)->getValue(),
                'village' => $worksheet->getCell('G'. $rowIndex)->getValue(),
                'landmark' => $worksheet->getCell('H'. $rowIndex)->getValue(),
                'communityLeaderName' => $worksheet->getCell('I'. $rowIndex)->getValue(),
                'numberOfHouseholdMembers' => $worksheet->getCell('J'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('J'. $rowIndex)->getValue(),
                'mainCaregiver' => $worksheet->getCell('K'. $rowIndex)->getValue(),
                'noOfHabitableRooms' => $worksheet->getCell('L'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('L'. $rowIndex)->getValue(),
                'dwellingTenure' => $worksheet->getCell('M'. $rowIndex)->getValue(),
                'roof' => $worksheet->getCell('N'. $rowIndex)->getValue(),
                'wall' => $worksheet->getCell('O'. $rowIndex)->getValue(),
                'floor' => $worksheet->getCell('P'. $rowIndex)->getValue(),
                'dwellingUnitRisk' => $worksheet->getCell('Q'. $rowIndex)->getValue(),

                'lightingFuel' => $worksheet->getCell('R'. $rowIndex)->getValue(),
                'waterSource' => $worksheet->getCell('S'. $rowIndex)->getValue(),
                'humanWasteDisposal' => $worksheet->getCell('T'. $rowIndex)->getValue(),

                'cookingFuel'=>$worksheet->getCell('U'. $rowIndex)->getValue(),
                'television'=>$worksheet->getCell('V'. $rowIndex)->getValue(),
                'motorcycle'=>$worksheet->getCell('W'. $rowIndex)->getValue(),
                'tuktuk'=>$worksheet->getCell('X'. $rowIndex)->getValue(),
                'refrigerator'=>$worksheet->getCell('Y'. $rowIndex)->getValue(),
                'car'=>$worksheet->getCell('Z'. $rowIndex)->getValue(),

                'mobilePhone'=>$worksheet->getCell('AA'. $rowIndex)->getValue(),
                'bicycle'=>$worksheet->getCell('AB'. $rowIndex)->getValue(),
                'exoticCattle'=>$worksheet->getCell('AC'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('AC'. $rowIndex)->getValue(),
                'indigenousCattle'=>$worksheet->getCell('AD'. $rowIndex)->getValue(),
                
                

                'sheep'=>$worksheet->getCell('AE'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('AE'. $rowIndex)->getValue(),
                'goats'=>$worksheet->getCell('AF'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('AF'. $rowIndex)->getValue(),
                'camels'=>$worksheet->getCell('AG'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('AG'. $rowIndex)->getValue(),
                'donkeys'=>$worksheet->getCell('AH'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('AH'. $rowIndex)->getValue(),
                'pigs'=>$worksheet->getCell('AI'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('AI'. $rowIndex)->getValue(),
                'chicken'=>$worksheet->getCell('AJ'. $rowIndex)->getValue() == 'NULL' ? 0 : $worksheet->getCell('AJ'. $rowIndex)->getValue(),
                'liveBirths'=>$worksheet->getCell('AK'. $rowIndex)->getValue(),
                'deaths'=>$worksheet->getCell('AL'. $rowIndex)->getValue(),
                'householdCondition'=>$worksheet->getCell('AM'. $rowIndex)->getValue(),
                'mealCut'=>$worksheet->getCell('AN'. $rowIndex)->getValue(),
                'nsnpBenefits'=>$worksheet->getCell('AO'. $rowIndex)->getValue(),
                'otherBenefits'=>$worksheet->getCell('AP'. $rowIndex)->getValue(),
                'otherProgrammes'=>$worksheet->getCell('AQ'. $rowIndex)->getValue(),
                'benefitType'=>$worksheet->getCell('AR'. $rowIndex)->getValue(),
                'benefitAmount'=>$worksheet->getCell('AS'. $rowIndex)->getValue(),
                'inKindBenefit'=>$worksheet->getCell('AT'. $rowIndex)->getValue(),
                'listingID'=>$worksheet->getCell('AU'. $rowIndex)->getValue(),
                'firstName'=>$worksheet->getCell('AV'. $rowIndex)->getValue(),
                'middleName'=>$worksheet->getCell('AW'. $rowIndex)->getValue(),
                'surname'=>$worksheet->getCell('AX'. $rowIndex)->getValue(),
                'idTypeID'=>$worksheet->getCell('AY'. $rowIndex)->getValue(),
                'idNumber'=>$worksheet->getCell('AZ'. $rowIndex)->getValue(),
                'phoneNumber'=>$worksheet->getCell('BA'. $rowIndex)->getValue(),
                'relationshipID'=>$worksheet->getCell('BB'. $rowIndex)->getValue(),
                'sex'=>$worksheet->getCell('BC'. $rowIndex)->getValue(),
                'dateOfBirth'=>$dob,
                'maritalStatusID'=>$worksheet->getCell('BE'. $rowIndex)->getValue(),

                'spouseLineNumber'=>$worksheet->getCell('BF'. $rowIndex)->getValue(),

                'fatherAlive'=>$worksheet->getCell('BG'. $rowIndex)->getValue(),
                'motherAlive'=>$worksheet->getCell('BH'. $rowIndex)->getValue(),
                'chronicIllness'=>$worksheet->getCell('BI'. $rowIndex)->getValue(),
                'disabilityTypeID'=>$worksheet->getCell('BJ'. $rowIndex)->getValue(),
                'disabilityFulltimeCare'=>$worksheet->getCell('BK'. $rowIndex)->getValue(),
                //'mainCargiver'=>$worksheet->getCell('BL'. $rowIndex)->getValue(),
                'learningStatusID'=>$worksheet->getCell('BM'. $rowIndex)->getValue(),
                'educationLevelID'=>$worksheet->getCell('BN'. $rowIndex)->getValue(),
                'occupationID'=>$worksheet->getCell('BO'. $rowIndex)->getValue(),
                'formalEmploymentID'=>$worksheet->getCell('BP'. $rowIndex)->getValue(),
                'pmtScore'=>$worksheet->getCell('BQ'. $rowIndex)->getValue(), 

                'user_created_id'=>auth()->user()->id,
                'user_updated_id'=>auth()->user()->id
            ]);
            $rowIndex++;
        }

        return back()->with('success', 'Done');
        


    }


    public function import2(Request $request)
    {
        set_time_limit(600); // Set to 600 seconds (10 minutes)

        $request->validate([
            'file' => 'required|mimes:xlsx,xls,csv|max:2048'
        ]);

        

        try {
            // Load the uploaded file
            $file = $request->file('file');
            $spreadsheet = IOFactory::load($file->getPathname());
            $worksheet = $spreadsheet->getActiveSheet();
        } catch (\Exception $e) {
            return back()->withErrors(['file' => 'Error reading Excel file: ' . $e->getMessage()]);
        }

        $rowIndex = 2;
        $omitted_rows = [];
        $learning_statuses = [];
        $occupation_options = [];
        $nsnp_options = [];

        while ($worksheet->getCell('A' . $rowIndex)->getValue() !== null) {

            // Fetch values
            $first_name = $worksheet->getCell('AV' . $rowIndex)->getValue();
            $middle_name = $worksheet->getCell('AW' . $rowIndex)->getValue() == 'NULL' ? null : $worksheet->getCell('AW' . $rowIndex)->getValue();
            $surname = $worksheet->getCell('AX' . $rowIndex)->getValue();
            $tel = '0' . $worksheet->getCell('BA' . $rowIndex)->getValue();
            $pmt = $worksheet->getCell('BQ' . $rowIndex)->getValue();
            
            // Find applicant
            $applicants = Applicant::where([
                ['surnames', ucwords($surname)],
                ['other_names', $middle_name !== null ? ucwords($middle_name) : null ],
                ['first_names', ucwords($first_name)],
            ])->orderBy('id', 'desc')->get();
            
            foreach($applicants as $applicant){
                if ($applicant && $applicant->contactDetails->first() && $applicant->contactDetails->first()->mobile == $tel) {
                    if (strtoupper($applicant->first_names) === $applicant->first_names) {
                        $applicant->first_names = ucwords(strtolower($applicant->first_names));
                    }
                    if (strtoupper($applicant->other_names) === $applicant->other_names) {
                        $applicant->other_names = ucwords(strtolower($applicant->other_names));
                    }
                    if (strtoupper($applicant->surnames) === $applicant->surnames) {
                        $applicant->surnames = ucwords(strtolower($applicant->surnames));
                    }

                    if($worksheet->getCell('BG' . $rowIndex)->getValue() == 'No' && $worksheet->getCell('BH' . $rowIndex)->getValue() == 'No' ){
                        $applicant->is_orphan = 1 ;
                    }else{
                        $applicant->is_orphan = 0 ;
                    }

                    if($worksheet->getCell('BN' . $rowIndex)->getValue() !== 'None' || $worksheet->getCell('BN' . $rowIndex)->getValue() !== 'NULL'){
                        $applicant->highest_education_level = $worksheet->getCell('BN' . $rowIndex)->getValue();
                    }

                    if($worksheet->getCell('AM' . $rowIndex)->getValue() !== 'None' || $worksheet->getCell('AM' . $rowIndex)->getValue() !== 'NULL'){
                        $applicant->household_condition = $worksheet->getCell('AM' . $rowIndex)->getValue();
                    }

                    if($applicant->idDetails()->count() < 1 ){

                        if($worksheet->getCell('AY' . $rowIndex)->getValue() !== 'None' || $worksheet->getCell('AY' . $rowIndex)->getValue() !== 'NULL'){

                            IdentificationDetail::create([
                                'id_type' => $worksheet->getCell('AY' . $rowIndex)->getValue(),
                                'id_number' => $worksheet->getCell('AZ' . $rowIndex)->getValue() ,
                                'applicant_number' => $applicant->regID,
                                'Status' => 'New',
                                'addedBy'=>auth()->user()->id
                            ]);

                        }
                        
                    }

                    if($applicant->idDetails()->count() < 1 ){

                        if($worksheet->getCell('AY' . $rowIndex)->getValue() !== 'None' || $worksheet->getCell('AY' . $rowIndex)->getValue() !== 'NULL'){

                            IdentificationDetail::create([
                                'id_type' => $worksheet->getCell('AY' . $rowIndex)->getValue(),
                                'id_number' => $worksheet->getCell('AZ' . $rowIndex)->getValue() ,
                                'applicant_number' => $applicant->regID,
                                'Status' => 'New',
                                'addedBy'=>auth()->user()->id
                            ]);

                        }
                        
                    }

                    if($applicant->maritalDetails()->count() < 1 ){

                        if($worksheet->getCell('BE' . $rowIndex)->getValue() !== 'None' || $worksheet->getCell('BE' . $rowIndex)->getValue() !== 'NULL'){

                            if($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Never Married'){
                                MaritalDetail::create([
                                    'applicant_number' => $applicant->regID,
                                    'marriage_status' => 'Single',
                                    'addedBy' => auth()->user()->id,
                                    'status'=>'New'
                                ]);
                            }

                            elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Married Monogamous'){

                                MaritalDetail::create([
                                    'applicant_number' => $applicant->regID,
                                    'marriage_status' => 'Married Monogamous',
                                    'spouse_name' => 'Unknown',
                                    'addedBy' => auth()->user()->id,
                                    'status'=>'New'
                                ]);

                            }

                            elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Married Polygamous'){

                                MaritalDetail::create([
                                    'applicant_number' => $applicant->regID,
                                    'marriage_status' => 'Married Polygamous',
                                    'spouse_name' => 'Unknown',
                                    'addedBy' => auth()->user()->id,
                                    'status'=>'New'
                                ]);
                                
                            }

                            elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Widowed'){

                                MaritalDetail::create([
                                    'applicant_number' => $applicant->regID,
                                    'marriage_status' => 'Ended By Death',
                                    'spouse_name' => 'Unknown',
                                    'addedBy' => auth()->user()->id,
                                    'status'=>'New'
                                ]);
                                
                            }

                            elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Divorced or Separated'){

                                MaritalDetail::create([
                                    'applicant_number' => $applicant->regID,
                                    'marriage_status' => 'Separated',
                                    'spouse_name' => 'Unknown',
                                    'addedBy' => auth()->user()->id,
                                    'status'=>'New'
                                ]);
                                
                            }

                        }
                        
                    }

                    if($applicant->healthDetails()->count() < 1){

                        if($worksheet->getCell('BI' . $rowIndex)->getValue() == 'Yes'){
                            HealthDetail::create([
                                'regID' => $applicant->regID,
                                'health_type' => 'Chronic Fatigue, Pain Or Seizure',
                                'addedBy' => auth()->user()->id,
                                'status'=>'New'
                            ]);
                        }

                        if($worksheet->getCell('BJ' . $rowIndex)->getValue() !== 'NULL' && str_contains($worksheet->getCell('BJ' . $rowIndex)->getValue(), 'Hearing')){
                            HealthDetail::create([
                                'regID' => $applicant->regID,
                                'health_type' => 'Hearing',
                                'addedBy' => auth()->user()->id,
                                'status'=>'New'
                            ]);
                        }

                        if($worksheet->getCell('BJ' . $rowIndex)->getValue() !== 'NULL' && str_contains($worksheet->getCell('BJ' . $rowIndex)->getValue(), 'Speech')){

                            HealthDetail::create([
                                'regID' => $applicant->regID,
                                'health_type' => 'Speech',
                                'addedBy' => auth()->user()->id,
                                'status'=>'New'
                            ]);

                        }

                    }

                    

                    $applicant -> father_alive = $worksheet->getCell('BG' . $rowIndex)->getValue() == 'Yes' ? 1 : 0 ;
                    $applicant -> mother_alive = $worksheet->getCell('BH' . $rowIndex)->getValue() == 'Yes' ? 1 : 0 ;
                    if($worksheet->getCell('BM' . $rowIndex)->getValue() !== 'NULL' && $worksheet->getCell('BM' . $rowIndex)->getValue() !== 'None'){
                        $applicant -> learning_status =  ucwords($worksheet->getCell('BM' . $rowIndex)->getValue());
                        if(!in_array(ucwords($worksheet->getCell('BM' . $rowIndex)->getValue()), $learning_statuses)){
                            $learning_statuses[] = ucwords($worksheet->getCell('BM' . $rowIndex)->getValue());
                        }
                    }
                    
                    if($worksheet->getCell('BO' . $rowIndex)->getValue() !== 'NULL' && $worksheet->getCell('BO' . $rowIndex)->getValue() !== 'None' && $worksheet->getCell('BO' . $rowIndex)->getValue() !== 'No'){
                        $applicant -> occupation =  ucwords($worksheet->getCell('BO' . $rowIndex)->getValue());

                        if(!in_array(ucwords($worksheet->getCell('BO' . $rowIndex)->getValue()) , $occupation_options)){
                            $occupation_options[] = ucwords($worksheet->getCell('BO' . $rowIndex)->getValue()) ;
                        }
                    }

                    if($worksheet->getCell('AO' . $rowIndex)->getValue() !== 'NULL' && $worksheet->getCell('AO' . $rowIndex)->getValue() !== 'None' && $worksheet->getCell('AO' . $rowIndex)->getValue() !== 'No'){
                        $applicant -> nsnp_benefits =  $worksheet->getCell('AO' . $rowIndex)->getValue();

                        if(!in_array($worksheet->getCell('AO' . $rowIndex)->getValue(), $nsnp_options)){
                            $nsnp_options[] = $worksheet->getCell('AO' . $rowIndex)->getValue();
                        }
                    }

                    //check if village exists if it does assign to applicant else create and assign

                    if($worksheet->getCell('G' . $rowIndex)->getValue() !== 'NULL' && $worksheet->getCell('G' . $rowIndex)->getValue() !== 'None' && $worksheet->getCell('G' . $rowIndex)->getValue() !== 'No'){

                        $village = Village::where('name', ucwords(strtolower($worksheet->getCell('G' . $rowIndex)->getValue())))->first();

                        if($village){
                            $applicant->village_id = $village->id;
                        }
                    }





                    $applicant->pmtScore = $pmt;
                    $applicant->save();
                } else {
                    $omitted_rows[] = $rowIndex;
                }
            }

            $rowIndex++;
        }

        if (count($omitted_rows) > 0) {
            return back()->with('warning', 'Beneficiary Records Updated. The following rows in the spreadsheet were omitted: ' . implode(', ', $omitted_rows) .'Learning Status Options include : '.implode(', ', $learning_statuses). ' Occupation Options include : '.implode(', ', $occupation_options).' and NSNP include :'.implode(', ', $nsnp_options));
        } else {
            return back()->with('success', 'Beneficiary Records Updated.');
        }
    }

    public function inventory(Request $request)
    {
        //dd(auth()->user()->deployments()->first()->items());
        $benefits = getActiveBenefit();
        $applications = Application::where('instance_id', '=', getActiveInstance())->count();
        $new = Application::submitted()->where('instance_id', '=', getActiveInstance())->count();
        $approved = Application::approved()->where('instance_id', '=', getActiveInstance())->count();
        $deployments = Deployment::orderBy('name')->get();
        
        
        if (canDo('administrator', 'settings')) {
            $inventory_items = InventoryItem::orderBy('id', 'desc')->get();
            $programs = Benefit::orderBy('titles')->get();
            if(isset($request->item_name)){

                $inventory_items = InventoryItem::where('item', 'like', '%'.$request->item_name.'%')->orderBy('id', 'desc')->get();
            }
        } else {
            $userDeploymentIds = auth()->user()->deployments->pluck('id');
            $inventory_items = InventoryItem::whereIn('deployment_id', $userDeploymentIds)->orderBy('id', 'desc')->get();
            $programs = Benefit::whereIn('instance_id', $userDeploymentIds)->orderBy('titles')->get();
            $deployments = auth()->user()->deployments;
            if(isset($request->item_name)){

                $inventory_items = InventoryItem::whereIn('deployment_id', $userDeploymentIds)->where('item', 'like', '%'.$request->item_name.'%')->orderBy('id', 'desc')->get();

            }
        }

        

        return view('inventory.index', compact('programs','request', 'benefits', 'new', 'approved', 'applications', 'deployments', 'inventory_items'));
    }

    public function create_inventory_item(Request $request)
    {
        $request->validate([
            'deployment'=> 'required',
            'category'=> 'required',
            'item'=> 'required',
            'cost'=>'required',
            'quantity' => 'required'
        ]);

        //dd($request);

        $deployment = Deployment::findOrFail($request->deployment);

        InventoryItem::create([
            'deployment_id'=>$deployment->id,
            'benefit_id' => $request->program ?? null,
            'category'=> $request->category,
            'item'=>$request->item,
            'monetary_value'=>$request->cost,
            'quantity'=>$request->quantity,
            'created_by'=>auth()->user()->id,
            'budget'=>$request->budget
        ]);

        return back()->with('success', 'New Inventory Item Created');
    }

    public function get_inventory_item($item_id)
    {
        $item = InventoryItem::findOrFail($item_id);
        return response()->json($item);
    }

    public function update_inventory_item(Request $request, $item_id)
    {
        $request->validate([
            'deployment'=> 'required',
            'category'=> 'required',
            'item'=> 'required',
            'cost'=>'required',
            'quantity' => 'required'
        ]);

        $item = InventoryItem::findOrFail($item_id);

        $item->item = $request->item;
        $item->category = $request->category;
        $item->monetary_value = $request->cost;
        $item->quantity = $request->quantity;
        $item->budget = $request->budget;

        $item->save();

        return redirect()->back()->with('success', 'Item Successfully Updated');

    }

    public function create_village(Request $request)
    {
        set_time_limit(600); // Set to 600 seconds (10 minutes)

        $request->validate([
            'file' => 'required|mimes:xlsx,xls,csv|max:2048'
        ]);

        

        try {
            // Load the uploaded file
            $file = $request->file('file');
            $spreadsheet = IOFactory::load($file->getPathname());
            $worksheet = $spreadsheet->getActiveSheet();
        } catch (\Exception $e) {
            return back()->withErrors(['file' => 'Error reading Excel file: ' . $e->getMessage()]);
        }

        $rowIndex = 2;
        $constituencies = [];
        $villages = [];
        $districts = [];

        while ($worksheet->getCell('A' . $rowIndex)->getValue() !== null) {

            if(
                $worksheet->getCell('B' . $rowIndex)->getValue() !== null && 
                $worksheet->getCell('B' . $rowIndex)->getValue() !== 'NULL' &&
                $worksheet->getCell('B' . $rowIndex)->getValue() !== 'None'
            ){
                if(!in_array(strtolower($worksheet->getCell('B' . $rowIndex)->getValue()), $constituencies)){

                    //check if constituency exists in db if not create
                    $constituency = Constituency::where('name', ucwords(strtolower($worksheet->getCell('B' . $rowIndex)->getValue())))->first();

                    if(!$constituency){
                        $constituency = Constituency::create([
                            'name'=> ucwords(strtolower($worksheet->getCell('B' . $rowIndex)->getValue())),
                            'country_id'=> 115
                        ]);
                    }

                    $constituencies[] = strtolower($worksheet->getCell('B' . $rowIndex)->getValue());
                    
                }

            }

            if(
                $worksheet->getCell('G' . $rowIndex)->getValue() !== null && 
                $worksheet->getCell('G' . $rowIndex)->getValue() !== 'NULL' &&
                $worksheet->getCell('G' . $rowIndex)->getValue() !== 'None'
            ){
                if(!in_array(strtolower($worksheet->getCell('G' . $rowIndex)->getValue()), $constituencies)){

                    //check if village exists in db if not create
                    $village = Village::where('name', ucwords(strtolower($worksheet->getCell('G' . $rowIndex)->getValue())))->first();

                    if(!$village){

                        Village::create([
                            'name' => ucwords(strtolower($worksheet->getCell('G' . $rowIndex)->getValue())),
                            'country_id'=> 115,
                            'division'=>ucwords(strtolower($worksheet->getCell('D' . $rowIndex)->getValue())),
                            'district_name' => ucwords(strtolower($worksheet->getCell('C' . $rowIndex)->getValue())),
                            'constituency_id'=> $constituency->id,
                            'location' => ucwords(strtolower($worksheet->getCell('E' . $rowIndex)->getValue())),
                            'sublocation' => ucwords(strtolower($worksheet->getCell('F' . $rowIndex)->getValue())),
                            'leader_name' => ucwords(strtolower($worksheet->getCell('I' . $rowIndex)->getValue()))
                        ]);

                    }

                    $villages[] = strtolower($worksheet->getCell('G' . $rowIndex)->getValue());

                    
                    
                }

            }

            $rowIndex++;

            


        }

        return back()->with('success', 'Village Records Updated');


    }

    public function create_districts(Request $request)
    {
        set_time_limit(600); // Set to 600 seconds (10 minutes)

        $request->validate([
            'file' => 'required|mimes:xlsx,xls,csv|max:2048'
        ]);

        

        try {
            // Load the uploaded file
            $file = $request->file('file');
            $spreadsheet = IOFactory::load($file->getPathname());
            $worksheet = $spreadsheet->getActiveSheet();
        } catch (\Exception $e) {
            return back()->withErrors(['file' => 'Error reading Excel file: ' . $e->getMessage()]);
        }

        $rowIndex = 2;
        $districts = [];

        while ($worksheet->getCell('A' . $rowIndex)->getValue() !== null) {

            if($worksheet->getCell('C' . $rowIndex)->getValue() !== 'None' && $worksheet->getCell('C' . $rowIndex)->getValue() !== 'NULL'){

                //check if district name is in array
                if(!in_array(strtolower($worksheet->getCell('C' . $rowIndex)->getValue()), $districts)){
                    $constituency = Constituency::where('name', ucwords($worksheet->getCell('B' . $rowIndex)->getValue()))->first();

                    $district = District::create([
                        'name' => ucwords(strtolower($worksheet->getCell('C' . $rowIndex)->getValue())),
                        'constituency_id'=> $constituency->id ?? null,
                        'country_id' => 115
                    ]);

                    $village = Village::where('name', ucwords(strtolower($worksheet->getCell('G' . $rowIndex)->getValue())))->first();
                    $village->district_id = $district->id;
                    $village->save();

                    $districts[] = strtolower($worksheet->getCell('C' . $rowIndex)->getValue());

                }


            }

            $rowIndex++;
            
        
        }

        return back()->with('success', 'District Records Updated');



    }



}
