<?php
namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Middleware\LoadUserPermissions;
use App\Models\DocUploads;
use App\Models\Faq;
use App\Models\Indicator;
use App\Models\IndicatorQuestion;
use App\Models\IndicatorQuestionGroup;
use App\Models\IndicatorQuestionOption;
use App\Models\ManageIndicatorQuestion;
use App\Models\ManageIndicators;
use App\Models\Office;
use App\Models\OfficeRank;
use App\Models\Status;
use App\Models\ToastNotification;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use App\Models\PageSettings;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;

class PagesContentController extends Controller
{

    public function __construct()
    {

        $this->middleware(['auth:web', LoadUserPermissions::class]);
    }


    public function faq()
    {
        $faq = $this->getPageContent('faq');

        $all_faqs = Faq::all();

        return view('faq.index', compact('faq','all_faqs'));
    }


    public function store_faq(Request $request)
    {
        $faq = $this->savePageContent('faq');
        if($request->has('faq_page_details')){
            if ($faq->update([
                'page_title' => $request->title,
                'page_description' => $request->sub_description,
            ])) {
                return redirect()->back()->with('success', 'Frequently Asked Questions Page Details updated successfully');
            }
        }

        if($request->has('add_new_faq')) {
            $faqs = $request->has('update_faq') ? Faq::find($request->update_faq) : new Faq();
            $faqs->question = $request->question;
            $faqs->answer = $request->page_content;
            $faqs->save();

            return redirect()->back()->with('success', 'Frequently Asked Questions saved successfully');

        }


    }

    public function contact_us()
    {
        $contact_us = $this->getPageContent('contact_us');

        return view('contact_us.index', compact('contact_us'));
    }

    public function store_contact_us(Request $request)
    {
        $contact_us = $this->savePageContent('contact_us');

        $contact_us->update([
            'page_content' => $request->page_content,
            'page_title' => $request->title,
            'page_description' => $request->sub_description,
        ]);

        return redirect()->back()->with('success','Contact Us content saved successfully');

    }

    public function about_us()
    {
        $about_us = $this->getPageContent('about_us');

        return view('about_us.index', compact('about_us'));
    }

    public function store_about_us(Request $request)
    {
        $about_us = $this->savePageContent('about_us');

        $about_us->update([
            'page_content' => $request->page_content,
            'page_title' => $request->title,
            'page_description' => $request->sub_description,
        ]);

        return redirect()->back()->with('success','About Us content saved successfully');
    }

    public function downloads()
    {
        $documents = DocUploads::query()
            ->where('upload_type', 'system_downloads')
            ->paginate(10);

        return view('downloads.index', compact('documents'));
    }

    public function store_downloads(Request $request)
    {
        $request->validate([
            'file' => 'required|mimes:png,jpg,jpeg,docx,pdf,xlx,xlsx,csv|max:2048',
        ]);
        $document_upload = new DocUploads();
        $document_upload->upload_type = 'system_downloads';
        $document_upload->created_by = auth()->user()->id;
        $document_upload->subregister_id = 0;
        $document_upload->description = $request->page_content;
        $document_upload->title = $request->title;
        if ($request->upload_dir != null) {
            $fileName = time() . '.' . $request->upload_dir->extension();
            $request->file->move(storage_path('app/public/documents_uploads/'), $fileName);
            $document_upload->upload_dir = $fileName;
        } else {
            $document_upload->upload_dir = "";
        }

        notify(new ToastNotification('Document Uploaded', 'Documents have been uploaded available for users to download', 'success'));

        return redirect()->back();

    }

    private function getPageContent($pageName)
    {
        $page_content = PageSettings::query()
            ->where('page_name', $pageName)
            ->first();

        return $page_content;
    }

    private function savePageContent($pageName)
    {
        $page_content = PageSettings::query()
            ->where('page_name', $pageName)
            ->first();

        return $page_content;
    }

    public function editFAQDetails(Faq $faq)
    {
        return response()->json($faq);
    }

    public function deleteFAQDetails(Faq $faq)
    {
        $faq->delete();
        session()->put('success','Frequently Asked Question Deleted successfully');
        return response()->json(["success"=>true,"route"=>route('page.faq')]);
    }

}
