<?php

namespace App\Http\Controllers;

use App\Models\BeneficiariesDetail;
use App\Models\BeneficiariesListView;
use App\Models\Benefit;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Models\Payroll;
use App\Models\PayrollHistory;
use App\Models\UserSaveReport;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Reader\Exception;
use PhpOffice\PhpSpreadsheet\Cell\DataValidation;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xls;

class ReportingController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        ini_set('max_execution_time', 0);
        ini_set('memory_limit', '4000M');
        $this->middleware(['auth', 'verified']);
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $table = "Beneficiaries";
        $alias_table = "Beneficiaries";

        $viewdata = $this->viewdata();
        $conditions = $this->conditions();
//        $fields = DB::connection('mysql')->select("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'beneficiaries_details'");
        $fields = getTableColumns('beneficiaries_list_view');
//        Arr::add($fields,'COLUMN_NAME','benefit_type');
//        dd($fields);
//        $fields = DB::connection('mysql')->select("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'beneficiaries_list_view'");

        return view('reports.index',compact('table','alias_table','fields','viewdata','conditions'));
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function benefitRecords()
    {
        $table = "Benefits";
        $alias_table = "Benefit Records";

        $viewdata = $this->viewdata();
        $conditions = $this->conditions();
        $fields = DB::connection('mysql')->select("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'benefits_list_view'");

        return view('reports.index',compact('table','alias_table','fields','viewdata','conditions'));
    }

    public function getfields($table){

        if($table == 'Beneficiaries') {
            $columns = \DB::connection('mysql')
                ->getSchemaBuilder()
                ->getColumnListing($table);

        }



        if($table == 'Benefits') {
            $columns = \DB::connection('mysql')
                ->getSchemaBuilder()
                ->getColumnListing($table);


            $value = collect($columns)->map(function ($column){
                return $column;
            });



            foreach ($columns as $columname) {
                $value = ltrim(preg_replace('/(?<!\ )[A-Z]/', ' $0', $columname));
                $dataset[$table] = [
                    $columname => $value,
                ];
            }

        }

    }

    public function viewdata(){

        $viewdata['view'] = [
            'raw' => 'Raw Data',
            'count' => 'Count of rows',
            'sum' => 'Sum of ...',
            'average' => 'Average of ...',
            'min' => 'Minimum of ...',
            'max' => 'Maximum of ...',
            'distinct' => 'Distinct',

        ];

        return $viewdata;
    }

    public function conditions(){

        $filterdata = [
            '<' => 'Less than',
            '>' => 'Greater than',
            '>=' => 'Greater than or equal to',
            '<=' => 'Less than or equal to',
            '=' => 'Is',
            '!=' => 'Is not',
            'in' => 'Contains',
            'not_in' => 'Does not contain',
            'empty' => 'Is empty',
            '!empty' => 'Not empty',
            'before%' => 'Starts with',
            '%after' => 'Ends with',
            '%like' => 'Like',
            'date' => 'Between',

        ];

        return $filterdata;
    }

    public function results(Request $request){

        $allfilters = array();

        $table = $request->input('data');
        $reportfields = $request->input('reportfields');

        $view = $request->input('view');
        $groupedby = $request->input('groupedby');

        $report_data = json_encode($request->except('_token'));

        $viewdisplay = null;
        $viewdisplayname = '';
        $data = array();


        if($table == 'Beneficiaries'){
            $alias_table = 'Beneficiaries';
            $query = (new BeneficiariesListView())->newQuery();

//            $fields = DB::connection('mysql')->select("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'beneficiaries_list_view'");
            $fields = getTableColumns('beneficiaries_list_view');
        }

        if($table == 'Benefits') {

            $alias_table = 'Benefits';
            $query = (new BeneficiariesListView())->newQuery();

            $fields = DB::connection('mysql')->select("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'benefits_list_view'");
        }

        if(!empty($request->filter)){
            foreach ($request->filter as $key => $filter) {
                if($filter == null){
                    continue;
                }

                if($request->condition[$key] == 'in'){
                    $values = explode(",", $request->value[$key]);
                    try{
                        $query->whereIn($filter, $values);
                    }catch (\Throwable $e){
                        alertNotify('(Where In) not applicable on the collective fields selected...', 'Not Applicable', 'error');
                    }

//                        $query->whereIn($filter,$request->value[$key]);
                }
                elseif($request->condition[$key] == 'not_in'){
                    $values = explode(",", $request->value[$key]);
                    try{
                        $query->whereNotIn($filter, $values);
                    }catch (\Throwable $e){
                        alertNotify('(Where Not In) not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }
                elseif($request->condition[$key] == 'empty'){
                    try{
                        $query->whereNull($filter);
                    }catch (\Throwable $e){
                        alertNotify('(Where Null) not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }
                elseif($request->condition[$key] == '!empty'){
                    try{
                        $query->whereNotNull($filter);
                    }catch (\Throwable $e){
                        alertNotify('(Where Not Null) not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }
                elseif($request->condition[$key] == 'before%'){
                    try{
                        $query->where($filter,'LIKE',$request->value[$key]."%");
                    }catch (\Throwable $e){
                        alertNotify('(Starts With) not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }
                elseif($request->condition[$key] == '%after'){
                    try{
                        $query->where($filter,'LIKE',"%".$request->value[$key]);
                    }catch (\Throwable $e){
                        alertNotify('(Ends With) not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }
                elseif($request->condition[$key] == '%like'){
                    try{
                        $query->where($filter,'LIKE',"%".$request->value[$key]."%");
                    }catch (\Throwable $e){
                        alertNotify('(Like) not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }
                elseif($request->condition[$key] == 'date'){
                    $values = explode(",", $request->value[$key]);
                    try{
                        $query->whereBetween($filter,$values);
                    }catch (\Throwable $e){
                        alertNotify('(Between) not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }
                else{
                    try{
                        $value = $request->value[$key];
                        if ($request->value[$key] == "true") {
                            $value = 1;
                        }
                        elseif($request->value[$key] == "false") {
                            $value = 0;
                        }
                        $query->where($filter,$request->condition[$key],$value);

                    }catch (\Throwable $e){

                        alertNotify('Condition Selected not applicable on the collective fields selected...','Not Applicable',  'error');
                    }
                }


                $allfilters[] = [
                    'filter' => $filter,
                    'condition' => $request->condition[$key],
                    'value' => $request->value[$key],
                ];
            }

        }

        if(!empty($reportfields)){
            if(!empty($groupedby) && $view == 'distinct'){
                $request->reportfields = (array)$groupedby;
                $reportfields = $request->reportfields;
            }
            $query->select($reportfields);
        }

        if(!empty($groupedby) && $view != 'distinct'){
            $query->groupBy($groupedby);
        }

        if(!empty($view)){

            if($view == 'count'){
                $viewdisplay = $query->count();
                $viewdisplayname = "Total Count";
            }

            if($view == 'sum'){
                foreach ($reportfields as $key => $fieldname) {
                    try{
                        $viewdisplay = collect($query->get())->sum($fieldname);
                    }catch (\Throwable $e){
                        $reportset = array();
                        if($key==0){
                            alertNotify('Total Sum not applicable on the collective fields selected...','Not Applicable',  'error');
                        }
                    }
                }

                $viewdisplayname = "Total Sum";
            }

            if($view == 'average'){
                foreach ($reportfields as $key => $fieldname) {
                    try{
                        $viewdisplay = collect($query->get())->average($fieldname);
                    }catch (\Throwable $e){
                        $reportset = array();
                        if($key==0){
                            alertNotify('Average Number not applicable on the collective fields selected...','Not Applicable',  'error');
                        }
                    }
                }
                $viewdisplayname = "Average Number";
            }

            if($view == 'min'){
                foreach ($reportfields as $key => $fieldname){
                    try{
                        $viewdisplay = collect($query->get())->min($fieldname);
                    }catch (\Throwable $e){
                        $reportset = array();
                        if($key==0){
                            alertNotify('Minimum Number not applicable on the collective fields selected...','Not Applicable',  'error');
                        }
                    }

                }
                $viewdisplayname = "Minimum Number";
            }

            if($view == 'max'){
                foreach ($reportfields as $key => $fieldname){
                    try{
                        $viewdisplay = collect($query->get())->max($fieldname);
                    }catch (\Throwable $e){
                        $reportset = array();
                        if($key==0){
                            alertNotify('Maximum Number not applicable on the collective fields selected...','Not Applicable',  'error');
                        }
                    }
                }
                $viewdisplayname = "Maximum Number";
            }

            if($view == 'distinct'){

                foreach ($reportfields as $key => $fieldname) {
                    try{
                        $viewdisplay = collect($query->get())->groupBy($fieldname)->map(function ($sub_query) {
                            return $sub_query->count();
                        });

                    }catch (\Throwable $e){
                        $reportset = array();
                        if($key==0){
                            alertNotify('Distinct Count not applicable on the collective fields selected...','Not Applicable',  'error');
                        }
                    }
                }
//                $query->groupBy($groupedby);
                $viewdisplayname = "Distinct Count";
            }
        }


        if ($request->query->get( 'export' ) ) {

            $reportset = $query->get();
            if($view == 'distinct'){
                $reportfields[] = 'Total Count';
            }
//            dd($viewdisplay);

            $spreadsheet = new Spreadsheet();

            // Set document properties
            $spreadsheet->getProperties()->setCreator(env('APP_NAME'))
                ->setLastModifiedBy(env('APP_NAME')." - Report")
                ->setTitle("CSV Report Sheet")
                ->setSubject("Report Sheet")
                ->setDescription(env('APP_NAME')."Generated Report from Portal")
                ->setKeywords(env('APP_NAME')." EXCEL")
                ->setCategory(env('APP_NAME')." REPORT");

            $styleArray3 = array(
                'font'  => array(
                    'bold'  => true,
                    'size'  => 12,
                ));

            $center_style = array(
                'alignment' => array(
                    'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                )
            );

            try {

                $rows_letters = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z'];
                $sheet = $spreadsheet->getActiveSheet();
                $data_start_row = 7;$last_count = 0;

                $spreadsheet->getActiveSheet()->mergeCells("A1:H3");
                $spreadsheet->getActiveSheet()->getStyle("A1:H3")->applyFromArray($center_style);
                $spreadsheet->getActiveSheet()->getRowDimension('1')->getRowHeight(100);
//                $spreadsheet->getActiveSheet()->getColumnDimension('A')->getWidth(90);

                //Inserting Logo
                $drawing = new \PhpOffice\PhpSpreadsheet\Worksheet\Drawing();
                $drawing->setName('Logo');
                $drawing->setDescription('Logo');
                $drawing->setPath(public_path('assets/media/custom/Coat_of_arms_of_the_Cook_Islands-01.png'));
                $drawing->setResizeProportional(false);
                $drawing->setWidthAndHeight(40,60);
                $drawing->setCoordinates('A1');
                $drawing->setOffsetX(100);
                $drawing->setOffsetY(0);
                $drawing->setWorksheet($spreadsheet->getActiveSheet());
                //End

                $spreadsheet->getActiveSheet()->mergeCells("A4:H4");
                $spreadsheet->getActiveSheet()->mergeCells("A5:H5");
                $spreadsheet->getActiveSheet()->getCell("A5")->setValue("Ministry of Internal Affairs");
                $spreadsheet->getActiveSheet()->getStyle("A5")->applyFromArray($center_style);
                $spreadsheet->getActiveSheet()->mergeCells("A6:H6");
                $spreadsheet->getActiveSheet()->getCell("A6")->setValue($request->report_title);
                $spreadsheet->getActiveSheet()->getStyle("A6")->applyFromArray($center_style);

                foreach($reportfields as $key => $data){
                    $name = strtoupper(ucwords(str_replace("_"," ",$data)));
                    $spreadsheet->setActiveSheetIndex(0)->setCellValue($rows_letters[$key]."$data_start_row", $name);
                    $last_count = $key+$data_start_row;
                }

                foreach(range("A$data_start_row",$rows_letters[$key]."$data_start_row") as $columnID) {
                    $spreadsheet->getActiveSheet()->getColumnDimension($columnID)
                        ->setAutoSize(true);
                }
//
                $spreadsheet->getActiveSheet()->getStyle("A1:".$rows_letters[$last_count]."$data_start_row")->applyFromArray($styleArray3);

                //use array data
                $count = $data_start_row+1;

                if($view == 'distinct'){
                    foreach($viewdisplay as $item => $value){
                        $spreadsheet->getActiveSheet()->SetCellValue($rows_letters[0].$count, ($item == "")?'-':$item);
                        $spreadsheet->getActiveSheet()->SetCellValue($rows_letters[1].$count, ($value == "")?'0':$value);
                        $count++;
                    }
                }
                else {
                    foreach($reportset as $key => $value){
                        foreach($reportfields as $mainkey => $data){
                            $spreadsheet->getActiveSheet()->SetCellValue($rows_letters[$mainkey].$count, ($value->$data == "")?'-':$value->$data);
                        }
                        $count++;
                    }
                }

            } catch (\PhpOffice\PhpSpreadsheet\Exception $e) {

            }

            $Excel_writer = new Xls($spreadsheet);
            $title = env('APP_NAME')." Report Sheet";
            header('Content-Type: application/vnd.ms-excel');
//                    header('Content-Disposition: attachment;filename="'.$title.'");
            header( 'Content-Disposition: attachment;filename="' . $title . ".xls" . '"' );
            header('Cache-Control: max-age=0');

            $Excel_writer->save('php://output');
            return;

        }

        $querystring = vsprintf(str_replace(['?'], ['\'%s\''], $query->toSql()), $query->getBindings());

        try{

            $reportset = $query->paginate(10);

        }catch (\Throwable $e){
            $reportset = array();
            alertNotify('The collective fields selected produced no dataset... Please check and try again',  'Search results not found...','warning');

        }

        $viewdata = $this->viewdata();
        $conditions = $this->conditions();

//        dd($request->all());

        return view('reports.index',compact('data', 'conditions','viewdisplay','viewdisplayname','querystring','request','table','alias_table','fields','viewdata','reportset','reportfields','allfilters','report_data'));

    }

    public function savereport(Request $request){

//        dd($request->input('groupedby'));

        $savereport = new UserSaveReport();
        $savereport->user_id = auth()->user()->id;
        $savereport->title = $request->input('reporttitle');
        $savereport->tablename = $request->input('tablename');
        $savereport->querystring = $request->input('reportquery');
        $savereport->report_data = $request->input('report_data');
        $savereport->report_title = $request->input('report_title');
        $savereport->displayfields = json_encode($request->input('displayfields'));
        if($request->has('data')){
            $savereport->data = $request->input('data');
        }
        if($request->has('chart_type')){
            $savereport->chart_type = $request->input('chart_type');
            $savereport->groupedby = $request->input('groupedby');
        }


        if($savereport->save()){
            echo 'yes';
        }else{
            echo 'no';
        }

    }

    public function allsaved(){
        $allsaved = UserSaveReport::where('user_id',auth()->user()->id)
            ->where('chart_type',null)
            ->paginate(10);

        return view('reports.managereport',compact('allsaved'));
    }

    public function showreport($id)
    {
        $showreport = UserSaveReport::where('user_id', auth()->user()->id)
            ->where('id', $id)
            ->orWhere('id','=','0')
            ->first();
        $reportdata = json_decode($showreport->report_data,true);


        return redirect()->route('generate.report.results',$reportdata);
    }

    public function addfilter(Request $request){
        $type = $request->input('type');
        $filtercount = rand ( 10000 , 99999 );
        $conditions = $this->conditions();

        if($type == 'Beneficiaries'){
//            $fields = DB::connection('mysql')->select("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME ='beneficiaries_list_view'");
            $fields = getTableColumns('beneficiaries_list_view');
        }elseif($type == 'Benefits'){
            $fields = DB::connection('mysql')->select("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME ='benefits_list_view'");
        }

        return view('reports.selectfilter',compact('fields','filtercount','conditions'));
    }

    private function exportToExcel($printdata){
        ini_set('max_execution_time', 0);
        ini_set('memory_limit', '4000M');

        try {
            $spreadSheet = new Spreadsheet();
            $spreadSheet->getActiveSheet()->getDefaultColumnDimension()->setWidth(20);
            $spreadSheet->getActiveSheet()->fromArray($printdata);

            $Excel_writer = new Xls($spreadSheet);
            header('Content-Type: application/vnd.ms-excel');
            header('Content-Disposition: attachment;filename="CI-SPMIS Report Sheet.xls"');
            header('Cache-Control: max-age=0');
            $Excel_writer->save('php://output');
            return;

        } catch (Exception $e) {
            return;
        }


    }

    public function mainBenefit(Request $request)
    {
        $islands = LookupCode::activeOptions(config('settings.island_code'));
        $benefits = Benefit::where('archived','=', 0)->pluck('titles','id')->toArray();

        $benefit_recipients = array();
        $grand_total = array();
        $island_names = array();
        $benefit_names = array();
        $export_titles = __('MAIN WELFARE BENEFIT SUMMARY');
        $header_title = __('');
        if($request->submit == 'submit') {

            $this->validate($request,[
                'islands'       =>  'required',
                'benefits'      =>  'required',
                'type'          =>  'required',
                'date_range'    =>  'required',
            ]);

            $from = date("Y-m-d",strtotime(now()));
            $to = date("Y-m-d",strtotime(now()));

            $getIslands = $request->islands;

            $island_names = Lookup::whereIn('id',$getIslands)->pluck('fullName','id')->toArray();
            $benefit_names = Benefit::whereIn('id',$request->benefits)->pluck('titles','id')->toArray();

            if(!is_null($request)) {
                //split date range value to from and to
                $split_dates = explode('/',str_replace(' ', '', $request->date_range));
                $from = date($split_dates[0]);
                $to = date($split_dates[1]);
            }

            if($request->type == 'present') {
                $export_titles = __('WELFARE BENEFIT SUMMARY');
                $header_title = date('d F Y',strtotime($from))." - ".date('d F Y',strtotime($to));
                //Filter through the Payroll with the date range and benefit
                $payroll_query = Payroll::whereIn('benefit_id',$request->benefits)
                    ->whereBetween('payment_date',[$from,$to])
                    ->get();

                //Make a total count and total amount for each benefit and Island
                foreach ($getIslands as $island) {

                    //Loop through the payroll result
                    foreach ($payroll_query as $payroll) {

                        $payroll_beneficiaries = $payroll->beneficiaries_id;
                        $benefit = $payroll->benefit_id;

                        //Get each beneficiary from the payroll, get the amount to be paid and find the total recipients
                        // and amount for each benefit under each selected island
                        foreach ($payroll_beneficiaries as $key=>$recipient) {

//                            dd($payroll_beneficiaries);

                            $recipient_details = BeneficiariesDetail::where('id','=',$recipient)
                            ->where('islands','=',$island)->first();

                            //Assign ZERO '0' to totals since this recipient does not belong to the island in question.
                            if (is_null($recipient_details)) {
                                $increase_number_by = 0;
                                $increase_amount_by = 0;
                            }
                            else {
                                $get_beneficiary_account = getBeneficiaryAgeGroupTaxByID($recipient,$benefit);
                                $increase_number_by = 1;
                                $increase_amount_by = $get_beneficiary_account['amount']*((100 - $get_beneficiary_account['tax'])/100);
                            }

                            //Assignment to benefit per island
                            if (isset($benefit_recipients[$island][$benefit]['recipients'])) {
                                $benefit_recipients[$island][$benefit]['recipients'] += $increase_number_by;
                            }
                            else {
                                $benefit_recipients[$island][$benefit]['recipients'] = $increase_number_by;
                            }

                            if (isset($benefit_recipients[$island][$benefit]['amount'])) {
                                $benefit_recipients[$island][$benefit]['amount'] += $increase_amount_by;
                            }
                            else {
                                $benefit_recipients[$island][$benefit]['amount'] = $increase_amount_by;
                            }
                            //End

                            //Assignment to Grand total
                            if (isset($grand_total[$benefit]['recipients'])) {
                                $grand_total[$benefit]['recipients'] += $increase_number_by;
                            }
                            else {
                                $grand_total[$benefit]['recipients'] = $increase_number_by;
                            }

                            if (isset($grand_total[$benefit]['amount'])) {
                                $grand_total[$benefit]['amount'] += $increase_amount_by;
                            }
                            else {
                                $grand_total[$benefit]['amount'] = $increase_amount_by;
                            }
                            //End
                        }

                    }

                }

//                dd($benefit_recipients);
            }
            elseif ($request->type == 'history') {
                $export_titles = __('WELFARE BENEFIT HISTORY SUMMARY');
                $header_title = date('d F Y',strtotime($from))." - ".date('d F Y',strtotime($to));

                $payroll_query = Payroll::whereIn('benefit_id',$request->benefits)
                    ->where('cleared','=',1)
                    ->whereBetween('payment_date',[$from,$to])
                    ->get();


                //Loop through the islands
                foreach ($getIslands as $island) {

                    foreach ($payroll_query as $payroll) {

                        $payroll_beneficiaries = $payroll->history->beneficiaries_data;
//                        dd($payroll_beneficiaries);
                        $benefit = $payroll->benefit_id;
//
                        //Get each beneficiary from the payroll, get the amount to be paid and find the total recipients
                        // and amount for each benefit under each selected island
                        foreach ($payroll_beneficiaries as $recipient=>$amount) {

                            $recipient_details = BeneficiariesDetail::where('id','=',$recipient)
                                ->where('islands','=',$island)->first();

                            //Assign ZERO '0' to totals since this recipient does not belong to the island in question.
                            if (is_null($recipient_details)) {
                                $increase_number_by = 0;
                                $increase_amount_by = 0;
                            }
                            else {
//                                    $get_beneficiary_account = $amount;
                                $increase_number_by = 1;
                                $increase_amount_by = $amount;
                            }

                            //Assignment to benefit per island
                            if (isset($benefit_recipients[$island][$benefit]['recipients'])) {
                                $benefit_recipients[$island][$benefit]['recipients'] += $increase_number_by;
                            }
                            else {
                                $benefit_recipients[$island][$benefit]['recipients'] = $increase_number_by;
                            }

                            if (isset($benefit_recipients[$island][$benefit]['amount'])) {
                                $benefit_recipients[$island][$benefit]['amount'] += $increase_amount_by;
                            }
                            else {
                                $benefit_recipients[$island][$benefit]['amount'] = $increase_amount_by;
                            }
                            //End

                            //Assignment to Grand total
                            if (isset($grand_total[$benefit]['recipients'])) {
                                $grand_total[$benefit]['recipients'] += $increase_number_by;
                            }
                            else {
                                $grand_total[$benefit]['recipients'] = $increase_number_by;
                            }

                            if (isset($grand_total[$benefit]['amount'])) {
                                $grand_total[$benefit]['amount'] += $increase_amount_by;
                            }
                            else {
                                $grand_total[$benefit]['amount'] = $increase_amount_by;
                            }
                        }
                    }
                }
            }
//            elseif ($request->type == 'history') {
//                $export_titles = __('WELFARE BENEFIT HISTORY SUMMARY');
//                $header_title = date('d F Y',strtotime($from))." - ".date('d F Y',strtotime($to));
//
//
//                //Loop through the islands
//                foreach ($getIslands as $island) {
//
//                    foreach ($request->benefits as $benefit) {
////                    if((int)$benefit != 6) continue;
//                        $beneficiaries = DB::table('beneficiaries_details')
//                            ->join('beneficiaries','beneficiaries.id','=','beneficiaries_details.beneficiary_id')
//                            ->join('forms','beneficiaries.form_id','=','forms.id')
//                            ->join('benefits','benefits.form_id','=','forms.id')
//                            ->where('benefits.id','=',(int)$benefit)
//                            ->where('beneficiaries_details.islands','=',"$island")
//                            ->where('beneficiaries.archived','=',0)
//                            ->pluck('beneficiaries.id')->toArray();
//
//                        $benefit_recipients[$island][$benefit]['recipients'] = sizeof($beneficiaries);
//
//                        if (isset($grand_total[$benefit]['recipients'])) {
//                            $grand_total[$benefit]['recipients'] += $benefit_recipients[$island][$benefit]['recipients'];
//                        }
//                        else {
//                            $grand_total[$benefit]['recipients'] = $benefit_recipients[$island][$benefit]['recipients'];
//                        }
//
//                        if (sizeof($beneficiaries) <= 0) {
//                            $benefit_recipients[$island][$benefit]['amount'] = 0;
//                            $grand_total[$benefit]['recipients'] = 0;
//                            continue;
//                        }
//
//                        $beneficiaries_data = PayrollHistory::join('payrolls','payroll_history.payroll_id','=','payrolls.id')
//                            ->where('payrolls.benefit_id','=',(int)$benefit)
//                            ->whereBetween('payrolls.payment_date',[$from,$to])
//                            ->pluck('payroll_history.beneficiaries_data');
//
////                    dd($beneficiaries);
//                        $total_amount = 0;
//                        foreach ($beneficiaries_data as $beneficiary_info) {
//                            $beneficiary_info = collect(json_decode($beneficiary_info));
//
//                            foreach ($beneficiary_info as $beneficiary_id=>$amount) {
////                            dd($amount);
//                                if (isset($beneficiary_info[$beneficiary_id])) {
//
////
//                                    if (isset($benefit_recipients[$island][$benefit]['amount'])) {
//                                        $benefit_recipients[$island][$benefit]['amount'] += $amount;
//                                    }
//                                    else {
//                                        $benefit_recipients[$island][$benefit]['amount'] = $amount;
//                                    }
//
//                                    if (isset($grand_total[$benefit]['amount'])) {
//                                        $grand_total[$benefit]['amount'] += $amount;
//                                    }
//                                    else {
//                                        $grand_total[$benefit]['amount'] = $amount;
//                                    }
//                                }
//                            }
//
//                        }
//
////                    dd($benefit_recipients);
//                    }
//                }
//            }
        }
        else {
            $request = null;
        }
//        dd($grand_total);

        return view('reports.main-benefit', compact('islands','benefits','request','benefit_recipients',
            'grand_total','island_names','benefit_names','export_titles','header_title'));
    }

}

