<?php

namespace App\Http\Controllers;

use App\Models\Applicant;
use App\Models\ApplicationApproval;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Illuminate\View\View;

class ReportsController extends Controller
{

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware(['auth', 'admin-access','check-permissions:administrator,reporting']);
    }

    public function index(Request $request)
    {
        $hasRequest = true;
        $months = config('settings.months_value_short');
        $users_demographic_array_empty = $users_demographic_count_roles_empty = $current_members_by_age_empty = false;

        $current_members_by_gender_members = $benefits_paid_by_gender_genders = $benefits_paid_by_gender_members =
        $current_members_by_age_groups = $current_members_by_age_groups_count = $active_members_by_age_groups = $active_members_by_age_groups_count =
        $members_demographic_count_gender =
        $benefits_demographic_gender = $benefits_paid_names = $benefits_paid_amount = $applications_by_status =
        $applications_by_status_count = $benefits_by_status_graph = $active_members_by_gender_members = null;

        $selected_year = date('Y');
        if (isset($request->year)) {
            $selected_year = $request->year;
        }

        $selected_from = date('m');
        if (isset($request->from)) {
            $selected_from = $request->from;
        }

        $selected_to = date('m');
        if (isset($request->to)) {
            $selected_to = $request->to;
        }

        if (isset($request->generate)) {
            $request->validate([
                'year' => 'required|integer',
                'from' => 'required|integer',
                'to' => 'required|integer',
            ]);

            $hasRequest = true;

            if (!isset($months[$request->from]) || !isset($months[$request->to])) {
                session()->flash('message', 'Invalid month selected');
                session()->flash('status', 'error');
                return back();
            }

            if ($request->from > $request->to) {
                session()->flash('message', 'The From Month must be less or equal to the To Month');
                session()->flash('status', 'error');
                return back();
            }
        }

        $start_date_string = date('Y-m-d H:i:s', strtotime($selected_year.'-'.$selected_from.'-01 00:00:00'));

        /**
         * Users Roles Tab
         */
        $users_demographic_array = DB::table('roles')
            ->join('role_user','role_user.role_id','=','roles.id')
            ->join('users','users.id','=','role_user.user_id')
            ->join('deployment_user','deployment_user.user_id','=','users.id')
            ->selectRaw('roles.display_name, count(users.id) as user_count')
            ->where('deployment_user.deployment_id','=',getActiveInstance())
            ->where('roles.status','=',1)
            ->whereYear('users.created_at','=',$selected_year)
            ->whereMonth('users.created_at','>=',$selected_from)
            ->whereMonth('users.created_at','<=',$selected_to)
            ->whereNotIn('users.id',config('settings.hidden_accounts'))
            ->groupBy('roles.display_name')
            ->orderBy('roles.display_name')
            ->pluck('user_count','roles.display_name')->toArray();

        if (sizeof($users_demographic_array) < 1) $users_demographic_array_empty = true;

        $users_demographic_roles = json_encode(array_keys($users_demographic_array),JSON_HEX_TAG);
        $users_demographic_count = json_encode(array_values($users_demographic_array),JSON_HEX_TAG);

        /*Users Role Tab by Month*/
        $users_demographic_count_roles = [];
        $role_counter = 0;
        foreach ($users_demographic_array as $key=>$value) {
            $users_demographic_count_roles[$role_counter]['label'] = Str::plural($key);
            $users_demographic_count_roles[$role_counter]['borderWidth'] = 1;
            for ($a = $selected_from; $a <= $selected_to; $a++) {
                $users_demographic_count_roles[$role_counter]['data'][] = DB::table('roles')
                    ->join('role_user','role_user.role_id','=','roles.id')
                    ->join('users','users.id','=','role_user.user_id')
                    ->join('deployment_user','deployment_user.user_id','=','users.id')
                    ->where('roles.status','=',1)
                    ->where('deployment_user.deployment_id','=',getActiveInstance())
                    ->where('roles.display_name','=',$key)
                    ->whereYear('users.created_at','=',$selected_year)
                    ->whereMonth('users.created_at','=',$a)
                    ->whereNotIn('users.id',config('settings.hidden_accounts'))
                    ->count();
            }
            $role_counter++;
        }

        if (sizeof($users_demographic_count_roles) < 1)  $users_demographic_count_roles_empty = true;

        $users_demographic_count_roles = json_encode($users_demographic_count_roles,JSON_HEX_TAG);
        /**
         * End of Users Roles Tab
         */

        /**
         * Graph Distribution for Users
         */
        for ($a = $selected_from; $a <= $selected_to; $a++) {
            $users_by_roles[$months[(integer)$a]] = DB::table('roles')
                ->join('role_user','role_user.role_id','=','roles.id')
                ->join('users','users.id','=','role_user.user_id')
                ->join('deployment_user','deployment_user.user_id','=','users.id')
                ->selectRaw('users.id,users.created_at')
                ->where('roles.status','=',1)
                ->where('deployment_user.deployment_id','=',getActiveInstance())
                ->whereYear('users.created_at','=',$selected_year)
                ->whereMonth('users.created_at','=',$a)
                ->whereNotIn('users.id',config('settings.hidden_accounts'))
                ->count();
        }

        $users_on_month = json_encode(array_keys($users_by_roles),JSON_HEX_TAG);
        $users_on_month_value = json_encode(array_values($users_by_roles),JSON_HEX_TAG);

        //Users BF
        for ($a = 1; $a <= 12; $a++) {
            $users_BF[$months[$a]] = DB::table('roles')
                ->join('role_user','role_user.role_id','=','roles.id')
                ->join('users','users.id','=','role_user.user_id')
                ->join('deployment_user','deployment_user.user_id','=','users.id')
                ->selectRaw('users.id,users.created_at')
                ->where('roles.status','=',1)
                ->where('deployment_user.deployment_id','=',getActiveInstance())
                ->where('roles.access_type','=','backend')
                ->whereDate('users.created_at','<',$start_date_string)
                ->whereMonth('users.created_at','=',$a)
                ->whereNotIn('users.id',config('settings.hidden_accounts'))
                ->count();
        }

        $users_on_month_bf = json_encode(array_keys($users_BF),JSON_HEX_TAG);
        $users_on_month_value_bf = json_encode(array_values($users_BF),JSON_HEX_TAG);

        /**
         * Table Distribution for Users
         */
        $users_demographic = DB::table('roles')
            ->join('role_user','role_user.role_id','=','roles.id')
            ->join('users','users.id','=','role_user.user_id')
            ->join('deployment_user','deployment_user.user_id','=','users.id')
            ->selectRaw('roles.display_name, count(users.id) as user_count')
            ->where('roles.status','=',1)
            ->where('deployment_user.deployment_id','=',getActiveInstance())
            ->whereYear('users.created_at','=',$selected_year)
            ->whereMonth('users.created_at','>=',$selected_from)
            ->whereMonth('users.created_at','<=',$selected_to)
            ->whereNotIn('users.id',config('settings.hidden_accounts'))
            ->groupBy('roles.display_name')
            ->orderBy('roles.display_name')
            ->get();


        $approved = DB::table('application_approvals')
            ->selectRaw('count(id) as applications, sum(amount) as amount')
            ->where('instance_id','=',getActiveInstance())
            ->whereYear('application_approvals.created_at','=',$selected_year)
            ->whereMonth('application_approvals.created_at','>=',$selected_from)
            ->whereMonth('application_approvals.created_at','<=',$selected_to)
            ->first();

        $rejected = DB::table('application_declines')
            ->selectRaw('count(id) as applications, sum(amount) as amount')
            ->where('instance_id','=',getActiveInstance())
            ->whereYear('application_declines.created_at','=',$selected_year)
            ->whereMonth('application_declines.created_at','>=',$selected_from)
            ->whereMonth('application_declines.created_at','<=',$selected_to)
            ->first();

        /**
         * Table Distribution for New and BF Members (Applicants)
         */
        $current_members_brought_forward = Applicant::join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
            ->where('applicant_deployment.deployment_id','=',getActiveInstance())->whereDate('applicant_deployment.created_at','<',$start_date_string)
            ->count();

        $new_current_members = DB::table('applicants')
        ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
        ->where('applicant_deployment.deployment_id','=',getActiveInstance())
            ->whereYear('applicant_deployment.created_at','=',$selected_year)
            ->whereMonth('applicant_deployment.created_at','>=',$selected_from)
            ->whereMonth('applicant_deployment.created_at','<=',$selected_to)
            ->count();

        /***************************************************************************************************************
         * Distribution for Current Members
         **************************************************************************************************************/
        $age_groups = config('settings.age_group');

        //For Doughnut and Horizontal Graph - AGE GROUP
        $current_members_by_age = DB::table('applicants')
        ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
        ->where('applicant_deployment.deployment_id','=',getActiveInstance())
            ->selectRaw('dobs,id')
            ->whereYear('applicant_deployment.created_at','=',$selected_year)
            ->whereMonth('applicant_deployment.created_at','>=',$selected_from)
            ->whereMonth('applicant_deployment.created_at','<=',$selected_to)
            ->get();

        if(!$current_members_by_age->isNotEmpty()) $current_members_by_age_empty = true;

        $age_group = [];
        foreach ($current_members_by_age as $key=>$current_applicant) {
            $age = getRawAgeByDate($current_applicant->dobs);
            foreach ($age_groups as $group=>$ages) {
                if (!isset($age_group[$group])) $age_group[$group] = 0;

                if ($age >= $ages[0] && $age <= $ages[1])  {
                    $age_group[$group] += 1;
                    break;
                }
            }
        }

        foreach ($age_group as $group=>$total_members) {
            $current_members_by_age_groups[] = $group;
            $current_members_by_age_groups_count[] = $total_members;
        }
        $current_members_by_age_groups = json_encode($current_members_by_age_groups,JSON_HEX_TAG);
        $current_members_by_age_groups_count = json_encode($current_members_by_age_groups_count,JSON_HEX_TAG);
        //End For Doughnut and Horizontal Graph

        //For stacked Graph
        $members_demographic_count_age_groups = [];
        $age_groups_counter = 0;
        foreach ($age_groups as $group=>$ages) {
            $members_demographic_count_age_groups[$age_groups_counter]['label'] = $group;
            $members_demographic_count_age_groups[$age_groups_counter]['borderWidth'] = 1;
            for ($a = $selected_from; $a <= $selected_to; $a++) {
                $members_demographic_count_age_groups[$age_groups_counter]['data'][] = DB::table('applicants')
                    ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
                    ->where('applicant_deployment.deployment_id','=',getActiveInstance())
                    ->whereYear('applicant_deployment.created_at', '=', $selected_year)
                    ->whereMonth('applicant_deployment.created_at','=',$a)
                    ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) >= ?",[$ages[0]])
                    ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) <= ?",[$ages[1]])
                    ->count('dobs');
            }
            $age_groups_counter++;
        }

        $members_demographic_count_age_groups = json_encode($members_demographic_count_age_groups,JSON_HEX_TAG);

        /**
         * Distribution for Current Members by Gender
         */
        $current_members_by_gender = DB::table('applicants')
            ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
            ->where('applicant_deployment.deployment_id','=',getActiveInstance())
            ->selectRaw('genders,count(id) as members')
            ->whereYear('applicant_deployment.created_at','=',$selected_year)
            ->whereMonth('applicant_deployment.created_at','>=',$selected_from)
            ->whereMonth('applicant_deployment.created_at','<=',$selected_to)
            ->groupBy('genders')
            ->orderBy('genders')
            ->get();

        $current_members_by_gender_genders_list = [];
        foreach ($current_members_by_gender as $key=>$current_members) {
            $current_members_by_gender_genders_list[] = $current_members->genders;
            $current_members_by_gender_members[] = $current_members->members;
        }
        $current_members_by_gender_genders = json_encode($current_members_by_gender_genders_list,JSON_HEX_TAG);
        $current_members_by_gender_members = json_encode($current_members_by_gender_members,JSON_HEX_TAG);

        /**
         * Members Distribution by Gender - Bar Stacked Graph
         */
        foreach ($current_members_by_gender_genders_list as $key=>$gender) {
            $members_demographic_count_gender[$key]['label'] = $gender;
            $members_demographic_count_gender[$key]['borderWidth'] = 1;
            for ($a = $selected_from; $a <= $selected_to; $a++) {
                $members_demographic_count_gender[$key]['data'][] = DB::table('applicants')
                    ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
                    ->where('applicant_deployment.deployment_id','=',getActiveInstance())
                    ->selectRaw('genders,count(id) as members')
                    ->where('genders', '=', $gender)
                    ->whereYear('applicant_deployment.created_at', '=', $selected_year)
                    ->whereMonth('applicant_deployment.created_at', '=', $a)
                    ->count();
            }
        }

        $members_demographic_count_gender = json_encode($members_demographic_count_gender,JSON_HEX_TAG);

        /**
         * Active Members Distribution - List
         */
        $new_members_list = DB::table('applicants')
            ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
            ->where('applicant_deployment.deployment_id','=',getActiveInstance())
            ->selectRaw('applicants.regID,applicants.surnames,applicants.first_names,applicants.dobs,
                applicants.genders,applicant_deployment.created_at as joined_on')
            ->whereYear('applicant_deployment.created_at', '=', $selected_year)
            ->whereMonth('applicant_deployment.created_at', '>=', $selected_from)
            ->whereMonth('applicant_deployment.created_at', '<=', $selected_to)
            ->get();
        /***************************************************************************************************************
         * End Distribution for Current Members
         **************************************************************************************************************/

        /***************************************************************************************************************
         * Distribution for Active Members
         **************************************************************************************************************/

        //For Doughnut and Horizontal Graph Active Members - Age Group
        $active_members_by_age = DB::table('applicants')
        ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
        ->where('applicant_deployment.deployment_id','=',getActiveInstance())
            ->selectRaw('dobs,id')
            ->whereYear('applicant_deployment.created_at','<=',$selected_year)
            ->whereMonth('applicant_deployment.created_at','<=',$selected_to)
            ->get();

        if(!$active_members_by_age->isNotEmpty()) $active_members_by_age_empty = true;

        $active_member_age_group = [];
        foreach ($active_members_by_age as $key=>$current_applicant) {
            $age = getRawAgeByDate($current_applicant->dobs);
            foreach ($age_groups as $group=>$ages) {
                if (!isset($active_member_age_group[$group])) $active_member_age_group[$group] = 0;

                if ($age >= $ages[0] && $age <= $ages[1])  {
                    $active_member_age_group[$group] += 1;
                    break;
                }
            }
        }

        foreach ($active_member_age_group as $group=>$total_members) {
            $active_members_by_age_groups[] = $group;
            $active_members_by_age_groups_count[] = $total_members;
        }
        $active_members_by_age_groups = json_encode($active_members_by_age_groups,JSON_HEX_TAG);
        $active_members_by_age_groups_count = json_encode($active_members_by_age_groups_count,JSON_HEX_TAG);
        //End For Doughnut and Horizontal Graph Active Members

        //For stacked Graph - Active Members
        $active_members_demographic_count_age_groups = [];
        $age_groups_counter = 0;
        $active_members_period = [];
        for ($a = 1; $a <= $selected_to; $a++) {
            $active_members_period[] = $months[$a];
        }
        foreach ($age_groups as $group=>$ages) {
            $active_members_demographic_count_age_groups[$age_groups_counter]['label'] = $group;
            $active_members_demographic_count_age_groups[$age_groups_counter]['borderWidth'] = 1;
            for ($a = 1; $a <= $selected_to; $a++) {
                $active_members_demographic_count_age_groups[$age_groups_counter]['data'][] = DB::table('applicants')
                ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
                ->where('applicant_deployment.deployment_id','=',getActiveInstance())
                ->whereYear('applicant_deployment.created_at', '<=', $selected_year)
                ->whereMonth('applicant_deployment.created_at','=',$a)
                ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) >= ?",[$ages[0]])
                ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) <= ?",[$ages[1]])
                ->count('dobs');
            }
            $age_groups_counter++;
        }

        $active_members_period = json_encode(array_values($active_members_period),JSON_HEX_TAG);
        $active_members_demographic_count_age_groups = json_encode($active_members_demographic_count_age_groups,JSON_HEX_TAG);

        /**
         * Distribution for Active Members by Gender
         */
        $active_members_by_gender = DB::table('applicants')
            ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
            ->where('applicant_deployment.deployment_id','=',getActiveInstance())
            ->selectRaw('genders,count(id) as members')
            ->whereYear('applicant_deployment.created_at','<=',$selected_year)
//            ->whereMonth('applicant_deployment.created_at','>=',$selected_from)
            ->whereMonth('applicant_deployment.created_at','<=',$selected_to)
            ->groupBy('genders')
            ->orderBy('genders')
            ->get();

        $active_members_by_gender_genders_list = [];
        foreach ($active_members_by_gender as $key=>$current_members) {
            $active_members_by_gender_genders_list[] = $current_members->genders;
            $active_members_by_gender_members[] = $current_members->members;
        }
        $active_members_by_gender_genders = json_encode($active_members_by_gender_genders_list,JSON_HEX_TAG);
        $active_members_by_gender_members = json_encode($active_members_by_gender_members,JSON_HEX_TAG);

        /**
         * Members Distribution by Gender - Bar Stacked Graph
         */
        $active_members_demographic_count_gender = [];
        foreach ($active_members_by_gender_genders_list as $key=>$gender) {
            $active_members_demographic_count_gender[$key]['label'] = $gender;
            $active_members_demographic_count_gender[$key]['borderWidth'] = 1;
            for ($a = 1; $a <= $selected_to; $a++) {
                $active_members_demographic_count_gender[$key]['data'][] = DB::table('applicants')
                    ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
                    ->where('applicant_deployment.deployment_id','=',getActiveInstance())
                    ->selectRaw('genders,count(id) as members')
                    ->where('genders', '=', $gender)
                    ->whereYear('applicant_deployment.created_at', '<=', $selected_year)
                    ->whereMonth('applicant_deployment.created_at', '=', $a)
                    ->count();
            }
        }

        $active_members_demographic_count_gender = json_encode($active_members_demographic_count_gender,JSON_HEX_TAG);

        /**
         * Active Members Distribution - List
         */
        $affected_members_list = DB::table('applicants')
            ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
            ->where('applicant_deployment.deployment_id','=',getActiveInstance())
            ->selectRaw('applicants.regID,applicants.surnames,applicants.first_names,applicants.dobs,
                applicants.genders,applicant_deployment.created_at as joined_on')
            ->whereYear('applicant_deployment.created_at', '<=', $selected_year)
            ->whereMonth('applicant_deployment.created_at', '<=', $selected_to)
            ->get();

        /***************************************************************************************************************
         * End Distribution for Active Members
         **************************************************************************************************************/


        /**
         * Beneficiaries Distribution by Age Group - Doughnut Graph
         */
        $applicant_list = DB::table('application_approvals')
            ->join('applications','applications.id','=','application_approvals.application_id')
            ->join('applicants','applicants.regID','=','applications.applicant_number')
            ->join('benefits','benefits.id','=','applications.benefit_id')
            ->whereYear('applications.created_at','=',$selected_year)
            ->distinct()
            ->where('applications.instance_id','=',getActiveInstance())
            ->whereMonth('applications.created_at','>=',$selected_from)
            ->whereMonth('applications.created_at','<=',$selected_to)
            ->pluck('applications.applicant_number')->toArray();

        $affected_beneficiaries_list = DB::table('application_approvals')
            ->join('applications','applications.id','=','application_approvals.application_id')
            ->join('applicants','applicants.regID','=','applications.applicant_number')
            ->join('benefits','benefits.id','=','applications.benefit_id')
            ->whereYear('applications.created_at','=',$selected_year)
            ->selectRaw('applicants.genders,applicants.surnames,applicants.first_names,applicants.regID,benefits.titles as benefits,
            application_approvals.amount')
            ->where('applications.instance_id','=',getActiveInstance())
            ->whereMonth('applications.created_at','>=',$selected_from)
            ->whereMonth('applications.created_at','<=',$selected_to)->get();

        $grouped_applicants = [];
        if (!is_null($applicant_list)) {
            foreach ($affected_beneficiaries_list as $application) {
                foreach ($applicant_list as $key=>$applicant_number) {
                    if (!isset($grouped_applicants[$applicant_number]) && $applicant_number == $application->regID) {
                        $grouped_applicants[$applicant_number]['name'] = $application->surnames.' '.$application->first_names;
                        $grouped_applicants[$applicant_number]['gender'] = $application->genders;
                        $grouped_applicants[$applicant_number]['amount'] = $application->amount;
                        $grouped_applicants[$applicant_number]['benefits'] = $application->benefits;
                    }
                    elseif($applicant_number == $application->applicant_number){
                        $grouped_applicants[$applicant_number]['benefits'] .= ', '.$application->benefits;
                        $grouped_applicants[$applicant_number]['amount'] += $application->amount;
                    }
                }
            }
        }
//        dd($grouped_applicants);
//        dd($grouped_applicants);

            $beneficiaries_demographic_age_groups_array = [];
            foreach ($age_groups as $group=>$ages) {
                $beneficiaries_demographic_age_groups_array[$group] = DB::table('applicants')
                    ->join('applications','applicants.regID','=','applications.applicant_number')
                    ->join('application_approvals','applications.id','=','application_approvals.application_id')
                    ->selectRaw("applicants.id,applicants.dobs")
                    ->distinct()
                    ->where('applications.instance_id','=',getActiveInstance())
                    ->whereYear('application_approvals.created_at','=',$selected_year)
                    ->whereMonth('application_approvals.created_at','>=',$selected_from)
                    ->whereMonth('application_approvals.created_at','<=',$selected_to)
                    ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) >= ?",[$ages[0]])
                    ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) <= ?",[$ages[1]])
                    ->count();
            }

            $beneficiaries_demographic_age_groups = json_encode(array_keys($beneficiaries_demographic_age_groups_array),JSON_HEX_TAG);
            $beneficiaries_demographic_age_groups_values = json_encode(array_values($beneficiaries_demographic_age_groups_array),JSON_HEX_TAG);

            /**
             * Beneficiaries Distribution by Age Group - Bar Stack Graph
             */
            $beneficiaries_demographic_count_age_groups = [];
            $b_age_groups_counter = 0;
            foreach ($age_groups as $group=>$ages) {
                $beneficiaries_demographic_count_age_groups[$b_age_groups_counter]['label'] = $group;
                $beneficiaries_demographic_count_age_groups[$b_age_groups_counter]['borderWidth'] = 1;
                for ($a = $selected_from; $a <= $selected_to; $a++) {
                    $beneficiaries_demographic_count_age_groups[$b_age_groups_counter]['data'][] = DB::table('applicants')
                        ->join('applications','applicants.regID','=','applications.applicant_number')
                        ->join('application_approvals','applications.id','=','application_approvals.application_id')
                        ->selectRaw("applicants.id,applicants.dobs")
                        ->distinct()
                        ->where('applications.instance_id','=',getActiveInstance())
                        ->whereYear('application_approvals.created_at', '=', $selected_year)
                        ->whereMonth('application_approvals.created_at','=',$a)
                        ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) >= ?",[$ages[0]])
                        ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) <= ?",[$ages[1]])
                        ->count();
                }
                $b_age_groups_counter++;
            }
            $beneficiaries_demographic_count_age_groups = json_encode($beneficiaries_demographic_count_age_groups,JSON_HEX_TAG);

            /**
             * Beneficiaries Distribution by Gender - Doughnut Graph
             */
            $beneficiaries_demographic_gender_list = DB::table('application_approvals')
                ->join('applications','applications.id','=','application_approvals.application_id')
                ->join('applicants','applicants.regID','=','applications.applicant_number')
                ->selectRaw('applicants.genders,count(distinct(applicants.id)) as beneficiaries')
                ->whereYear('application_approvals.created_at','=',$selected_year)
                ->whereMonth('application_approvals.created_at','>=',$selected_from)
                ->whereMonth('application_approvals.created_at','<=',$selected_to)
                ->where('applications.instance_id','=',getActiveInstance())
                ->groupBy('applicants.genders')
                ->orderBy('applicants.genders')
                ->get();

            $beneficiaries_demographic_gender_array = $beneficiaries_demographic_gender_beneficiaries = [];
            foreach ($beneficiaries_demographic_gender_list as $key=>$current_members) {
                $beneficiaries_demographic_gender_array[] = $current_members->genders;
                $beneficiaries_demographic_gender_beneficiaries[] = $current_members->beneficiaries;
            }
            $beneficiaries_demographic_gender_array = json_encode($beneficiaries_demographic_gender_array,JSON_HEX_TAG);
            $beneficiaries_demographic_gender_beneficiaries = json_encode($beneficiaries_demographic_gender_beneficiaries,JSON_HEX_TAG);

            /**
             * Beneficiaries Distribution by Gender - Bar Stacked Graph
             */
        $beneficiaries_demographic_count_gender = null;
            foreach ($current_members_by_gender_genders_list as $key=>$gender) {
                $beneficiaries_demographic_count_gender[$key]['label'] = $gender;
                $beneficiaries_demographic_count_gender[$key]['borderWidth'] = 1;
                for ($a = $selected_from; $a <= $selected_to; $a++) {
                    $beneficiaries_demographic_count_gender[$key]['data'][] = DB::table('applicants')
                        ->join('applications','applicants.regID','=','applications.applicant_number')
                        ->join('application_approvals','applications.id','=','application_approvals.application_id')
                        ->select('applicants.id')->distinct()
                        ->where('applications.instance_id','=',getActiveInstance())
                        ->whereYear('application_approvals.created_at','=',$selected_year)
                        ->whereMonth('application_approvals.created_at', '=', $a)
                        ->where('applicants.genders', '=', $gender)
                        ->get()->count();
                }
            }

            $beneficiaries_demographic_count_gender = json_encode($beneficiaries_demographic_count_gender,JSON_HEX_TAG);


            /**
             * Expenditure Distribution by Benefit - Doughnut
             */
            $benefits_paid = DB::table('application_approvals')
                ->join('applications','applications.id','=','application_approvals.application_id')
                ->join('benefits','benefits.id','=','applications.benefit_id')
                ->selectRaw('benefits.titles,sum(application_approvals.amount) as amount')
                ->where('applications.instance_id','=',getActiveInstance())
                ->whereYear('application_approvals.created_at','=',$selected_year)
                ->whereMonth('application_approvals.created_at','>=',$selected_from)
                ->whereMonth('application_approvals.created_at','<=',$selected_to)
                ->groupBy('benefits.titles')
                ->orderBy('benefits.titles')
                ->get();

            $benefits_names = $benefits_amount = [];
            foreach ($benefits_paid as $key=>$benefit_paid) {
                $benefits_names[] = $benefit_paid->titles;
                $benefits_amount[] = $benefit_paid->amount;
            }
            if (!is_null($benefits_amount))$benefits_paid_amount = json_encode(array_values($benefits_amount),JSON_HEX_TAG);
            if (!is_null($benefits_names)) $benefits_paid_names = json_encode(array_values($benefits_names),JSON_HEX_TAG);


            /**
             * Expenditure Distribution by Benefit - Bar Stack
             */
            $benefits_demographic_amount = [];
            foreach ($benefits_names as $key=>$benefit) {
                $benefits_demographic_amount[$key]['label'] = $benefit;
                $benefits_demographic_amount[$key]['borderWidth'] = 1;
                for ($a = $selected_from; $a <= $selected_to; $a++) {
                    $benefits_demographic_amount[$key]['data'][] = DB::table('application_approvals')
                        ->join('applications','applications.id','=','application_approvals.application_id')
                        ->join('benefits','benefits.id','=','applications.benefit_id')
                        ->selectRaw("application_approvals.amount as amount")
                        ->where('applications.instance_id','=',getActiveInstance())
                        ->where('benefits.titles', '=', $benefit)
                        ->whereYear('application_approvals.created_at', '=', $selected_year)
                        ->whereMonth('application_approvals.created_at','=',$a)
                        ->sum('amount');
                }
            }

            $benefits_demographic_amount = json_encode($benefits_demographic_amount,JSON_HEX_TAG);

            /**
             * Expenditure Distribution by Age Group - Doughnut
             */
            $benefits_paid_by_age_group = DB::table('application_approvals')
                ->join('applications','applications.id','=','application_approvals.application_id')
                ->join('applicants','applicants.regID','=','applications.applicant_number')
                ->selectRaw('applicants.dobs,application_approvals.amount')
                ->where('applications.instance_id','=',getActiveInstance())
                ->whereYear('application_approvals.created_at','=',$selected_year)
                ->whereMonth('application_approvals.created_at','>=',$selected_from)
                ->whereMonth('application_approvals.created_at','<=',$selected_to)
                ->get();

            $age_group = [];
            foreach ($benefits_paid_by_age_group as $key=>$current_applicant) {
                $age = getRawAgeByDate($current_applicant->dobs);
                foreach ($age_groups as $group=>$ages) {
                    if (!isset($age_group[$group])) $age_group[$group] = 0;

                    if ($age >= $ages[0] && $age <= $ages[1])  {
                        $age_group[$group] += $current_applicant->amount;
                        break;
                    }
                }
            }
            $benefits_paid_by_age = json_encode(array_keys($age_group),JSON_HEX_TAG);
            $benefits_paid_by_age_amount = json_encode(array_values($age_group),JSON_HEX_TAG);

            /**
             * Expenditure Distribution by Age Group - Bar Stack
             */
            $benefits_demographic_age_groups = [];
            $b_age_groups_counter = 0;
            foreach ($age_groups as $group=>$ages) {
                $benefits_demographic_age_groups[$b_age_groups_counter]['label'] = $group;
                $benefits_demographic_age_groups[$b_age_groups_counter]['borderWidth'] = 1;
                for ($a = $selected_from; $a <= $selected_to; $a++) {
                    $benefits_demographic_age_groups[$b_age_groups_counter]['data'][] = DB::table('applicants')
                        ->join('applications','applicants.regID','=','applications.applicant_number')
                        ->join('application_approvals','applications.id','=','application_approvals.application_id')
                        ->selectRaw("applicants.dobs, application_approvals.amount as amount")
                        ->where('applications.instance_id','=',getActiveInstance())
                        ->whereYear('application_approvals.created_at', '=', $selected_year)
                        ->whereMonth('application_approvals.created_at','=',$a)
                        ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) >= ?",[$ages[0]])
                        ->havingRaw("DATE_FORMAT(CURDATE(), '%Y') - DATE_FORMAT(dobs, '%Y') - (DATE_FORMAT(CURDATE(), '00-%m-%d') < DATE_FORMAT(dobs, '00-%m-%d')) <= ?",[$ages[1]])
                        ->sum('amount');
                }
                $b_age_groups_counter++;
            }

            $benefits_demographic_age_groups = json_encode($benefits_demographic_age_groups,JSON_HEX_TAG);


            /**
             * Benefits by Status - Doughnut
             */

//          $benefits_paid_list
            $applications_list = DB::table('applications')
                ->join('applicants','applicants.regID','=','applications.applicant_number')
                ->join('benefits','benefits.id','=','applications.benefit_id')
                ->selectRaw('benefits.titles, benefits.financial, applications.status, applications.id, applications.created_at,
                applications.applicant_number,applicants.surnames,applicants.first_names')
                ->whereYear('applications.created_at','=',$selected_year)
                ->where('applications.instance_id','=',getActiveInstance())
                ->whereMonth('applications.created_at','>=',$selected_from)
                ->whereMonth('applications.created_at','<=',$selected_to)
                ->groupBy('applications.applicant_number','applicants.surnames','applicants.first_names',
                    'benefits.titles','applications.status','applications.id')
                ->orderBy('applications.id')
                ->get();
//            dd($grouped_applicants);

            $recorded_statuses = $recorded_benefits = [];
            foreach ($applications_list as $list) {
                if (!in_array($list->status,$recorded_statuses)) $recorded_statuses[] = $list->status;
                if (!in_array($list->titles,$recorded_benefits))$recorded_benefits[] = $list->titles;
            }

            /**
             * Benefits by Status - Bar Stack
             */

            $b_by_status_counter = 0;
            foreach ($recorded_benefits as $key=>$recorded_benefit) {
                $benefits_by_status_graph[$b_by_status_counter]['label'] = $recorded_benefit;
                $benefits_by_status_graph[$b_by_status_counter]['borderWidth'] = 1;

                foreach ($recorded_statuses as $key=>$recorded_status) {

                    $counting = 0;
                    foreach ($applications_list as $benefit_list) {
                        if ($benefit_list->status == $recorded_status && $benefit_list->titles == $recorded_benefit) {
                            $counting += 1;
                        }
                    }

                    $benefits_by_status_graph[$b_by_status_counter]['data'][] = $counting;

                }
                $b_by_status_counter++;
            }

            $benefits_by_status_graph = json_encode($benefits_by_status_graph,JSON_HEX_TAG);
            $recorded_statuses = json_encode($recorded_statuses,JSON_HEX_TAG);


            /**
             * Expenditure Distribution by Gender - Doughnut
             */
            $benefits_paid_by_gender = DB::table('application_approvals')
                ->join('applications','applications.id','=','application_approvals.application_id')
                ->join('applicants','applicants.regID','=','applications.applicant_number')
                ->selectRaw('applicants.genders,sum(application_approvals.amount) as amount')
                ->where('applications.instance_id','=',getActiveInstance())
                ->whereYear('application_approvals.created_at','=',$selected_year)
                ->whereMonth('application_approvals.created_at','>=',$selected_from)
                ->whereMonth('application_approvals.created_at','<=',$selected_to)
                ->groupBy('applicants.genders')
                ->orderBy('applicants.genders')
                ->get();

            foreach ($benefits_paid_by_gender as $key=>$current_members) {
                $benefits_paid_by_gender_genders[] = $current_members->genders;
                $benefits_paid_by_gender_members[] = $current_members->amount;
            }
            $benefits_paid_by_gender_genders = json_encode($benefits_paid_by_gender_genders,JSON_HEX_TAG);
            $benefits_paid_by_gender_members = json_encode($benefits_paid_by_gender_members,JSON_HEX_TAG);

            /**
             * Expenditure Distribution by Gender - Bar Stacked Graph
             */
            foreach ($current_members_by_gender_genders_list as $key=>$gender) {
                $benefits_demographic_gender[$key]['label'] = $gender;
                $benefits_demographic_gender[$key]['borderWidth'] = 1;
                for ($a = $selected_from; $a <= $selected_to; $a++) {
                    $benefits_demographic_gender[$key]['data'][] = DB::table('applicants')
                        ->join('applications','applicants.regID','=','applications.applicant_number')
                        ->join('application_approvals','applications.id','=','application_approvals.application_id')
                        ->selectRaw("applicants.genders, application_approvals.amount as amount")
                        ->where('applications.instance_id','=',getActiveInstance())
                        ->whereYear('application_approvals.created_at','=',$selected_year)
                        ->whereMonth('application_approvals.created_at', '=', $a)
                        ->where('applicants.genders', '=', $gender)
                        ->where('applicants.genders', '=', $gender)
                        ->sum('amount');
                }
            }
            $benefits_demographic_gender = json_encode($benefits_demographic_gender,JSON_HEX_TAG);


            /**
             * Benefit Expenditure By Status
             */
            $benefits_demographic_status = DB::table('applications')
                ->selectRaw('count(id) as applications, status')
                ->where('applications.instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',$selected_year)
                ->whereMonth('created_at','>=',$selected_from)
                ->whereMonth('created_at','<=',$selected_to)
                ->groupBy('status')
                ->orderBy('status')
                ->get();

            foreach ($benefits_demographic_status as $key=>$by_status) {
                $applications_by_status[] = $by_status->status;
                $applications_by_status_count[] = $by_status->applications;
            }
            $applications_by_status = json_encode($applications_by_status,JSON_HEX_TAG);
            $applications_by_status_count = json_encode($applications_by_status_count,JSON_HEX_TAG);

            //Other Statistics
            $employees = DB::table('roles')
                ->join('role_user','role_user.role_id','=','roles.id')
                ->join('users','users.id','=','role_user.user_id')
                ->join('deployment_user','deployment_user.user_id','=','users.id')
                ->where('deployment_user.deployment_id','=',getActiveInstance())
                ->where('roles.status','=',1)
                ->where('roles.access_type','=','backend')
                ->whereYear('users.created_at','=',$selected_year)
                ->whereNotIn('users.id',config('settings.hidden_accounts'))
                ->count();

            $members = DB::table('applicants')
                ->join('applicant_deployment','applicant_deployment.applicant_id','=','applicants.id')
                ->where('applicant_deployment.deployment_id','=',getActiveInstance())
                ->where('status','=',1)
                ->whereYear('applicant_deployment.created_at','=',$selected_year)
                ->count();

            $applications = DB::table('applications')
                ->where('applications.instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',$selected_year)
                ->count();

            $expenditure = DB::table('application_approvals')
                ->where('instance_id','=',getActiveInstance())
                ->whereYear('created_at','=',$selected_year)
                ->sum('amount');

            if ($request->generate == 'download') {
                dd('Downloading...');
            }
//        }

        return view('reports.index', compact('hasRequest','months','users_demographic',
            'approved','rejected','current_members_by_gender','current_members_by_gender_members','current_members_by_gender_genders',
        'benefits_paid_by_gender_genders','benefits_paid_by_gender_members','current_members_by_age_groups_count','current_members_by_age_groups',
        'new_current_members','current_members_brought_forward','users_on_month','users_on_month_value','users_on_month_bf','users_on_month_value_bf',
        'benefits_paid_by_age','benefits_paid_by_age_amount','users_demographic_roles','users_demographic_count','users_demographic_count_roles',
        'members_demographic_count_age_groups','members_demographic_count_gender','beneficiaries_demographic_count_age_groups',
        'beneficiaries_demographic_age_groups','beneficiaries_demographic_gender_beneficiaries','beneficiaries_demographic_gender_array',
        'beneficiaries_demographic_count_gender','benefits_demographic_age_groups','benefits_demographic_gender','benefits_paid_names',
        'benefits_paid_amount','benefits_demographic_amount','employees','members','expenditure','applications','affected_beneficiaries_list',
        'applications_list','grouped_applicants','applications_by_status','applications_by_status_count','benefits_demographic_status','benefits_by_status_graph',
        'recorded_statuses','selected_to','selected_from','selected_year','users_demographic_array_empty','users_demographic_count_roles_empty',
        'current_members_by_age_empty','beneficiaries_demographic_age_groups_values','active_members_by_age_groups','active_members_by_age_groups_count',
        'active_members_demographic_count_age_groups','active_members_period','active_members_by_gender_genders','active_members_by_gender_members',
        'active_members_demographic_count_gender','affected_members_list','new_members_list'));
    }

    /**
     * @param Request $request
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function result(Request $request)
    {
        return view('reports.result');
    }
}
