<?php

namespace App\Http\Controllers;

//use App\Classes\ToastNotification;
use App\Models\EmailTemplate;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Models\RequestStatus;
use App\Models\Role;
use App\Models\StageEmail;
use Illuminate\Http\Request;

class StageEmailsController extends Controller
{
    //
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index()
    {
        $email_templates = EmailTemplate::orderBy('title','asc')->get();
        $stages = LookupCode::activeApprovalProcessesOptions();
        $roles = Role::orderBy('name','asc')->get();
        $stage_emails = StageEmail::orderBy('lookup_id','asc')->orderBy('order','asc')->get();

        return view('stage-emails.index', compact('email_templates','stages','stage_emails','roles'));
    }

    public function edit(Request $request)
    {
        $email_templates = EmailTemplate::orderBy('title','asc')->get();
        $stages = LookupCode::activeApprovalProcessesOptions();
        $roles = Role::orderBy('name','asc')->get();
        $data = StageEmail::find($request->id);

        $theView = view('stage-emails.edit', compact('email_templates','stages','data','roles'))->render();

        return response()->json(['theView'=>$theView]);
    }

    public function save(Request $request)
    {
        try {
            $this->validate($request, [
                'status' => 'required|numeric|min:1',
                'email' => 'required|numeric|min:1',
                'order' => 'required|numeric',
                'roles' => 'required|array'
            ]);

            $getLookup = Lookup::find($request->status);
            $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->pluck('id')->toArray();
            $checkOrder = StageEmail::whereIn('lookup_id', $get_other_options)
                ->where('order','=',$request->order)
                ->where('lookup_id','<>',$request->status)
                ->count();
            if ($checkOrder > 0) {
                alertNotify('The Approval Process Stage with order '.$request->order.', already exists.',null,'danger');
                return back();
            }

            $getData = StageEmail::where('lookup_id','=', $request->status)
                ->first();
            if ($getData){
                alertNotify('The Approval Process Stage has been assigned already!',null,'danger');
                return back();
            }

            $data = new StageEmail();
            $data->email_template_id = $request->email;
            $data->lookup_id = $request->status;
            $data->order = $request->order;
            $data->roles = implode(',',$request->roles);
            $data->active = true;

            if ($data->save()){
                alertNotify('Information has been saved',null,'success');
            }
            else{
                alertNotify('Sorry! Data was not able to save. Please try again.',null,'danger');
            }
        }
        catch (\Exception $exception){
            alertNotify('Sorry! Something went wrong. Please try again.'.$exception->getMessage(),null,'danger');
        }
        return back();
    }

    public function update(Request $request)
    {
        try {
            $this->validate($request, [
                'status' => 'required|numeric|min:1',
                'email' => 'required|numeric|min:1',
                'order' => 'required|numeric',
                'roles' => 'required|array'
            ]);

            //Checking if the updated Stage has an already existing Order Number
            $getLookup = Lookup::find($request->status);
            $get_other_options = Lookup::where('lookup_code_id','=',$getLookup->lookup_code_id)->pluck('id')->toArray();

            $checkOrder = StageEmail::whereIn('lookup_id', $get_other_options)
                ->where('order','=',$request->order)
                ->where('lookup_id','<>',$request->status)
                ->count();
            if ($checkOrder > 0) {
                alertNotify('The Approval Process Stage with order '.$request->order.', already exists.',null,'danger');
                return back();
            }
            //End of Checking Order Number

            //Checking if the Stage already exists
            $getData = StageEmail::where('lookup_id','=', $request->status)
                ->where('id','<>', $request->id)
                ->first();
            if ($getData){
                alertNotify('The Approval Process Stage has been assigned already!',null,'danger');
            }
            else{

                $data = StageEmail::find($request->id);
                $data->email_template_id = $request->email;
                $data->lookup_id = $request->status;
                $data->order = $request->order;
                $data->roles = implode(',',$request->roles);
                $data->active = true;

                if ($data->save()){
                    alertNotify('Information has been updated',null,'success');
                }
                else{
                    alertNotify('Sorry! Data was not able to save. Please try again.',null,'danger');
                }
            }
        }
        catch (\Exception $exception){
            alertNotify('Sorry! Something went wrong. Please try again.',null,'danger');
        }
        return back();
    }

    public function destroy($id)
    {
        if (StageEmail::destroy($id)) {
            alertNotify('Setup has been deleted.',null,'success');
        }
        else {
            alertNotify('Sorry! Something went wrong. Try again.',null,'danger');
        }
        return back();
    }
}
