<?php
namespace App\Http\Traits;

use App\Models\BenefitRegistration;
use App\Models\ClearanceRequest;
use App\Models\EmailTemplate;
use App\Models\Role;
use App\Models\StageEmail;
use App\Models\User;
use App\Models\UserClearanceRequest;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;


trait EmailSender
{
    protected $data = array();

    /**
     * @param $subject_item_id
     * @param array $email_list
     * @param bool $excludeGroup
     * @param null $order
     * @param null $fileName
     * @param null $additional_messages
     * @return bool
     */
    public function send_mail($subject_item_id, $email_list = array(), $excludeGroup = false,
                              $order = null, $fileName = null, $additional_messages = null)
    {
        //GETTING THE SUBJECT STATUS INFORMATION ON WHICH THE ACTION HAS BEEN PERFORMED
        $subject_item = BenefitRegistration::find($subject_item_id);
        $request_status_id = $subject_item->request_status_id; //RETRIEVE REQUEST_STATUS_ID

        //GETTING ALL THE EMAIL TEMPLATES LINKED TO THE PROVIDED REQUEST_STATUS_ID(THE NEXT STATUS AFTER CHANGE IN ACTION) PROVIDED
        $email_templates = $this->load_email_template($request_status_id,$order);

        //SET MESSAGE VARIABLE TO EMPTY
        $message = "";

        if ($email_templates->count() > 0){

            //LOOPING THROUGH THE SELECTED EMAIL TEMPLATES
            foreach ($email_templates as $email_template){

                //SELECTING ONE EMAIL TEMPLATE AT A TIME
                $template = EmailTemplate::find($email_template->email_template_id);
                //USING THE EMAIL TEMPLATE TO GET THE POSSIBLE EMAILS TO RECEIVE THE NOTIFICATION
                $emails = $this->get_mail_receivers($subject_item,$template,$email_list,$excludeGroup);

                //LOOPING THROUGH THE EMAILS
                if (sizeof($emails) > 0){
                    //FINDING THE USER WITH THIS EMAIL
                    $user = null;

                    //REFORMATTING THE MESSAGE TEMPLATE BY REPLACING THE VARIABLES IN THE CONTENT
                    $message = $this->format_message($template->message,$subject_item,$user,$additional_messages);
                    $subject = $this->format_message($template->subject,$subject_item,$user,$additional_messages);
                    $ccs = $this->present_cc_emails($template->cc,$subject_item,$request_status_id);

                    if (!empty($ccs)) {
                        //SENDING THE EMAIL NOTIFICATION TO THE USER
                        $this->blast_mails($message, $subject, $emails, $ccs, $fileName);
                    } else {
                        $this->blast_mails($message, $subject, $emails, null, $fileName);
                    }
                }
                else{
                    continue;
                }
            }

        }
        else{
            return false;
        }

    }

    public function send_email($new_status_id,$table_name,$subject_item_id, $email_list = array(), $excludeGroup = false,
                              $order = null, $fileName = null, $additional_messages = null)
    {
        //GETTING THE SUBJECT STATUS INFORMATION ON WHICH THE ACTION HAS BEEN PERFORMED
        $subject_item = DB::table($table_name)->where('id','=',$subject_item_id)->first();
        $request_status_id = $new_status_id; //RETRIEVE REQUEST_STATUS_ID

        //GETTING ALL THE EMAIL TEMPLATES LINKED TO THE PROVIDED REQUEST_STATUS_ID(THE NEXT STATUS AFTER CHANGE IN ACTION) PROVIDED
        $email_templates = $this->load_email_template($request_status_id,$order);

        //SET MESSAGE VARIABLE TO EMPTY
        $message = "";

        if ($email_templates->count() > 0){

            //LOOPING THROUGH THE SELECTED EMAIL TEMPLATES
            foreach ($email_templates as $email_template){

                //SELECTING ONE EMAIL TEMPLATE AT A TIME
                $template = EmailTemplate::find($email_template->email_template_id);
                //USING THE EMAIL TEMPLATE TO GET THE POSSIBLE EMAILS TO RECEIVE THE NOTIFICATION
                $emails = $this->get_mail_receivers($subject_item,$template,$email_list,$excludeGroup);

                //LOOPING THROUGH THE EMAILS
                if (sizeof($emails) > 0){
                    //FINDING THE USER WITH THIS EMAIL
                    $user = null;

                    //REFORMATTING THE MESSAGE TEMPLATE BY REPLACING THE VARIABLES IN THE CONTENT
                    $message = $this->format_message($template->message,$subject_item,$request_status_id,$user,$additional_messages);
                    $subject = $this->format_message($template->subject,$subject_item,$request_status_id,$user,$additional_messages);
                    $ccs = $this->present_cc_emails($template->cc,$subject_item,$request_status_id);
//                    dd($message);

                    if (!empty($ccs)) {
                        //SENDING THE EMAIL NOTIFICATION TO THE USER
                        $this->blast_mails($message, $subject, $emails, $ccs, $fileName);
                    } else {
                        $this->blast_mails($message, $subject, $emails, null, $fileName);
                    }
                }
                else{
                    continue;
                }
            }

        }
        else{
            return false;
        }

    }

    public function sendDetachedMail($subject_item_id,$table_name,$new_status_id,$email_set_id, $email_list = array(), $excludeGroup = false,
                                     $order = null, $fileName = null, $additional_messages = null)
    {
        //GETTING THE CLEARANCE REQUEST INFORMATION ON WHICH THE ACTION HAS BEEN PERFORMED
        $subject_item = DB::table($table_name)->where('id','=',\Crypt::decrypt($subject_item_id))
        ->first();
        //SELECTING ONE EMAIL TEMPLATE AT A TIME
        $template = EmailTemplate::find($email_set_id);
        //USING THE EMAIL TEMPLATE TO GET THE POSSIBLE EMAILS TO RECEIVE THE NOTIFICATION
        $emails = $this->get_mail_receivers($subject_item,$template,$email_list,$excludeGroup);

        //LOOPING THROUGH THE EMAILS
        if (sizeof($emails) > 0){
            //FINDING THE USER WITH THIS EMAIL
            $user = null;

            //REFORMATTING THE MESSAGE TEMPLATE BY REPLACING THE VARIABLES IN THE CONTENT
            $message = $this->format_message($template->message,$subject_item,$new_status_id,$user,$additional_messages);
            $subject = $this->format_message($template->subject,$subject_item,$new_status_id,$user,$additional_messages);
            $ccs = $this->present_cc_emails($template->cc,$subject_item,$new_status_id);

            if (!empty($ccs)) {
                //SENDING THE EMAIL NOTIFICATION TO THE USER
                $this->blast_mails($message, $subject, $emails, $ccs, $fileName);
            } else {
                $this->blast_mails($message, $subject, $emails, null, $fileName);
            }
            return "true";
        } else {
            return "false";
        }
    }

    /**
     * @param $request_status_id
     * @param $order
     * @return mixed
     */
    public function has_mail($request_status_id, $order)
    {
        if (empty($order)) {
            return StageEmail::where('lookup_id','=',$request_status_id)->get();
        } else {
            return StageEmail::where('lookup_id','=',$request_status_id)->where('order', $order)->get();
        }

    }

    /**
     * @param $request_status_id
     * @param $order
     * @return mixed
     */
    public function load_email_template($request_status_id, $order)
    {
        $emails_templates = $this->has_mail($request_status_id, $order);

        return $emails_templates;
    }

    /**
     * @param $subject_item
     * @param $template
     * @param array $email_list
     * @param bool $excludeGroup
     * @return array
     */
    public function get_mail_receivers($subject_item,$template,$email_list=array(),$excludeGroup=false)
    {

        $subject_item_id = $subject_item->id;
        $emails = $the_emails = array();
        if ($template != null){
            if (sizeof($email_list) <= 0){

                if ($template->receiver_type == "specific"){
                    $rc_emails = array();
                    $template_emails = json_decode($template->emails);
                    foreach ($template_emails as $key=>$rc){

                        if (!empty($rc->r_email) || !empty($rc->r_name)) {
                            $rc_emails[$rc->r_email] = $rc->r_name;
                            $emails = array_merge($emails,$rc_emails);
                        }
                    }

                }

                elseif ($template->receiver_type == "group"){
                    $groups = explode(',',$template->receivers);
                    foreach ($groups as $key=>$group){
                        $role = Role::find($group);
                        $the_emails = User::whereRoleIs($role->name)
                            ->where('email_verified_at','<>',NULL)
                            ->pluck('name','email')->toArray();
                        $emails = array_merge($emails,$the_emails);
                    }
                }


//                elseif ($template->receiver_type == "specific_in_group"){
//                    $rc_emails = array();
//
//                    $template_emails = json_decode($template->emails);
//
//                    foreach ($template_emails as $key=>$rc){
//                        $r_name = $this->format_message($rc->r_name,$subject_item);
//                        $r_email = $this->format_message($rc->r_email,$subject_item);
//                        $rc_emails[$r_email] = $r_name;
//                        $emails = array_merge($emails,$rc_emails);
//                    }
//                }

            }
            else{
                if ($template->receiver_type == "group"){
                    $groups = explode(',',$template->receivers);
                    
                    foreach ($groups as $key=>$group){
                        $role = Role::find($group);
                        $the_emails = User::whereRoleIs($role->name)->pluck('email')->toArray();
                        $emails = array_merge($emails,$the_emails);
                    }
                    
                    if ($excludeGroup && sizeof($email_list) > 0){
                        $emails = $email_list;
                    }
                    
                    if (sizeof($email_list) > 0){
                        $emails=array_merge($emails,$email_list);
                        $emails=array_unique($emails);
                    }
                    
                }
//                elseif ($template->receiver_type == "specific_in_group"){
//                    $groups = explode(',',$template->receivers);
//
//                    $specific_emails = $this->get_specific_emails($subject_item_id,$groups);
//
//                    foreach ($groups as $key=>$group){
//                        $role = Role::find($group);
//                        $the_emails = User::whereRoleIs($role->name)->pluck('email')->toArray();
//                        $emails = array_merge($emails,$the_emails);
//                    }
//                    $emails = array_intersect($specific_emails,$emails);
//                    if ($excludeGroup && sizeof($email_list) > 0){
//                        $emails = $email_list;
//                    }
//                    else{
//                        if (sizeof($email_list) > 0){
//                            $emails=array_merge($emails,$email_list);
//                            $emails=array_unique($emails);
//                        }
//                    }
//
//                }
            }
        }
        return $emails;
    }

    /**
     * @param $subject_item_id
     * @param array $roles_id
     * @return array
     */
    public function get_specific_emails($subject_item_id,$roles_id=array())
    {
        $specific_emails = \DB::table('stage_emails')
            ->join('roles','stage_emails.roles','=','roles.id')
            ->join('role_user','roles.id','=','role_user.role_id')
            ->join('users','role_user.user_id','=','users.id')
            ->where('request_id','=',$subject_item_id)
            ->whereIn('role_id',$roles_id)
            ->pluck('users.name','users.email')->toArray();

        return $specific_emails;
    }

//    public function get_watchers_on_request($subject_item_id)
//    {
//        $user = \DB::table('user_clearance_requests')
//            ->join('users','user_clearance_requests.user_id','=','users.id')
//            ->join('roles','user_clearance_requests.role_id','=','roles.id')
//            ->where('user_clearance_requests.request_id','=',$subject_item_id)
//            ->select('users.name as username','users.email as email', 'roles.name as role')
//            ->get();
//
//        return $user;
//    }

    public function get_watchers_on_request($new_status_id)
    {
        $user = \DB::table('stage_emails')
            ->join('roles','stage_emails.roles','=','roles.id')
            ->join('role_user','roles.id','=','role_user.role_id')
            ->join('users','role_user.user_id','=','users.id')
            ->where('stage_emails.lookup_id','=',$new_status_id)
            ->where('users.email_verified_at','<>',NULL)
            ->select('users.name as username','users.email as email', 'roles.name as role')
            ->get();

        return $user;
    }

    /**
     * @param $message
     * @param $subject_item
     * @param $new_status_id
     * @param null $user
     * @param null $additional_messages
     * @return mixed
     */
    public function format_message($message,$subject_item,$new_status_id,$user=null,$additional_messages=null)
    {
        $watchers = $this->get_watchers_on_request($new_status_id);
//            dd($watchers);
        $roles = Role::all();

//        foreach ($watchers as $watcher){
//            foreach ($roles as $role) {
//                if ($role->name == $watcher->role) {
//
//                }
//            }
//            switch ($watcher->role) {
//                case "coordinators":
//                    $coordinator = $watcher->username;
//                    $coordinator_email = $watcher->email;
//                    break;
//                case "inspectors":
//                    $inspector = $watcher->username;
//                    $inspector_email = $watcher->email;
//                    break;
//                case "managements":
//                    $management = $watcher->username;
//                    $management_email = $watcher->email;
//                    break;
//                case "monitors":
//                    $monitor = $watcher->username;
//                    $monitor_email = $watcher->email;
//                    break;
//                case "project_manager":
//                    $project_manager = $watcher->username;
//                    $project_manager_email = $watcher->email;
//                    break;
//               case "project-manager":
//                    $project_manager = $watcher->username;
//                    $project_manager_email = $watcher->email;
//                    break;
//               case "oic":
//                    $oic = $watcher->username;
//                    $oic_email = $watcher->email;
//                    break;
//                case "super-admin":
//                    $super_admin = $watcher->username;
//                    $super_admin_email = $watcher->email;
//                    break;
//                default:
//            }
//        }

        $find = array(
            "[name]",
            "[email]",
            "[additional_messages]"
        );

        if ($user == null){
            $user_name = "Sir/Madam";
            $user_email = "to all";
        }else{
            $user_name = $user->name;
            $user_email = $user->email;
        }

        $replace = array(
            $user_name,
            $user_email,
            $additional_messages
        );
        $message = str_replace($find,$replace,$message);
        return $message;
    }

    public function blast_mails($data, $subject, $emails, $ccs=null, $fileName=null)
    {
        $attachLink = null;
        if (!empty($fileName)) {
//            $attachLink = \Storage::url('app/public/workflow_attachments/'.$fileName);
            $attachLink = \Storage::url('app/'.$fileName);
            $attachLink = base_path().''.$attachLink;
        }

        \Mail::send('emails.general', ['data'=>$data,'subject'=>$subject], function($message)use($emails,$subject,$ccs,$attachLink) {
            $message->to($emails)->subject($subject);
            if ($ccs != null){
                $message->cc($ccs);
            }
            if ($attachLink != null){
                $message->attach($attachLink);
            }
        });
    }

    public function present_cc_emails($template_emails,$subject_item,$new_status_id)
    {
        $emails= array();
        if (!empty($template_emails)) {
            $d_template_emails = json_decode($template_emails);

            foreach ($d_template_emails as $key=>$rc){
                $email = $this->format_message($rc->cc_email,$subject_item,$new_status_id);
                $name = $this->format_message($rc->cc_name,$subject_item,$new_status_id);
                $cc_emails[$email] = $name;

                if (!empty($cc_emails[$email])) {
                    $emails = array_merge($emails,$cc_emails);
                }
            }
        }
        return $emails;
    }

    public function present_emails($template_emails)
    {
        $d_template_emails = json_decode($template_emails);
        $emails= array();
        foreach ($d_template_emails as $key=>$rc){
            $r_emails[$rc->r_email] = $rc->r_name;
            $emails = array_merge($emails,$r_emails);
        }

        return $emails;
    }
}
