<?php
namespace App\Http\Traits;

use App\Models\AccountDetail;
use App\Models\Applicant;
use App\Models\Bank;
use App\Models\BankBranch;
use App\Models\ContactDetail;
use App\Models\County;
use App\Models\EmploymentDetail;
use App\Models\Family;
use App\Models\HealthDetail;
use App\Models\HouseHold;
use App\Models\IdentificationDetail;
use App\Models\Lookup;
use App\Models\MaritalDetail;
use App\Models\Village;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

trait MembershipTrait
{
    /**
     * Function for upload image
     *
     * @param  string  $folder
     * @param  string  $key
     * @param  string  $validation
     *
     * @return false|string|null
     */
    public function uploadProfile($key = 'app_logo', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $location = 'applicant/profile/'.auth()->id();
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($location, request()->file($key),$name);
            $file = 'storage/'.$file;
        }

        return $file;
    }
    public function uploadFile($key,$location, $validation = 'file|mimes:jpeg,png,jpg,gif,docx,pdf,txt,xlsx|max:10048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('local')->putFileAs($location, request()->file($key),$name);
        }
        return $file;
    }
    public function uploadMultipleFiles($files,$location)
    {
        $file = [];
        foreach($files as $evidence) {
            $name = $evidence->hashName();
            $file[] = Storage::disk('local')->putFileAs($location, $evidence,$name);
        }
        return $file;
    }
    /**
     * Generate Applicant Reference ID
     * @param Request $request
     *
     * @return string
     */
    function generateRefID(Request $request)
    {
        $base_code = $this->getBaseRegID($request);
        // $existing_applicants = Applicant::where('regID','like',"$base_code%")->count();
        $existing_applicants = Applicant::orderBy('created_at','desc')->pluck('id')->first();
        $base_code.=sprintf("%03d", $existing_applicants+1);
        return $base_code;
    }
    function getBaseRegID(Request $request)
    {
        $prefix = 'MB';
        $dob = date('Ymd',strtotime($request->dobs));
        $gender = ($request->genders == 'Male')?1:2;

        $refID = "$prefix-".$gender;
        return $refID;
    }

    function updateMembershipRecordWithRecords(Request $request) {
        $applicant_id = Crypt::decrypt($request->id);
        $applicant = Applicant::find($applicant_id);
        $flash = '#add-'.$request->form_type;

        $extra = $doc = $record_id = null;
        $model = config('tracker.model','RecordTracker');

        if ($request->form_type == 'personal') {

            $this->validate($request,[
                'surnames' => 'required|string',
                'first_names' => 'required|string',
                'other_names' => 'nullable|string',
                'dobs' => 'required|string',
                'genders' => 'required|string',
                'county'=>'required',
                'village'=>'required',
                //'place_of_births' => 'required|string',
                //'country_of_births' => 'required|string',
                'nationalities' => 'required|string',
                'reason' => 'required|string',
            ]);

            if($request->file('upload_passport_pic')) {
                $file = $this->uploadProfile('upload_passport_pic');
                $extra['upload_passport_pic'] = $file;
                $extra['avatar'] = $file;
            }

            if ($record_references = $this->track('applicants',$applicant_id,$request->all(),$extra)){
                if (!is_null($extra)) addApplicantDocument($applicant_id,$extra['upload_passport_pic'],'Profile Image');

                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/personal');
                }

                $model::whereIn('id',$record_references)->update([
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);
                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                return back();
            }
            else {
                $message = 'Something went wrong. The applicant was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'family') {
            $this->validate($request,[
                'type' => 'required|string',
                'surname' => 'required|string',
                'first_name' => 'required|string',
                'other_name' => 'nullable|string',
                'date_of_birth' => 'required|string',
                'dependent_reference_id' => 'nullable|string',
                'id_type' => 'required|string',
                'is_dependant' => 'required|string'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = Family::find($record_id);
            }
            else {
                $record = new Family;
                $record->applicant_number = $applicant->regID;
            }
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->type = $request->type;
            $record->surname = $request->surname;
            $record->first_name = $request->first_name;
            $record->other_name = $request->other_name;
            $record->date_of_birth = date('Y-m-d', strtotime($request->date_of_birth));
            $record->id_type = $request->id_type;
            $record->dependent_reference_id = $request->dependent_reference_id;
            $record->is_dependent = $request->is_dependant;
            $record->removed = 0;
            $record->status = "New";
            $record->addedBy = Auth::id();

            if ($record->save()){

                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/family');
                }

                $model::whereIn('id',$record_references)->update([
                    'record_id'    =>  $record->id,
                    'applicant_number'    =>  $applicant->regID,
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);

                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'contact_details') {

            if (isset($request->request_delete)){
                if ($request->request_delete == "true"){
                    if(isset($request->mobile)) {
                        if (ContactDetail::where('mobile','=',$request->mobile)->count() > 0) {
                            $message = 'Sorry! This mobile number is already in use.';
                            session()->flash('status', 'error');
                            session()->flash('message', $message);
                            return back();
                        }
                    }
                }

                if(isset($request->email)) {
                    if (ContactDetail::where('email','=',$request->email)->count() > 0) {
                        $message = 'Sorry! This email is already in use.';
                        session()->flash('status', 'error');
                        session()->flash('message', $message);
                        return back();
                    }
                }
            }

            if (isset($request->household_number)) {
                //Check if applicant belongs to a household
                $belongs_to_a_household = ContactDetail::where('applicant_number','=',$applicant->regID)
                    ->where('type_name','=','Household')
                    ->where('household_number','!=',null)->count();

                if($belongs_to_a_household > 0){
                    $message = 'Sorry! This applicant already belongs to a household. Remove the current household and then try again.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return back();
                }

                //Check if household number is valid
                $is_valid = HouseHold::where('reference_id','=',$request->household_number)->first();
                if(is_null($is_valid)){
                    $message = 'Sorry! This Household Number does not exist.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    session()->flash('load-new-form',$flash);
                    return back();
                }

                //check if household has a head already
                if($is_valid->head != null && $is_valid->head != $applicant->regID){
                    $message = 'Sorry! This Household Number already has Head. Please detach the current head before you can set a new head.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    session()->flash('load-new-form',$flash);
                    return back();
                }
            }

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = ContactDetail::find($record_id);
            }
            else {
                $record = new ContactDetail();
                $record->applicant_number = $applicant->regID;
            }

            $res_document_path = $this->uploadFile('upload_residency','applicants/'.$applicant->regID.'/proof-residence');
            $record->upload_residency = $res_document_path;
            $extra['upload_residency'] = $res_document_path;

            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->type = $request->type;
            $record->type_name = getLookupSelectedOptionTextByValue($request->type);
            $message = 'The Request for Amendment has been received successfully and is pending approval.';
            if (isset($request->mobile)) {

                $message = 'Your mobile number has been recorded. A 4-letter code has been sent to the number via sms for verification.';
                $code = strtoupper(Str::random(4));
                $record->mobile = $request->mobile;
                $record->code = $code;
            }
            if (isset($request->village)) $record->village = $request->village;
            if (isset($request->telephone)) $record->telephone = $request->telephone;
            if (isset($request->email)) {
                $message = 'Your email has been recorded. A 4-letter code has been sent to the email for verification.';
                $code = strtoupper(Str::random(4));
                $record->email = $request->email;
                $record->code = $code;
            }
            if (isset($request->address)) $record->address = $request->address;
            if (isset($request->postal_address)) $record->postal_address = $request->postal_address;
            if (isset($request->contact_region)) $record->region = $request->contact_region;
            if (isset($request->contact_city)) $record->city = $request->contact_city;
            if (isset($request->contact_country)) $record->country = $request->contact_country;
            if (isset($request->landmark)) $record->landmark = $request->landmark;
            if (isset($request->house_address)) $record->address = $request->house_address;
            if (isset($request->household_number)) $record->household_number = $request->household_number;
            if (isset($request->is_household_head)) $record->is_household_head = $request->is_household_head;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){
                if (isset($request->mobile) && $request->request_delete != "true") {
                    $sms = setAppDefault('app_project_name').": Your verification code is ".$record->code.".";
                    sendSMS($request->mobile,$sms);
                }
                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/contact-details');
                }
                $model::whereIn('id',$record_references)->update([
                    'record_id'    =>  $record->id,
                    'applicant_number'    =>  $applicant->regID,
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);
                addApplicantDocument($applicant->id,$res_document_path,'Proof of Residence');
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'identification_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'id_types' => 'required|string',
                'id_number' => 'required|string',
                'expiry' => 'nullable|string',
                'document' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = IdentificationDetail::find($record_id);
            }
            else {
                $record = new IdentificationDetail;
                $record->applicant_number = $applicant->regID;
            }

            $res_document_path = $this->uploadFile('document','applicants/'.$applicant->regID.'/identifications');
            $record->document = $res_document_path;
            $extra['document'] = $res_document_path;

            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->id_type = $request->id_types;
            $record->id_number = $request->id_number;
            if (!is_null($request->expiry)) {
                $record->expiry = date('Y-m-d', strtotime($request->expiry));
            }
            else {
                $record->expiry = null;
            }
            $record->removed = 0;
            $record->status = "New";
            $record->addedBy = Auth::id();

            if ($record->save()){
                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/identification');
                }
                $model::whereIn('id',$record_references)->update([
                    'record_id'    =>  $record->id,
                    'applicant_number'    =>  $applicant->regID,
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);
                addApplicantDocument($applicant->id,$res_document_path,$record->id_type);
                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'marital_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'marriage_status' => 'required|string',
                'marriage_type' => 'nullable|string',
                'spouse_name' => 'nullable|string',
                'spouse_ref_id' => 'nullable|string',
                'marriage_start_date' => 'nullable|string',
                'marriage_end_date' => 'nullable|string'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = MaritalDetail::find(Crypt::decrypt($request->record_id));
            }
            else {
                $record = new MaritalDetail;
                $record->applicant_number = $applicant->regID;
            }
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->marriage_status = $request->marriage_status;
            $record->marriage_type = $request->marriage_type;
            $record->spouse_name = $request->spouse_name;
            $record->spouse_ref_id = $request->spouse_ref_id;
            $record->marriage_start_date = (!is_null($request->marriage_start_date))?date('Y-m-d', strtotime($request->marriage_start_date)):null;
            $record->marriage_end_date = (!is_null($request->marriage_end_date))?date('Y-m-d', strtotime($request->marriage_end_date)):null;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){
                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/marital');
                }
                $model::whereIn('id',$record_references)->update([
                    'record_id'    =>  $record->id,
                    'applicant_number'    =>  $applicant->regID,
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);

                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'employment_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'employment_type' => 'required|string',
                'employment_sector' => 'required|string',
                'employer_name' => 'required|string',
                'employer_mobile' => 'nullable|string',
                'employer_email' => 'nullable|string',
                'employer_bank_account_name' => 'nullable|string',
                'employer_bank_account_number' => 'nullable|string',
                'employer_postal_address' => 'nullable|string',
                'employee_position' => 'nullable|string',
                'employment_start_date' => 'nullable|string',
                'employment_end_date' => 'nullable|string',
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = EmploymentDetail::find($record_id);
            }
            else {
                $record = new EmploymentDetail;
                $record->applicant_number = $applicant->regID;
            }
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->employment_type = $request->employment_type;
            $record->employment_sector = $request->employment_sector;
            $record->employer_name = $request->employer_name;
//            $record->employer_telephone = $request->employer_telephone;
            $record->employer_mobile = $request->employer_mobile;
            $record->employer_email = $request->employer_email;
//            $record->employer_rmd = $request->employer_rmd;
            $record->employer_bank_account_name = $request->employer_bank_account_name;
            $record->employer_bank_account_number = $request->employer_bank_account_number;
            $record->employer_postal_address = $request->employer_postal_address;
//            $record->employer_island = $request->employer_island;
            $record->employee_position = $request->employee_position;
//            $record->employee_manager = $request->employee_manager;
            $record->employment_start_date = (isset($request->employment_start_date))?date('Y-m-d', strtotime($request->employment_start_date)):null;
            $record->employment_end_date = (isset($request->employment_end_date))?date('Y-m-d', strtotime($request->employment_end_date)):null;
            $record->removed = 0;
            $record->status = "New";
            $record->addedBy = Auth::id();

            if ($record->save()){
                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/employment');
                }
                $model::whereIn('id',$record_references)->update([
                    'record_id'    =>  $record->id,
                    'applicant_number'    =>  $applicant->regID,
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);

                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'account_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'bank_name' => 'required|integer',
                'account_type' => 'required|string',
                'account_holder_name' => 'required|string',
                'account_number' => 'required|string',
                'branch' => 'nullable|string',
                'country' => 'required|numeric',
                'upload_account' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = AccountDetail::find($record_id);
            }
            else {
                $record = new AccountDetail();
                $record->regID = $applicant->regID;
            }
            $bankName = getMerchant($request->bank_name,'name');
            $branchName = getMerchantBranch($request->branch,'name');
            $merchant = getMerchant($request->bank_name);
//            dd($merchant['type']['name']);

            $res_document_path = $this->uploadFile('upload_account','applicants/'.$applicant->regID.'/account-details');
            $record->upload_account = $res_document_path;
            $extra['upload_account'] = $res_document_path;


            $extra['bank_name'] = $bankName;
            $extra['branch'] = $branchName;
            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $record->country_id = $request->country;
            $record->bank_name = $bankName;
            $record->bank_id = $request->bank_name;
            $record->account_holder_name = $request->account_holder_name;
            $record->bank_branch = $branchName;
            $record->branch_id = $request->branch;
            $record->account_type = $request->account_type;
            $record->type = $merchant['type']['name'];
            $record->account_number = $request->account_number;
            $record->removed = 0;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){
                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/account-details');
                }
                $model::whereIn('id',$record_references)->update([
                    'record_id'    =>  $record->id,
                    'applicant_number'    =>  $applicant->regID,
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);
                if($request->file('upload_account')) {
                    addApplicantDocument($applicant->id, $res_document_path, 'Proof of Account');
                }
                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'health_details') {
            $this->validate($request,[
                'form_type' => 'required|string',
                'health_type' => 'required|string',
                'date_started' => 'nullable|string',
                'confirmed_by' => 'nullable|string',
                'supporting_document' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = HealthDetail::find($record_id);
            }
            else {
                $record = new HealthDetail();
                $record->regID = $applicant->regID;
            }

            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $res_document_path = $this->uploadFile('supporting_document','applicants/'.$applicant->regID.'/health');
            $record->supporting_document = $res_document_path;
            $extra['supporting_document'] = $res_document_path;

            $record->health_type = $request->health_type;
            $record->date_started = date('Y-m-d', strtotime($request->date_started));
            $record->confirmed_by = $request->confirmed_by;
            $record->removed = 0;
            $record->status = 'New';
            $record->addedBy = Auth::id();

            if ($record->save()){

                if(isset($request->evidences)) {
                    $doc = $this->uploadMultipleFiles($request->evidences,'member-records/'.$applicant->regID.'/health');
                }
                $model::whereIn('id',$record_references)->update([
                    'record_id'    =>  $record->id,
                    'applicant_number'    =>  $applicant->regID,
                    'reason'    =>  $request->reason,
                    'evidences'    =>  $doc
                ]);

                addApplicantDocument($applicant->id,$res_document_path,'Proof of Health State');
                $message = 'The Request for Amendment has been received successfully and is pending approval.';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }

        return back();
    }

    function updateMembershipRecord(Request $request) {
        if (isset($request->id)){
            $applicant_id = Crypt::decrypt($request->id);
            $applicant = Applicant::find($applicant_id);
        }

        $flash = '#add-'.$request->form_type;

        $extra = $doc = $record_id = null;

        if ($request->form_type == 'personal') {
            
            $this->validate($request,[
                'surnames' => 'required|string',
                'first_names' => 'required|string',
                'other_names' => 'nullable|string',
                'dobs' => 'required|string',
                'genders' => 'required|string',
                //'place_of_births' => 'required|string',
                'county' => 'required',
                'village'=>'required',
                //'country_of_births' => 'required|string',
                //'nationalities' => 'required|string',
            ]);


            $file = null;
            if($request->file('upload_passport_pic')) {
                $file = $this->uploadProfile('upload_passport_pic');
                $applicant->upload_passport_pic = $file;
                $applicant->avatar = $file;
            }

            //$country_name = getCountry($request->country_of_births,'country_name');
            $country_name = 'Kenya';
            if(isset($request->county)){
                $state_name = County::find($request->county)->county;
            }

            $village = Village::findOrFail($request->village);
            //$state_name = getState($request->state_of_births,'name');
            //$county_name = getCounty($request->place_of_births,'county');

            $applicant->surnames = $request->surnames;
            $applicant->first_names = $request->first_names;
            $applicant->other_names = $request->other_names;
            $applicant->dobs = date('Y-m-d', strtotime($request->dobs));
            $applicant->genders = $request->genders;
            $applicant->country_id = $request->country_of_births ?? 115;
            $applicant->state_id = $request->state_of_births  ?? $village->state_id;
            $applicant->city_id = $request->county;
            $applicant->country_of_births = $country_name ?? 'Kenya';
            $applicant->state_of_births = $state_name;
            $applicant->place_of_births = $country_name;
            $applicant->nationalities = 'Kenyan';
            $applicant->type = $request->type;
            $applicant->village_id = $request->village;
            $applicant->updated_by = Auth::id();


            if ($applicant->save()){

                if (!is_null($file)) addApplicantDocument($applicant_id,$file,'Profile Image');
                $message = 'Information has been updated';
                session()->flash('status', 'success');
                session()->flash('message', $message);
                return back();
            }
            else {
                $message = 'Something went wrong. The applicant was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'family') {

            if ($request->request_delete == "true"){
                if (Family::destroy([Crypt::decrypt($request->record_id)])) {
                    $message = 'Record has been removed';
                    $status = 'success';
                }
                else{
                    $message = 'Sorry! Invalid record or something went wrong.';
                    $status = 'error';
                }
                session()->flash('status', $status);
                session()->flash('message', $message);
                return back();
            }

            $this->validate($request,[
                'type' => 'required|string',
                'surname' => 'required|string',
                'first_name' => 'required|string',
                'other_name' => 'nullable|string',
                'date_of_birth' => 'required|string',
                'dependent_reference_id' => 'nullable|string',
                'id_type' => 'required|string',
                'is_dependant' => 'required|string'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = Family::find($record_id);
                $message = 'Information has been updated.';
            }
            else {
                $record = new Family;
                $record->applicant_number = $applicant->regID;
                $message = 'A new family has been added.';
            }

            $record->type = $request->type;
            $record->surname = $request->surname;
            $record->first_name = $request->first_name;
            $record->other_name = $request->other_name;
            $record->date_of_birth = date('Y-m-d', strtotime($request->date_of_birth));
            $record->id_type = $request->id_type;
            $record->dependent_reference_id = $request->dependent_reference_id;
            $record->is_dependent = $request->is_dependant;
            $record->removed = 0;
            $record->status = "Verified";
            $record->addedBy = Auth::id();

            if ($record->save()){
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'contact_details') {

            if ($request->request_delete == "true"){
                if (ContactDetail::destroy([Crypt::decrypt($request->record_id)])) {
                    $message = 'Record has been removed';
                    $status = 'success';
                }
                else{
                    $message = 'Sorry! Invalid record or something went wrong.';
                    $status = 'error';
                }
                session()->flash('status', $status);
                session()->flash('message', $message);
                return back();
            }

            if (isset($request->household_number)) {
                //Check if applicant belongs to a household
                $belongs_to_a_household = ContactDetail::where('applicant_number','=',$applicant->regID)
                    ->where('type_name','=','Household')
                    ->where('household_number','!=',null)->count();

                if($belongs_to_a_household > 0){
                    $message = 'Sorry! This applicant already belongs to a household. Remove the current household and then try again.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    return back();
                }

                //Check if household number is valid
                $is_valid = HouseHold::where('reference_id','=',$request->household_number)->first();
                if(is_null($is_valid)){
                    $message = 'Sorry! This Household Number does not exist.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    session()->flash('load-new-form',$flash);
                    return back();
                }

                //check if household has a head already
                if($is_valid->head != null && $is_valid->head != $applicant->regID){
                    $message = 'Sorry! This Household Number already has Head. Please detach the current head before you can set a new head.';
                    session()->flash('status', 'error');
                    session()->flash('message', $message);
                    session()->flash('load-new-form',$flash);
                    return back();
                }
            }

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = ContactDetail::find($record_id);
                $message = 'Information has been updated.';
            }
            else {
                $record = new ContactDetail();
                $record->applicant_number = $applicant->regID;
                $message = 'A new contact record has been added.';
            }

            $res_document_path = $this->uploadFile('upload_residency','applicants/'.$applicant->regID.'/proof-residence');
            $record->upload_residency = $res_document_path;
            $record->type = $request->type;
            $record->type_name = getLookupSelectedOptionTextByValue($request->type);

            if (isset($request->mobile)) {
                $message = 'Your mobile number has been recorded. A 4-letter code has been sent to the number via sms for verification.';
                $code = strtoupper(Str::random(4));
                $record->mobile = $request->mobile;
                $record->code = $code;
            }
            if (isset($request->village)) $record->village = $request->village;
            if (isset($request->telephone)) $record->telephone = $request->telephone;
            if (isset($request->email)) {
                $message = 'Your email has been recorded. A 4-letter code has been sent to the email for verification.';
                $code = strtoupper(Str::random(4));
                $record->email = $request->email;
                $record->code = $code;
            }
            if (isset($request->address)) $record->address = $request->address;
            if (isset($request->postal_address)) $record->postal_address = $request->postal_address;
            if (isset($request->contact_region)) $record->region = $request->contact_region;
            if (isset($request->contact_city)) $record->city = $request->contact_city;
            if (isset($request->contact_country)) $record->country = $request->contact_country;
            if (isset($request->landmark)) $record->landmark = $request->landmark;
            if (isset($request->house_address)) $record->address = $request->house_address;
            if (isset($request->household_number)) $record->household_number = $request->household_number;
            if (isset($request->is_household_head)) $record->is_household_head = $request->is_household_head;
            $record->status = 'Verified';
            $record->addedBy = Auth::id();

            if ($record->save()){
                if (isset($request->mobile) && $request->request_delete != "true") {
                    $sms = setAppDefault('app_project_name').": Your verification code is ".$record->code.".";
                    sendSMS($request->mobile,$sms);
                }

                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'identification_details') {

            if ($request->request_delete == "true"){
                if (IdentificationDetail::destroy([Crypt::decrypt($request->record_id)])) {
                    $message = 'Record has been removed';
                    $status = 'success';
                }
                else{
                    $message = 'Sorry! Invalid record or something went wrong.';
                    $status = 'error';
                }
                session()->flash('status', $status);
                session()->flash('message', $message);
                return back();
            }

            $this->validate($request,[
                'form_type' => 'required|string',
                'id_types' => 'required|string',
                'id_number' => 'required|string',
                'expiry' => 'nullable|string',
                'document' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = IdentificationDetail::find($record_id);
                $message = 'Information has been updated.';
            }
            else {
                $record = new IdentificationDetail;
                $record->applicant_number = $applicant->regID;
                $message = 'A new identification record has been added.';
            }

            $res_document_path = $this->uploadFile('document','applicants/'.$applicant->regID.'/identifications');
            $record->document = $res_document_path;

            $record->id_type = $request->id_types;
            $record->id_number = $request->id_number;
            if (!is_null($request->expiry)) {
                $record->expiry = date('Y-m-d', strtotime($request->expiry));
            }
            else {
                $record->expiry = null;
            }
            $record->removed = 0;
            $record->status = "Verified";
            $record->addedBy = Auth::id();

            if ($record->save()){
                addApplicantDocument($applicant->id,$res_document_path,$record->id_type);
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'marital_details') {

            if ($request->request_delete == "true"){
                if (MaritalDetail::destroy([Crypt::decrypt($request->record_id)])) {
                    $message = 'Record has been removed';
                    $status = 'success';
                }
                else{
                    $message = 'Sorry! Invalid record or something went wrong.';
                    $status = 'error';
                }
                session()->flash('status', $status);
                session()->flash('message', $message);
                return back();
            }

            $this->validate($request,[
                'form_type' => 'required|string',
                'marriage_status' => 'required|string',
                'marriage_type' => 'nullable|string',
                'spouse_name' => 'nullable|string',
                'spouse_ref_id' => 'nullable|string',
                'marriage_start_date' => 'nullable|string',
                'marriage_end_date' => 'nullable|string'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = MaritalDetail::find(Crypt::decrypt($request->record_id));
                $message = 'Information has been updated.';
            }
            else {
                $record = new MaritalDetail;
                $record->applicant_number = $applicant->regID;
                $message = 'A new marital record has been added.';
            }

            $record->marriage_status = $request->marriage_status;
            $record->marriage_type = $request->marriage_type;
            $record->spouse_name = $request->spouse_name;
            $record->spouse_ref_id = $request->spouse_ref_id;
            $record->marriage_start_date = (!is_null($request->marriage_start_date))?date('Y-m-d', strtotime($request->marriage_start_date)):null;
            $record->marriage_end_date = (!is_null($request->marriage_end_date))?date('Y-m-d', strtotime($request->marriage_end_date)):null;
            $record->status = 'Verified';
            $record->addedBy = Auth::id();

            if ($record->save()){
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'employment_details') {

            if ($request->request_delete == "true"){
                if (EmploymentDetail::destroy([Crypt::decrypt($request->record_id)])) {
                    $message = 'Record has been removed';
                    $status = 'success';
                }
                else{
                    $message = 'Sorry! Invalid record or something went wrong.';
                    $status = 'error';
                }
                session()->flash('status', $status);
                session()->flash('message', $message);
                return back();
            }

            $this->validate($request,[
                'form_type' => 'required|string',
                'employment_type' => 'required|string',
                'employment_sector' => 'required|string',
                'employer_name' => 'required|string',
                'employer_mobile' => 'nullable|string',
                'employer_email' => 'nullable|string',
                'employer_bank_account_name' => 'nullable|string',
                'employer_bank_account_number' => 'nullable|string',
                'employer_postal_address' => 'nullable|string',
                'employee_position' => 'nullable|string',
                'employment_start_date' => 'nullable|string',
                'employment_end_date' => 'nullable|string',
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = EmploymentDetail::find($record_id);
                $message = 'Information has been updated.';
            }
            else {
                $record = new EmploymentDetail;
                $record->applicant_number = $applicant->regID;
                $message = 'A new employment record has been added.';
            }

            $record->employment_type = $request->employment_type;
            $record->employment_sector = $request->employment_sector;
            $record->employer_name = $request->employer_name;
            $record->employer_mobile = $request->employer_mobile;
            $record->employer_email = $request->employer_email;
            $record->employer_bank_account_name = $request->employer_bank_account_name;
            $record->employer_bank_account_number = $request->employer_bank_account_number;
            $record->employer_postal_address = $request->employer_postal_address;
            $record->employee_position = $request->employee_position;
            $record->employment_start_date = (isset($request->employment_start_date))?date('Y-m-d', strtotime($request->employment_start_date)):null;
            $record->employment_end_date = (isset($request->employment_end_date))?date('Y-m-d', strtotime($request->employment_end_date)):null;
            $record->removed = 0;
            $record->status = "Verified";
            $record->addedBy = Auth::id();

            if ($record->save()){
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'account_details') {

            if ($request->request_delete == "true"){
                if (AccountDetail::destroy([Crypt::decrypt($request->record_id)])) {
                    $message = 'Record has been removed';
                    $status = 'success';
                }
                else{
                    $message = 'Sorry! Invalid record or something went wrong.';
                    $status = 'error';
                }
                session()->flash('status', $status);
                session()->flash('message', $message);
                return back();
            }

            $this->validate($request,[
                'form_type' => 'required|string',
                'bank_name' => 'required|integer',
                'account_type' => 'required|string',
                'account_holder_name' => 'required|string',
                'account_number' => 'required|string',
                'branch' => 'nullable|string',
                'country' => 'required|numeric',
                'upload_account' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = AccountDetail::find($record_id);
                $message = 'Information has been updated.';
            }
            else {
                $record = new AccountDetail();
                $record->regID = $applicant->regID;
                $message = 'A new merchant account has been added.';
            }
            $bankName = getMerchant($request->bank_name,'name');
            $branchName = getMerchantBranch($request->branch,'name');
            $merchant = getMerchant($request->bank_name);
//            dd($merchant['type']['name']);

            $res_document_path = $this->uploadFile('upload_account','applicants/'.$applicant->regID.'/account-details');
            $record->upload_account = $res_document_path;
            $record->country_id = $request->country;
            $record->bank_name = $bankName;
            $record->bank_id = $request->bank_name;
            $record->account_holder_name = $request->account_holder_name;
            $record->bank_branch = $branchName;
            $record->branch_id = $request->branch;
            $record->account_type = $request->account_type;
            $record->type = $merchant['type']['name'];
            $record->account_number = $request->account_number;
            $record->removed = 0;
            $record->status = 'Verified';
            $record->addedBy = Auth::id();

            if ($record->save()){
                if($request->file('upload_account')) {
                    addApplicantDocument($applicant->id, $res_document_path, 'Proof of Account');
                }
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }
        elseif ($request->form_type == 'health_details') {

            if ($request->request_delete == "true"){
                if (HealthDetail::destroy([Crypt::decrypt($request->record_id)])) {
                    $message = 'Record has been removed';
                    $status = 'success';
                }
                else{
                    $message = 'Sorry! Invalid record or something went wrong.';
                    $status = 'error';
                }
                session()->flash('status', $status);
                session()->flash('message', $message);
                return back();
            }

            $this->validate($request,[
                'form_type' => 'required|string',
                'health_type' => 'required|string',
                'date_started' => 'nullable|string',
                'confirmed_by' => 'nullable|string',
                'supporting_document' => 'nullable|file'
            ]);

            if (isset($request->record_id)) {
                $record_id = Crypt::decrypt($request->record_id);
                $record = HealthDetail::find($record_id);
                $message = 'Information has been updated.';
            }
            else {
                $record = new HealthDetail();
                $record->regID = $applicant->regID;
                $message = 'A new health record has been added.';
            }

            $record_references = $this->track($request->form_type,$record_id,$request->all(),$extra);

            $res_document_path = $this->uploadFile('supporting_document','applicants/'.$applicant->regID.'/health');
            $record->supporting_document = $res_document_path;
            $extra['supporting_document'] = $res_document_path;

            $record->health_type = $request->health_type;
            $record->date_started = date('Y-m-d', strtotime($request->date_started));
            $record->confirmed_by = $request->confirmed_by;
            $record->removed = 0;
            $record->status = 'Verified';
            $record->addedBy = Auth::id();

            if ($record->save()){
                addApplicantDocument($applicant->id,$res_document_path,'Proof of Health State');
                session()->flash('status', 'success');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back();
            }
            else {
                $message = 'Something went wrong. The record was not saved. Please try again';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                session()->flash('load-new-form',$flash);
                return back()->withInput();
            }
        }

        return back();
    }

    function createMemberShip(Request $request) {
        //dd('here');
        $this->validate($request,[
            'surnames' => 'required|string',
            'first_names' => 'required|string',
            'other_names' => 'nullable|string',
            'dobs' => 'required|string',
            'genders' => 'required|string',
            //'state_of_births' => 'required|string',
            //'place_of_births' => 'required|string',
            //'country_of_births' => 'required|string',
            //'nationalities' => 'required|string',
            'village' => 'required',
            'county'=>'required',
        ]);

        $village = Village::findOrFail($request->village);

        if (!isset($request->id)){
            if ($regID = $this->generateRefID($request)) {
                $applicant = new Applicant();
                $applicant->regID = $regID;
                $applicant->added_by = Auth::id();
            }else {
                alertNotify(__('Something went wrong. Please try again'),'','danger');
                return back()->withInput();
            }
        }
        else {
            $applicant = Applicant::find(Crypt::decrypt($request->id));
        }

        $country_name = getCountry($request->country_of_births,'country_name');
        //$state_name = getState($request->state_of_births,'name');
        $state_name = getState($village->state_id,'name');
        $county_name = getCounty($request->place_of_births,'county');

        $passport_pic_path = $avatar = null;
        if($request->file('upload_passport_pic')) {
            $file = $this->uploadProfile('upload_passport_pic');
            $passport_pic_path = $file;
            $avatar = $file;
        }

        

        $applicant->surnames = $request->surnames;
        $applicant->first_names = $request->first_names;
        $applicant->other_names = $request->other_names;
        $applicant->dobs = date('Y-m-d', strtotime($request->dobs));
        $applicant->genders = $request->genders;
        $applicant->country_id = $request->country_of_births ?? 115;
        $applicant->state_id = $request->state_of_births  ?? $village->state_id;
        $applicant->city_id = $request->place_of_births ?? 1104;
        $applicant->country_of_births = $country_name ?? 'Kenya';
        $applicant->state_of_births = $state_name;
        $applicant->place_of_births = $county_name ?? 'Kenya';
        $applicant->nationalities = $request->nationalities ?? 'Kenyan';
        $applicant->upload_passport_pic = $passport_pic_path;
        $applicant->avatar = $avatar;
        $applicant->type = $request->type;
        $applicant->updated_by = Auth::id();
        $applicant->village_id = $request->village;

        if ($applicant->save()){
            if (!isMember()) {
                $applicant->deployments()->attach([auth()->user()->last_instance_opened]);
            }

            addApplicantDocument($applicant->id,$passport_pic_path,'Profile Image');
            $message = 'New beneficiary has been added successfully';
            if (auth()->user()->hasRole('beneficiary')) {
                auth()->user()->applicant_number = $applicant->regID;
                auth()->user()->save();
            }
            session()->flash('status', 'success');
            session()->flash('message', $message);

            if($request->type == 'Household') {
                $household = HouseHold::findOrFail(Crypt::decrypt($request->household_id));
                setNewMember($applicant->regID,$household->reference_id);
                return redirect()->route('household.view',[$request->household_id]);
            }

            $selected_village = Village::find($request->village);

            ContactDetail::create([
                'type_name' => 'Residential Address',
                'type' => 107,
                'applicant_number'=> $refID ?? $applicant->regID,
                'country'=>115,
                'region'=> $selected_village->state_id,
                'city'=> $selected_village->county_id,
                'village'=>$selected_village->name,
                'status'=>'Verified',
                'addedBy'=>auth()->user()->id
            ]);
            return redirect()->route('applicant.applicantRegistration',[cryptThis($applicant->id)]);
        }
        else {
            $message = 'Something went wrong. The beneficiary was not saved. Please try again';
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }
}
