<?php
namespace App\Http\Traits;


use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

trait ModelTracker
{
    protected $user_type = 'App\Models\User';
    protected $event = 'updated';
    protected $tracking_table = 'App\Models\RecordTracker';

    public function getTrackerTable()
    {
        return $this->tracking_table = config('tracker.model','RecordTracker');
    }

    protected function getData($table_name,$auditable_id)
    {
        $data = DB::table($table_name)->where('id','=',$auditable_id)->first();
        return $data;
    }

    protected function getModifiedData($oldData,$newData,$table_name,$auditable_id=null)
    {
        $event_type = 'Addition';
        if (!is_null($oldData)) {
            $oldData = (array)$oldData;
            $oldData = Arr::except($oldData, ['updated_at','created_at','added_by','updated_by','default','status']);
            $event_type = 'Updated';
        }

        //Check if the applicant has requested for a deletion
        if (isset($newData['request_delete'])) {
            if ($newData['request_delete'] == "true")
                $event_type = "Deletion";
        }

        $newData = Arr::except($newData, ['updated_at','_token','_method','id','form_type','reason','evidences','request_delete']);

        $results = [];

        if ($table_name == 'applicants') {
            foreach ($newData as $key=>$value) {

                if (isset($oldData[$key])) {

                    if ($oldData[$key] == $value)  continue;

                    $old_value = $oldData[$key];
                    $new_value = $value;
                    if (($old_value != "" && $new_value == "") || (is_null($old_value) && is_null($new_value))) {
                        $action = config('tracker.clear_action');
                    }
                    else {
                        $action = config('tracker.update_action');
                    }
                }
                else {
                    $old_value = "";
                    $new_value = $value;
                    if ($new_value != "") $action = config('tracker.update_action');
                    else $action = 'New';
                }

                $results[] = [
                    'applicant_number'  =>  $oldData['regID'],
                    'old_values'  =>  $old_value,
                    'new_values'  =>  $new_value,
                    'event'         =>  $action,
                    'table_name'  =>  $table_name,
                    'field'  =>  $key,
                    'record_id'  =>  $oldData['id'],
                    'request_by'  =>  Auth::id(),
                ];

            }
        }
        else {
            $applicant_number = null;
            if (!is_null($oldData)) {
                if (isset($oldData['applicant_number'])) {
                    $applicant_number = $oldData['applicant_number'];
                }
                $oldData = Arr::except($oldData, ['id','applicant_number','removed','addedBy','deleted_at']);
            }
            if (!is_null($newData)) {
                $newData = Arr::except($newData, ['record_id']);
            }

            $results[] = [
                'applicant_number'  =>  $applicant_number,
                'old_values'  =>  $oldData,
                'new_values'  =>  $newData,
                'event'         =>  $event_type,
                'table_name'  =>  $table_name,
                'field'  =>  Str::headline(Str::singular($table_name)),
                'record_id'  =>  $auditable_id,
                'request_by'  =>  Auth::id(),
            ];
        }

        return $results;
    }

    public function track($table_name,$auditable_id,array $newData,array $extra = null)
    {
        $tracking_table = $this->getTrackerTable();
        $record_references = [];
        if ($extra != null) {
            $newData = array_merge($newData,$extra);
        }
        $oldData = null;
        $newRecord = true;
        if (!is_null($auditable_id)) {
            //Get the old records from the original table
            $oldData = $this->getData($table_name,$auditable_id);

            //Check the status of the old record to verify for insertion of a new record or update the existing record
            if ( !in_array($oldData->status,['Verified','Declined'])) {
                $newRecord = false;
            }
        }

        $modified = $this->getModifiedData($oldData,$newData,$table_name,(int)$auditable_id);

        if(sizeof($modified) > 0) {
            foreach ($modified as $key=>$data) {
                $field = null;
                $data['instance_id'] = getActiveInstance();
                if ($newRecord) {
                    $record = $tracking_table::create($data);
                }
                else {
                    if (isset($data['field'])) $field = $data['field'];
                    $data['status'] = "New";
                    $record = $tracking_table::updateOrCreate([
                        'table_name'    =>  $table_name,
                        'record_id'    =>  (int)$auditable_id,
                        'field'    =>  $field,
                    ],$data);
                }
                $record_references[] = $record->id;
            }
        }

        return $record_references;

    }


}
