<?php

namespace App\Mail;

use App\Models\Deployment;
use App\Models\Plan;
use App\Models\PlanSubscription;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Mail\Mailable;
use Illuminate\Mail\Mailables\Address;
use Illuminate\Mail\Mailables\Content;
use Illuminate\Mail\Mailables\Envelope;
use Illuminate\Queue\SerializesModels;

class ShipRedeployment extends Mailable
{
    use Queueable, SerializesModels;

    public $deployment;
    public $plan;
    public $subscription;

    /**
     * Create a new message instance.
     *
     * @return void
     */
    public function __construct($deployment)
    {
        $this->deployment = $deployment;
        $this->plan = Plan::whereSlug($deployment->package)->firstOrFail();
        $this->subscription = PlanSubscription::whereSubscriberId($deployment->id)->first();
    }

    /**
     * Get the message envelope.
     *
     * @return \Illuminate\Mail\Mailables\Envelope
     */
    public function envelope()
    {
        return new Envelope(
            from: new Address(env('MAIL_FROM_ADDRESS'), env('MAIL_FROM_NAME')),
            subject: env('APP_NAME').' Re-deployment',
        );
    }

    /**
     * Get the message content definition.
     *
     * @return \Illuminate\Mail\Mailables\Content
     */
    public function content()
    {
        return new Content(
            view: 'emails.re-deployment',
        );
    }

    /**
     * Get the attachments for the message.
     *
     * @return array
     */
    public function attachments()
    {
        return [];
    }
}
