<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use OwenIt\Auditing\Contracts\Auditable;

class Application extends Model implements Auditable
{
    use HasFactory, \OwenIt\Auditing\Auditable;

    protected $guarded = [];

    protected $casts = [
        'responsible_role'  =>  'json',
        'documentations'  =>  'array',
    ];

    public function form() {
        return $this->belongsTo('App\Models\Form');
    }

    public function benefitPayment()
    {
        return $this->hasOne(BenefitPayment::class);
    }

    public function benefit() {
        return $this->belongsTo('App\Models\Benefit','benefit_id');
    }

    public function items(){
        return $this->hasMany(ItemIssuance::class, 'application_id');
    }

    public function applicant() {
        return $this->belongsTo('App\Models\Applicant','applicant_number','regID');
    }

    public function applicationable()
    {
        return $this->morphTo();
    }

    public function addedBy() {
        return $this->belongsTo(User::class,'applicationable_id');
    }

    public function lookup() {
        return $this->belongsTo('App\Models\Lookup','stage');
    }

    public function modifiedBy() {
        return $this->belongsTo('App\Models\User','modified_by');
    }

    public function processLog() {
        return $this->hasMany('App\Models\BenefitVerificationProcessLog','benefit_registration_id');
    }

    public function applicationProcessings() {
        return $this->hasMany(ApplicationsProcessing::class);
    }

    public function latestReturnForCorrection() {
        return $this->applicationProcessings->where('action','=','Returned For Correction')->sortByDesc('created_at')->first();
    }

    public function approval() {
        return $this->hasOne(ApplicationApproval::class);
    }

    public function declines() {
        return $this->hasMany(ApplicationDecline::class);
    }

    public function scopeIsReturnedForCorrection($query)
    {
        return $query->where('status','=','Returned For Correction');
    }
    public function scopeActive($query)
    {
        $query->where('archived','=',0);
    }

    public function scopeSubmitted($query)
    {
        $query->whereIn('status',['Submitted','Resubmitted']);
    }

    public function scopeApproved($query)
    {
        $query->where('status','=','Approved');
    }

    public function markRead()
    {
        $receivers = GeneralNotification::whereJsonContains('receivers',auth()->id())
            ->where('table','=','applications')
            ->where('subject_id','=',$this->id)->pluck('receivers')
            ->first();
        if ($receivers) {
            $new_receivers = findReplaceItem(auth()->id(),$receivers);
            DB::table('general_notifications')
                ->where('table','=','applications')
                ->where('subject_id','=',$this->id)
                ->update(['receivers'=>$new_receivers]);
        }
    }
}
