<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Str;

class Form extends Model
{
    protected $fillable = ['title','slug','table_name','description','step','active','form_type'];

    public function fields()
    {
        return $this->hasMany('App\Models\Field');
    }
    public function sub_forms()
    {
        return $this->hasMany('App\Models\FormFieldChildren','parent_form_id');
    }
//
    public function setTableNameAttribute($value)
    {
        $this->attributes['table_name'] = Str::snake(Str::plural($value));
    }
//
    public function scopeActiveForms($query)
    {
        return $query->where('active', '=', 1)->where('collective', '=', 0);
    }
//
    public function scopeIsCollective($query)
    {
        $isTrue = $query->where('collective', '=', 1)->count();

        if ($isTrue > 0) {
            return true;
        }
        return false;
    }
//
    public function scopeCollectives($query)
    {
        return $query->where('collective', '=', 1);
    }
//
    public function scopeActive($query)
    {
        return $query->where('active', '=', 1);
    }
//
    public function scopeActiveCollectives($query)
    {
        return $query->where('collective', '=', 1)->where('active', '=', 1);
    }
//
    public function form_collective()
    {
        return $this->hasOne('App\Models\FormCollective','form_id');
    }

    public function form_collectives()
    {
        return $this->belongsToMany('App\Models\FormCollective','form_id');
    }

    public function form_collectives_forms()
    {
        return $this->hasMany('App\Models\FormCollectivesForm','form_id');
    }

    public function questionnaire()
    {
        return $this->hasOne('App\Models\Questionnaire','form_id');
    }

    public function scopeActiveQuestionnaires($query)
    {
        return $query->where('form_type', '=', 'survey')->where('active', '=', 1);
    }

    public function benefit()
    {
        return $this->hasOne('App\Models\Benefit','form_id');
    }

}
