<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Team extends Model
{
    use HasFactory,SoftDeletes;

    protected $fillable = ['name','code','program_id','type','survey','status','created_by','updated_by',
        'instance_id'];

    public function createdBy()
    {
        return $this->belongsTo(User::class,'created_by');
    }

    public function updatedBy()
    {
        return $this->belongsTo(User::class,'updated_by');
    }

    public function program()
    {
        return $this->belongsTo(Program::class);
    }

    public function _type()
    {
        return $this->belongsTo(Lookup::class,'team_type');
    }

    public function survey()
    {
        return $this->belongsToMany(Survey::class);
    }

    public function members()
    {
        return $this->hasMany(TeamMember::class);
    }

    public function memberInvitations()
    {
        return $this->hasMany(TeamInvitation::class);
    }

    public function scopeActive($query)
    {
        return $query->where('status',1);
    }
}
