<?php

return [

    // All the sections for the settings page
    'sections' => [
        'app' => [
            'title' => 'General Settings',
            'descriptions' => 'Application general settings.', // (optional)
            'icon' => 'fa fa-cog', // (optional)

            'inputs' => [
                [
                    'name' => 'app_name', // unique key for setting
                    'type' => 'text', // type of input can be text, number, textarea, select, boolean, checkbox etc.
                    'label' => 'App Name', // label for input
                    // optional properties
                    'placeholder' => 'Application Name', // placeholder for input
                    'class' => 'form-control', // override global input_class
                    'style' => '', // any inline styles
                    //'rules' => 'required|min:2|max:20', // validation rules for this input
                    'value' => 'QCode', // any default value
                    'hint' => 'You can set the app name here' // help block text for input
                ],
                [
                    'name' => 'logo',
                    'type' => 'image',
                    'label' => 'Upload logo',
                    'hint' => 'Must be an image and cropped in desired size',
                    'rules' => 'image|max:500',
                    'disk' => 'logo', // which disk you want to upload
                    'path' => 'uploads/system_logo', // path on the disk,
                    'preview_class' => 'thumbnail',
                    'preview_style' => 'height:40px'
                ],
                [
                    'type' => 'textarea',
                    'name' => 'footer_details',
                    'label' => 'Footer Details',
                    'rows' => 4,
                    'cols' => 10,
                    'placeholder' => 'Provide footer details here'
                ],
                [
                    'name' => 'password_expiry_days', // unique key for setting
                    'type' => 'number', // type of input can be text, number, textarea, select, boolean, checkbox etc.
                    'label' => 'Password Expiry Days', // label for input
                    // optional properties
                    'placeholder' => 'Enter Password Expiry Days', // placeholder for input
                    'class' => 'form-control', // override global input_class
                    'style' => '', // any inline styles
                    //'rules' => 'required|min:2|max:20', // validation rules for this input
                    'value' => '30', // any default value
                    'hint' => 'Set password expiry days to prompt user to change password' // help block text for input
                ],
            ]
        ],
        'email' => [
            'title' => 'Email Settings',
            'descriptions' => 'How app email will be sent.',
            'icon' => 'fa fa-envelope',

            'inputs' => [
                [
                    'name' => 'from_email',
                    'type' => 'email',
                    'label' => 'From Email',
                    'placeholder' => 'Application from email',
                    'rules' => 'required|email',
                ],
                [
                    'name' => 'from_name',
                    'type' => 'text',
                    'label' => 'Email from Name',
                    'placeholder' => 'Email from Name',
                ]
            ]
        ]
    ],

    // Setting page url, will be used for get and post request
    'url' => 'settings',

    // Any middleware you want to run on above route
    'middleware' => ['auth:web'],

    // View settings
//    'setting_page_view' => 'app_settings::settings_page',
    'setting_page_view' => 'system_settings.settings',
    'flash_partial' => 'app_settings::_flash',

    // Setting section class setting
    'section_class' => 'card mb-3',
    'section_heading_class' => 'card-header',
    'section_body_class' => 'card-body row',

    // Input wrapper and group class setting
    'input_wrapper_class' => 'col-lg-6 mb-3',
    'input_class' => 'form-control',
    'input_error_class' => 'has-error',
    'input_invalid_class' => 'is-invalid',
    'input_hint_class' => 'form-text text-muted',
    'input_error_feedback_class' => 'text-danger',

    // Submit button
    'submit_btn_text' => 'Save Settings',
    'submit_success_message' => 'Settings has been saved.',

    // Remove any setting which declaration removed later from sections
    'remove_abandoned_settings' => false,

    // Controller to show and handle save setting
    'controller' => '\App\Http\Controllers\SettingController',

    // settings group
    'setting_group' => function() {
        // return 'user_'.auth()->id();
        return 'default';
    },

    //Set Administrator code
    'admin_group'    => 'administrator',

    //Default Components
    'app_name'  =>  env('APP_NAME'),
    'logo'  =>  'images/icons/Alleviate.png',
    'app_copyright' =>  'Alleviate @2022 Powered by <a href="https://bewsys.com" target="_blank"> Bewsys</a>.',
    'app_linkedIn_url' =>  'https://gh.linkedin.com/company/bewsys',
    'app_twitter_url' =>  'https://twitter.com/bewsys1?lang=en',
    'app_facebook_url' =>  'https://www.facebook.com/betterworldsystems/',
    'app_whatsApp_url' =>  'https://wa.me/233505668284',
];
