<x-app-layout>
    @push('vendor-css')
        <link rel="stylesheet" href="{{ asset('assets/plugins/custom/datatables/datatables.bundle.css') }}">
    @endpush

    <x-slot name="breadcrumb">
        <ol class="breadcrumb bg-transparent mb-0">
            <li class="breadcrumb-item"><a class="text-secondary" href="{{ route('dashboard') }}">Dashboard</a></li>
            <li class="breadcrumb-item"><a class="text-secondary" href="{{ route('benefit_registration.list') }}">Applications</a>
            </li>
            <li class="breadcrumb-item active" aria-current="page">{{ __('Eligibility') }} @if($result)
                    {{ " - ".$result->titles }}
                @endif</li>
        </ol>
    </x-slot>
    <!-- start: page toolbar -->
    <x-slot name="page_heading">
        <div class="row align-items-center">
            <div class="col-auto page-header-title">
                <h1 class="color-900 mt-1 mb-0">{{ __('Eligibility') }} @if($result)
                        {{ " - ".$result->titles }}
                    @endif</h1>
                <small class="text-muted">You have 12 new messages and 7 new notifications.</small>
            </div>
            <div class="col d-flex justify-content-lg-end mt-2 mt-md-0">
            </div>
        </div> <!-- .row end -->
    </x-slot>

    <div class="page-body px-xl-4 px-sm-2 px-0 py-lg- py-1 mt-0 mt-lg-">
        <div class="container-fluid">
            @component('components.announcement')@endcomponent

            <!--begin::Row-->
            <div class="row g-3">
                @if(true)
                    <div class="col-12">
                        <div class="card">
                            <div class="card-body">
                                <small class="pb-1">In the Program Eligibility module, select a benefit from the drop-down list, then click "Get Eligible Applicants" to retrieve the list of individuals who qualify for the selected social protection program. This list will be used for further processing.</small>
                                <form action="#" method="get">
                                    @csrf
                                    <div class="input-group">
                                        <select name="benefit" id="benefit" class="form-select">
                                            <option value="">{{ __('-- Select Program --') }}</option>
                                            @foreach($benefits as $benefit)
                                                <option value="{{$benefit->id}}"
                                                        {{ (request()->get('benefit') == $benefit->id)?'selected':'' }}>{{ $benefit->titles }}</option>
                                            @endforeach
                                        </select>
                                        <button class="btn btn-secondary" type="submit"><i class="fa fa-search"></i> Get
                                            Eligible Applicants
                                        </button>
                                    </div>
                                </form>
                            </div>
                        </div>
                    </div>
                @endif

                @if ($result)
                    <form action="{{ route('benefit.eligibility.group') }}" method="post" class="d-none" id="eligibility-group-app">
                        @csrf
                        <input type="text" value="{{request()->get('benefit')}}" name="benefit">
                        @foreach ($applicants as $data)
                            <input type="text" name="applicants[]" value="{{$data->id}}" id="" class="form-control">
                        @endforeach
                    </form>
                    
                @endif

                @if($result)
                    <div class="col-12">
                        <div class="card mb-5">
                            <div class="card-body">
                                <p class="text-info fw-bolder">Displaying memberships who match the criteria below</p>
                                {!! $note !!}
                            </div>
                        </div>
                        
                        
                        <div class="table-responsiv mt-10">
                            <table id="forms_table"
                                   class="table table-sm myDataTable align-middle table-bordered mb-3 custom-table">
                                <thead>
                                <tr>
                                    <th style="width: 30px">{{ __('#') }}</th>
                                    <th>{{ __('Membership ID') }}</th>
                                    <th style="width: 80px;">{{ __('Status') }}</th>
                                    <th style="width: 80px;">{{ __('Action') }}</th>
                                </tr>
                                </thead>
                                <tbody>
                                @php $count=1; @endphp
                                @foreach($applicants as $applicant)
                                    <tr>
                                        <td>{{$count++}}</td>
                                        <td>{{$applicant->regID}}</td>
                                        <td>
                                            @if($applicant->applications->where('benefit_id','=',$result->id)->count() > 0)
                                                <span class="text-danger">Enrolled</span>
                                            @else
                                                <span class="text-success">Eligible</span>
                                            @endif
                                        </td>
                                        <td>
                                            @if(canDo('administrator','view-member-info'))
                                                <a href="{{route('applicant.viewApplicantInfo',['app_id'=>cryptThis($applicant->id)])}}"
                                                   data-toggle="tooltip" title="{{ __('View Membership') }}"
                                                   class="btn btn-link text-primary">
                                                    <i class="fa fa-user"></i></a>
                                            @endif
                                        </td>
                                    </tr>
                                @endforeach
                                </tbody>
                            </table>
                        </div>
                        {{ $applicants->links() }}
                        <button class="btn btn-primary" onclick="document.getElementById('eligibility-group-app').submit();">Apply</button>
                        <div class="table-responsiv mt-10">
                            <table id=""class="mt-2 table table-sm myDataTable align-middle table-bordered mb-3 custom-table">
                                <thead>
                                <tr>
                                    <th style="">{{ __('Date') }}</th>
                                    <th>{{ __('Application ID') }}</th>
                                    <th style="">{{ __('Group ID') }}</th>
                                    <th style="">{{ __('Status') }}</th>
                                    <th style="">{{ __('Applicants') }}</th>
                                    <th style="">{{ __('Action') }}</th>
                                </tr>
                                </thead>
                                <tbody>
                                
                                </tbody>
                            </table>
                        </div>
                    </div>
                @else
                    @include('components.no-data')
                @endif
            </div>
        </div>
    </div>


    <!--end::Row-->
    @push('vendor-scripts')
    @endpush
    @push('scripts')
        <script src="{{ asset('assets/js/bundle/dataTables.bundle.js') }}"></script>
        <script>
            $('#forms_table').addClass('nowrap').dataTable({
                responsive: false,
                searching: false,
                paging: false,
                ordering: false,
                info: false,
            });
        </script>
    @endpush
</x-app-layout>
