<x-app-layout>
    @push('vendor-css')

    @endpush

    <x-slot name="breadcrumb">
        <ol class="breadcrumb bg-transparent mb-0">
            <li class="breadcrumb-item"><a class="text-secondary"
                                           href="{{ route('dashboard') }}">{{__('Deployments')}}</a></li>
            {{--            <li class="breadcrumb-item"><a class="text-secondary" href="{{ route('case.list') }}">Cases</a></li>--}}
            <li class="breadcrumb-item active" aria-current="page">{{__('Set-up New Deployment')}}</li>
        </ol>
    </x-slot>
    <div class="page-body px-xl-4 px-sm-2 px-0 py-lg- py- mt-0 mt-lg-">
        <div class="container-fluid">
            @component('components.announcement')@endcomponent
            <div class="card card-custom">
                <div class="card-body">
                    @if(!$deployment->isReadyForDeployment())
                        <h3 class="text-danger text-center fw-lighter mt-4 mb-4">
                            {{__('This Project is not ready for deployment.')}}
                            <br>
                            <br>
                            <small class="fs-6"><a
                                        href="{{route('deployment.projectSettings',[$deployment->client_id,$deployment->id])}}"> {{__('Click here to complete.')}}</a></small>
                        </h3>
                    @else
                        @php
                            $package = $domain = $username = $password  = $expiry = $abbreviation = $db_name = $db_user = $db_password = null;
                            if (!is_null($deployment->package)) $package = $deployment->package;
                            if (!is_null($deployment->domain)) {
                                if ($deployment->self_domain) {
                                    $domain = $deployment->domain;
                                }
                                else {
                                    $domain = getDeploymentUrl($deployment);
                                }
                            }
                            if (!is_null($deployment->username)) $username = $deployment->username;
                            if (!is_null($deployment->password)) $password = getRandomString(8);
                            if (!is_null($deployment->abbreviation)) $abbreviation = $deployment->abbreviation;
                            if (!is_null($deployment->db_name)) $db_name = $deployment->db_name;
                            if (!is_null($deployment->db_user)) $db_user = $deployment->db_user;
                            if (!is_null($deployment->db_password)) $db_password = $deployment->db_password;
                        @endphp
                        <form action="{{route('deployment.save')}}" method="POST" enctype="multipart/form-data">@csrf
                            <input type="hidden" name="deployment_id" value="{{$deployment->id}}">
                            <div class="row">
                                <div class="col-12 mb-3">
                                    <h6 class="card-title">{{ __('Complete Deployment Request') }}</h6>
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="domain" class="form-label">{{ __('Domain URL') }}</label>
                                    <input class="form-control" disabled value="{{$domain}}">
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="package" class="form-label">{{ __('Subscription Plan') }}</label>
                                    <input class="form-control" disabled value="{{Str::headline($package)}}">
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="subscription_interval"
                                           class="form-label">{{ __('Subscription Interval') }}</label>
                                    <select name="subscription_interval" class="form-select" id="subscription_interval">
                                        <option value="1" @selected($deployment->subscription_interval == 1)>
                                            Per Monthly
                                        </option>
                                        <option value="12" @selected($deployment->subscription_interval == 12)>
                                            Per Year
                                        </option>
                                    </select>
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="abbreviation" class="form-label">{{ __('Abbreviation') }}<span
                                                class="required">*</span></label>
                                    <input type="text" class="form-control" required name="abbreviation"
                                           id="abbreviation" maxlength="3"
                                           value="{{old('abbreviation')?old('abbreviation'):$abbreviation}}">
                                    @if ($errors->has('abbreviation'))
                                        <p class="text-right">
                                            <span class="text-warning form-text">{{ $errors->first('abbreviation') }}</span>
                                        </p>
                                    @endif
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="db_name" class="form-label">{{ __('Database Name') }}<span
                                                class="required">*</span></label>
                                    <input type="text" class="form-control" required name="db_name" id="db_name"
                                           value="{{old('db_name')?old('db_name'):$db_name}}">
                                    @if ($errors->has('db_name'))
                                        <p class="text-right">
                                            <span class="text-warning form-text">{{ $errors->first('db_name') }}</span>
                                        </p>
                                    @endif
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="db_user" class="form-label">{{ __('Database User') }}<span
                                                class="required">*</span></label>
                                    <input type="text" class="form-control" required name="db_user" id="db_user"
                                           value="{{old('db_user')?old('db_user'):$db_user}}">
                                    @if ($errors->has('db_user'))
                                        <p class="text-right">
                                            <span class="text-warning form-text">{{ $errors->first('db_user') }}</span>
                                        </p>
                                    @endif
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="db_password" class="form-label">{{ __('Database Password') }}<span
                                                class="required">*</span></label>
                                    <input type="text" class="form-control" required name="db_password" id="db_password"
                                           value="{{old('db_password')?old('db_password'):$db_password}}">
                                    @if ($errors->has('db_password'))
                                        <p class="text-right">
                                            <span class="text-warning form-text">{{ $errors->first('db_password') }}</span>
                                        </p>
                                    @endif
                                </div>
                                <div class="col-md-3 mb-3">
                                    <label for="username" class="form-label">{{ __('Admin Email') }}<span
                                                class="required">*</span></label>
                                    <input type="email" class="form-control" required name="username" id="username"
                                           value="{{old('username')?old('username'):$username}}">
                                    @if ($errors->has('username'))
                                        <p class="text-right">
                                            <span class="text-warning form-text">{{ $errors->first('username') }}</span>
                                        </p>
                                    @endif
                                </div>
                                <div class="col-md-12 mt-3">
                                    <button type="submit" class="btn btn-primary"><i
                                                class="fa fa-cloud-upload mx-1"></i>
                                        @if ($deployment->instance == 'Suspended')
                                            {{ __('Submit') }}
                                        @else
                                            {{ __('Submit') }}
                                        @endif
                                    </button>
                                    <a href="{{route('deployment.projectSettings',[$deployment->client_id,$deployment->id])}}"
                                       class="btn btn-light-primary" data-bs-toggle="tooltip"
                                       data-bs-placement="top" title="{{__('Configurations')}}"><i
                                                class="fa fa-cogs fs-6"></i> Configurations</a>
                                    <a href="{{ route('dashboard') }}" class="btn btn-light-secondary"><i
                                                class="fa fa-arrow-left fs-6"></i> {{ __('Back to Deployments') }}</a>
                                </div>
                            </div>
                        </form>

                        <div class="mt-4 bg-light p-3">
                            <h6 class="mb-4"><span class="text-uppercase mb-4">{{__('Environment Variables')}}</span>
                                <br>
                                <small class="text-muted fw-light">Copy and paste or replace the variables in the '.env'
                                    file</small>
                            </h6>

                            <div class="p-4 bg-dark"
                                 style="color: #e3e3e3; font-size: 13px; font-family: 'Source Code Pro', monospace, Calibri, sans-serif; line-height: 25px;">
                                <code class="m-0">
                                    APP_NAME="{{$deployment->configurations['app_project_name']}}" <br>
                                    APP_ABBREV={{$deployment->abbreviation}} <br>
                                    APP_KEY=base64:DgZ7fjo3AKcqptewsodNiQlzurNzMg3lZRPYx9WeRlY= <br>
                                    APP_ENV=production<br>
                                    APP_URL={{getDeploymentUrl($deployment)}} <br>
                                    APP_DEBUG=false <br>
                                    <br>
                                    SYSTEM_DEBUG_MODE=false <br>
                                    CONSOLE_API=https://my.alleviate.pro/api/ <br>
                                    END_SUBSCRIPTION_URL=/trial-expired <br>
                                    CONSOLE_DEPLOYMENT_CODE={{$deployment->code}}<br>
                                    <br>
                                    ALLOW_MULTI_LOGIN=off<br>
                                    PASSWORD_DURATION_LIMIT=30<br>
                                    <br>
                                    DEFAULT_DEPLOYMENT_URL=alleviate.pro<br>
                                    DEFAULT_COUNTRY_ID={{$deployment->configurations['app_country']}}<br>
                                    DEFAULT_CALL_CODE={{$deployment->configurations['app_main_currency_code']}}<br>
                                    DEFAULT_CURRENCY_SYMBOL={{$deployment->configurations['app_main_currency_code']}}
                                    <br>
                                    DEFAULT_CURRENCY_CODE={{$deployment->configurations['app_main_currency_code']}}<br>
                                    <br>
                                    DB_CONNECTION=mysql<br>
                                    DB_HOST=127.0.0.1<br>
                                    DB_PORT=3306<br>
                                    DB_DATABASE={{$deployment->db_name}}<br>
                                    DB_USERNAME={{$deployment->db_user}}<br>
                                    DB_PASSWORD={{$deployment->db_password}}<br>
                                    <br>
                                    LOG_CHANNEL=daily<br>
                                    LOG_DEPRECATIONS_CHANNEL=null<br>
                                    LOG_LEVEL=debug<br>
                                    <br>
                                    BROADCAST_DRIVER=log<br>
                                    CACHE_DRIVER=file<br>
                                    FILESYSTEM_DRIVER=local<br>
                                    QUEUE_CONNECTION=sync<br>
                                    SESSION_DRIVER=file<br>
                                    SESSION_LIFETIME=120<br>
                                    <br>
                                    MAIL_MAILER=smtp<br>
                                    MAIL_HOST=mail.alleviate.pro<br>
                                    MAIL_PORT=465<br>
                                    MAIL_USERNAME=info@alleviate.pro<br>
                                    MAIL_PASSWORD=+=[oAd;258M$<br>
                                    MAIL_ENCRYPTION=tls<br>
                                    MAIL_FROM_ADDRESS='info@alleviate.pro'<br>
                                    MAIL_FROM_NAME="${APP_NAME}"<br>
                                    <br>
                                    SMS_SENDER_ID=ALLEVIATE<br>
                                    SMS_API_KEY=62ab11bcae4fe7a31c7931d1c5459ba892dc91bdc200551e52a3c23006a21dc9<br>
                                    SMS_URI=https://api.smsonlinegh.com/v4/message/sms/send<br>
                                </code>
                            </div>
                        </div>
                    @endif
                </div>
            </div>
        </div>
    </div>

    @push('vendor-scripts')

    @endpush
    @push('scripts')
        <script src="{{asset('assets/plugins/custom/ckeditor/ckeditor-classic.bundle.js')}}"
                type="text/javascript"></script>
        <script>
            $(function () {
                ClassicEditor.create(document.querySelector('#description'));
            });
        </script>
    @endpush
</x-app-layout>
