<x-app-layout>
    <x-slot name="breadcrumb">
        <ol class="breadcrumb bg-transparent mb-0">
            <li class="breadcrumb-item"><a class="text-secondary" href="{{ route('dashboard') }}">Dashboard</a></li>
            <li class="breadcrumb-item"><a class="text-secondary" href="{{ route('partner.list') }}">Partners</a></li>
            <li class="breadcrumb-item active" aria-current="page">{{$page_title}} - {{$partner->name}}</li>
        </ol>
    </x-slot>
    <!-- start: page toolbar -->
    <x-slot name="page_heading">
        <div class="row align-items-center">
            <div class="col-auto page-header-title">
                <h1 class="color-900 mt-1 mb-0">{{$partner->name}}</h1>
                <small class="text-muted">You have 12 new messages and 7 new notifications.</small>
            </div>
            <div class="col d-flex justify-content-lg-end mt-2 mt-md-0">
                <div class="p-2 me-md-3">
                    <div><span class="h6 mb-0">8.18K</span> <small class="text-secondary"><i class="fa fa-angle-up"></i>
                            1.3%</small></div>
                    <small class="text-muted text-uppercase">Active</small>
                </div>
                <div class="p-2 me-md-3">
                    <div><span class="h6 mb-0">1.11K</span> <small class="text-secondary"><i class="fa fa-angle-up"></i>
                            4.1%</small></div>
                    <small class="text-muted text-uppercase">Inactive</small>
                </div>
            </div>
        </div> <!-- .row end -->
    </x-slot>

    <div class="page-body px-xl-4 px-sm-2 px-0 py-lg- py-1 mt-0 mt-lg-">
        <div class="container-fluid">
            @if(session()->has('message'))
                <div class="mb-5">
                    @if(session()->get('status') == 'success')
                        <div role="alert" class="alert alert-success">{{ session()->get('message') }}</div>
                    @else
                        <div role="alert" class="alert alert-danger">{{ session()->get('message') }}</div>
                    @endif
                </div>
            @endif

            <div class="row g-3 mt-4">
                <div class="col-md-12">
                    @if(canDo('administrator','edit-partner'))
                    <div class="dropdown text-end mb-3">
                        <button class="btn btn-light-primary dropdown-toggle" type="button" id="dropdownMenuButton" data-bs-toggle="dropdown" aria-expanded="false">
                            <i class="fa fa-cog"></i>
                        </button>
                        <ul class="dropdown-menu border-0 shadow">
                            <li><a class="dropdown-item py-2 rounded" href="{{route('partner.edit',[cryptThis($partner->id)])}}">Edit</a></li>
                        </ul>
                    </div>
                    @endif

                    <div class="card">
                        <div class="card-body">
                            <div class="row">
                                <div class="col-md-3">
                                    @php
                                        $logo = asset('assets/PlaceholderLogo.png');
                                        if($partner->logo_url != null) {
                                            $logo = asset($partner->logo_url);
                                        }
                                    @endphp
                                    <img src="{{$logo}}" alt="" class="circle no-filter" alt="{{$partner->name}}" style="width: 150px; height: 150px; border-radius: 100%;">
                                </div>
                                <div class="col-md-9">
                                    <p>
                                        <span class="fw-bold me-3">Type:</span> {{$partner->_type->fullName}}
                                    </p>
                                    <p>
                                        <span class="fw-bold me-3">Contact:</span>
                                        <span class="mx-3"><i class="bi-phone"></i> {{$partner->phone}}</span>|
                                        <span class="mx-3"><i class="bi-envelope"></i> {{$partner->email}}</span>|
                                        <span class="mx-3"><i class="bi-globe"></i> {{$partner->website}}</span>
                                    </p>
                                    <p>
                                        <span class="fw-bold me-3">Status:</span> {{($partner->status == 1)?'Active':'Disabled'}}
                                    </p>
                                    <p>
                                        <span class="fw-bold me-2">Created By:</span> {{$partner->createdBy->fullName.' ('.easyDateNormal($partner->created_at,true).')'}}

                                        <span class="fw-bold mx-2">Last Updated By:</span> {{$partner->updatedBy?->fullName.' ('.easyDateNormal($partner->updated_at,true).')'}}
                                    </p>
                                </div>
                                {{-- <div class="col-md-12">

                                </div> --}}
                            </div>

                        </div>
                    </div>
                </div>

                <div class="col-md-12">
                    <div class="card">
                        <div class="card-body">
                            <h6 class="mb-4">Partner Description</h6>
                            {!! $partner->description !!}
                        </div>
                    </div>
                </div>
            </div>
            <div class="row g-3 mt-4">
                <div class="col-12">
                    <div class="card">
                        <div class="card-body">
                            <h6 class="mb-4">{{__('Programs')}}</h6>
                            @if ($partner->programs->count() > 0)
                            <ul class="list-group list-group-custom list-group-flush">
                                @foreach ($partner->programs as $program)
                                <li class="list-group-item d-flex justify-content-between align-items-center flex-wrap py-3">
                                    <p class="mb-0 fw-light">{{$program->name}}
                                    <br> <small class="text-muted">{{($program->status == 1)?'Active':'Disabled'}}</small>
                                    </p>
                                    @if(canDo('administrator','view-program-info'))
                                    <a href="{{route('program.view',[cryptThis($program->id)])}}" class="text-primary small">{{__('Open')}}</a>
                                    @endif
                                  </li>
                                @endforeach
                              </ul>
                            @else
                             <x-no-data></x-no-data>
                            @endif
                        </div>
                    </div>
                </div>
        </div>
    </div>

    @push('vendor-scripts')
    @endpush
    @push('scripts')
        <script src="{{ asset('assets/js/bundle/dataTables.bundle.js') }}"></script>
        <script src="{{ asset('assets/js/bundle/bootstrapdatepicker.bundle.js') }}"></script>
        <script src="{{ asset('assets/js/custom.js') }}"></script>
        <script src="{{ asset('assets/js/customjs/custom-programs-datatable.js') }}"></script>
    @endpush
</x-app-layout>
