<x-app-layout>
    @push('vendor-css')
        <link rel="stylesheet" href="{{ asset('assets/plugins/custom/datatables/datatables.bundle.css') }}">
    @endpush
    <x-slot name="page_title">
        {{ __('Payment Schedules') }}
    </x-slot>
    <x-slot name="breadcrumb">
        <ul class="breadcrumb breadcrumb-transparent breadcrumb-dot font-weight-bold p-0 my-2 font-size-sm">
            <li class="breadcrumb-item text-muted">
                <a href="{{ route('dashboard') }}" class="text-muted">{{ __('Dashboard') }}</a>
            </li>
            <li class="breadcrumb-item text-muted">
                <a href="" class="text-muted">{{ __('Payment Schedules') }}</a>
            </li>
        </ul>
    </x-slot>
    <!--begin::Row-->
        <div class="row">
            <div class="col-12 mb-7">
                <div class="card card-custom">
                    <div class="card-body">
                        <h5 class="text-primary">{{ __('Save Payment Schedule') }}</h5>
                        <form action="{{ route('payment.createPaymentSchedule') }}" method="get">
                            @csrf
                            <div class="row">
                                <div class="col-md-6">
                                    <div class="form-group{{ $errors->has('benefit') ? ' form-control-warning' : '' }}">
                                        <select name="benefit" class="form-control" required>
                                            <option>{{ __('Select Benefit') }}</option>
                                            @foreach($benefits as $benefit)
                                                <option value="{{ $benefit->id }}">{{ $benefit->titles }}</option>
                                            @endforeach
                                        </select>
                                        @if ($errors->has('benefit'))
                                            <span class="form-text text-warning">{{ $errors->first('benefit') }}</span>
                                        @endif
                                    </div>
                                </div>
                                <div class="col-md-6">
                                    <div class="form-group">
                                        <button class="btn btn-light-primary" type="submit">
                                            <i class="la la-box-open mr-1"></i> {{ __('Open') }}</button>
                                    </div>
                                </div>
                            </div>
                        </form>
                    </div>
                </div>
            </div>
            <div class="col-12 mb-7">
                <div class="card card-custom">
                    <div class="card-header">
                            <h3 class="card-title">{{ __('List of Payment Schedules') }}</h3>
                    </div>
                    <div class="card-body">
                        <table class="table table-sm table-bordered table-hover" id="forms_table" style="width: 100%;margin-top: 13px !important">
                            <thead>
                            <tr>
                                <th style="width: 100px;">{{ __('Benefit') }}</th>
                                <th style="width: 100px;">{{ __('Payment Type') }}</th>
                                <th style="width: 100px;">{{ __('Payment Days') }}</th>
                                <th style="width: 30px">{{ __('Action') }}</th>
                            </tr>
                            </thead>
                            <tbody>
                            @foreach($schedules as $schedule)
                                <tr>
                                    <td>{{$schedule->benefit->titles}}</td>
                                    <td>{{$schedule->type}}</td>
                                    <td>{{$schedule->day_of_payment}}</td>
                                    <td>
                                        {{--<a href="{{ route('payment.paymentSchedule') }}#" title="Edit"--}}
                                           {{--class="btn btn-outline-primary btn-icon btn-sm" onclick="editSchedule({{$schedule->id}})">--}}
                                            {{--<i class="la la-edit"></i></a>--}}

                                        <a class="btn btn-danger btn-icon btn-sm" href="javascript:void(0)"
                                           onclick="event.preventDefault(); destroy({{ $schedule->id }})" title="Delete">
                                            <i class="la la-trash"></i>
                                        </a>
                                        <form id="delete-form-{{$schedule->id}}" action="{{ route('payment.deletePaymentSchedule',[cryptThis($schedule->id)]) }}" method="POST" style="display: none;">
                                            @csrf {{method_field('DELETE')}}
                                        </form>
                                    </td>
                                </tr>
                            @endforeach
                            </tbody>
                        </table>
                        {{--                        {{ $forms->links() }}--}}
                    </div>
                </div>
            </div>
    </div>
    <!--end::Row-->
    @push('vendor-scripts')
        <script src="{{ asset('assets/plugins/custom/datatables/datatables.bundle.js') }}"></script>
    @endpush
    @push('scripts')
        <script>
            function destroy(key) {
                if (confirm("Data will be lost after Deletion. Delete?")){
                    document.getElementById('delete-form-'+key).submit();
                }
            }
            $('#forms_table').dataTable( {
                "ordering": false
            } );
            function editSchedule(key) {
                var path = "{{route('payment.editPaymentSchedule')}}";
{{--                var path_update = "{{route('user_management.permissions.update')}}";--}}
                $.ajaxSetup(    {
                    headers: {
                        'X-CSRF-Token': $('meta[name=_token]').attr('content')
                    }
                });
                $.ajax({
                    url: path,
                    type: 'GET',
                    data: {id:key},
                    beforeSend: function(){
                        $('#action-card').block({
                            message: '<div class="ft-loader icon-spin font-large-1"></div>',
                            overlayCSS: {
                                backgroundColor: '#ccc',
                                opacity: 0.8,
                                cursor: 'wait'
                            },
                            css: {
                                border: 0,
                                padding: 0,
                                backgroundColor: 'transparent'
                            }
                        });
                    },
                    success: function(data){
                        $('select#type').val(data.type);
                        $('select#benefit').val(data.benefit_id);
                        $('input#day_of_payment').val(data.day_of_payment);
                        $('input#title').val(data.title);
                        // $("form#add-form").attr("action", path_update);
                        $('div#hidden-input').html('<input type="hidden" name="id" value="'+data.id+'">');
                    },
                    complete:function(){
                        $('#action-card').unblock();
                    }
                    ,
                    error: function (data) {
                    }
                });
            }
        </script>
    @endpush
</x-app-layout>