<x-app-layout>
    @push('vendor-css')
        <link rel="stylesheet" href="{{ asset('assets/plugins/custom/datatables/datatables.bundle.css') }}">
    @endpush

        <x-slot name="breadcrumb">
            <ol class="breadcrumb bg-transparent mb-0">
                <li class="breadcrumb-item"><a class="text-secondary" href="{{ route('dashboard') }}">Dashboard</a></li>
                <li class="breadcrumb-item active" aria-current="page">Access Logs</li>
            </ol>
        </x-slot>
        <!-- start: page toolbar -->
        <x-slot name="page_heading">
            <div class="row align-items-center">
                <div class="col-auto">
                    <h1 class="fs-5 color-900 mt-1 mb-0">Access Logs</h1>
                    <small class="text-muted">You have 12 new messages and 7 new notifications.</small>
                </div>
                <div class="col d-flex justify-content-lg-end mt-2 mt-md-0">
                    <div class="p-2 me-md-3">
                        <div><span class="h6 mb-0">8.18K</span> <small class="text-secondary"><i class="fa fa-angle-up"></i>
                                1.3%</small></div>
                        <small class="text-muted text-uppercase">Active</small>
                    </div>
                    <div class="p-2 me-md-3">
                        <div><span class="h6 mb-0">1.11K</span> <small class="text-secondary"><i class="fa fa-angle-up"></i>
                                4.1%</small></div>
                        <small class="text-muted text-uppercase">Inactive</small>
                    </div>
                </div>
            </div> <!-- .row end -->
        </x-slot>

        <div class="page-body px-xl-4 px-sm-2 px-0 py-lg- py-1 mt-0 mt-lg-">
            <div class="container-fluid">
                @if(session()->has('message'))
                    <div class="mb-5">
                        @if(session()->get('status') == 'success')
                            <div role="alert" class="alert alert-success">{{ session()->get('message') }}</div>
                        @else
                            <div role="alert" class="alert alert-danger">{{ session()->get('message') }}</div>
                        @endif
                    </div>
                @endif
                <!--begin::Row-->
                <div class="row g-3">
                    <div class="col-12">
                        <div class="card">
                            <div class="card-body">
                                <form action="#" method="get">
                                    @csrf
                                    <div class="input-group" style="min-width: 200px">
                                        <select name="user" id="user" class="form-select select2" style="min-width: 200px">
                                            <option value="">{{ __('-- User --') }}</option>
                                            @foreach($users as $user)
                                                <option value="{{$user->id}}" {{ (request()->get('user') == $user->id)?'selected':'' }}>
                                                    {!! userName($user) !!}</option>
                                            @endforeach
                                        </select>
                                        <button class="btn btn-secondary" type="submit"><i class="fa fa-search"></i></button>
                                        <a href="{{ route('settings.accessLog') }}" class="btn btn-light-primary" ><i class="bi-arrow-repeat"></i></a>
                                    </div>
                                </form>
                            </div>
                        </div>
                    </div>

                    <div class="col-12">
                        <div class="table-responsive mt-10">
                            <table id="forms_table" class="table myDataTable align-middle table-bordered mb-3 custom-table">
                                <thead>
                                <tr>
                                    <th>{{ __('User') }}</th>
                                    <th>{{ __('IP Address') }}</th>
                                    <th>{{ __('Login Date') }}</th>
                                    <th>{{ __('Logout Date') }}</th>
                                </tr>
                                </thead>
                                <tbody>
                                @foreach($logs as $log)
                                    <tr>
                                        <td>{!! userName($log->user) !!}</td>
                                        <td>{{$log->ip_address}}</td>
                                        <td>{{date('d M Y H:i:s', strtotime($log->created_at))}}</td>
                                        <td>
                                            @if($log->updated_at != $log->created_at)
                                            {{date('d M Y H:i:s', strtotime($log->updated_at))}}
                                            @else
                                                <a href="{{ route('settings.forceLogout',[cryptThis($log->user_id),$log->id]) }}" class="text-danger">Force Logout</a>
                                            @endif
                                        </td>
                                    </tr>
                                @endforeach
                                </tbody>
                            </table>
                        </div>
                        {{ $logs->links('vendor.pagination.bootstrap-5') }}
                    </div>
                </div>
            </div>
        </div>


    <!--end::Row-->
    @push('vendor-scripts')
    @endpush
    @push('scripts')
            <script src="{{ asset('assets/js/bundle/dataTables.bundle.js') }}"></script>
        <script>
            $('#forms_table').addClass('nowrap').dataTable({
                responsive: false,
                searching: false,
                paging: false,
                ordering: true,
                info: false,
            });
        </script>
    @endpush
</x-app-layout>
