<x-app-layout>
    <x-slot name="page_title">
        {{ __('Permissions') }}
    </x-slot>
    <x-slot name="breadcrumb">
        <ul class="breadcrumb breadcrumb-transparent breadcrumb-dot font-weight-bold p-0 my-2 font-size-sm">
            <li class="breadcrumb-item text-muted">
                <a href="{{ route('dashboard') }}" class="text-muted">{{ __('Dashboard') }}</a>
            </li>
            <li class="breadcrumb-item text-muted">
                <a href="" class="text-muted">{{ __('Permissions') }}</a>
            </li>
        </ul>
    </x-slot>

    <!--begin::Row-->
    <div class="row">
        <div class="col-12 mb-7">
            <div class="card card-custom" id="action-card">
                <div class="card-header">
                    <h3 class="card-title">Add Permission</h3>
                </div>
                <div class="card-body">
                    <form id="add-form" action="{{route('user_management.permissions.create')}}" method="post">
                        @csrf
                        <div id="hidden-input"></div>
                        <div class="row">
                            <div class="col-md-6">
                                <div class="form-group{{ $errors->has('permission_name') ? ' form-control-warning' : '' }}">
                                    <label for="permission_name">{{ __('Permission Name') }}<span class="required">*</span></label>
                                    <input type="text" required min="2" name="permission_name" class="form-control" value="{{ old('permission_name') }}" id="permission_name">
                                    @if ($errors->has('permission_name'))
                                        {{--<p class="text-right">--}}
                                            <span class="form-text text-warning">{{ $errors->first('permission_name') }}</span>
                                        {{--</p>--}}
                                    @endif
                                </div>
                            </div>
                            <div class="col-md-6">
                                <div class="form-group{{ $errors->has('display_name') ? ' form-control-warning' : '' }}">
                                    <label for="display_name">{{ __('Permission Display Name') }} <span class="required">*</span></label>
                                    <input type="text" name="display_name" class="form-control" value="{{ old('display_name') }}" id="display_name">
                                    @if ($errors->has('display_name'))
                                        <span class="form-text text-warning">{{ $errors->first('display_name') }}</span>
                                    @endif
                                </div>
                            </div>
                            <div class="col-md-12">
                                <div class="form-group{{ $errors->has('permission_desp') ? ' form-control-warning' : '' }}">
                                    <label for="group_desp">{{ __('Permission Description') }}</label>
                                    <input type="text" name="permission_desp" placeholder="Permission Description" value="{{ old('permission_desp') }}" class="form-control" id="permission_desp">
                                    @if ($errors->has('permission_desp'))
                                        <span class="form-text text-warning">{{ $errors->first('permission_desp') }}</span>
                                    @endif
                                </div>
                            </div>
                        </div>
                        <div class="form-group">
                            <button class="btn btn-primary square" type="submit">
                                <i class="la la-save mr-1"></i> Save</button>
                        </div>
                    </form>
                </div>
            </div>
        </div>
        <div class="col-12">
            <div class="card card-custom">
                <div class="card-header">
                    <h3 class="card-title">{{ __('Permissions') }}</h3>
                </div>
                <div class="card-body">
                    <div class="table-responsive">
                        <table class="table table-hover" id="permissions_table">
                            <thead>
                            <tr>
                                <th width="50px">#</th>
                                <th>Permission Name</th>
                                <th>Permission code</th>
                                <th>Permission Description</th>
                                <th width="120px">Actions</th>
                            </tr>
                            </thead>
                            <tbody>
                            @if($permissions->count() > 0)
                                @php $counter=0; @endphp
                                @foreach($permissions as $permission)
                                    <tr>
                                        <td>{{$counter+=1}}</td>
                                        <td>{{$permission->display_name}}</td>
                                        <td>{{$permission->name}}</td>
                                        <td>{{$permission->description}}</td>
                                        <td>
                                            <button class="btn btn-icon btn-light-primary" href="#action-card" onclick="editPermission('{{$permission->id}}')">
                                                <i class="la la-edit"></i>
                                            </button>
                                            <button class="btn btn-icon btn-danger">
                                                <i class="la la-trash"></i>
                                            </button>
                                        </td>
                                    </tr>
                                @endforeach
                            @else
                                <tr>
                                    <td colspan="4" class="text-center">No Permission Set</td>
                                </tr>
                            @endif
                            </tbody>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>
    <!--end::Row-->
    @push('vendor-scripts')
    @endpush
    @push('scripts')
        <script>
            $('#permissions_table').dataTable( {
                "ordering": false
            } );
            function editPermission(key) {
                var path = "{{route('user_management.permissions.edit')}}";
                var path_update = "{{route('user_management.permissions.update')}}";
                $.ajaxSetup(    {
                    headers: {
                        'X-CSRF-Token': $('meta[name=_token]').attr('content')
                    }
                });
                $.ajax({
                    url: path,
                    type: 'GET',
                    data: {id:key},
                    beforeSend: function(){
                        $('#action-card').block({
                            message: '<div class="ft-loader icon-spin font-large-1"></div>',
                            overlayCSS: {
                                backgroundColor: '#ccc',
                                opacity: 0.8,
                                cursor: 'wait'
                            },
                            css: {
                                border: 0,
                                padding: 0,
                                backgroundColor: 'transparent'
                            }
                        });;
                    },
                    success: function(data){
                        $('input#permission_name').val(data.name);
                        $('input#display_name').val(data.display_name);
                        $('input#permission_desp').val(data.description);
                        $("form#add-form").attr("action", path_update);
                        $('div#hidden-input').html('<input type="hidden" name="id" value="'+data.id+'"><input type="hidden" name="_method" value="PUT">');
                    },
                    complete:function(){
                        $('#action-card').unblock();
                    }
                    ,
                    error: function (data) {
                    }
                });
            }
        </script>
    @endpush
</x-app-layout>