<?php
use \App\Http\Controllers\UsersController;
use \App\Http\Controllers\PermissionsController;
use \App\Http\Controllers\RolesController;
use Illuminate\Support\Facades\Route;
use \App\Http\Controllers\AjaxController;

Route::middleware(['auth'])->name('account.')->group(function () {
    Route::get('/get_higher_roles/{roleId}', [AjaxController::class, 'getHigherRoles']);
    Route::get('/get_role_supervisors/{roleId}', [AjaxController::class, 'getRoleSupervisors']);

    Route::prefix('account')->group(function () {

        Route::get('users', [UsersController::class, 'users'])
            ->name('users');

        Route::get('users/tree', [UsersController::class, 'hierarchy'])->name('hierarchy');

        Route::get('profile', [UsersController::class, 'profile'])
            ->name('profile');

        Route::get('profile/{userID}', [UsersController::class, 'userProfile'])
            ->name('userProfile');

        Route::get('create', [UsersController::class, 'addAccount'])
            ->name('addAccount');

        Route::post('save', [UsersController::class, 'savePersonal'])
            ->name('savePersonal');

        Route::post('activate-invitation', [UsersController::class, 'activateInvitation'])
            ->name('activateInvitation');

        Route::get('reset-account-password/{id}', [UsersController::class, 'resetPasswordRandom'])
            ->name('resetPasswordRandom');

        Route::get('user-deletion/{user}', [UsersController::class, 'deleteAccount'])
            ->name('deleteAccount');

        Route::get('account-activation', [UsersController::class, 'activation'])
            ->name('activation');

        Route::get('edit/{userID?}', [UsersController::class, 'editAccount'])
            ->name('editAccount');

        Route::get('edit-card-details', [UsersController::class, 'loadCardEditForm'])
            ->name('loadCardEditForm');

        Route::post('save-card', [UsersController::class, 'saveCard'])
            ->name('saveCard');

        Route::post('save-skills', [UsersController::class, 'saveSkills'])
            ->name('saveSkills');

        Route::post('change-password', [UsersController::class, 'changePassword'])
            ->name('changePassword');

        Route::delete('remove-card', [UsersController::class, 'removeCard'])
            ->name('removeCard');

        //Permissions Routes
        Route::group(['middleware' => ['check-permissions:administrator,roles']], function() {
            Route::get('/permissions', [PermissionsController::class, 'index'])->name('permissions');
            Route::post('/permissions/create', [PermissionsController::class, 'create'])->name('permissions.create');
            Route::get('/permissions/edit', [PermissionsController::class, 'edit'])->name('permissions.edit');
            Route::put('/permissions/update', [PermissionsController::class, 'update'])->name('permissions.update');

            //Roles Routes
            Route::get('/roles', [RolesController::class, 'index'])->name('roles');
            Route::post('/roles/create', [RolesController::class, 'create'])->name('roles.create');
            Route::get('/roles/edit', [RolesController::class, 'edit'])->name('roles.edit');
            Route::get('/roles/newFormView', [RolesController::class, 'emptyForm'])->name('roles.emptyForm');
            Route::get('/roles/load-permissions', [RolesController::class, 'loadPermissions'])->name('roles.loadPermissions');
            Route::put('/roles/update', [RolesController::class, 'update'])->name('roles.update');
            Route::get('/roles/remove/{id}', [RolesController::class, 'remove'])->name('role.remove');
            Route::get('/roles/activate/{id}', [RolesController::class, 'activation'])->name('role.activation');
            Route::get('/roles/tree', [RolesController::class, 'hierarchy'])->name('role.hierarchy');
            Route::get('/line_of_sight', [RolesController::class, 'line_of_sight'])->name('line_of_sight');
            Route::post('/roles/toggle_status', [RolesController::class, 'toggleStatus'])->name('roles.toggleStatus');
        });

        //Member Records
        Route::get('/support-tickets', [UsersController::class,'supportTickets'])->name('supportTickets');
        Route::get('/payslip', [UsersController::class,'paySlip'])->name('paySlip');
        Route::get('/payment-batch', [UsersController::class,'paymentBatch'])->name('paymentBatch');
        Route::get('/payment-voucher', [UsersController::class,'paymentVouchers'])->name('paymentVouchers');

    });
});
