<?php
use Illuminate\Support\Facades\Route;
use \App\Http\Controllers\CaseManagementControllerOld;
use \App\Http\Controllers\CaseManagementController;

$applicant = config('app.portal_base_url','portal');
//Benefits Management Routes
Route::name('case.')->group(function () use($applicant) {
    Route::prefix('cases')->group(function () {
        Route::get('/', [CaseManagementController::class,'index'])->name('list')->middleware(['check-permissions:administrator,view-cases-list']);
        Route::get('create', [CaseManagementController::class,'createCase'])->name('createCase')->middleware(['check-permissions:administrator,add-case']);
        Route::get('get-case-list', [CaseManagementController::class,'getActiveCasesList'])->name('getCasesList');
        Route::get('get-archived-case-list', [CaseManagementController::class,'getArchivedCasesList'])->name('getArchivedCasesList');
        Route::get('archives', [CaseManagementController::class,'archives'])->name('archives')->middleware(['check-permissions:administrator,archive-case']);
        Route::get('edit/{case_id}', [CaseManagementController::class,'editCase'])->name('editCase')->middleware(['check-permissions:administrator,edit-case']);
        Route::post('save', [CaseManagementController::class,'saveCase'])->name('saveCase')->middleware(['check-permissions:administrator,add-case|edit-case']);
        Route::get('view/{case_id}', [CaseManagementController::class,'viewCase'])->name('viewCase')->middleware(['check-permissions:administrator,view-case-info']);
        Route::get('delete/{case_id}', [CaseManagementController::class,'deleteCase'])->name('deleteCase')->middleware(['check-permissions:administrator,delete-case']);
        Route::get('archive/{case_id}', [CaseManagementController::class,'archiveCase'])->name('archiveCase')->middleware(['check-permissions:administrator,archive-case']);
        Route::post('assign/{case_id}', [CaseManagementController::class,'assignStage'])->name('assignStage');
        Route::post('save-response/{case_id}', [CaseManagementController::class,'saveResponse'])->name('saveResponse')->middleware(['check-permissions:administrator|beneficiary,respond-to-case']);
        Route::post('update-category/{case_id}', [CaseManagementController::class,'updateCategory'])->name('updateCategory')->middleware(['check-permissions:administrator,respond-to-case']);
        Route::post('update-severity/{case_id}', [CaseManagementController::class,'updateSeverity'])->name('updateSeverity')->middleware(['check-permissions:administrator,respond-to-case']);
        Route::post('update-type/{case_id}', [CaseManagementController::class,'updateType'])->name('updateType')->middleware(['check-permissions:administrator,respond-to-case']);
        Route::get('escalate/{case_id}', [CaseManagementController::class,'escalateCase'])->name('escalateCase')->middleware(['check-permissions:administrator|beneficiary,respond-to-case']);
    });

    Route::prefix($applicant)->group(function () {
        //Beneficiary Routes
        Route::get('my-support-tickets', [CaseManagementController::class,'myCases'])->name('myCases')->middleware(['customer-access']);
        Route::post('my-support-ticket/new', [CaseManagementController::class,'NewCase'])->name('NewCase')->middleware(['customer-access']);
        Route::get('my-support-ticket/view/{case_id}', [CaseManagementController::class,'viewMyCase'])->name('viewMyCase')->middleware(['customer-access']);
    });
});
